@@ -21,10 +21,8 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <math.h>
-#include <dirent.h>
 
-#define GRETLCLI
+#include <dirent.h>
 
 #include "libgretl.h"
 #include "monte_carlo.h"
@@ -32,15 +30,17 @@
 #include "system.h"
 #include "gretl_restrict.h"
 #include "gretl_func.h"
-#include "modelspec.h"
 #include "libset.h"
 #include "forecast.h"
 #include "cmd_private.h"
+#include "flow_control.h"
 #include "objstack.h"
 #include "gretl_xml.h"
 #include "gretl_panel.h"
 #include "usermat.h"
 #include "gretl_string_table.h"
+#include "dbread.h"
+#include "boxplots.h"
 
 #ifdef WIN32
 # include <windows.h>
@@ -51,11 +51,6 @@
 # include <unistd.h>
 #endif 
 
-enum {
-    ERRFATAL_AUTO,
-    ERRFATAL_FORCE
-};
-
 #ifdef HAVE_READLINE
 # include <readline/readline.h>
 /* readline functions from complete.c */
@@ -63,14 +58,14 @@ extern char *rl_gets (char **line_read, const char *prompt);
 extern void initialize_readline (void);
 #endif /* HAVE_READLINE */
 
+char datafile[MAXLEN];
 char cmdfile[MAXLEN];
 char outfile[MAXLEN];
 char hdrfile[MAXLEN];
 char syscmd[MAXLEN];
-PATHS paths;                  /* useful paths */
 PRN *cmdprn;
 FILE *fb;
-int errfatal, batch;
+int batch;
 int runit;
 int data_status;
 char linebak[MAXLINE];      /* for storing comments */
@@ -83,9 +78,10 @@ static int saved_object_action (const char *line, double ***pZ,
 				DATAINFO *pdinfo, MODEL **models,
 				PRN *prn);
 
-static void usage(void)
+static void usage (int err)
 {
-    logo();
+    logo(0);
+
     printf(_("\nYou may supply the name of a data file on the command line.\n"
 	     "Options:\n"
 	     " -b or --batch     Process a command script and exit.\n"
@@ -93,17 +89,17 @@ static void usage(void)
 	     " -h or --help      Print this info and exit.\n"
 	     " -v or --version   Print version info and exit.\n"
 	     " -e or --english   Force use of English rather than translation.\n"
-	     " -q or --basque    Force use of Basque translation.\n"
+	     " -q or --quiet     Print less verbose program information.\n"
 	     "Example of batch mode usage:\n"
 	     " gretlcli -b myfile.inp >myfile.out\n"
 	     "Example of run mode usage:\n"
 	     " gretlcli -r myfile.inp\n"));
-    exit(EXIT_SUCCESS);
-}
 
-static int cli_workdir_callback (const char *s)
-{
-    return set_gretl_work_dir(s, &paths);
+    if (err) {
+	exit(EXIT_FAILURE);
+    } else {
+	exit(EXIT_SUCCESS);
+    }
 }
 
 static void gretl_abort (char *line)
@@ -113,38 +109,51 @@ static void gretl_abort (char *line)
     exit(EXIT_FAILURE);
 }
 
-static void noalloc (const char *str)
+static void noalloc (void)
 {
-    fprintf(stderr, _("Couldn't allocate memory for %s\n"), str);
+    fputs(_("Out of memory!\n"), stderr);
     exit(EXIT_FAILURE);
 }
 
-static void file_get_line (char *line, CMD *cmd)
+static int file_get_line (ExecState *s)
 {
+    char *line = s->line;
+    int len = 0;
+
     clear(line, MAXLINE);
-    fgets(line, MAXLINE - 1, fb);
+    
+    if (fgets(line, MAXLINE, fb) == NULL) {
+	/* no more input from current source */
+	gretl_exec_state_uncomment(s);
+    } else {
+	len = strlen(line);
+    }
 
     if (*line == '\0') {
 	strcpy(line, "quit");
+    } else if (len == MAXLINE - 1 && line[len-1] != '\n') {
+	return E_TOOLONG;
     } else {
 	*linebak = 0;
-	strncat(linebak, line, MAXLINE - 1);
+	strncat(linebak, line, MAXLINE-1);
+	tailstrip(linebak);
     }
 
     if (!strncmp(line, "noecho", 6)) {
 	set_gretl_echo(0);
     }
 
-    if (gretl_echo_on() && cmd->ci == RUN && batch && *line == '(') {
+    if (gretl_echo_on() && s->cmd->ci == RUN && batch && *line == '(') {
 	printf("%s", line);
 	*linebak = 0;
     }
-}
 
-#ifdef ENABLE_NLS
+    return 0;
+}
 
 static void nls_init (void)
 {
+#ifdef ENABLE_NLS
 # ifdef WIN32
     char gretldir[MAXLEN], LOCALEDIR[MAXLEN];
 
@@ -162,39 +171,15 @@ static void nls_init (void)
     putenv("LC_NUMERIC=");
     setlocale(LC_NUMERIC, "");
     reset_local_decpoint();
-}
-
-static void force_language (int f)
-{
-    if (f == ENGLISH) {
-	setlocale(LC_ALL, "C");
-    } else if (f == BASQUE) {
-# ifdef WIN32
-	setlocale(LC_ALL, "eu");
-# else
-	setlocale(LC_ALL, "eu_ES");
-# endif
-    }
-
-# ifdef WIN32
-    if (f == ENGLISH) { 
-	SetEnvironmentVariable("LC_ALL", "C");
-	putenv("LC_ALL=C");
-    } else if (f == BASQUE) {
-	SetEnvironmentVariable("LC_ALL", "eu");
-	putenv("LC_ALL=eu");
-    }	
-# endif
-}
-
 #endif /* ENABLE_NLS */
+}
 
 static int cli_clear_data (CMD *cmd, double ***pZ, DATAINFO *pdinfo,
 			   MODEL **models)
 {
     int err = 0;
 
-    *paths.datfile = 0;
+    *datafile = 0;
 
     if (pZ != NULL && *pZ != NULL) {
 	err = restore_full_sample(pZ, pdinfo, NULL); 
@@ -209,8 +194,6 @@ static int cli_clear_data (CMD *cmd, double ***pZ, DATAINFO *pdinfo,
     clear_model(models[0]);
     clear_model(models[1]);
 
-    free_modelspec();
-
     if (cmd->opt & OPT_P) {
 	libgretl_session_cleanup(SESSION_PRESERVE_MATRICES);
     } else {
@@ -230,48 +213,78 @@ static void get_a_filename (char *fname)
     fgets(fname, MAXLEN - 1, stdin);
 }
 
-static int 
-get_an_input_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
+static const char *get_prompt (ExecState *s)
+{
+    if (s->flags & DEBUG_EXEC) {
+	return "$ ";
+    } else if (gretl_compiling_function() ||
+	       gretl_compiling_loop()) {
+	return "> ";
+    } else {
+	return "? ";
+    }
+}
+
+/* this function is set up as it is to make it available for debugging
+   purposes */
+
+static int get_interactive_line (void *p)
+{
+    ExecState *s = (ExecState *) p;
+    const char *prompt = get_prompt(s);
+    int err = 0;
+
+#ifdef HAVE_READLINE
+    rl_gets(&line_read, prompt);
+
+    if (line_read == NULL) {
+	strcpy(s->line, "quit");
+    } else if (strlen(line_read) > MAXLINE - 2) {
+	err = E_TOOLONG;
+    } else {
+	*s->line = '\0';
+	strncat(s->line, line_read, MAXLINE - 2);
+	strcat(s->line, "\n");
+    }
+#else
+    printf("%s", prompt);
+    fflush(stdout);
+    file_get_line(s); /* note: "file" = stdin here */
+#endif
+
+    return err;
+}
+
+static int cli_get_input_line (ExecState *s)
 {
-    int coding = gretl_compiling_function() || 
-	gretl_compiling_loop();
     int err = 0;
 
     if (runit || batch) {
 	/* reading from script file */
-	file_get_line(s->line, s->cmd);
+	err = file_get_line(s);
     } else {
 	/* interactive use */
-#ifdef HAVE_READLINE
-	rl_gets(&line_read, (coding)? "> " : "? ");
-	if (line_read == NULL) {
-	    strcpy(s->line, "quit");
-	} else {
-	    strcpy(s->line, line_read);
-	}
-#else
-	printf("%s", (coding)? "> " : "? ");
-	fflush(stdout);
-	file_get_line(s->line, s->cmd); /* note: "file" = stdin here */
-#endif
+	err = get_interactive_line(s);
     }
 
     return err;
 }
 
-/* allow for backslash continuation of lines */
+/* allow for continuation of lines */
 
 static int maybe_get_input_line_continuation (char *line)
 {
     char tmp[MAXLINE];
-    int err = 0;
+    int contd, err = 0;
 
     if (!strncmp(line, "quit", 4)) {
 	return 0;
     }
 
-    while (top_n_tail(line)) {
-	tmp[0] = '\0';
+    contd = top_n_tail(line, MAXLINE, &err);
+
+    while (contd && !err) {
+	*tmp = '\0';
 
 	if (batch || runit) {
 	    fgets(tmp, MAXLINE - 1, fb);
@@ -286,41 +299,19 @@ static int maybe_get_input_line_continuation (char *line)
 
 	if (*tmp != '\0') {
 	    if (strlen(line) + strlen(tmp) > MAXLINE - 1) {
-		fprintf(stderr, _("Maximum length of command line "
-				  "(%d bytes) exceeded\n"), MAXLINE);
-		err = 1;
+		err = E_TOOLONG;
 		break;
 	    } else {
 		strcat(line, tmp);
 		compress_spaces(line);
 	    }
 	}
+	contd = top_n_tail(line, MAXLINE, &err);
     }
 
     return err;
 }
 
-static void set_errfatal (int code)
-{
-    static int hoe = -1;
-
-    if (hoe < 0) {
-	hoe = libset_get_bool(HALT_ON_ERR);
-    }
-
-    if (code == ERRFATAL_FORCE) {
-	/* for contexts where continuation on error is
-	   bound to make a big mess */
-	errfatal = 1;
-    } else if (hoe == 0) {
-	/* we've been explicitly told to keep going on error */
-	errfatal = 0;
-    } else {
-	/* default: errors are fatal in batch mode only */
-	errfatal = batch;
-    }
-}
-
 static int xout;
 
 #ifdef HAVE_READLINE
@@ -335,21 +326,23 @@ static int ctrl_x (int count, int key)
 
 int main (int argc, char *argv[])
 {
+#ifdef WIN32
+    char *callname = argv[0];
+#endif
     double **Z = NULL;
     DATAINFO *datainfo = NULL;
     MODEL **models = NULL;
     ExecState state;
     char *line = NULL;
-    int cli_get_data = 0;
+    int quiet = 0;
+    int load_datafile = 1;
     char filearg[MAXLEN];
     char runfile[MAXLEN];
     CMD cmd;
     PRN *prn;
     int err = 0;
 
-#ifdef ENABLE_NLS
     nls_init();
-#endif
 
 #ifdef HAVE_READLINE
     rl_bind_key(0x18, ctrl_x);
@@ -357,52 +350,60 @@ int main (int argc, char *argv[])
 
     datainfo = datainfo_new();
     if (datainfo == NULL) {
-	noalloc(_("data information"));
+	noalloc();
     }
-    
-    if (argc > 1) {
-	int force_lang = 0;
-	int opt = parseopt((const char **) argv, argc, filearg, &force_lang);
 
-	switch (opt) {
-	case OPT_BATCH:
-	    batch = 1;
-	    if (*filearg == '\0') usage();
-	    strcpy(runfile, filearg);
-	    cli_get_data = 1;
-	    break;
-	case OPT_HELP:
-	case OPT_DBOPEN:
-	case OPT_WEBDB:
-	    usage();
-	    break;
-	case OPT_VERSION:
-	    logo();
-	    exit(EXIT_SUCCESS);
-	    break;
-	case OPT_RUNIT:
-	    runit = 1;
-	    if (*filearg == '\0') {
-		usage();
-	    }
-	    strcpy(runfile, filearg); 
-	    cli_get_data = 1;
-	    break;
-	default:
-	    break;
-	}
-
-#ifdef ENABLE_NLS
-	if (force_lang) {
-	    force_language(force_lang);
-	    if (argc == 2) {
-		cli_get_data = 1;
-	    }
-	}
-#endif
+    if (argc < 2) {
+	load_datafile = 0;
     } else {
-	cli_get_data = 1;
-    }
+	gretlopt opt;
+
+	err = parseopt(&argc, &argv, &opt, filearg);
+
+	if (!err && (opt & (OPT_DBOPEN | OPT_WEBDB))) {
+	    /* catch GUI-only options */
+	    err = E_BADOPT;
+	}
+
+	if (err) {
+	    /* bad option */
+	    usage(1);
+	} else if (opt & (OPT_HELP | OPT_VERSION)) {
+	    /* we'll exit in these cases */
+	    if (opt & OPT_HELP) {
+		usage(0);
+	    } else {
+		logo(0);
+		exit(EXIT_SUCCESS);
+	    }
+	}
+	    
+	if (opt & (OPT_BATCH | OPT_RUNIT)) {
+	    if (*filearg == '\0') {
+		/* we're missing a filename argument */
+		usage(1);
+	    } else {
+		/* record argument (not a datafile) */
+		strcpy(runfile, filearg);
+		load_datafile = 0;
+		if (opt & OPT_BATCH) {
+		    batch = 1;
+		} else {
+		    runit = 1;
+		}
+	    }
+	} else if (*filearg == '\0') {
+	    load_datafile = 0;
+	}
+
+	if (opt & OPT_QUIET) {
+	    quiet = 1;
+	}
+
+	if (opt & OPT_ENGLISH) {
+	    force_language(LANG_C);
+	}
+    } 
 
 #ifdef WIN32
     if (!batch) {
@@ -411,31 +412,28 @@ int main (int argc, char *argv[])
 #endif
 
     libgretl_init();
-    logo(); 
+
+    logo(quiet); 
     session_time(NULL);
 
     prn = gretl_print_new(GRETL_PRINT_STDOUT, &err);
     if (err) {
-	noalloc(_("output stream"));
+	noalloc();
     }
 
     line = malloc(MAXLINE);
     if (line == NULL) {
-	noalloc(_("command line"));
+	noalloc();
     } 
 
-    gretl_set_paths(&paths, OPT_D); /* defaults, not gui */
-
 #ifdef WIN32
-    cli_read_registry(argv[0], &paths);
+    cli_read_registry(callname);
 #else
-    cli_read_rc(&paths);
+    cli_read_rc();
 #endif /* WIN32 */
 
-    set_workdir_callback(cli_workdir_callback);
-
     if (!batch) {
-	strcpy(cmdfile, gretl_work_dir());
+	strcpy(cmdfile, gretl_workdir());
 	strcat(cmdfile, "session.inp");
 	cmdprn = gretl_print_new_with_filename(cmdfile, &err);
 	if (err) {
@@ -444,14 +442,14 @@ int main (int argc, char *argv[])
 	}
     }
 
-    if (!cli_get_data) {
+    if (load_datafile) {
 	char given_file[MAXLEN];
 	int ftype;
 
 	strcpy(given_file, filearg);
-	strcpy(paths.datfile, filearg);
+	strcpy(datafile, filearg);
 
-	ftype = detect_filetype(paths.datfile, &paths, prn);
+	ftype = detect_filetype(datafile, OPT_P);
 
 	switch (ftype) {
 	case GRETL_UNRECOGNIZED:
@@ -460,41 +458,44 @@ int main (int argc, char *argv[])
 	    exit(EXIT_FAILURE);
 	    break;
 	case GRETL_NATIVE_DATA:
-	    err = gretl_get_data(&Z, datainfo, paths.datfile, &paths, 
-				 OPT_NONE, prn);
+	    err = gretl_get_data(datafile, &Z, datainfo, OPT_NONE, prn);
 	    break;
 	case GRETL_XML_DATA:
-	    err = gretl_read_gdt(&Z, datainfo, paths.datfile, &paths, 
-				 OPT_NONE, prn);
+	    err = gretl_read_gdt(datafile, &Z, datainfo, OPT_NONE, prn);
 	    break;
-	case GRETL_CSV_DATA:
-	    err = import_csv(&Z, datainfo, paths.datfile, OPT_NONE, prn);
-	    break;
-	case GRETL_OCTAVE:
-	    err = import_octave(&Z, datainfo, paths.datfile, OPT_NONE, prn);
+	case GRETL_CSV:
+	    err = import_csv(datafile, &Z, datainfo, OPT_NONE, prn);
 	    break;
+	case GRETL_XLS:
 	case GRETL_GNUMERIC:
-	case GRETL_EXCEL:
-	case GRETL_WF1:
+	case GRETL_ODS:
+	    err = import_spreadsheet(datafile, ftype, NULL, NULL,
+				     &Z, datainfo, OPT_NONE, prn);
+	    break;
 	case GRETL_DTA:
-	    err = import_other(&Z, datainfo, ftype, paths.datfile, 
+	case GRETL_SAV:
+	case GRETL_SAS:
+	case GRETL_JMULTI:
+	case GRETL_OCTAVE:
+	case GRETL_WF1:
+	    err = import_other(datafile, ftype, &Z, datainfo, 
 			       OPT_NONE, prn);
 	    break;
 	case GRETL_SCRIPT:
 	    runit = 1;
-	    strcpy(runfile, paths.datfile); 
-	    clear(paths.datfile, MAXLEN);
-	    cli_get_data = 1;
+	    strcpy(runfile, datafile); 
+	    clear(datafile, MAXLEN);
+	    load_datafile = 0;
 	    break;
 	default:
 	    break;
 	}
 
-	if (!cli_get_data) {
+	if (load_datafile) {
 	    if (err) {
 		errmsg(err, prn);
 		if (err == E_FOPEN) {
-		    show_paths(&paths);
+		    show_paths();
 		}
 		return EXIT_FAILURE;
 	    }
@@ -508,11 +509,12 @@ int main (int argc, char *argv[])
     /* allocate memory for models */
     models = allocate_working_models(2);
     if (models == NULL) {
-	noalloc("models"); 
+	noalloc(); 
     }
 
     gretl_cmd_init(&cmd);
     gretl_exec_state_init(&state, 0, line, &cmd, models, prn);
+    set_debug_read_func(get_interactive_line);
 
     /* print list of variables */
     if (data_status) {
@@ -521,15 +523,15 @@ int main (int argc, char *argv[])
 
     /* check for help file */
     if (!batch) {
-	FILE *fp = fopen(paths.helpfile, "r");
+	const char *hpath = helpfile_path(GRETL_HELPFILE);
+	FILE *fp = fopen(hpath, "r");
 
 	if (fp != NULL) { 
 	    printf(_("\n\"help\" gives a list of commands\n"));
 	    fclose(fp);
 	} else {
-	    printf(_("help file %s is not accessible\n"), 
-		   paths.helpfile);
-	    show_paths(&paths);
+	    printf(_("help file %s is not accessible\n"), hpath);
+	    show_paths();
 	}
     } 
 
@@ -543,25 +545,22 @@ int main (int argc, char *argv[])
     }
 
 #ifdef HAVE_READLINE
-    if (!batch) initialize_readline();
+    /* note: this was conditional on !batch */
+    initialize_readline();
 #endif
 
     if (batch || runit) {
 	/* re-initialize: will be incremented by "run" cmd */
 	runit = 0;
-	sprintf(line, "run %s", runfile);
+	sprintf(line, "run %s\n", runfile);
 	exec_line(&state, &Z, datainfo);
     }
 
-    /* should we stop immediately on error, in batch mode? */
-    set_errfatal(ERRFATAL_AUTO);
-
     /* main command loop */
     while (cmd.ci != QUIT && fb != NULL && !xout) {
 	char linecopy[MAXLINE];
-	int overflow;
 
-	if (err && errfatal) {
+	if (err && gretl_error_is_fatal()) {
 	    gretl_abort(linecopy);
 	}
 
@@ -569,24 +568,34 @@ int main (int argc, char *argv[])
 	    if (gretl_loop_exec(&state, &Z, datainfo)) {
 		return 1;
 	    }
-	    set_errfatal(ERRFATAL_AUTO);
 	} else {
-	    err = get_an_input_line(&state, &Z, datainfo);
+	    err = cli_get_input_line(&state);
 	    if (err) {
 		errmsg(err, prn);
-		continue;
+		break;
 	    }
 	}
 
-	overflow = maybe_get_input_line_continuation(line); 
-	if (overflow) {
-	    break;
-	} else {
-	    strcpy(linecopy, line);
-	    err = exec_line(&state, &Z, datainfo);
-	}
+	if (!state.in_comment) {
+	    err = maybe_get_input_line_continuation(line); 
+	    if (err) {
+		errmsg(err, prn);
+		break;
+	    }
+	} 
+
+	strcpy(linecopy, line);
+	tailstrip(linecopy);
+	err = exec_line(&state, &Z, datainfo);
     } /* end of get commands loop */
 
+    if (!err) {
+	err = gretl_if_state_check(0);
+	if (err) {
+	    errmsg(err, prn);
+	}
+    }
+
     /* leak check -- try explicitly freeing all memory allocated */
 
     destroy_working_models(models, 2);
@@ -598,7 +607,6 @@ int main (int argc, char *argv[])
 
     free(line);
 
-    free_modelspec();
     gretl_print_destroy(prn);
     gretl_cmd_free(&cmd);
     libgretl_cleanup();
@@ -606,19 +614,8 @@ int main (int argc, char *argv[])
     return 0;
 }
 
-static void printf_strip (char *s)
+static void printline (const char *s)
 {
-    int i, n;
-
-    while (isspace((unsigned char) *s)) s++;
-
-    n = strlen(s);
-
-    for (i=n-1; i>0; i--) {
-	if (isspace(s[i]) || s[i] == '\r') s[i] = '\0';
-	else break;
-    }
-
     if (gretl_compiling_loop()) {
 	printf("> %s\n", s);
     } else {
@@ -626,45 +623,64 @@ static void printf_strip (char *s)
     }
 }
 
-static void cli_exec_callback (ExecState *s, double ***pZ,
-			       DATAINFO *pdinfo)
+static void cli_exec_callback (ExecState *s, void *ptr,
+			       GretlObjType type)
 {
     int ci = s->cmd->ci;
 
-    if (ci == VAR || ci == VECM) {
-	maybe_stack_var(s->var, s->cmd);
-    } else if (ci == END && !strcmp(s->cmd->param, "restrict")) {
-	maybe_stack_var(s->var, s->cmd);
+    if (ci == MODELTAB || ci == GRAPHPG) {
+	pprintf(s->prn, _("%s: command not available\n"), s->cmd->word);
     } 
+
+    /* otherwise, no-op */
+}
+
+static int cli_renumber_series (const char *s, double **Z, 
+				DATAINFO *pdinfo, PRN *prn)
+{
+    int err, fixmax = highest_numbered_var_in_saved_object(pdinfo);
+
+    err = renumber_series_with_checks(s, fixmax, Z, pdinfo, prn);
+    if (err) {
+	errmsg(err, prn);
+    }
+
+    return err;
 }
 
 static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 			    DATAINFO *pdinfo, MODEL **models,
 			    PRN *prn)
 {
-    char datfile[MAXLEN] = {0};
+    char newfile[MAXLEN] = {0};
     char response[3];
-    int k, dbdata = 0;
+    int ftype, dbdata = 0;
     int err = 0;
 
-    err = getopenfile(line, datfile, &paths, (cmd->opt & OPT_W)?
-		      OPT_W : OPT_NONE);
-    if (err) {
-	errmsg(err, prn);
-	return err;
+    if (!(cmd->opt & OPT_O)) {
+	/* not using ODBC */
+	err = getopenfile(line, newfile, (cmd->opt & OPT_W)?
+			  OPT_W : OPT_NONE);
+	if (err) {
+	    errmsg(err, prn);
+	    return err;
+	}
     }
 
     if (cmd->opt & OPT_W) {
-	k = GRETL_NATIVE_DB_WWW;
+	ftype = GRETL_NATIVE_DB_WWW;
+    } else if (cmd->opt & OPT_O) {
+	ftype = GRETL_ODBC;
     } else {
-	k = detect_filetype(datfile, &paths, prn);
+	ftype = detect_filetype(newfile, OPT_P);
     }
 
-    dbdata = (k == GRETL_NATIVE_DB || k == GRETL_NATIVE_DB_WWW ||
-	      k == GRETL_RATS_DB || k == GRETL_PCGIVE_DB);
+    dbdata = (ftype == GRETL_NATIVE_DB || ftype == GRETL_NATIVE_DB_WWW ||
+	      ftype == GRETL_RATS_DB || ftype == GRETL_PCGIVE_DB ||
+	      ftype == GRETL_ODBC);
 
     if (data_status && !batch && !dbdata && cmd->ci != APPEND &&
-	strcmp(datfile, paths.datfile)) {
+	strcmp(newfile, datafile)) {
 	fprintf(stderr, _("Opening a new data file closes the "
 			  "present one.  Proceed? (y/n) "));
 	fgets(response, sizeof response, stdin);
@@ -678,20 +694,21 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
 	cli_clear_data(cmd, pZ, pdinfo, models);
     } 
 
-    if (k == GRETL_CSV_DATA) {
-	err = import_csv(pZ, pdinfo, datfile, cmd->opt, prn);
-    } else if (k == GRETL_OCTAVE) {
-	err = import_octave(pZ, pdinfo, datfile, cmd->opt, prn);
-    } else if (WORKSHEET_IMPORT(k)) {
-	err = import_other(pZ, pdinfo, k, datfile, cmd->opt, prn);
-    } else if (k == GRETL_XML_DATA) {
-	err = gretl_read_gdt(pZ, pdinfo, datfile, &paths, 
-			     cmd->opt, prn);
+    if (ftype == GRETL_CSV) {
+	err = import_csv(newfile, pZ, pdinfo, cmd->opt, prn);
+    } else if (SPREADSHEET_IMPORT(ftype)) {
+	err = import_spreadsheet(newfile, ftype, cmd->list, cmd->extra,
+				 pZ, pdinfo, cmd->opt, prn);
+    } else if (OTHER_IMPORT(ftype)) {
+	err = import_other(newfile, ftype, pZ, pdinfo, cmd->opt, prn);
+    } else if (ftype == GRETL_XML_DATA) {
+	err = gretl_read_gdt(newfile, pZ, pdinfo, cmd->opt, prn);
+    } else if (ftype == GRETL_ODBC) {
+	err = set_odbc_dsn(line, prn);
     } else if (dbdata) {
-	err = set_db_name(datfile, k, &paths, prn);
+	err = set_db_name(newfile, ftype, prn);
     } else {
-	err = gretl_get_data(pZ, pdinfo, datfile, &paths, 
-			     cmd->opt, prn);
+	err = gretl_get_data(newfile, pZ, pdinfo, cmd->opt, prn);
     }
 
     if (err) {
@@ -700,28 +717,28 @@ static int cli_open_append (CMD *cmd, const char *line, double ***pZ,
     }
 
     if (!dbdata && cmd->ci != APPEND) {
-	strncpy(paths.datfile, datfile, MAXLEN - 1);
+	strncpy(datafile, newfile, MAXLEN - 1);
     }
 
     data_status = 1;
 
-    if (pdinfo->v > 0 && !dbdata) {
+    if (pdinfo->v > 0 && !dbdata && !(cmd->opt & OPT_Q)) {
 	varlist(pdinfo, prn);
     }
 
     return err;
 }
 
-static gretlopt plot_opt (gretlopt opt, int batch)
-{
-    if (batch) {
-	opt |= OPT_B;
-    }
-    return opt;
-}
-
 #define ENDRUN (NC + 1)
 
+/* exec_line: this is called to execute both interactive and script
+   commands.  Note that most commands get passed on to the libgretl
+   function gretl_cmd_exec(), but some commands that require special
+   action are dealt with here.
+
+   see also gui_exec_line() in gui2/library.c
+*/
+
 static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 {
     char *line = s->line;
@@ -747,26 +764,23 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	return err;
     } 
 
-    if (!s->in_comment) {
+    if (!s->in_comment && !cmd->context) {
 	/* catch requests relating to saved objects, which are not
 	   really "commands" as such */
 	k = saved_object_action(line, pZ, pdinfo, models, prn);
-	if (k == 1) return 0;   /* action was OK */
-	if (k == -1) return 1;  /* action was faulty */
-
-	/* are we ready for this? */
-	if (!data_status && !cmd_ignore(cmd) && !ready_for_command(line)) {
-	    fprintf(stderr, _("You must open a data file first\n"));
-	    return 1;
+	if (k == 1) {
+	    return 0; /* action was OK, or ignored */
+	} else if (k == -1) {
+	    return 1; /* action was faulty */
 	}
     }
 
     /* tell libgretl if we're in batch mode */
     gretl_set_batch_mode(batch);
 
-    /* if we're stacking commands for a loop, parse "lightly" */
     if (gretl_compiling_loop()) {
-	err = get_command_index(line, cmd, pdinfo);
+	/* if we're stacking commands for a loop, parse "lightly" */
+	err = get_command_index(line, cmd);
     } else {
 	err = parse_command_line(line, cmd, pZ, pdinfo);
     }
@@ -777,12 +791,11 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	return err;
     }
 
-    /* are we in a multi-line comment block? */
-    s->in_comment = (cmd_ignore(cmd))? 1 : 0;
+    gretl_exec_state_transcribe_flags(s, cmd);
 
     /* if in batch mode, echo comments from input */
     if (batch && cmd->ci == CMD_COMMENT && gretl_echo_on()) {
-	printf_strip(linebak);
+	printline(linebak);
     }
 
     if (cmd->ci < 0) {
@@ -806,22 +819,17 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
    
     if (cmd->ci == LOOP || gretl_compiling_loop()) {  
 	/* accumulating loop commands */
-	if (!ok_in_loop(cmd->ci)) {
-	    printf(_("Command '%s' ignored; not available in loop mode\n"), line);
-	} else {
-	    if (gretl_echo_on() && (!gretl_compiling_loop() || batch || runit)) {
-		eflag = gretl_compiling_loop()? CMD_STACKING : CMD_BATCH_MODE;
-		/* straight visual echo */
-		echo_cmd(cmd, pdinfo, line, eflag, prn);
-	    }
-	    err = gretl_loop_append_line(s, pZ, pdinfo);
-	    if (err) {
-		set_errfatal(ERRFATAL_FORCE);
-		errmsg(err, prn);
-	    } else if (!batch && !runit) {
-		/* echo to record */
-		echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
-	    }
+	if (gretl_echo_on() && (!gretl_compiling_loop() || batch || runit)) {
+	    eflag = gretl_compiling_loop()? CMD_STACKING : CMD_BATCH_MODE;
+	    /* straight visual echo */
+	    echo_cmd(cmd, pdinfo, line, eflag, prn);
+	}
+	err = gretl_loop_append_line(s, pZ, pdinfo);
+	if (err) {
+	    errmsg(err, prn);
+	} else if (!batch && !runit) {
+	    /* echo to record */
+	    echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
 	}
 	return err;
     }
@@ -834,72 +842,33 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     check_for_loop_only_options(cmd->ci, cmd->opt, prn);
 
-    s->callback = cli_exec_callback;
+    gretl_exec_state_set_callback(s, cli_exec_callback, OPT_NONE);
 
     switch (cmd->ci) {
 
     case DELEET:
+	k = 0;
 	if (cmd->opt & OPT_D) {
 	    err = db_delete_series_by_name(cmd->param, prn);
-	    if (err) {
-		errmsg(err, prn);
-	    } 
-	    break;
-	}
-	if (get_matrix_by_name(cmd->param)) {
-	    err = user_matrix_destroy_by_name(cmd->param, prn);
-	    if (err) {
-		errmsg(err, prn);
-	    } 
-	    break;
-	}
-	if (get_string_by_name(cmd->param)) {
-	    err = delete_saved_string(cmd->param, prn);
-	    if (err) {
-		errmsg(err, prn);
-	    } 
-	    break;
-	}	    
-	err = dataset_drop_listed_variables(cmd->list, pZ, pdinfo, 
+	} else if (*cmd->param != '\0') {
+	    err = gretl_delete_var_by_name(cmd->param, prn);
+	} else if (get_list_by_name(cmd->extra)) {
+	    err = delete_list_by_name(cmd->extra);
+	} else {
+	    err = dataset_drop_listed_variables(cmd->list, pZ, pdinfo, 
 					    &k, prn);
+	}
 	if (err) {
 	    errmsg(err, prn);
-	} else {
-	    if (k) {
-		pputs(prn, _("Take note: variables have been renumbered"));
-		pputc(prn, '\n');
-		maybe_list_vars(pdinfo, prn);
-	    }
-	}
-	break;
-
-    case GNUPLOT:
-    case SCATTERS:
-	if (cmd->ci == GNUPLOT) {
-	    if ((cmd->opt & OPT_Z) && 
-		(cmd->list[0] != 3 || 
-		 !gretl_isdummy(pdinfo->t1, pdinfo->t2, (*pZ)[cmd->list[3]]))) { 
-		pputs(prn, _("You must supply three variables, the last of "
-			     "which is a dummy variable\n(with values 1 or 0)\n"));
-		break;
-	    }
-	    
-	    err = gnuplot(cmd->list, cmd->param, (const double **) *pZ, 
-			  pdinfo, plot_opt(cmd->opt, batch));
-	} else {
-	    err = multi_scatters(cmd->list, (const double **) *pZ, pdinfo, 
-				 plot_opt(cmd->opt, batch));
-	}
-
-	if (err) {
-	    pputs(prn, _("gnuplot command failed\n"));
-	} else if (batch) {
-	    pprintf(prn, _("wrote %s\n"), gretl_plotfile());
+	} else if (k) {
+	    pputs(prn, _("Take note: variables have been renumbered"));
+	    pputc(prn, '\n');
+	    maybe_list_vars(pdinfo, prn);
 	}
 	break;
 
     case HELP:
-	cli_help(cmd->param, &paths, cmd->opt, prn);
+	cli_help(cmd->param, cmd->opt, prn);
 	break;
 
     case OPEN:
@@ -908,8 +877,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	break;
 
     case NULLDATA:
-	k = gretl_int_from_string(cmd->param, (const double **) *pZ, 
-				  pdinfo, &err);
+	k = gretl_int_from_string(cmd->param, &err);
 	if (!err && k < 2) {
 	    err = 1;
 	}
@@ -920,7 +888,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	cli_clear_data(cmd, pZ, pdinfo, models);
 	err = open_nulldata(pZ, pdinfo, data_status, k, prn);
 	if (err) { 
-	    pputs(prn, _("Failed to create empty data set\n"));
+	    errmsg(err, prn);
 	} else {
 	    data_status = 1;
 	}
@@ -948,17 +916,16 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
 	printf(_("type a filename to store output (enter to quit): "));
 	get_a_filename(outfile);
-	top_n_tail(outfile);
+	top_n_tail(outfile, 0, NULL);
 
 	if (*outfile != 0 && *outfile != '\n' && *outfile != '\r' 
 	    && strcmp(outfile, "q")) {
-	    const char *udir = gretl_work_dir();
+	    const char *udir = gretl_workdir();
 
 	    printf(_("writing session output to %s%s\n"), udir, outfile);
 #ifdef WIN32
-	    sprintf(syscmd, "\"%s\\gretlcli\" -b \"%s\" > \"%s%s\"", 
-		    paths.gretldir, cmdfile, udir, outfile);
-	    /* WinExec(syscmd, SW_SHOWMINIMIZED); */
+	    sprintf(syscmd, "\"%sgretlcli\" -b \"%s\" > \"%s%s\"", 
+		    gretl_home(), cmdfile, udir, outfile);
 	    system(syscmd);
 #else
 	    sprintf(syscmd, "gretlcli -b \"%s\" > \"%s%s\"", 
@@ -971,11 +938,18 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     case RUN:
     case INCLUDE:
-	err = getopenfile(line, runfile, &paths, OPT_S);
+	if (cmd->ci == INCLUDE) {
+	    err = getopenfile(line, runfile, OPT_I);
+	} else {
+	    err = getopenfile(line, runfile, OPT_S);
+	}
 	if (err) { 
 	    pputs(prn, _("Command is malformed\n"));
 	    break;
 	}
+	if (gretl_messages_on()) {
+	    pprintf(prn, " %s\n", runfile);
+	}
 	if (cmd->ci == INCLUDE && gretl_is_xml_file(runfile)) {
 	    err = load_user_XML_file(runfile);
 	    if (err) {
@@ -997,8 +971,11 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    fprintf(stderr, _("Couldn't open script \"%s\"\n"), runfile);
 	    fb = pop_input_file();
 	} else {
+	    gretl_set_current_dir(runfile);
 	    strcpy(s->runfile, runfile);
-	    fprintf(stderr, _("%s opened OK\n"), runfile);
+	    if (libset_get_bool(VERBOSE_INCLUDE)) {
+		pprintf(prn, _("%s opened OK\n"), runfile);
+	    }
 	    if (cmd->ci == INCLUDE) {
 		pprintf(cmdprn, "include \"%s\"\n", runfile);
 	    } else {
@@ -1013,33 +990,17 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    err = cli_clear_data(cmd, pZ, pdinfo, models);
 	    pputs(prn, _("Dataset cleared\n"));
 	    break;
+	} else if (cmd->aux == DS_RENUMBER) {
+	    err = cli_renumber_series(cmd->param, *pZ, pdinfo, prn);
+	    break;
 	}
-	/* else fall through */
+	/* else fall-through intended */
 
     default:
 	err = gretl_cmd_exec(s, pZ, pdinfo);
 	break;
     }
 
-    if (!err && (is_model_cmd(cmd->word) || s->alt_model)
-	&& !is_quiet_model_test(cmd->ci, cmd->opt)) { 
-	attach_subsample_to_model(models[0], pdinfo);
-#if MSPEC_DEBUG
-	fprintf(stderr, "\ngretlcli: saving spec: model.ID = %d, model_count = %d\n",
-		(models[0])->ID, get_model_count());
-#endif
-	if (is_model_cmd(cmd->word)) {
-	    err = modelspec_save(models[0]);
-	}
-	maybe_stack_model(models[0], cmd, prn);
-    }
-
-    if (system_save_flag_is_set(s->sys)) {
-	/* only warrants action in GUI program */
-	system_unset_save_flag(s->sys);
-	s->sys = NULL;
-    }
-
     if (!err && cmd->ci != QUIT && gretl_echo_on() && !batch && !old_runit) {
 	/* record a successful interactive command */
 	echo_cmd(cmd, pdinfo, line, CMD_RECORDING, cmdprn);
