@@ -24,8 +24,11 @@
 #include "database.h"
 #include "datafiles.h"
 #include "gretl_www.h"
+#include "gretl_untar.h"
 #include "menustate.h"
 #include "treeutils.h"
+#include "winstack.h"
+#include "toolbar.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -50,6 +53,7 @@ static int add_rats_db_series_list (windata_t *vwin);
 static int add_pcgive_db_series_list (windata_t *vwin);
 static dbwrapper *get_series_info (windata_t *vwin, int action);
 static int *db_get_selection_list (windata_t *vwin);
+static void gui_get_db_series (windata_t *vwin, int cmd);
 
 enum db_data_actions {
     DB_DISPLAY,
@@ -105,22 +109,6 @@ static void set_time_series (DATAINFO *pdinfo)
     }
 }
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
-typedef struct {
-    long frac;
-    short exp;
-} netfloat;
-
-float retrieve_float (netfloat nf)
-{
-    short exp = ntohs(nf.exp);
-    long frac = ntohl(nf.frac);
-    double receive = frac / 10e6;
-    
-    return ldexp(receive, exp);
-}
-#endif
-
 void show_network_error (windata_t *vwin)
 {
     const char *msg = gretl_errmsg_get();
@@ -199,20 +187,16 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbinfo)
     }
 
     if (dbinfo->structure == CROSS_SECTION) {
-	err = boxplots(list, NULL, dbZ, dbinfo, 0);
-	if (err) {
-	    errbox(_("boxplot command failed"));
-	}
+	err = boxplots(list, dbZ, dbinfo, OPT_NONE);
     } else {
 	err = gnuplot(list, NULL, (const double **) *dbZ, dbinfo,
 		      OPT_G | OPT_O | OPT_T);
-	if (err) {
-	    errbox(_("gnuplot command failed"));
-	}
     }
 
-    if (!err) {
-	register_graph();
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	register_graph(NULL);
     }
 
     free(list);
@@ -298,7 +282,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	}
 
 	/* is there already a var of this name? */
-	dbv = varindex(datainfo, sinfo->varname);
+	dbv = series_index(datainfo, sinfo->varname);
 	if (dbv < datainfo->v) {
 	    if (dw->nv == 1) {
 		resp = yes_no_dialog ("gretl",                      
@@ -324,7 +308,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	    xvec = expand_db_series(dbZ[v], sinfo, datainfo->pd);
 	} else if (sinfo->pd > datainfo->pd) {
 	    if (!chosen) {
-		data_compact_dialog(vwin->w, sinfo->pd, &datainfo->pd, NULL, 
+		data_compact_dialog(vwin->main, sinfo->pd, &datainfo->pd, NULL, 
 				    &method, NULL);
 		if (method == COMPACT_NONE) {
 		    if (!overwrite) {
@@ -427,23 +411,23 @@ add_dbdata (windata_t *vwin, double **dbZ, DATAINFO *dbinfo,
     }
 
     if (!err) {
-	register_data(NULL, NULL, 0);
+	register_data(DATA_APPENDED);
     }
 }
 
-static void gui_display_series (GtkWidget *w, windata_t *vwin)
+static void db_display_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_db_series(vwin, DB_DISPLAY, NULL);
+    gui_get_db_series(vwin, DB_DISPLAY);
 }
 
-static void gui_graph_series (GtkWidget *w, windata_t *vwin)
+static void db_graph_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_db_series(vwin, DB_GRAPH, NULL);
+    gui_get_db_series(vwin, DB_GRAPH);
 }
 
-static void gui_import_series (GtkWidget *w, windata_t *vwin)
+static void db_import_series (GtkWidget *w, windata_t *vwin)
 {
-    gui_get_db_series(vwin, DB_IMPORT, NULL);
+    gui_get_db_series(vwin, DB_IMPORT);
 }
 
 void sync_db_windows (void)
@@ -451,31 +435,35 @@ void sync_db_windows (void)
     const char *dname = get_db_name();
 
     if (*dname != '\0') {
-	GtkWidget *w = match_window_by_filename(dname);
-	windata_t *vwin = NULL;
+	GtkWidget *w = match_db_window_by_filename(dname);
 
 	if (w != NULL) {
-	    vwin = g_object_get_data(G_OBJECT(w), "object");
-	}
+	    windata_t *vwin = g_object_get_data(G_OBJECT(w), "vwin");
 
-	if (vwin != NULL) {
-	    add_local_db_series_list(vwin);
+	    if (vwin != NULL) {
+		add_local_db_series_list(vwin);
+	    }
 	}
     }
 }
 
-static void gui_delete_series (GtkWidget *w, windata_t *vwin)
+static void db_delete_callback (GtkWidget *w, windata_t *vwin)
 {
     int *list = db_get_selection_list(vwin);
+    gchar *query;
     int resp, err = 0;
 
     if (list == NULL) {
 	return;
     }
 
-    resp = yes_no_dialog ("gretl", 
-			  _("Really delete the selected series?"),
-			  0);
+    query = g_strdup_printf(_("Really delete the selected series\n"
+			      "from the database '%s'?"), 
+			    gtk_window_get_title(GTK_WINDOW(vwin->main)));
+
+    resp = yes_no_dialog ("gretl", query, 0);
+
+    g_free(query);
 
     if (resp == GRETL_YES) { 	
 	err = db_delete_series_by_number(list, vwin->fname);
@@ -490,7 +478,7 @@ static void gui_delete_series (GtkWidget *w, windata_t *vwin)
 
 void import_db_series (windata_t *vwin)
 {
-    gui_get_db_series(vwin, DB_IMPORT, NULL);
+    gui_get_db_series(vwin, DB_IMPORT);
 }
 
 static int diffdate (double d1, double d0, int pd)
@@ -596,9 +584,8 @@ static DATAINFO *new_dataset_from_dbwrapper (dbwrapper *dw,
     return dinfo;
 }
 
-void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
+static void gui_get_db_series (windata_t *vwin, int cmd)
 {
-    windata_t *vwin = (windata_t *) p;
     int dbcode = vwin->role;
     DATAINFO *dbinfo = NULL;
     double **dbZ = NULL;
@@ -631,7 +618,7 @@ void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
 	    err = get_pcgive_db_data(vwin->fname, sinfo, dbZ);
 	}
 
-	if (action == DB_IMPORT && err == DB_MISSING_DATA) {
+	if (cmd == DB_IMPORT && err == DB_MISSING_DATA) {
 	    warnbox(_("Warning: series has missing observations"));
 	}
 
@@ -644,11 +631,11 @@ void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
 	strcpy(VARLABEL(dbinfo, i+1), sinfo->descrip);
     }
 
-    if (action == DB_DISPLAY) {
+    if (cmd == DB_DISPLAY) {
 	display_dbdata((const double **) dbZ, dbinfo);
-    } else if (action == DB_GRAPH) {
+    } else if (cmd == DB_GRAPH) {
 	graph_dbdata(&dbZ, dbinfo);
-    } else if (action == DB_IMPORT) { 
+    } else if (cmd == DB_IMPORT) { 
 	add_dbdata(vwin, dbZ, dbinfo, dw, &freeit);
     }
 
@@ -660,6 +647,13 @@ void gui_get_db_series (gpointer p, guint action, GtkWidget *w)
     }
 
     dbwrapper_destroy(dw);
+}
+
+/* double-click callback */
+
+void display_db_series (windata_t *vwin)
+{
+    gui_get_db_series(vwin, DB_DISPLAY);
 } 
 
 static void db_view_codebook (GtkWidget *w, windata_t *vwin)
@@ -672,17 +666,6 @@ static void db_view_codebook (GtkWidget *w, windata_t *vwin)
     view_file(cbname, 0, 0, 78, 350, VIEW_CODEBOOK);
 }
 
-static void 
-book_callback_wrapper (gpointer p, guint u, GtkWidget *w)
-{
-    db_view_codebook(w, p);
-}
-
-static void db_menu_find (GtkWidget *w, windata_t *vwin)
-{
-    menu_find(vwin, 1, NULL);
-}
-
 static void build_db_popup (windata_t *vwin, int cb, int del)
 {
     if (vwin->popup != NULL) {
@@ -692,22 +675,20 @@ static void build_db_popup (windata_t *vwin, int cb, int del)
     vwin->popup = gtk_menu_new();
 
     add_popup_item(_("Display"), vwin->popup, 
-		   G_CALLBACK(gui_display_series), 
+		   G_CALLBACK(db_display_series), 
 		   vwin);
     add_popup_item(_("Graph"), vwin->popup, 
-		   G_CALLBACK(gui_graph_series), 
+		   G_CALLBACK(db_graph_series), 
 		   vwin);
     add_popup_item(_("Import"), vwin->popup, 
-		   G_CALLBACK(gui_import_series), 
+		   G_CALLBACK(db_import_series), 
 		   vwin);
     if (del) {
 	add_popup_item(_("Delete"), vwin->popup, 
-		       G_CALLBACK(gui_delete_series), 
+		       G_CALLBACK(db_delete_callback), 
 		       vwin);
     }	
-    add_popup_item(_("Find..."), vwin->popup, 
-		   G_CALLBACK(db_menu_find), 
-		   vwin);
+
     if (cb) {
 	add_popup_item(_("Codebook"), vwin->popup, 
 		       G_CALLBACK(db_view_codebook), 
@@ -715,49 +696,51 @@ static void build_db_popup (windata_t *vwin, int cb, int del)
     }
 }
 
-static void 
-delete_series_callback (gpointer p, guint u, GtkWidget *w)
+static void db_close (GtkWidget *w, windata_t *vwin)
 {
-    gui_delete_series(NULL, p);
+    gtk_widget_destroy(vwin->main);
 }
 
-static void 
-close_db_callback (windata_t *vwin, guint u, GtkWidget *w)
-{
-    gtk_widget_destroy(vwin->w);
-}
+enum {
+    DEL_BTN = 1,
+    CB_BTN
+};
 
-static void set_up_db_menu (windata_t *vwin, int cb, int del)
-{
-    GtkItemFactoryEntry db_items[] = {
-	{ N_("/_File"), NULL, NULL, 0, "<Branch>", GNULL },
-	{ N_("/File/_Close"), NULL, close_db_callback, 0, "<StockItem>", GTK_STOCK_CLOSE },
-	{ N_("/_Series/_Display"), NULL, gui_get_db_series, DB_DISPLAY, NULL, GNULL },
-	{ N_("/_Series/_Graph"), NULL, gui_get_db_series, DB_GRAPH, NULL, GNULL },
-	{ N_("/_Series/_Import"), NULL, gui_get_db_series, DB_IMPORT, NULL, GNULL },
-	{ N_("/_Series/_Delete"), NULL, delete_series_callback, 0, NULL, GNULL },
-	{ N_("/_Find"), NULL, NULL, 0, "<Branch>", GNULL },   
-	{ N_("/Find/_Find in window"), NULL, menu_find, 1, "<StockItem>", GTK_STOCK_FIND },
-	{ N_("/_Codebook"), NULL, NULL, 0, "<Branch>", GNULL },    
-	{ N_("/Codebook/_Open"), NULL, book_callback_wrapper, 0, "<StockItem>", 
-	  GTK_STOCK_OPEN }
-    };
-    gint n_items = sizeof db_items / sizeof db_items[0];
+static GretlToolItem db_items[] = {
+    { N_("Display values"), GTK_STOCK_OPEN,   G_CALLBACK(db_display_series), 0 },
+    { N_("Graph"),          GRETL_STOCK_TS,   G_CALLBACK(db_graph_series), 0 },
+    { N_("Add to dataset"), GTK_STOCK_ADD,    G_CALLBACK(db_import_series), 0 },
+    { N_("Delete"),         GTK_STOCK_DELETE, G_CALLBACK(db_delete_callback), DEL_BTN },
+    { N_("Codebook"),       GRETL_STOCK_BOOK, G_CALLBACK(db_view_codebook), CB_BTN },
+    { N_("Close"),          GTK_STOCK_CLOSE,  G_CALLBACK(db_close), 0 }
+};
 
-    if (!cb) {
-	n_items -= 2;
+static int n_db_items = G_N_ELEMENTS(db_items);
+
+static void make_db_toolbar (windata_t *vwin, int cb, int del)
+{
+    GtkWidget *hbox;
+    GretlToolItem *item;
+    int i;
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
+
+    vwin->mbar = gretl_toolbar_new();
+
+    for (i=0; i<n_db_items; i++) {
+	item = &db_items[i];
+	if (!del && item->flag == DEL_BTN) {
+	    continue;
+	} if (!cb && item->flag == CB_BTN) {
+	    continue;
+	}
+	gretl_toolbar_insert(vwin->mbar, item, item->func, vwin, -1);
     }
 
-    vwin->ifac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", NULL);
-#ifdef ENABLE_NLS
-    gtk_item_factory_set_translate_func(vwin->ifac, menu_translate, NULL, NULL);
-#endif
-    gtk_item_factory_create_items(vwin->ifac, n_items, db_items, vwin);
-    vwin->mbar = gtk_item_factory_get_widget(vwin->ifac, "<main>");
-
-    if (!del) {
-	flip(vwin->ifac, "/Series/Delete", FALSE);
-    }
+    gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
+    vwin_add_finder(vwin);
+    gtk_widget_show_all(hbox);
 }
 
 static void destroy_db_win (GtkWidget *w, windata_t *vwin)
@@ -844,31 +827,18 @@ maybe_adjust_descrip_column (windata_t *vwin)
     }
 }
 
-static void db_select_first_series (windata_t *vwin)
-{
-    GtkTreeView *view = GTK_TREE_VIEW(vwin->listbox);
-    GtkTreeSelection *selection;
-    GtkTreeIter iter;
-    GtkListStore *store;
-
-    store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
-    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-    selection = gtk_tree_view_get_selection(view);
-    gtk_tree_selection_select_iter(selection, &iter);
-}
-
 static int 
 make_db_series_window (int action, char *fname, char *buf)
 {
-    GtkWidget *listbox;
-    GtkWidget *w, *main_vbox;
-    char *titlestr;
+    GtkWidget *w, *listbox;
+    gchar *title;
     windata_t *vwin;
     int db_width = 700, db_height = 420;
     int cb = 0, del = 0;
-    int err = 0;
+    int record, err = 0;
+
+    w = match_db_window_by_filename(fname);
 
-    w = match_window_by_filename(fname);
     if (w != NULL) {
 	gtk_window_present(GTK_WINDOW(w));
 	return 0;
@@ -878,37 +848,25 @@ make_db_series_window (int action, char *fname, char *buf)
 	return 1;
     }
 
-    vwin = mymalloc(sizeof *vwin);
+    if (buf == NULL && strrchr(fname, SLASH) != NULL) {
+	title = strrchr(fname, SLASH) + 1;
+    } else {
+	title = fname;
+    } 
+
+    record = (action == NATIVE_SERIES);
+
+    vwin = gretl_browser_new(action, title, record);
     if (vwin == NULL) {
 	return 1;
     }
 
-    windata_init(vwin);
-    vwin->role = action;
-
-    vwin->w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-
     db_width *= gui_scale;
     db_height *= gui_scale;
-    gtk_window_set_default_size(GTK_WINDOW(vwin->w), db_width, db_height);
+    gtk_window_set_default_size(GTK_WINDOW(vwin->main), db_width, db_height);
     
-    if (buf == NULL && strrchr(fname, SLASH) != NULL) {
-	titlestr = strrchr(fname, SLASH) + 1;
-    } else {
-	titlestr = fname;
-    }
-
-    gtk_window_set_title(GTK_WINDOW(vwin->w), titlestr);
-
-    if (action == NATIVE_SERIES) {
-	g_object_set_data(G_OBJECT(vwin->w), "object", vwin);
-	g_object_set_data(G_OBJECT(vwin->w), "role", 
-			  GINT_TO_POINTER(vwin->role));
-	winstack_add(vwin->w);
-	g_signal_connect(G_OBJECT(vwin->w), "destroy",
-			 G_CALLBACK(free_windata), vwin);	
-    } else {
-	g_signal_connect(G_OBJECT(vwin->w), "destroy", 
+    if (action != NATIVE_SERIES) {
+	g_signal_connect(G_OBJECT(vwin->main), "destroy", 
 			 G_CALLBACK(destroy_db_win), vwin);
     }
 
@@ -917,29 +875,26 @@ make_db_series_window (int action, char *fname, char *buf)
     }
 
     strcpy(vwin->fname, fname);
-    
-    /* set up grids */
-    main_vbox = gtk_vbox_new(FALSE, 5);
-    gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 10);
-    gtk_container_add(GTK_CONTAINER(vwin->w), main_vbox);
+
+    vwin->vbox = gtk_vbox_new(FALSE, 1);
+    gtk_box_set_spacing(GTK_BOX(vwin->vbox), 4);
+    gtk_container_set_border_width(GTK_CONTAINER(vwin->vbox), 4);
+    gtk_container_add(GTK_CONTAINER(vwin->main), vwin->vbox);
 
     cb = db_has_codebook(fname);
     del = db_is_writable(action, fname);
 
-    set_up_db_menu(vwin, cb, del);
+    make_db_toolbar(vwin, cb, del);
     build_db_popup(vwin, cb, del);
 
-    gtk_box_pack_start(GTK_BOX(main_vbox), vwin->mbar, FALSE, TRUE, 0);
-    gtk_widget_show(vwin->mbar);
-
     listbox = database_window(vwin);
-    gtk_box_pack_start(GTK_BOX(main_vbox), listbox, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), listbox, TRUE, TRUE, 0);
 
     if (action == REMOTE_SERIES) {
 	GtkWidget *hbox; 
 
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
 	vwin->status = gtk_label_new(_("Network status: OK"));
 	gtk_label_set_justify(GTK_LABEL(vwin->status), GTK_JUSTIFY_LEFT);
 	gtk_box_pack_start(GTK_BOX(hbox), vwin->status, FALSE, FALSE, 0);
@@ -956,11 +911,11 @@ make_db_series_window (int action, char *fname, char *buf)
     }
 
     if (err) {
-	gtk_widget_destroy(vwin->w);
+	gtk_widget_destroy(vwin->main);
     } else {
-	gtk_widget_show_all(vwin->w); 
+	gtk_widget_show_all(vwin->main); 
 	maybe_adjust_descrip_column(vwin);
-	db_select_first_series(vwin);
+	listbox_select_first(vwin);
     }
 
     return err;
@@ -1022,7 +977,7 @@ static void db_drag_connect (windata_t *vwin, int i)
 			&gretl_drag_targets[i],
 			1, GDK_ACTION_COPY);
 
-    g_signal_connect(G_OBJECT(vwin->listbox), "drag_data_get",
+    g_signal_connect(G_OBJECT(vwin->listbox), "drag-data-get",
 		     G_CALLBACK(do_db_drag),
 		     vwin);
 }
@@ -1342,7 +1297,7 @@ static GtkWidget *database_window (windata_t *vwin)
 
     box = gtk_vbox_new(FALSE, 0);
     vwin_add_list_box(vwin, GTK_BOX(box), 4, TRUE, types, titles, 0);
-    g_signal_connect(G_OBJECT(vwin->listbox), "button_press_event",
+    g_signal_connect(G_OBJECT(vwin->listbox), "button-press-event",
 		     G_CALLBACK(popup_menu_handler), 
 		     vwin->popup);
     gtk_widget_show(box);
@@ -1560,8 +1515,6 @@ void open_named_remote_db_index (char *dbname)
     free(getbuf);
 }
 
-#define KEEP_BROWSER_OPEN 1
-
 void open_db_index (GtkWidget *w, gpointer data)
 {
     gchar *fname = NULL, *dbdir = NULL;
@@ -1585,12 +1538,6 @@ void open_db_index (GtkWidget *w, gpointer data)
     g_free(dbdir);
 
     make_db_series_window(action, dbfile, NULL); 
-
-#ifndef KEEP_BROWSER_OPEN
-    if (vwin != NULL && vwin->w != NULL && GTK_IS_WIDGET(vwin->w)) {
-	gtk_widget_destroy(GTK_WIDGET(vwin->w));
-    }
-#endif
 }
 
 void open_remote_db_index (GtkWidget *w, gpointer data)
@@ -1713,7 +1660,7 @@ static int ggz_extract (char *ggzname)
 	goto bailout;
     } 
 
-    memset(gzbuf, GRETL_BUFSIZE, 0);
+    memset(gzbuf, 0, GRETL_BUFSIZE);
     gzread(fgz, gzbuf, INFOLEN);
 
     if (parse_db_header(gzbuf, &idxlen, &datalen, &cblen)) {
@@ -1791,10 +1738,10 @@ static int ggz_extract (char *ggzname)
     if (fcbk != NULL) fclose(fcbk);
 
     if (cblen == 0) {
-	remove(cbname);
+	gretl_remove(cbname);
     }
 
-    remove(ggzname);
+    gretl_remove(ggzname);
 
     return err;
 }
@@ -1819,8 +1766,14 @@ enum {
     TMP_INSTALL
 };
 
+/* try to find a suitable path, for which the user has write
+   permission, for installing a database, collection of
+   data files, or function package 
+*/
+
 static char *get_writable_target (int code, int op, char *objname)
 {
+    const char *ext;
     FILE *fp;
     char *targ;
     int err = 0;
@@ -1831,16 +1784,29 @@ static char *get_writable_target (int code, int op, char *objname)
     }
 
     if (code == REMOTE_DB) {
-	build_path(targ, paths.binbase, objname, ".ggz");
+	ext = ".ggz";
+    } else if (code == REMOTE_FUNC_FILES) {
+	ext = ".gfn";
+    } else {
+	ext = ".tar.gz";
+    }
+
+    if (code == REMOTE_DB) {
+	build_path(targ, gretl_binbase(), objname, ext);
+    } else if (code == REMOTE_DATA_PKGS) {
+	char pkgdir[MAXLEN];
+
+	get_default_dir(pkgdir, SAVE_DATA_PKG);
+	build_path(targ, pkgdir, objname, ext);
     } else {
 	if (op == TMP_INSTALL) {
-	    build_path(targ, paths.dotdir, "dltmp", NULL);
+	    build_path(targ, gretl_dotdir(), "dltmp", NULL);
 	    err = gretl_tempname(targ);
 	} else {
-	    char fndir[MAXLEN];
+	    char pkgdir[MAXLEN];
 
-	    get_default_dir(fndir, SAVE_FUNCTIONS);
-	    build_path(targ, fndir, objname, ".gfn");
+	    get_default_dir(pkgdir, SAVE_FUNCTIONS);
+	    build_path(targ, pkgdir, objname, ext);
 	}
     } 
 
@@ -1854,7 +1820,7 @@ static char *get_writable_target (int code, int op, char *objname)
 	if (fp == NULL) {
 	    if (errno == EACCES && code != REMOTE_FUNC_FILES) { 
 		/* write to user dir instead? */
-		build_path(targ, paths.workdir, objname, ".ggz");
+		build_path(targ, gretl_workdir(), objname, ext);
 	    } else {
 		file_write_errbox(targ);
 		free(targ);
@@ -1868,15 +1834,46 @@ static char *get_writable_target (int code, int op, char *objname)
     return targ;
 }
 
+static int unpack_book_data (const char *fname)
+{
+    char *p, path[FILENAME_MAX];
+    int err = 0;
+
+    errno = 0;
+
+    strcpy(path, fname);
+
+    p = strrchr(path, SLASH);
+    if (p == NULL && SLASH == '\\') {
+	p = strrchr(path, '/');
+    }
+    if (p != NULL) {
+	*p = '\0';
+    }
+
+    chdir(path);
+
+    if (errno != 0) {
+        gretl_errmsg_set_from_errno("chdir");
+	err = E_FOPEN;
+    }
+
+    if (!err) {
+	err = gretl_untar(fname);
+    }
+
+    return err;
+}
+
 /* note : 'vwin' here is the source viewer window displaying the
-   remote file (database or function package) that is being installed
-   oto the local machine.
+   remote file (database or datafiles package or function package)
+   that is being installed onto the local machine.
 */
 
-static int real_install_file_from_server (windata_t *vwin, int op)
+void install_file_from_server (GtkWidget *w, windata_t *vwin)
 {
     gchar *objname = NULL;
-    char *target = NULL;
+    char *path = NULL;
     int err = 0;
 
     if (vwin->role == REMOTE_DB) {
@@ -1889,30 +1886,31 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	    gtk_tree_model_get(model, &iter, 0, &objname, -1);
 	}
     } else {
-	/* remote function packages */
+	/* remote datafiles or function package */
 	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			     vwin->active_var, 0, &objname);
-	
     }
 
     if (objname == NULL || *objname == '\0') {
-	free(objname);
-	return 1;
+	g_free(objname);
+	return;
     }    
 
-    target = get_writable_target(vwin->role, op, objname);
-    if (target == NULL) {
-	free(objname);
-	return 1;
+    path = get_writable_target(vwin->role, REAL_INSTALL, objname);
+    if (path == NULL) {
+	g_free(objname);
+	return;
     }
 
     if (vwin->role == REMOTE_FUNC_FILES) {
-	err = retrieve_remote_function_package(objname, target);
+	err = retrieve_remote_function_package(objname, path);
+    } else if (vwin->role == REMOTE_DATA_PKGS) {
+	err = retrieve_remote_datafiles_package(objname, path);
     } else {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-	err = retrieve_remote_db(objname, target, GRAB_NBO_DATA);
+	err = retrieve_remote_db(objname, path, GRAB_NBO_DATA);
 #else
-	err = retrieve_remote_db(objname, target, GRAB_DATA);
+	err = retrieve_remote_db(objname, path, GRAB_DATA);
 #endif
     }
 
@@ -1922,47 +1920,70 @@ static int real_install_file_from_server (windata_t *vwin, int op)
 	windata_t *local = get_local_viewer(vwin->role);
 
 	if (vwin->role == REMOTE_FUNC_FILES) {
-	    if (op == REAL_INSTALL) {
-		if (local != NULL) {
-		    populate_filelist(local, NULL);
-		} else {
-		    infobox(_("Function package installed"));
-		}
+	    list_store_set_string(GTK_TREE_VIEW(vwin->listbox),
+				  vwin->active_var, 3,
+				  _("Up to date"));
+	    if (local != NULL) {
+		populate_filelist(local, NULL);
+	    } 
+	} else if (vwin->role == REMOTE_DATA_PKGS) {
+	    fprintf(stderr, "downloaded '%s'\n", path);
+	    err = unpack_book_data(path);
+	    remove(path);
+	    if (err) {
+		errbox(_("Error unzipping compressed data"));
 	    } else {
-		gui_show_function_info(target, VIEW_FUNC_INFO);
+		infobox("Restart gretl to access this database");
 	    }
 	} else {
-	    err = ggz_extract(target);
+	    fprintf(stderr, "downloaded '%s'\n", path);
+	    err = ggz_extract(path);
 	    if (err) {
 		if (err != E_FOPEN) {
 		    errbox(_("Error unzipping compressed data"));
 		}
-	    } else if (local != NULL) {
-		populate_filelist(local, NULL);
 	    } else {
-		offer_db_open(target);
+		tree_store_set_string(GTK_TREE_VIEW(vwin->listbox),
+				      vwin->active_var, 2,
+				      _("Up to date"));
+		if (local != NULL) {
+		    populate_filelist(local, NULL);
+		} else {
+		    offer_db_open(path);
+		}
 	    }
 	}
     }
 
-    free(objname);
-    free(target);
-
-    return err;
+    g_free(objname);
+    free(path);
 }
 
-void install_file_from_server (GtkWidget *w, gpointer data)
+void pkg_info_from_server (GtkWidget *w, windata_t *vwin)
 {
-    windata_t *vwin = (windata_t *) data;
+    gchar *objname = NULL;
+    char *path = NULL;
+    int err = 0;
 
-    real_install_file_from_server(vwin, REAL_INSTALL);
-}
+    tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
+			 vwin->active_var, 0, &objname);
 
-void file_info_from_server (GtkWidget *w, gpointer data)
-{
-    windata_t *vwin = (windata_t *) data;
+    path = get_writable_target(vwin->role, TMP_INSTALL, objname);
+    if (path == NULL) {
+	g_free(objname);
+	return;
+    }
 
-    real_install_file_from_server(vwin, TMP_INSTALL);
+    err = retrieve_remote_function_package(objname, path);
+
+    if (err) {
+	show_network_error(NULL);
+    } else {
+	display_function_package_data(objname, path, VIEW_PKG_INFO);
+    }
+
+    g_free(objname);
+    free(path);
 }
 
 static gchar *
@@ -2061,7 +2082,7 @@ int write_db_description (const char *binname, const char *descrip)
 	}
 	fclose(fnew);
 	fclose(fbak);
-	remove(idxtmp);
+	gretl_remove(idxtmp);
     }
 
     return err;
@@ -2094,56 +2115,86 @@ read_db_files_in_dir (DIR *dir, int dbtype, const char *dbdir,
     return ndb;
 }
 
-static void get_local_object_status (char *fname, int role, char *status, 
+static void get_local_object_status (const char *fname, int role, 
+				     const char **status, 
 				     time_t remtime)
 {
-    char fndir[MAXLEN];
+    char filedir[MAXLEN];
     char fullname[MAXLEN];
     struct stat fbuf;
     int err;
 
     if (role == REMOTE_DB) {
-	build_path(fullname, paths.binbase, fname, NULL);
+	build_path(fullname, gretl_binbase(), fname, NULL);
+    } else if (role == REMOTE_DATA_PKGS) {
+	build_path(filedir, gretl_home(), "data", NULL);
+	build_path(fullname, filedir, fname, NULL);
     } else {
-	build_path(fndir, paths.gretldir, "functions", NULL);
-	build_path(fullname, fndir, fname, NULL);
+	build_path(filedir, gretl_home(), "functions", NULL);
+	build_path(fullname, filedir, fname, NULL);
     }
 
-    if ((err = stat(fullname, &fbuf)) == -1) {
+    if ((err = gretl_stat(fullname, &fbuf)) == -1) {
 	if (errno == ENOENT) {
-#ifdef G_OS_WIN32
-	    strcpy(status, _("Not installed"));
+#ifdef G_OS_WIN32 /* FIXME? */
+	    *status = N_("Not installed");
 #else
 	    /* try user dir too, if not on Windows */
 	    if (role == REMOTE_DB) {
-		build_path(fullname, paths.workdir, fname, NULL);
+		build_path(fullname, gretl_workdir(), fname, NULL);
+	    } else if (role == REMOTE_DATA_PKGS) {
+		build_path(fullname, gretl_workdir(), fname, NULL);
 	    } else {
-		build_path(fndir, paths.workdir, "functions", NULL);
-		build_path(fullname, fndir, fname, NULL);
+		build_path(filedir, gretl_workdir(), "functions", NULL);
+		build_path(fullname, filedir, fname, NULL);
 	    }
-	    if ((err = stat(fullname, &fbuf)) == -1) {
-		strcpy(status, _("Not installed"));
+	    err = gretl_stat(fullname, &fbuf);
+	    if (err == -1) {
+		if (role == REMOTE_FUNC_FILES || role == REMOTE_DATA_PKGS) {
+		    /* try default working dir */
+		    char *tmp = gretl_default_workdir();
+
+		    if (tmp != NULL) {
+			if (role == REMOTE_FUNC_FILES) {
+			    build_path(filedir, tmp, "functions", NULL);
+			    build_path(fullname, filedir, fname, NULL);
+			} else {
+			    build_path(fullname, tmp, fname, NULL);
+			}
+			err = stat(fullname, &fbuf);
+			free(tmp);
+		    }
+		} 
+		if (err == -1) {
+		    *status = N_("Not installed");
+		}
 	    } 
 #endif
 	} else {
-	    strcpy(status, _("Unknown: access error"));
+	    *status = N_("Unknown: access error");
 	}
     }
 
     if (!err) {
-	if (difftime(remtime, fbuf.st_ctime) > 360) {
-	    strcpy(status, _("Not up to date"));
+	double dt;
+
+	dt = difftime(remtime, fbuf.st_ctime);
+	if (dt > 360) {
+	    dt = difftime(remtime, fbuf.st_mtime);
+	}
+	if (dt > 360) {
+	    *status = N_("Not up to date");
 	} else {
-	    strcpy(status, _("Up to date"));
+	    *status = N_("Up to date");
 	}
     }
 }
 
-static int read_remote_filetime (char *line, char *fname, time_t *date)
+static int read_remote_filetime (char *line, char *fname, time_t *date,
+				 char *buf)
 {
-    char mon[4], hrs[9];
-    int mday, yr;
-    struct tm mytime;
+    char month[4], hrs[9];
+    int mday, yr, mon = 0;
     const char *months[] = {
 	"Jan", "Feb", "Mar", "Apr",
 	"May", "Jun", "Jul", "Aug",
@@ -2160,34 +2211,40 @@ static int read_remote_filetime (char *line, char *fname, time_t *date)
     */
 
     if (sscanf(line, "%*s%*s%3s%2d%8s%4d%31s", 
-	       mon, &mday, hrs, &yr, fname) != 5) {
+	       month, &mday, hrs, &yr, fname) != 5) {
 	return 1;
     }
 
-    hrs[2] = 0;
-
-    mytime.tm_sec = 0;
-    mytime.tm_min = 0;   
-    mytime.tm_wday = 0;   
-    mytime.tm_yday = 0;   
-    mytime.tm_isdst = -1; 
-    mytime.tm_hour = atoi(hrs);
-    mytime.tm_year = yr - 1900;
-    mytime.tm_mday = mday;
-    mytime.tm_mon = 0;
-
     for (i=0; i<12; i++) {
-	if (strcmp(mon, months[i]) == 0) {
-	    mytime.tm_mon = i;
+	if (!strcmp(month, months[i])) {
+	    mon = i;
 	}
-    }
+    }    
 
-    *date = mktime(&mytime);
+    if (buf != NULL) {
+	sprintf(buf, "%d-%02d-%02d", yr, mon + 1, mday);
+    } else {
+	struct tm mytime;
+
+	hrs[2] = 0;
+
+	mytime.tm_sec = 0;
+	mytime.tm_min = 0;   
+	mytime.tm_wday = 0;   
+	mytime.tm_yday = 0;   
+	mytime.tm_isdst = -1; 
+	mytime.tm_hour = atoi(hrs);
+	mytime.tm_year = yr - 1900;
+	mytime.tm_mday = mday;
+	mytime.tm_mon = mon;
+
+	*date = mktime(&mytime);
+    }
 
     return 0;
 }
 
-/* below: mechanism for tucking individual databases under a twisty,
+/* below: mechanism for tucking individual databases under a 'twisty',
    when there's a large number of databases from a single source.
 */
 
@@ -2261,8 +2318,9 @@ gint populate_remote_db_list (windata_t *vwin)
     GtkTreeIter iter, child_iter; 
     char *getbuf = NULL;
     char line[1024];
-    char fname[32], status[20];
+    char fname[32];
     char src[96], srcbak[96];
+    const char *status;
     gchar *row[3];
     time_t remtime;
     int start, parent, kids;
@@ -2287,7 +2345,7 @@ gint populate_remote_db_list (windata_t *vwin)
 	if (strstr(line, "idx")) {
 	    continue;
 	}
-	if (read_remote_filetime(line, fname, &remtime)) {
+	if (read_remote_filetime(line, fname, &remtime, NULL)) {
 	    continue;
 	}
 	if (bufgets(line, sizeof line, getbuf)) {
@@ -2331,13 +2389,15 @@ gint populate_remote_db_list (windata_t *vwin)
 	if (strstr(line, "idx")) {
 	    continue;
 	}
-	if (read_remote_filetime(line, fname, &remtime)) {
+	if (read_remote_filetime(line, fname, &remtime, NULL)) {
 	    continue;
 	}
 
-	get_local_object_status(fname, vwin->role, status, remtime);
+	status = "";
+	get_local_object_status(fname, vwin->role, &status, remtime);
 	row[0] = strip_extension(fname);
-	row[2] = status;
+	row[1] = NULL;
+	row[2] = _(status);
 
 	if (bufgets(line, sizeof line, getbuf)) {
 	    tailstrip(line);
@@ -2349,9 +2409,7 @@ gint populate_remote_db_list (windata_t *vwin)
 		parent = 1;
 		kids = ndb;
 	    }	
-	} else {
-	    row[1] = NULL;
-	}
+	} 
 
 	if (parent) {
 	    /* header for child databases */
@@ -2371,7 +2429,7 @@ gint populate_remote_db_list (windata_t *vwin)
 	    /* insert at top level */
 	    gtk_tree_store_append(store, &iter, NULL);
 	    gtk_tree_store_set(store, &iter, 0, row[0], 1, row[1],
-			   2, row[2], -1);
+			       2, row[2], -1);
 	}	    
 
 	i++;
@@ -2398,7 +2456,8 @@ gint populate_remote_func_list (windata_t *vwin)
     GtkTreeIter iter;  
     char *getbuf = NULL;
     char line[1024];
-    char fname[32], status[20];
+    char fname[32];
+    const char *status;
     char *basename;
     char *descrip;
     char *version;
@@ -2424,11 +2483,12 @@ gint populate_remote_func_list (windata_t *vwin)
 	descrip = NULL;
 	version = NULL;
 
-	if (read_remote_filetime(line, fname, &remtime)) {
+	if (read_remote_filetime(line, fname, &remtime, NULL)) {
 	    continue;
 	}
 
-	get_local_object_status(fname, vwin->role, status, remtime);
+	status = "";
+	get_local_object_status(fname, vwin->role, &status, remtime);
 	basename = strip_extension(fname);
 
 	if (bufgets(line, sizeof line, getbuf)) {
@@ -2452,7 +2512,7 @@ gint populate_remote_func_list (windata_t *vwin)
 			   0, basename, 
 			   1, version,
 			   2, descrip, 
-			   3, status,
+			   3, _(status),
 			   -1);
 
 	free(descrip);
@@ -2474,7 +2534,89 @@ gint populate_remote_func_list (windata_t *vwin)
     return err;
 }
 
-static DIR *open_gretl_db_dir (char *dbdir)
+/* Fill a list box with names and short descriptions
+   of data file packages, retrieved from server.
+*/
+
+gint populate_remote_data_pkg_list (windata_t *vwin)
+{
+    GtkListStore *store;
+    GtkTreeIter iter;  
+    char *getbuf = NULL;
+    char line[256];
+    char fname[32];
+    char tstr[16];
+    const char *status;
+    char *basename;
+    char *descrip;
+    int n, err = 0;
+
+    err = list_remote_data_packages(&getbuf);
+
+    if (err) {
+	show_network_error(NULL);
+	free(getbuf);
+	return err;
+    }
+
+    store = GTK_LIST_STORE(gtk_tree_view_get_model 
+			   (GTK_TREE_VIEW(vwin->listbox)));
+    gtk_list_store_clear(store);
+    gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+
+    n = 0;
+    bufgets_init(getbuf);
+
+    while (bufgets(line, sizeof line, getbuf)) {
+	descrip = NULL;
+	*tstr = '\0';
+
+	if (read_remote_filetime(line, fname, NULL, tstr)) {
+	    continue;
+	}
+
+	status = "";
+	basename = strip_extension(fname);
+
+	if (bufgets(line, sizeof line, getbuf)) {
+	    tailstrip(line);
+	    utf8_correct(line);
+	    descrip = gretl_strdup(line + 2);
+	} 
+
+	if (descrip == NULL) {
+	    descrip = gretl_strdup("");
+	}
+
+	gtk_list_store_append(store, &iter);
+	gtk_list_store_set(store, &iter, 
+			   0, basename, 
+			   1, descrip, 
+			   2, tstr,
+			   -1);
+
+	free(descrip);
+	n++;
+    }
+
+    bufgets_finalize(getbuf);
+    free(getbuf);
+
+    if (n == 0) {
+	warnbox(_("No data packages found"));
+	err = 1;
+    }
+
+#if 0
+    if (!err) {
+	funcpkg_drag_connect(vwin);
+    }
+#endif
+
+    return err;
+}
+
+static DIR *open_gretl_db_dir (const char *dbdir)
 {
     DIR *dir = NULL;
     int tries = 0;
@@ -2487,7 +2629,7 @@ static DIR *open_gretl_db_dir (char *dbdir)
 #endif
 	if (dir == NULL) {
 	    errbox(_("Can't open folder %s"), dbdir);
-	    if (++tries == 2 || options_dialog(TAB_DBS, "binbase")) {
+	    if (++tries == 2 || options_dialog(TAB_DBS, "binbase", mdata->main)) {
 		/* canceled */
 		return NULL;
 	    }
@@ -2501,7 +2643,7 @@ gint populate_dbfilelist (windata_t *vwin)
 {
     GtkListStore *store;
     GtkTreeIter iter;
-    char *dbdir;
+    const char *dbdir;
     DIR *dir = NULL;
     int ndb = 0;
 
@@ -2510,7 +2652,7 @@ gint populate_dbfilelist (windata_t *vwin)
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
     /* pick up any databases in the shared gretl dir */
-    dbdir = paths.binbase;
+    dbdir = gretl_binbase();
     dir = open_gretl_db_dir(dbdir);
     if (dir != NULL) {
 	ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
@@ -2518,7 +2660,7 @@ gint populate_dbfilelist (windata_t *vwin)
     }
 
     /* pick up any databases in the user's personal dir */
-    dbdir = paths.workdir;
+    dbdir = gretl_workdir();
 #ifdef G_OS_WIN32 
     dir = win32_opendir(dbdir);
 #else
@@ -2529,6 +2671,20 @@ gint populate_dbfilelist (windata_t *vwin)
 	closedir(dir);
     }
 
+    /* and in the default working dir? */
+    dbdir = gretl_default_workdir();
+    if (dbdir != NULL) {
+#ifdef G_OS_WIN32 
+	dir = win32_opendir(dbdir);
+#else
+	dir = opendir(dbdir);
+#endif
+	if (dir != NULL) {
+	    ndb += read_db_files_in_dir(dir, vwin->role, dbdir, store, &iter);
+	    closedir(dir);
+	}
+    }
+
     if (ndb == 0) {
 	errbox(_("No database files found"));
     }
@@ -2536,6 +2692,26 @@ gint populate_dbfilelist (windata_t *vwin)
     return (ndb == 0);
 }
 
+void set_db_dir_callback (windata_t *vwin, char *path)
+{
+    DIR *dir = opendir(path);
+    int ndb = 0;
+
+    if (dir != NULL) {
+	GtkListStore *store;
+	GtkTreeIter iter;
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(vwin->listbox)));
+
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	ndb = read_db_files_in_dir(dir, vwin->role, path, store, &iter);
+    }
+
+    if (ndb == 0) {
+	warnbox(_("No database files found"));
+    }
+}
+
 static void set_compact_info_from_default (int method)
 {
     int i;
@@ -2562,7 +2738,7 @@ void do_compact_data_set (void)
 	pmonstart = &monstart;
     }
 
-    data_compact_dialog(mdata->w, datainfo->pd, &newpd, pmonstart, 
+    data_compact_dialog(mdata->main, datainfo->pd, &newpd, pmonstart, 
 			&method, &repday);
 
     if (method == COMPACT_NONE) {
@@ -2588,7 +2764,7 @@ void do_expand_data_set (void)
 	return;
     }
 
-    data_expand_dialog(mdata->w, datainfo->pd, &newpd);
+    data_expand_dialog(mdata->main, datainfo->pd, &newpd);
     if (newpd < 0) {
 	return;
     }
