@@ -42,12 +42,16 @@ void infobox (const char *template, ...);
 
 void warnbox (const char *template, ...);
 
+void maybe_warn (void);
+
 void file_read_errbox (const char *fname);
 
 void file_write_errbox (const char *fname);
 
 gint yes_no_dialog (const char *title, const char *msg, int cancel);
 
+void gretl_dialog_add_message (GtkWidget *dlg, const char *msg);
+
 int make_default_storelist (void);
 
 gboolean exit_check (void);
@@ -56,12 +60,12 @@ void menu_exit_check (void);
 
 double gui_double_from_string (const char *str, int *err);
 
-int delimiter_dialog (gretlopt *optp);
-
-void varinfo_dialog (int varnum, int full);
+int csv_options_dialog (gretlopt *optp);
 
 void rand_seed_dialog (void);
 
+void select_list_dialog (char *listname, int *cancel);
+
 void database_description_dialog (const char *binname);
 
 int select_var_from_list_with_opt (const int *list, 
@@ -71,7 +75,9 @@ int select_var_from_list_with_opt (const int *list,
 
 int select_var_from_list (const int *list, const char *query);
 
-void sample_range_dialog (gpointer p, guint u, GtkWidget *w);
+void sample_range_dialog (GtkAction *action, gpointer p);
+
+void sample_restrict_dialog (GtkAction *action, gpointer p);
 
 void panel_structure_dialog (DATAINFO *pdinfo);
 
@@ -91,16 +97,36 @@ int radio_dialog_with_spinner (const char *title, const char **opts,
 			       int *spinvar, const char *spintxt,
 			       int spinmin, int spinmax);
 
+int radio_dialog_with_check (const char *title, const char *label, 
+			     const char **opts, int nopts, int deflt, int hcode,
+			     int *checkvar, const char *checktxt);
+
+GtkWidget *
+build_checks_dialog (const char *title, const char *blurb,
+		     const char **opts, int nopts,
+		     int *active, int nradios, int *rvar, int *spinvar, 
+		     const char *spintxt, int spinmin, int spinmax, 
+		     int hcode, int *ret);
+
 int checks_dialog (const char *title, const char *blurb,
 		   const char **opts, int nopts, 
 		   int *active, int nradios, int *rvar,
 		   int *spinvar, const char *spintxt, 
 		   int spinmin, int spinmax, int helpcode);
 
+int checks_only_dialog (const char *title, const char *blurb,
+			const char **opts, int nopts,
+			int *active, int hcode);
+
 int spin_dialog (const char *title, const char *blurb,
 		 int *spinvar, const char *spintxt, 
 		 int spinmin, int spinmax, int helpcode);
 
+int combo_selector_dialog (GList *list, const char *msg,
+			   int deflt);
+
+int yes_no_help_dialog (const char *msg, int hcode);
+
 int get_obs_dialog (const char *title, const char *text,
 		    const char *t1str, const char *t2str,
 		    int t1min, int t1max, int *t1, 
@@ -110,23 +136,32 @@ int add_obs_dialog (const char *blurb, int addmin);
 
 int forecast_dialog (int t1min, int t1max, int *t1, 
 		     int t2min, int t2max, int *t2,
-		     int pmin, int pmax, int *p,
+		     int *k, int pmin, int pmax, int *p,
 		     int dyn, gretlopt *optp,
-		     MODEL *pmod);
+		     double *conf, MODEL *pmod);
+
+void dialog_add_confidence_selector (GtkWidget *dlg, double *conf,
+				     gretlopt *gopt);
 
 int freq_dialog (const char *title, const char *blurb,
 		 int *nbins, int nbmax, double *f0, double *fwid,
-		 double xmin, double xmax, int *dist);
+		 double xmin, double xmax, int *dist, int *plot);
+
+int model_table_dialog (int *colhead_opt, int *se_opt, int *pv_opt,
+			int *ast_opt, int *figs);
 
 void bootstrap_dialog (windata_t *vwin, int *pp, int *pB,
 		       gretlopt *popt, int *cancelled);
 
-void compute_default_ts_info (DATAINFO *dwinfo, int newdata);
-
-void data_structure_wizard (gpointer p, guint create, GtkWidget *w);
-
 void lmax_dialog (double *lmax, double ymax);
 
-void tex_format_dialog (gpointer p, guint u, GtkWidget *w);
+int chow_dialog (int tmin, int tmax, int *t, int *dumv);
+
+void iter_control_dialog (int *optim, int *pmaxit, double *ptol, 
+			  int *plmem, int *cancel, GtkWidget *parent);
+
+void tex_format_dialog (void);
+
+int gretl_all_done (void);
 
 #endif /* DIALOGS_H */
