@@ -17,12 +17,9 @@
  * 
  */
 
-/* fileselect.c for gretl -- use the gtk file chooser under X11,
-   the native MS file selector under MS Windows */
-
 #include "gretl.h"
-#include "boxplots.h"
 #include "gpt_control.h"
+#include "gpt_dialog.h"
 #include "session.h"
 #include "textbuf.h"
 #include "textutil.h"
@@ -30,20 +27,30 @@
 #include "fileselect.h"
 #include "fnsave.h"
 #include "database.h"
+#include "datafiles.h"
+#include "graphics.h"
 #include "bootstrap.h"
 
+#include <sys/stat.h>
+#include <unistd.h>
+
+#ifdef G_OS_WIN32
+# include "gretlwin32.h"
+#endif
+
 #define IS_DAT_ACTION(i) (i == SAVE_DATA || \
                           i == SAVE_DATA_AS || \
                           i == OPEN_DATA)
 
 #define OPEN_DATA_ACTION(i)  (i == OPEN_DATA || \
                               i == OPEN_CSV || \
-                              i == OPEN_ASCII || \
                               i == OPEN_OCTAVE || \
                               i == OPEN_GNUMERIC || \
-	                      i == OPEN_EXCEL || \
+	                      i == OPEN_XLS || \
                               i == OPEN_WF1 || \
                               i == OPEN_DTA || \
+	                      i == OPEN_SAV || \
+			      i == OPEN_SAS || \
                               i == OPEN_JMULTI || \
                               i == OPEN_ODS)
 
@@ -51,18 +58,14 @@
                                i == APPEND_CSV || \
                                i == APPEND_OCTAVE || \
                                i == APPEND_GNUMERIC || \
-                               i == APPEND_EXCEL || \
-                               i == APPEND_ASCII || \
+                               i == APPEND_XLS || \
                                i == APPEND_WF1 || \
                                i == APPEND_DTA || \
+                               i == APPEND_SAV || \
+			       i == APPEND_SAS || \
                                i == APPEND_JMULTI || \
                                i == APPEND_ODS)
 
-#define SAVE_GRAPH_ACTION(i) (i == SAVE_GNUPLOT || \
-                              i == SAVE_BOXPLOT_EPS || \
-                              i == SAVE_BOXPLOT_PS || \
-                              i == SAVE_BOXPLOT_XPM)
-
 #define EXPORT_ACTION(a,s) ((a == EXPORT_OCTAVE || \
                              a == EXPORT_R || \
                              a == EXPORT_CSV || \
@@ -75,7 +78,8 @@
                        i == OPEN_DATA || \
                        i == APPEND_DATA)
 
-#define SET_DIR_ACTION(i) (i == SET_DIR || i == SET_WDIR)
+#define SET_DIR_ACTION(i) (i == SET_DIR || i == SET_WDIR || \
+                           i == SET_FDIR || i == SET_DBDIR)
 
 struct extmap {
     int action;
@@ -85,13 +89,17 @@ struct extmap {
 static struct extmap action_map[] = {
     { SAVE_DBDATA,       ".bin" },
     { SAVE_SCRIPT,       ".inp" },
-    { SAVE_CONSOLE,      ".inp" },
+    { SAVE_FUNCTIONS_AS, ".inp" },
+    { SAVE_CMD_LOG,      ".inp" },
+    { SAVE_CONSOLE,      ".txt" },
     { SAVE_SESSION,      ".gretl" },
     { SAVE_GP_CMDS,      ".plt" },
-    { SAVE_BOXPLOT_EPS,  ".eps" },
-    { SAVE_BOXPLOT_PS,   ".ps" },
-    { SAVE_BOXPLOT_XPM,  ".xpm" },
+    { SAVE_R_CMDS,       ".R" },
+    { SAVE_OX_CMDS,      ".ox" },
+    { SAVE_OCTAVE_CMDS,  ".m" },    
     { SAVE_FUNCTIONS,    ".gfn" },
+    { SAVE_MARKERS,      ".txt" },
+    { SAVE_LABELS,       ".txt" },
     { EXPORT_CSV,        ".csv" },
     { EXPORT_R,          ".R" },
     { OPEN_OCTAVE,       ".m" },
@@ -106,22 +114,26 @@ static struct extmap action_map[] = {
     { OPEN_SESSION,      ".gretl" },
     { OPEN_CSV,          ".csv" },
     { APPEND_CSV,        ".csv" },
-    { OPEN_ASCII,        ".txt" },
-    { APPEND_ASCII,      ".txt" },
     { OPEN_GNUMERIC,     ".gnumeric" },
     { APPEND_GNUMERIC,   ".gnumeric" },
-    { OPEN_EXCEL,        ".xls" },
-    { APPEND_EXCEL,      ".xls" },
+    { OPEN_XLS,          ".xls" },
+    { APPEND_XLS,        ".xls" },
     { OPEN_WF1,          ".wf1" },
     { APPEND_WF1,        ".wf1" },
     { OPEN_DTA,          ".dta" },
     { APPEND_DTA,        ".dta" },
+    { OPEN_SAV,          ".sav" },
+    { APPEND_SAV,        ".sav" },
+    { OPEN_SAS,          ".xpt" },
+    { APPEND_SAS,        ".xpt" },
     { OPEN_JMULTI,       ".dat" },
     { APPEND_JMULTI,     ".dat" },
     { OPEN_ODS,          ".ods" },
     { APPEND_ODS,        ".ods" },
     { OPEN_RATS_DB,      ".rat" },
     { OPEN_PCGIVE_DB,    ".bn7" },
+    { OPEN_GFN,          ".gfn" },
+    { OPEN_BARS,         ".txt" },
     { FILE_OP_MAX,       NULL }
 };
 
@@ -142,6 +154,10 @@ static gretlopt save_action_to_opt (int action, gpointer p)
     default: break;
     }
 
+    if (action == SAVE_DATA_AS && session_file_is_open()) {
+	opt |= OPT_X; /* "exporting" to GDT */
+    }
+
     if (p != NULL) {
 	opt |= GPOINTER_TO_INT(p);
     }
@@ -162,36 +178,14 @@ static const char *get_gp_ext (int ttype)
     else return "*";
 }
 
-static int dat_ext (const char *str, int showerr)
-{
-    const char *suff;
-    int err = 0;
-
-    if (str == NULL) {
-	return 0;
-    }
-
-    suff = strrchr(str, '.');
-
-    if (suff != NULL && !strcmp(suff, ".gdt")) {
-	if (showerr) {
-	    errbox(_("The suffix you selected should be used\n"
-		   "only for gretl datafiles"));
-	}
-	err = 1;
-    }
-
-    return err;
-}
-
 static const char *get_ext (int action, gpointer data)
 {
     const char *s = NULL;
 
     if (GDT_ACTION(action)) {
 	return ".gdt";
-    } else if (action == SAVE_GNUPLOT) {
-	int ttype = gp_term_code(data);
+    } else if (action == SAVE_GNUPLOT || action == SAVE_GRAPHIC) {
+	int ttype = gp_term_code(data, action);
 
 	s = get_gp_ext(ttype);
     } else {
@@ -217,7 +211,7 @@ static int check_maybe_add_ext (char *fname, int action, gpointer data)
     }
 
     /* don't mess if the fname is really a dir */
-    if (isdir(fname)) {
+    if (gretl_isdir(fname)) {
 	return !SET_DIR_ACTION(action);
     }
 
@@ -249,7 +243,7 @@ static void script_window_update (windata_t *vwin, const char *fname)
 
     /* update the window title */
     title = g_strdup_printf("gretl: %s", trfname);
-    gtk_window_set_title(GTK_WINDOW(vwin->dialog), title);
+    gtk_window_set_title(GTK_WINDOW(vwin->main), title);
     g_free(trfname);
     g_free(title);
 
@@ -257,15 +251,18 @@ static void script_window_update (windata_t *vwin, const char *fname)
     strcpy(vwin->fname, fname);
 
     if (vwin->role == VIEW_LOG || vwin->role == VIEW_SCRIPT ||
-	vwin->role == VIEW_FUNC_CODE) {
+	vwin->role == VIEW_PKG_CODE) {
 	/* change role of window for editing */
 	vwin->role = EDIT_SCRIPT;
+    } else if (vwin->role == EDIT_GP) {
+	/* plot file no longer under session control */
+	vwin->flags &= ~VWIN_SESSION_GRAPH;
     }
 
-    mark_content_saved(vwin);
+    mark_vwin_content_saved(vwin);
 
     /* make the window editable */
-    if (!gtk_text_view_get_editable(GTK_TEXT_VIEW(vwin->w))) {
+    if (!gtk_text_view_get_editable(GTK_TEXT_VIEW(vwin->text))) {
 	view_window_set_editable(vwin);
     }
 }
@@ -275,11 +272,9 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 {
     FILE *fp;
     gchar *buf;
-#ifdef ENABLE_NLS
     gchar *trbuf;
-#endif
 
-    buf = textview_get_text(vwin->w);
+    buf = textview_get_text(vwin->text);
     if (buf == NULL) {
 	errbox("Couldn't retrieve buffer");
 	return;
@@ -292,15 +287,12 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	return;
     }
 
-#ifdef ENABLE_NLS
     trbuf = my_locale_from_utf8(buf);
     if (trbuf != NULL) {
+	/* UTF minuses? */
 	system_print_buf(trbuf, fp);
 	g_free(trbuf);
     }
-#else
-    system_print_buf(buf, fp);
-#endif
 
     g_free(buf);
     fclose(fp);
@@ -309,18 +301,12 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	strcpy(scriptfile, fname);
 	mkfilelist(FILE_LIST_SCRIPT, scriptfile);
 	script_window_update(vwin, fname);
-    }
-}
-
-static void set_startdir (char *startdir, int action)
-{
-    get_default_dir(startdir, action);
-
-#ifndef G_OS_WIN32
-    if (startdir[strlen(startdir) - 1] != '/') {
-	strcat(startdir, "/");
-    }
-#endif
+    } else if (action == SAVE_GP_CMDS || 
+	       action == SAVE_R_CMDS ||
+	       action == SAVE_OX_CMDS ||
+	       action == SAVE_OCTAVE_CMDS) {
+	script_window_update(vwin, fname);
+    } 
 }
 
 static void filesel_save_prn_buffer (PRN *prn, const char *fname)
@@ -337,14 +323,26 @@ static void filesel_save_prn_buffer (PRN *prn, const char *fname)
     }
 }
 
-static void filesel_open_script (const char *fname)
+static void filesel_open_script (const char *fname, windata_t *vwin)
 {
-    strcpy(tryfile, fname);
-
-    if (view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT) != NULL) {
-	strcpy(scriptfile, tryfile);
-	mkfilelist(FILE_LIST_SCRIPT, scriptfile);
-	set_currdir_from_filename(scriptfile);
+    if (vwin != NULL) {
+	strcpy(tryfile, fname);
+	sourceview_insert_file(vwin, fname);
+    } else if (has_suffix(fname, ".R")) {
+	view_file(fname, 1, 0, 78, 370, EDIT_R);
+    } else if (has_suffix(fname, ".plt")) {
+	view_file(fname, 1, 0, 78, 370, EDIT_GP);
+    } else if (ox_support && has_suffix(fname, ".ox")) {
+	view_file(fname, 1, 0, 78, 370, EDIT_OX);
+    } else if (has_suffix(fname, ".m")) {
+	view_file(fname, 1, 0, 78, 370, EDIT_OCTAVE);
+    } else {
+	strcpy(tryfile, fname);
+	if (view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT) != NULL) {
+	    strcpy(scriptfile, tryfile);
+	    mkfilelist(FILE_LIST_SCRIPT, scriptfile);
+	    gretl_set_current_dir(scriptfile);
+	}
     }
 }
 
@@ -352,12 +350,13 @@ static void filesel_open_session (const char *fname)
 {
     strcpy(tryfile, fname);
 
-    if (is_session_file(fname)) {
+    if (gretl_is_pkzip_file(fname)) {
 	verify_open_session();
     } else {
+	/* old script-style session file? */
 	windata_t *vwin;
 
-	if (has_system_prefix(tryfile, &paths, SCRIPT_SEARCH)) {
+	if (has_system_prefix(tryfile, SCRIPT_SEARCH)) {
 	    vwin = view_file(tryfile, 0, 0, 78, 370, VIEW_SCRIPT);
 	} else {
 	    vwin = view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT);
@@ -443,20 +442,34 @@ static void bootstrap_save_callback (const char *fname)
     } 
 }
 
+static void maybe_set_fsel_status (int action, FselDataSrc src, 
+				   gpointer p, int val)
+{
+    if (src == FSEL_DATA_STATUS && p != NULL) {
+	* (int *) p = val;
+    } else if (action == OPEN_BARS && val) {
+	/* error or cancel */
+	set_plotbars_filename(NULL, p);
+    }
+}
+
 static void
 file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 			      gpointer data)
 {
     char fname[FILENAME_MAX];
+    int err = 0;
 
     *fname = 0;
     strncat(fname, in_fname, FILENAME_MAX - 1);
 
     if (action < END_OPEN) {
+	/* opening a file: check that the file is accessible */
 	FILE *fp = gretl_fopen(fname, "r");
 
 	if (fp == NULL) {
 	    file_read_errbox(fname);
+	    maybe_set_fsel_status(action, src, data, E_FOPEN);
 	    return;
 	} else {
 	    fclose(fp);
@@ -468,13 +481,23 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 	verify_open_data(NULL, action);
     } else if (APPEND_DATA_ACTION(action)) {
 	strcpy(tryfile, fname);
-	do_open_data(NULL, NULL, action);
+	do_open_data(NULL, action);
     } else if (action == OPEN_SCRIPT) {
-	filesel_open_script(fname);
+	if (src == FSEL_DATA_VWIN) {
+	    filesel_open_script(fname, data);
+	} else {
+	    filesel_open_script(fname, NULL);
+	}
     } else if (action == OPEN_SESSION) {
 	filesel_open_session(fname);
     } else if (action == OPEN_MARKERS) {
 	do_add_markers(fname);
+    } else if (action == OPEN_LABELS) {
+	do_add_labels(fname);
+    } else if (action == OPEN_BARS) {
+	set_plotbars_filename(fname, data);
+    } else if (action == OPEN_GFN) {
+	edit_function_package(fname);
     } else if (action == OPEN_RATS_DB) {
 	open_rats_window(fname);
     } else if (action == OPEN_PCGIVE_DB) {
@@ -487,10 +510,6 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
 
     /* now for the save/export options */
 
-    if (action > SAVE_DBDATA && action != EXPORT_DAT && dat_ext(fname, 1)) { 
-	return;
-    }
-
     if (check_maybe_add_ext(fname, action, data)) {
 	return;
     }
@@ -504,310 +523,42 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
     } else if (src == FSEL_DATA_PRN) {
 	filesel_save_prn_buffer((PRN *) data, fname);
     } else if (SAVE_DATA_ACTION(action)) {
-	do_store(fname, save_action_to_opt(action, data));
+	err = do_store(fname, save_action_to_opt(action, data));
     } else if (action == SAVE_GNUPLOT) {
 	save_graph_to_file(data, fname);
-    } else if (action == SAVE_BOXPLOT_EPS || action == SAVE_BOXPLOT_PS) {
-	int err;
-
-	err = ps_print_plots(fname, action, data);
-	if (err) {
-	    errbox(_("boxplot save failed"));
-	}
+    } else if (action == SAVE_GRAPHIC) {
+	save_graphic_to_file(data, fname);
     } else if (action == SAVE_SESSION) {
-	save_session(fname);
+	err = save_session(fname);
+    } else if (action == SAVE_CMD_LOG) {
+	err = save_session_commands(fname);
     } else if (action == SAVE_FUNCTIONS) {
-	save_user_functions(fname, data);
+	err = save_function_package(fname, data);
+    } else if (action == SAVE_FUNCTIONS_AS) {
+	err = save_function_package_as_script(fname, data);
     } else if (action == SAVE_BOOT_DATA) {
 	bootstrap_save_callback(fname);
+    } else if (action == SAVE_MARKERS) {
+	err = do_save_markers(fname);
+    } else if (action == SAVE_LABELS) {
+	err = do_save_labels(fname);
     } else if (action == SET_PROG || action == SET_DIR) {
 	set_path_callback(data, fname);
     } else if (action == SET_WDIR) {
 	set_working_dir_callback(data, fname);
+    } else if (action == SET_FDIR) {
+	set_funcs_dir_callback(data, fname);
+    } else if (action == SET_DBDIR) {
+	set_db_dir_callback(data, fname);
     } else {
 	windata_t *vwin = (windata_t *) data;
 
 	save_editable_content(action, fname, vwin);
     }
+
+    maybe_set_fsel_status(action, src, data, err);
 }
 
-#ifdef G_OS_WIN32 
-
-/* ........................................................... */
-
-          /* MS Windows version of file selection code */
-
-/* ........................................................... */
-
-
-#include <windows.h>
-#include <shlobj.h>
-
-struct winfilter {
-    const char *descrip;
-    const char *pat;
-} winfilter;
-
-struct win32_filtermap {
-    int action;
-    struct winfilter filter;
-};
-
-static struct winfilter get_gp_filter (int ttype)
-{
-    static struct winfilter gpfilt[] = {
-	{ N_("postscript files (*.eps)"), "*.eps" },
-	{ N_("PDF files (*.pdf)"), "*.pdf" },
-	{ N_("xfig files (*.fig)"), "*.fig" },
-	{ N_("LaTeX files (*.tex)"), "*.tex" },
-	{ N_("PNG files (*.png)"), "*.png" },
-	{ N_("Windows metafiles (*.emf)"), "*.emf" },
-	{ N_("gnuplot files (*.plt)"), "*.plt" },
-	{ N_("all files (*.*)"), "*.*" }
-    };
-
-    if (ttype == GP_TERM_EPS) 
-	return gpfilt[0];
-    else if (ttype == GP_TERM_PDF) 
-	return gpfilt[1];
-    else if (ttype == GP_TERM_FIG) 
-	return gpfilt[2];
-    else if (ttype == GP_TERM_TEX) 
-	return gpfilt[3];
-    else if (ttype == GP_TERM_PNG) 
-	return gpfilt[4];
-    else if (ttype == GP_TERM_EMF) 
-	return gpfilt[5];
-    else if (ttype == GP_TERM_PLT)
-	return gpfilt[6];
-    else
-	return gpfilt[7];
-}
-
-static struct winfilter get_filter (int action, gpointer data)
-{
-    static struct win32_filtermap map[] = {
-	{ SAVE_DATA,    { N_("gretl data files (*.gdt)"), "*.gdt" }},
-	{ SAVE_DBDATA,  { N_("gretl database files (*.bin)"), "*.bin" }},
-	{ SAVE_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
-	{ SAVE_CONSOLE, { N_("gretl command files (*.inp)"), "*.inp" }},
-	{ SAVE_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
-	{ SAVE_BOXPLOT_EPS, { N_("postscript files (*.eps)"), "*.eps" }},
-	{ SAVE_BOXPLOT_PS,  { N_("postscript files (*.ps)"), "*.ps" }},
-	{ SAVE_GP_CMDS, { N_("gnuplot files (*.plt)"), "*.plt" }},
-	{ SAVE_FUNCTIONS,   { N_("gretl function files (*.gfn)"), "*.gfn" }},
-	{ EXPORT_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
-	{ EXPORT_R,     { N_("GNU R files (*.R)"), "*.R" }},
-	{ EXPORT_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
-	{ OPEN_OCTAVE,  { N_("GNU Octave files (*.m)"), "*.m" }},
-	{ APPEND_OCTAVE, { N_("GNU Octave files (*.m)"), "*.m" }},
-	{ EXPORT_DAT,   { N_("PcGive files (*.dat)"), "*.dat" }},
-	{ EXPORT_JM,    { N_("JMulti files (*.dat)"), "*.dat" }},
-	{ SAVE_OUTPUT,  { N_("text files (*.txt)"), "*.txt" }},
-	{ SAVE_TEX,     { N_("TeX files (*.tex)"), "*.tex" }},
-	{ SAVE_RTF,     { N_("RTF files (*.rtf)"), "*.rtf" }},
-	{ SAVE_TEXT,    { N_("ASCII files (*.txt)"), "*.txt" }},
-	{ OPEN_SCRIPT,  { N_("gretl script files (*.inp)"), "*.inp" }},
-	{ OPEN_SESSION, { N_("session files (*.gretl)"), "*.gretl" }},
-	{ OPEN_CSV,     { N_("CSV files (*.csv)"), "*.csv" }},
-	{ APPEND_CSV,   { N_("CSV files (*.csv)"), "*.csv" }},
-	{ OPEN_ASCII,   { N_("ASCII files (*.txt)"), "*.txt" }},
-	{ APPEND_ASCII, { N_("ASCII files (*.txt)"), "*.txt" }},
-	{ OPEN_GNUMERIC,   { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
-	{ APPEND_GNUMERIC, { N_("Gnumeric files (*.gnumeric)"), "*.gnumeric" }},
-	{ OPEN_EXCEL,   { N_("Excel files (*.xls)"), "*.xls" }},
-	{ APPEND_EXCEL, { N_("Excel files (*.xls)"), "*.xls" }},
-	{ OPEN_WF1,     { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
-	{ APPEND_WF1,   { N_("Eviews workfiles (*.wf1)"), "*.wf1" }},
-	{ OPEN_DTA,     { N_("Stata files (*.dta)"), "*.dta" }},
-	{ APPEND_DTA,   { N_("Stata files (*.dta)"), "*.dta" }},
-	{ OPEN_JMULTI,  { N_("JMulTi files (*.dat)"), "*.dat" }},
-	{ APPEND_JMULTI,{ N_("JMulTi files (*.dat)"), "*.dat" }},
-	{ OPEN_ODS,     { N_("Open Document Spreadsheet (*.ods)"), "*.ods" }},
-	{ APPEND_ODS,   { N_("Open Document Spreadsheet (*.ods)"), "*.ods" }},
-	{ OPEN_RATS_DB, { N_("RATS databases (*.rat)"), "*.rat" }},
-	{ OPEN_PCGIVE_DB, { N_("PcGive data files (*.bn7)"), "*.bn7" }},
-	{ SET_PROG,     { N_("program files (*.exe)"), "*.exe" }}
-    };
-    static struct winfilter default_filter = {
-	N_("all files (*.*)"), "*.*" 
-    };
-    struct winfilter filt;
-    int i;
-
-    if (action == SAVE_GNUPLOT) {
-	int ttype = gp_term_code(data);
-
-	filt = get_gp_filter(ttype);
-    } else {
-	filt = default_filter;
-
-	for (i=0; i< sizeof map / sizeof *map; i++) {
-	    if (action == map[i].action) {
-		filt = map[i].filter;
-		break;
-	    }
-	}
-    }
-
-    return filt;
-}
-
-static char *make_winfilter (int action, gpointer data)
-{
-    struct winfilter filter;
-    char *ret, *p;
-
-    ret = calloc(128, 1);
-    if (ret == NULL) {
-	return NULL;
-    }
-
-    p = ret;
-
-    if (GDT_ACTION(action)) {
-	filter = get_filter(SAVE_DATA, data);
-    } else {
-	filter = get_filter(action, data);
-    }
-
-    strcpy(p, I_(filter.descrip));
-    p += strlen(p) + 1;
-    strcpy(p, filter.pat);
-
-    if (strncmp(filter.descrip, "all", 3)) {
-	p += strlen(p) + 1;
-	strcpy(p, I_("all files (*.*)"));
-	p += strlen(p) + 1;
-	strcpy(p, "*.*");
-    }
-
-    return ret;
-}
-
-static int select_dirname (char *fname, char *trmsg)
-{
-    BROWSEINFO bi;
-    LPITEMIDLIST pidl;
-    char dirname[MAX_PATH];
-    int ret = 1;
-
-    CoInitialize(NULL);
-
-    bi.hwndOwner = NULL;
-    bi.pidlRoot = NULL; /* FIXME? */
-    bi.pszDisplayName = dirname;
-    bi.lpszTitle = trmsg;
-    bi.ulFlags = BIF_USENEWUI;
-    bi.lpfn = NULL;
-    bi.lParam = 0;
-    bi.iImage = 0;   
-
-    pidl = SHBrowseForFolder(&bi);
-    if (pidl == NULL) {
-	ret = 0;
-    } else {
-	SHGetPathFromIDList(pidl, fname);
-	CoTaskMemFree(pidl);
-    }
-
-    return ret;
-}
-
-static void win32_file_selector (const char *msg, int action, FselDataSrc src, 
-				 gpointer data) 
-{
-    OPENFILENAME of;
-    int retval;
-    char fname[MAXLEN], endname[64], startdir[MAXLEN];
-    char *filter = NULL;
-    gchar *trmsg = NULL;
-
-    *fname = '\0';
-    *endname = '\0';
-
-    set_startdir(startdir, action);
-
-    /* special cases */
-    if (action == SAVE_DATA && *paths.datfile != '\0') {
-	char *savename = suggested_savename(paths.datfile);
-
-	strcpy(fname, savename);
-	g_free(savename);
-	if (!(data_status & BOOK_DATA)) {
-	    get_base(startdir, paths.datfile, SLASH);
-	}
-    } else if (EXPORT_ACTION(action, src) && *paths.datfile != '\0') {
-	char *savename = suggested_exportname(paths.datfile, action);
-
-	strcpy(fname, savename);
-	g_free(savename);
-	get_base(startdir, paths.datfile, SLASH);
-    } else if (action == SET_PROG) {
-	char *strvar = (char *) data;
-
-	if (strvar != NULL && *strvar != '\0') {
-	    if (get_base(startdir, strvar, SLASH)) {
-		strcpy(fname, strvar + slashpos(strvar) + 1);
-	    } 
-	}
-    } else if (action == SAVE_FUNCTIONS) {
-	get_default_package_name(fname, data);
-    }
-
-    if (doing_nls()) {
-	trmsg = my_locale_from_utf8(msg);
-    } else {
-	trmsg = g_strdup(msg);
-    }
-
-    if (SET_DIR_ACTION(action)) {
-	retval = select_dirname(fname, trmsg);
-    } else {
-	/* initialize file dialog info struct */
-	memset(&of, 0, sizeof of);
-#ifdef OPENFILENAME_SIZE_VERSION_400
-	of.lStructSize = OPENFILENAME_SIZE_VERSION_400;
-#else
-	of.lStructSize = sizeof of;
-#endif
-	of.hwndOwner = NULL;
-	filter = make_winfilter(action, data);
-	of.lpstrFilter = filter;
-	of.lpstrCustomFilter = NULL;
-	of.nFilterIndex = 1;
-	of.lpstrFile = fname;
-	of.nMaxFile = sizeof fname;
-	of.lpstrFileTitle = endname;
-	of.nMaxFileTitle = sizeof endname;
-	of.lpstrInitialDir = startdir;
-	of.lpstrTitle = trmsg;
-	of.lpstrDefExt = NULL;
-	of.Flags = OFN_HIDEREADONLY;
-
-	if (action < END_OPEN || action == SET_PROG) {
-	    retval = GetOpenFileName(&of);
-	} else {
-	    /* a file save action */
-	    retval = GetSaveFileName(&of);
-	}
-    }
-
-    free(filter);
-    g_free(trmsg);
-
-    if (!retval) {
-	if (CommDlgExtendedError()) {
-	    errbox(_("File dialog box error"));
-	}
-	return;
-    }
-
-    file_selector_process_result(fname, action, src, data);
-}
-
-#else /* End of MS Windows file selection code, start GTK */
-
 static char *get_filter_suffix (int action, gpointer data, char *suffix)
 {
     
@@ -834,134 +585,242 @@ static GtkFileFilter *get_file_filter (int action, gpointer data)
     return filter;
 }
 
-static void gtk_file_selector (const char *msg, int action, FselDataSrc src, 
+#ifdef G_OS_WIN32
+
+static gchar *inplace_windows_filename (gchar *s)
+{
+    char tmp[MAXLEN];
+    int err;
+
+    err = filename_to_win32(tmp, s);
+
+    if (!err) {
+	g_free(s);
+	s = g_strdup(tmp);
+    }
+
+    return s;
+}
+
+static char *win32_correct_path (char *s)
+{
+    if (!g_utf8_validate(s, -1, NULL)) {
+	gchar *tmp = my_locale_to_utf8(s);
+
+	if (tmp != NULL) {
+	    strcpy(s, tmp);
+	    g_free(tmp);
+	}
+    }
+
+    return s;
+}
+
+#endif
+
+static void set_default_progs_path (GtkFileChooser *fsel)
+{
+#ifdef G_OS_WIN32
+    char *progs = program_files_path();
+
+    if (progs != NULL) {
+	gchar *path = my_filename_to_utf8(progs);
+
+	gtk_file_chooser_set_current_folder(fsel, path);
+	g_free(path);
+	free(progs);
+    }
+#else
+    gtk_file_chooser_set_current_folder(fsel, "/usr/bin");
+#endif
+}
+
+static void filesel_maybe_set_current_name (GtkFileChooser *filesel,
+					    int action, FselDataSrc src,
+					    gpointer data)
+{
+    gchar *currname;
+
+    if (action == SAVE_DATA && *datafile != '\0') {
+	currname = suggested_savename(datafile);
+	gtk_file_chooser_set_current_name(filesel, currname);
+	g_free(currname);
+    } else if (EXPORT_ACTION(action, src) && *datafile != '\0') {
+	currname = suggested_exportname(datafile, action);
+	gtk_file_chooser_set_current_name(filesel, currname);
+	g_free(currname);
+    } else if (action == SAVE_CMD_LOG) {
+	gtk_file_chooser_set_current_name(filesel, "session.inp");
+    } else if (action == SET_PROG || action == SET_DIR) {
+	currname = (gchar *) data;
+	if (currname != NULL && g_path_is_absolute(currname)) {
+	    gtk_file_chooser_set_filename(filesel, currname);
+	} else if (action == SET_PROG) {
+	    set_default_progs_path(filesel);
+	}	    
+    } else if (action == SAVE_FUNCTIONS || action == SAVE_FUNCTIONS_AS) {
+	char fname[MAXLEN];
+
+	*fname = '\0';
+	get_default_package_name(fname, data, action);
+	gtk_file_chooser_set_current_name(filesel, fname);
+    } else if (action == SAVE_MARKERS) {
+	gtk_file_chooser_set_current_name(filesel, "markers.txt");
+    } else if (action == SAVE_LABELS) {
+	gtk_file_chooser_set_current_name(filesel, "labels.txt");
+    }
+	
+}
+
+static void filesel_add_filter (GtkWidget *filesel,
+				const char *desc, 
+				const char *pat)
+{
+    GtkFileFilter *filt = gtk_file_filter_new();
+
+    gtk_file_filter_set_name(filt, desc);
+    gtk_file_filter_add_pattern(filt, pat);
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filt);
+}
+
+static void filesel_set_filters (GtkWidget *filesel, int action,
+				 FselDataSrc src, gpointer data)
+{
+    if (action == OPEN_CSV || action == APPEND_CSV) {
+	filesel_add_filter(filesel, _("CSV files (*.csv)"), "*.csv");
+	filesel_add_filter(filesel, _("ASCII files (*.txt)"), "*.txt");
+	filesel_add_filter(filesel, _("all files (*.*)"), "*");
+    } else if (action == OPEN_SCRIPT) {
+	filesel_add_filter(filesel, _("gretl script files (*.inp)"), "*.inp");
+	if (src != FSEL_DATA_FNPKG) {
+	    filesel_add_filter(filesel, _("GNU R files (*.R)"), "*.R");
+	    filesel_add_filter(filesel, _("gnuplot files (*.plt)"), "*.plt");
+	    filesel_add_filter(filesel, _("GNU Octave files (*.m)"), "*.m");
+	    if (ox_support) {
+		filesel_add_filter(filesel, _("Ox files (*.ox)"), "*.ox");
+	    }
+	}
+    } else if (action == OPEN_LABELS) {
+	filesel_add_filter(filesel, _("ASCII files (*.txt)"), "*.txt");
+	filesel_add_filter(filesel, _("all files (*.*)"), "*");
+    } else {
+	GtkFileFilter *filter = get_file_filter(action, data);
+
+	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filesel), filter);
+    }
+}
+
+static void remember_folder (GtkFileChooser *chooser, char *savedir)
+{
+    gchar *folder = gtk_file_chooser_get_current_folder(chooser);
+
+    strcpy(savedir, folder);
+    g_free(folder);
+}
+
+static void gtk_file_selector (int action, FselDataSrc src, 
 			       gpointer data, GtkWidget *parent) 
 {
-    GtkWidget *filesel;
+    static char savedir[MAXLEN];
     char startdir[MAXLEN];
-    GtkFileFilter *filter;
+    GtkWidget *filesel;
     GtkFileChooserAction fa;
     const gchar *okstr;
-
-    set_startdir(startdir, action);
+    int remember = get_keep_folder();
+    gint response;
 
     if (SET_DIR_ACTION(action)) {
 	fa = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 	okstr = GTK_STOCK_OK;
+	remember = 0;
     } else if (action == SET_PROG) {
 	fa = GTK_FILE_CHOOSER_ACTION_OPEN;
 	okstr = GTK_STOCK_OK;
+	remember = 0;
     } else if (action < END_OPEN) {
 	fa = GTK_FILE_CHOOSER_ACTION_OPEN;
 	okstr = GTK_STOCK_OPEN;
     } else {
 	fa = GTK_FILE_CHOOSER_ACTION_SAVE;
 	okstr = GTK_STOCK_SAVE;
-	parent = NULL;
+	if (action == SAVE_FUNCTIONS || action == SAVE_DATA_PKG) {
+	    remember = 0;
+	}
     }
 
-    /* FIXME: parent window below should not always be mdata->w,
-       in particular when action == SAVE_GNUPLOT
-    */
+    if (remember && *savedir != '\0') {
+	strcpy(startdir, savedir);
+    } else {
+	get_default_dir(startdir, action);
+    }
+
+#ifdef G_OS_WIN32
+    win32_correct_path(startdir);
+#endif
 
     if (parent == NULL) {
-	parent = mdata->w;
+	parent = mdata->main;
     }
 
-    filesel = gtk_file_chooser_dialog_new(msg, GTK_WINDOW(parent), fa,
+    filesel = gtk_file_chooser_dialog_new(NULL, GTK_WINDOW(parent), fa,
 					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					  okstr, GTK_RESPONSE_ACCEPT,
 					  NULL);
 
     gtk_dialog_set_default_response(GTK_DIALOG(filesel), GTK_RESPONSE_ACCEPT);
 
-    filter = get_file_filter(action, data);
-    if (action == OPEN_ASCII || action == APPEND_ASCII) {
-	gtk_file_filter_set_name(filter, _("ASCII files (*.txt)"));
-	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    }
-
-    gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(filesel), filter);
-
-    if (action == OPEN_ASCII || action == APPEND_ASCII) {
-	filter = gtk_file_filter_new();
-	gtk_file_filter_set_name(filter, _("all files (*.*)"));
-	gtk_file_filter_add_pattern(filter, "*.*");
-	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filter);
-    }
-
-    /* FIXME session dir */
-    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
-
-    /* special cases */
-
-    if (action == SAVE_DATA && *paths.datfile != '\0') {
-	char *savename = suggested_savename(paths.datfile);
-
-	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
-					  savename);
-	g_free(savename);
-    } else if (EXPORT_ACTION(action, src) && *paths.datfile != '\0') {
-	char *savename = suggested_exportname(paths.datfile, action);
-
-	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
-					  savename);
-	g_free(savename);
-    } else if (action == SET_PROG) {
-	char *strvar = (char *) data;
-
-	if (strvar != NULL && g_path_is_absolute(strvar)) {
-	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
-					  strvar);
-	} else {
-	    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), 
-						"/usr/bin");
-	}	    
-    } else if (action == SET_DIR) {
-	char *strvar = (char *) data;
-
-	if (strvar != NULL && g_path_is_absolute(strvar)) {
-	    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), 
-					  strvar);
-	} 
-    } else if (action == SAVE_FUNCTIONS) {
-	char fname[MAXLEN];
-
-	*fname = '\0';
-	get_default_package_name(fname, data);
-	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filesel), 
-					  fname);
-    }	
-
-    if (gtk_dialog_run(GTK_DIALOG(filesel)) == GTK_RESPONSE_ACCEPT) {
-	char *fname;
-
-	fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
-	gtk_widget_destroy(filesel);
-	file_selector_process_result(fname, action, src, data);
-	g_free(fname);
+    if (SET_DIR_ACTION(action)) {
+	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), startdir);
     } else {
+	filesel_set_filters(filesel, action, src, data);
+	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
+	filesel_maybe_set_current_name(GTK_FILE_CHOOSER(filesel), action,
+				       src, data);
+    }
+
+    response = gtk_dialog_run(GTK_DIALOG(filesel));
+
+    if (response == GTK_RESPONSE_ACCEPT) {
+	gchar *fname;
+	
+	fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
+	if (remember) {
+	    remember_folder(GTK_FILE_CHOOSER(filesel), savedir);
+	}
+	gtk_widget_destroy(filesel);
+	filesel = NULL;
+
+	if (fname != NULL) {
+#ifdef G_OS_WIN32
+	    fname = inplace_windows_filename(fname);
+#endif
+	    file_selector_process_result(fname, action, src, data);
+	    g_free(fname);
+	}
+    } else {
+	maybe_set_fsel_status(action, src, data, GRETL_CANCEL);
+    }
+
+    if (filesel != NULL) {
 	gtk_widget_destroy(filesel);
     }
 }
 
-#endif /* end of non-MS Windows code */
-
-void file_selector (const char *msg, int action, FselDataSrc src, gpointer data)
+void file_selector (int action, FselDataSrc src, gpointer data)
 {
-#ifdef G_OS_WIN32
-    win32_file_selector(msg, action, src, data);
-#else
-    gtk_file_selector(msg, action, src, data, NULL);
-#endif
+    GtkWidget *w = NULL;
+
+    if (src == FSEL_DATA_VWIN) {
+	windata_t *vwin = (windata_t *) data;
+
+	w = vwin->main;
+    }
+
+    gtk_file_selector(action, src, data, w);
 }
 
-void file_selector_with_parent (const char *msg, int action, FselDataSrc src, 
+void file_selector_with_parent (int action, FselDataSrc src, 
 				gpointer data, GtkWidget *w)
 {
-#ifdef G_OS_WIN32
-    win32_file_selector(msg, action, src, data);
-#else
-    gtk_file_selector(msg, action, src, data, w);
-#endif
+    gtk_file_selector(action, src, data, w);
 }
