@@ -20,14 +20,26 @@
 #ifndef FNSAVE_H
 #define FNSAVE_H
 
-void edit_function_package (const char *fname, int *loaderr);
+void edit_function_package (const char *fname);
 
-void prepare_functions_save (void);
+void edit_new_function_package (void);
 
-int save_user_functions (const char *fname, gpointer p);
+int save_function_package (const char *fname, gpointer p);
+
+int save_function_package_as_script (const char *fname, gpointer p);
 
 int no_user_functions_check (void);
 
-void get_default_package_name (char *fname, gpointer p);
+void get_default_package_name (char *fname, gpointer p, int mode);
+
+gchar *package_sample_get_script (windata_t *vwin);
+
+void update_sample_script (windata_t *vwin);
+
+int update_func_code (windata_t *vwin);
+
+void edit_package_at_startup (const char *fname);
+
+void revise_function_package (void *p);
 
 #endif /* FNSAVE_H */
