@@ -24,38 +24,54 @@
 
 typedef struct png_plot_t png_plot;
 
-int remove_png_term_from_plotfile_by_name (const char *fname);
+int remove_png_term_from_plot_by_name (const char *fname);
 
-void display_session_graph_png (const char *pltname);
+void display_session_graph (const char *path, const char *title);
 
-void gnuplot_show_png_by_name (const char *fname);
+void display_new_graph (void);
+
+void saver_preview_graph (GPT_SPEC *spec, char *termstr);
+
+int saver_save_graph (GPT_SPEC *spec, char *termstr, const char *fname);
 
 void plot_label_position_click (GtkWidget *w, png_plot *plot);
 
 int redisplay_edited_plot (png_plot *plot);
 
-void plot_remove_controller (png_plot *plot);
-
 void set_plot_has_y2_axis (png_plot *plot, gboolean s);
 
+int plot_is_saved (const png_plot *plot);
+
 int plot_is_mouseable (const png_plot *plot);
 
 GtkWidget *plot_get_shell (png_plot *plot);
 
+double plot_get_xmin (png_plot *plot);
+
+double plot_get_ymin (png_plot *plot);
+
+int plot_get_coordinates (png_plot *plot,
+			  double *xmin,
+			  double *xmax,
+			  double *ymin,
+			  double *ymax);
+
 GPT_SPEC *plot_get_spec (png_plot *plot);
 
 void revise_distribution_plotspec (png_plot *plot, int d, int df1, int df2);
 
-int gp_term_code (gpointer p);
+int gp_term_code (gpointer p, int action);
 
 void save_graph_to_file (gpointer p, const char *fname);
 
-void filter_gnuplot_file (int ttype, int latin, int mono, int recolor, 
+void filter_gnuplot_file (int ttype, int latin, int mono, 
 			  FILE *fpin, FILE *fpout);
 
-void gp_send_callback (GtkWidget *w, gpointer data);
+void run_gp_script (gchar *buf);
 
 int dump_plot_buffer (const char *buf, const char *fname,
 		      int addpause);
 
+void close_plot_windows (void);
+
 #endif /* GPT_CONTROL_H */
