@@ -23,6 +23,7 @@ enum extra_cmds {
     GR_XY,
     GR_IMP,
     GR_DUMMY,
+    GR_XYZ,
     GR_BOX,
     GR_NBOX,
     GR_3D,
@@ -35,15 +36,14 @@ enum extra_cmds {
     TRAMO,
     X12A,
     VIEW_SERIES,
-    VIEW_SCALAR,
     VIEW_MODEL,
     VIEW_LOG,
     VIEW_DATA,
     VIEW_SCRIPT,
     VIEW_CODEBOOK,
     VIEW_MODELTABLE,
-    VIEW_FUNC_INFO,
-    VIEW_FUNC_CODE,
+    VIEW_PKG_INFO,
+    VIEW_PKG_CODE,
     VIEW_FILE,
     DATA_REPORT,
     SCRIPT_OUT,
@@ -51,19 +51,17 @@ enum extra_cmds {
     EDIT_HEADER,
     EDIT_SCRIPT,
     EDIT_NOTES,
-    EDIT_FUNC_CODE,
+    EDIT_PKG_CODE,
+    EDIT_PKG_SAMPLE,
+    EDIT_GP,
+    EDIT_R,
+    EDIT_OX,
+    EDIT_OCTAVE,
     CLI_HELP,
     GUI_HELP,
     CLI_HELP_EN,
     GUI_HELP_EN,
     FUNCS_HELP,
-    MODELTABLE,
-    GRAPHPAGE,
-    LMTEST_LOGS,
-    LMTEST_SQUARES,
-    LMTEST_WHITE,
-    LMTEST_BG,
-    LMTEST_GROUPWISE,
     KERNEL_DENSITY,
     CREATE_DATASET,
     HCCME,
@@ -77,7 +75,6 @@ enum extra_cmds {
     VECM_RESTR,
     ELLIPSE,
     LAGS_DIALOG,
-    COPY_FORMATS,
     MINIBUF,
     VLAGSEL,
     VAROMIT,
@@ -99,36 +96,52 @@ enum extra_cmds {
     HILU,
     PWE,
     AFR, /* actual, fitted, residual */
+    GPT_ADDLINE,
+    GPT_CURVE,
+    OLOGIT,
+    MLOGIT,
+    OPROBIT,
+    IV_LIML,
+    IV_GMM,
+    BFGS_CONFIG,
+    COUNTMOD,
     GUI_CMD_MAX
 };
 
+#define HELP_ROLE(r) (r >= CLI_HELP && r <= FUNCS_HELP)
+
 enum file_ops {
     OPEN_DATA = GUI_CMD_MAX + 1, /* don't collide with extra_cmds */
+    OPEN_CSV,
+    OPEN_OCTAVE,
+    OPEN_GNUMERIC,
+    OPEN_XLS,
+    OPEN_ODS,
+    OPEN_WF1,
+    OPEN_DTA,
+    OPEN_SAV,
+    OPEN_SAS,
+    OPEN_JMULTI,     /* end of regular "open data" entries */
     OPEN_RATS_DB,
     OPEN_PCGIVE_DB,
     OPEN_SCRIPT,
-    OPEN_CSV,
     APPEND_DATA,
     APPEND_CSV,
-    OPEN_ASCII,
-    APPEND_ASCII,
-    OPEN_OCTAVE,
     APPEND_OCTAVE,
-    OPEN_GNUMERIC,
     APPEND_GNUMERIC,
-    OPEN_EXCEL,
-    APPEND_EXCEL,
-    OPEN_WF1,
-    APPEND_WF1,
-    OPEN_DTA,
-    APPEND_DTA,
-    OPEN_JMULTI,
-    APPEND_JMULTI,
-    OPEN_ODS,
+    APPEND_XLS,
     APPEND_ODS,
+    APPEND_WF1,
+    APPEND_DTA,
+    APPEND_SAV,
+    APPEND_SAS,
+    APPEND_JMULTI,   /* end of "data append" entries */
     OPEN_SESSION,
     OPEN_MARKERS,
-    END_OPEN,      /* marker for end of file open section */
+    OPEN_LABELS,
+    OPEN_BARS,
+    OPEN_GFN,
+    END_OPEN,        /* marker for end of file open section */
     SAVE_DATA,
     SAVE_DATA_AS,
     SAVE_DBDATA,
@@ -145,17 +158,26 @@ enum file_ops {
     SAVE_SCRIPT,
     SAVE_OUTPUT,
     SAVE_SESSION,
-    SAVE_BOXPLOT_EPS,
-    SAVE_BOXPLOT_PS,
-    SAVE_BOXPLOT_XPM,
     SAVE_GNUPLOT,
+    SAVE_GRAPHIC,
     SAVE_GP_CMDS,
+    SAVE_R_CMDS,
+    SAVE_OX_CMDS,
+    SAVE_OCTAVE_CMDS,
     SAVE_CONSOLE,
+    SAVE_CMD_LOG,
     SAVE_FUNCTIONS,
+    SAVE_FUNCTIONS_AS,
     SAVE_BOOT_DATA,
+    SAVE_MARKERS,
+    SAVE_LABELS,
+    EDIT_FUNCTIONS,
     SET_PROG,
     SET_DIR,
     SET_WDIR,
+    SET_FDIR,
+    SET_DBDIR,
+    SAVE_DATA_PKG,
     FILE_OP_MAX
 };
 
@@ -168,6 +190,7 @@ enum browser_codes {
     REMOTE_FUNC_FILES,
     NATIVE_DB,
     REMOTE_DB,
+    REMOTE_DATA_PKGS,
     NATIVE_SERIES,
     RATS_SERIES,
     PCGIVE_SERIES,
@@ -175,6 +198,8 @@ enum browser_codes {
     MAINWIN
 };
 
+#define BROWSER_ROLE(r) (r >= TEXTBOOK_DATA && r < MAINWIN)
+
 enum pref_tabs {
     TAB_NONE = 0,
     TAB_MAIN,
@@ -185,19 +210,8 @@ enum pref_tabs {
     TAB_MAX
 };
 
-enum stat_codes {
-    ESS = 1,
-    R2,
-    TR2,
-    DF,
-    SIGMA,
-    LNL,
-    AIC,
-    BIC,
-    HQC
-};
-
 enum clipstuff {
+    TARGET_UTF8_STRING,
     TARGET_STRING,
     TARGET_TEXT,
     TARGET_COMPOUND_TEXT,
@@ -211,7 +225,16 @@ enum data_status_flags {
     IMPORT_DATA   = 1 << 3,
     GUI_DATA      = 1 << 4,
     MODIFIED_DATA = 1 << 5,
-    GZIPPED_DATA  = 1 << 6
+    GZIPPED_DATA  = 1 << 6,
+    SESSION_DATA  = 1 << 7
+};
+
+enum register_data_flags {
+    DATAFILE_OPENED = 1,
+    OPENED_VIA_CLI,
+    OPENED_VIA_SESSION,
+    DATA_APPENDED,
+    NULLDATA_STARTED
 };
 
 enum drag_types {
@@ -239,8 +262,7 @@ enum varclick_actions {
 
 enum font_selections {
     FIXED_FONT_SELECTION,
-    APP_FONT_SELECTION,
-    GRAPH_FONT_SELECTION
+    APP_FONT_SELECTION
 };
 
 enum calc_functions {
@@ -250,7 +272,9 @@ enum calc_functions {
     CALC_NPTEST,
     CALC_GRAPH,
     CALC_GRAPH_ADD,
-    CALC_RAND
+    CALC_RAND,
+    CALC_PLOT,
+    CALC_MAX
 };
 
 enum auto_dummies {
@@ -259,18 +283,4 @@ enum auto_dummies {
     PANEL_TIME_DUMMIES
 };
 
-enum dynamic_forecast_status {
-    DYNAMIC_NA,
-    DYNAMIC_OK,
-    DYNAMIC_FORCED
-};
-
-#define MULTI_FORMAT_ENABLED(c) (c == SUMMARY || c == VAR_SUMMARY || \
-                                 c == ALL_SUMMARY || c == AFR || \
-	                         c == CORR || c == ALL_CORR || \
-	                         c == FCAST || c == COEFFINT || \
-	                         c == COVAR || c == VIEW_MODEL || \
-                                 c == VIEW_MODELTABLE || c == VAR || c == VECM || \
-                                 c == VAR_IRF || c == VAR_DECOMP)
-
 #endif /* GRETL_ENUMS_H */
