@@ -89,10 +89,11 @@ static int ws_startup (void)
 	fprintf(stderr, I_("Couldn't find usable socket driver\n"));
 	WSACleanup();
 	return 1;
+    } else {
+	/* arrange for cleanup on exit */
+	atexit(ws_cleanup);
     }
 
-    atexit(ws_cleanup);
-
     return 0;
 }
 
@@ -140,196 +141,82 @@ int create_child_process (char *prog)
     return real_create_child_process(prog, 1);
 }
 
-void startR (char *Rcommand)
+/* try registry for path to Rgui.exe */
+
+static int Rgui_path_from_registry (void)
 {
-    char Rprofile[MAXLEN], Rdata[MAXLEN], Rline[MAXLEN];
-    const char *supp1 = "--no-init-file";
-    const char *supp2 = "--no-restore-data";
-    int *list;
-    FILE *fp;
-    int err = 0;
+    char tmp[MAX_PATH];
+    int err;
 
-    if (!data_status) {
-	warnbox(_("Please open a data file first"));
-	return;
-    }
+    err = R_path_from_registry(tmp, RGUI);
 
-    build_path(Rprofile, paths.dotdir, "gretl.Rprofile", NULL);
-    fp = gretl_fopen(Rprofile, "w");
-    if (fp == NULL) {
-	errbox(_("Couldn't write R startup file"));
-	return;
-    }
-
-    err = ! SetEnvironmentVariable("R_PROFILE", Rprofile);
-    if (err) {
-	errbox(_("Couldn't set R_PROFILE environment variable"));
-	fclose(fp);
-	return;
-    } 	
-
-    build_path(Rdata, paths.dotdir, "Rdata.tmp", NULL);
-
-    sprintf(Rline, "store \"%s\" -r", Rdata);
-    list = command_list_from_string(Rline);
-
-    if (list == NULL ||
-	write_data(Rdata, list, (const double **) Z, datainfo, 
-		   OPT_R, NULL)) {
-	errbox(_("Write of R data file failed"));
-	fclose(fp);
-	return; 
-    }
-
-    free(list);
-
-    if (dataset_is_time_series(datainfo)) {
-	fputs("# load data from gretl\n", fp);
-	fputs("vnum <- as.double(R.version$major) + (as.double(R.version$minor) / 10.0)\n", fp);
-	fputs("if (vnum > 1.89) library(stats) else library(ts)\n", fp);
-	fprintf(fp, "source(\"%s\", echo=TRUE)\n", 
-		slash_convert(Rdata, FROM_BACKSLASH));
-    } else {
-	char Rtmp[MAXLEN];
-	FILE *fq;
-
-	build_path(Rtmp, paths.dotdir, "Rtmp", NULL);
-	fq = gretl_fopen(Rtmp, "w");
-	if (fq != NULL) {
-	    fputs("# load data from gretl\n", fq);
-	    fputs("library(stats)\n", fq);
-	    fputs("vnum <- as.double(R.version$major) + (as.double(R.version$minor) / 10.0)\n", fq);
-	    fputs("if (vnum > 2.41) library(utils)\n", fq);
-	    fprintf(fq, "gretldata <- read.table(\"%s\", header=TRUE)\n", 
-		    slash_convert(Rdata, FROM_BACKSLASH));
-	    fprintf(fq, "attach(gretldata)\n");
-	    fclose(fq);
-	}
-	fprintf(fp, "source(\"%s\", echo=TRUE)\n", 
-		slash_convert(Rtmp, FROM_BACKSLASH));
-    }
-
-    fclose(fp);
-
-    sprintf(Rline, "\"%s\" %s %s", Rcommand, supp1, supp2);
-    err = real_create_child_process(Rline, 0);
-
-    if (err) {
-	/* failed: try registry for Rgui.exe path? */
-	char tmp[MAX_PATH] = {0}; 
-
-	err = read_reg_val(HKEY_LOCAL_MACHINE, "R-core\\R", "InstallPath", tmp);
-	if (err) {
-	    err = read_reg_val(HKEY_LOCAL_MACHINE, "R", "InstallPath", tmp);
-	}
-	if (!err) {
-	    strcat(tmp, "\\bin\\Rgui.exe");
-	    sprintf(Rline, "\"%s\" %s %s", tmp, supp1, supp2);
-	    err = real_create_child_process(Rline, 1);
-	    if (!err) {
-		/* got a good R path, so record it */
-		*Rcommand = '\0';
-		strncat(Rcommand, tmp, MAXSTR - 1);
-	    }
-	}
-    }
-}
-
-char *slash_convert (char *str, int which)
-{
-    char *p;
-
-    if (str == NULL) {
-	return NULL;
-    }
-
-    p = str;
-    while (*p) {
-	if (which == FROM_BACKSLASH) {
-	    if (*p == '\\') *p = '/';
-	} else if (which == TO_BACKSLASH) {
-	    if (*p == '/') *p = '\\';
-	}
-	p++;
-    }
-
-    return str;
-}
-
-static char *substr (char *targ, const char *src, const char *p)
-{
-    *targ = '\0';
-
-    if (p == NULL) {
-	strcpy(targ, src);
-    } else {
-	strncat(targ, src, p - src);
-    }
-
-    return targ;
-}
-
-int unmangle (const char *dosname, char *longname)
-{
-    HANDLE handle;
-    WIN32_FIND_DATA fdata;
-    char tmp[MAXLEN];
-    const char *p;
-    int err = 0, done = 0;
-    char drive;
-
-    *longname = '\0';
-    
-    if (sscanf(dosname, "%c:\\", &drive) != 1 ||
-	dosname[strlen(dosname) - 1] == '\\') {
-	strcpy(longname, dosname);
-	return 0;
-    }
-
-    sprintf(longname, "%c:", tolower(drive));
-    p = dosname + 2;
-
-    while (!done) {
-	p = strchr(p + 1, '\\');
-	if (p == NULL) {
-	    done = 1;
-	} 
-	substr(tmp, dosname, p);
-	handle = FindFirstFile(tmp, &fdata);
-	if (handle != INVALID_HANDLE_VALUE) {
-	    strcat(longname, "\\");
-	    strcat(longname, fdata.cFileName);
-	    FindClose(handle);
-	} else {
-	    *longname = '\0';
-	    err = 1;
-	    break;
-	}
+    if (!err) {
+	*Rcommand = '\0';
+	strncat(Rcommand, tmp, MAXSTR - 1);
     }
 
     return err;
 }
 
-void win_help (void)
-{
-    char hlpshow[MAXLEN];
-    int found = 0;
+/* start R in asynchronous (interactive) mode */
 
-    sprintf(hlpshow, "hh.exe \"%s\\%s\"", paths.gretldir, _("gretl.chm"));
+void win32_start_R_async (void)
+{
+    const char *supp1 = "--no-init-file";
+    const char *supp2 = "--no-restore-data";
+    gchar *Rline = NULL;
+    int err;
     
-    if (WinExec(hlpshow, SW_SHOWNORMAL) < 32) {
-	if (strcmp("gretl.chm", _("gretl.chm"))) {
-	    /* try falling back on untranslated helpfile */
-	    sprintf(hlpshow, "hh.exe \"%s\\gretl.chm\"", paths.gretldir);
-	    if (WinExec(hlpshow, SW_SHOWNORMAL) >= 32) found = 1;
+    Rline = g_strdup_printf("\"%s\" %s %s", Rcommand, supp1, supp2);
+    err = real_create_child_process(Rline, 0);
+
+    if (err) {
+	err = Rgui_path_from_registry();
+	if (!err) {
+	    g_free(Rline);
+	    Rline = g_strdup_printf("\"%s\" %s %s", Rcommand, supp1, supp2);
+	    real_create_child_process(Rline, 1);
+	} else {
+	    gui_errmsg(E_EXTERNAL);
+	}
+    }
+
+    g_free(Rline);
+}
+
+/* Convert @src, a filename that might be in UTF-8 and might contain
+   forward slashes, to canonical win32 form.  We assume that @targ can
+   hold MAXLEN bytes.
+*/
+
+int filename_to_win32 (char *targ, const char *src)
+{
+    GError *gerr = NULL;
+    gsize bytes;
+    int err = 0;
+
+    *targ = '\0';
+
+    if (string_is_utf8((const unsigned char *) src)) {
+	gchar *tr = g_locale_from_utf8(src, -1, NULL, &bytes, &gerr);
+
+	if (gerr != NULL) {
+	    fprintf(stderr, "Couldn't handle '%s': %s\n", src, gerr->message);
+	    g_error_free(gerr);
+	    err = 1;
+	} else {
+	    strncat(targ, tr, MAXLEN - 1);
+	    g_free(tr);
 	}
     } else {
-	found = 1;
+	strncat(targ, src, MAXLEN - 1);
     }
 
-    if (!found) {
-	errbox(_("Couldn't access help file"));
+    if (!err) {
+	slash_convert(targ, TO_BACKSLASH);
     }
+
+    return err;
 }
 
 static void dummy_output_handler (const gchar *log_domain,
@@ -419,9 +306,9 @@ static void try_to_get_windows_font (void)
     }
 }
 
-int use_wimp;
+int use_wimp; /* note: published via gretlwin32.h */
 
-static int wimp_init (void)
+static void wimp_init (void)
 {
     char tmp[4] = {0};
 
@@ -439,12 +326,13 @@ static int wimp_init (void)
 void set_up_windows_look (void)
 {
     if (use_wimp) { 
-	size_t n = strlen(paths.gretldir);
-	int needslash = (paths.gretldir[n-1] != SLASH);
+	const char *gretldir = gretl_home();
+	size_t n = strlen(gretldir);
+	int needslash = (gretldir[n-1] != SLASH);
 	gchar *wimprc;
 
 	wimprc = g_strdup_printf("%s%sshare\\themes\\MS-Windows\\gtk-2.0\\gtkrc", 
-				 paths.gretldir, (needslash)? "\\" : "");
+				 gretldir, (needslash)? "\\" : "");
 	gtk_rc_parse(wimprc);
 	g_free(wimprc);
     } else {
@@ -452,81 +340,24 @@ void set_up_windows_look (void)
     }
 }
 
-static char inifile[FILENAME_MAX];
+/* carry out some Windows-specific start-up tasks, and
+   call read_rc to get configuration info */
 
-const char *get_network_cfg_filename (void)
+void gretl_win32_init (const char *progname, int debug)
 {
-    return inifile;
-}
-
-static int set_network_cfg_filename (const char *prog)
-{
-    const char *p;
-    int n;
-
-    *inifile = '\0';
-    
-    n = strlen(prog);
-    p = prog + n - 1;
-    while (p - prog >= 0) {
-	if (*p == '\\' || *p == '/') {
-	    strncpy(inifile, prog, n - strlen(p));
-	    strcat(inifile, "\\gretlnet.txt");
-	    break;
-	}
-	p--;
+    if (debug) {
+        redirect_io_to_console();
     }
 
-    return 0;
-}
-
-#if 0
-static int set_gd_fontpath (void)
-{
-    char fpath[MAX_PATH];
-    char *envstr;
-    UINT usz;
-
-    usz = GetWindowsDirectory(fpath, MAX_PATH);
-    if (usz == 0 || usz > MAX_PATH - 6) {
-        return 1;
-    }
-
-    fpath[2] = '/';
-    strcat(fpath, "/fonts");
-
-    SetEnvironmentVariable("GDFONTPATH", fpath);
-
-    envstr = malloc(strlen(fpath) + 12);
-    if (envstr != NULL) {
-	sprintf(envstr, "GDFONTPATH=%s", fpath);
-	putenv(envstr);
-	free(envstr);
-    }
-
-    return 0;        
-}
-#endif
-
-void gretl_win32_init (const char *progname)
-{
-    set_network_cfg_filename(progname);
+    set_gretlnet_filename(progname);
 
     wimp_init();
-    read_rc(); /* get config info from registry */
+    read_win32_config(debug);
     set_gretl_startdir();
-#if 0
-    set_gd_fontpath();
-#endif
     hush_warnings();
     ws_startup(); 
 }
 
-void gretl_win32_debug (void)
-{
-    redirect_io_to_console();
-}
-
 static int win_copy_buf (const char *buf, int fmt, size_t buflen)
 {
     HGLOBAL winclip;
@@ -735,7 +566,7 @@ int emf_to_clipboard (char *emfname)
     HENHMETAFILE hemf, hemfclip;
     HANDLE htest;
 
-    mainw = GDK_WINDOW_HWND(mdata->w->window);
+    mainw = GDK_WINDOW_HWND(mdata->main->window);
     if (mainw == NULL) {
 	errbox("Got NULL HWND");
 	return 1;
@@ -856,16 +687,73 @@ int win32_open_file (const char *fname)
     return err;
 }
 
-void win32_raise_window (GtkWidget *w)
+static const char *font_weight_string (int weight)
 {
-    GtkWidget *top = gtk_widget_get_toplevel(w);
+    if (weight >= FW_THIN && weight <= FW_LIGHT) {
+	return " Light";
+    }
+    if (weight >= FW_NORMAL && weight <= FW_DEMIBOLD) {
+	return "";
+    }
+    if (weight >= FW_BOLD) {
+	return " Bold";
+    }
+    return "";
+}
 
-    if (top != NULL && top->window != NULL) {
-	HWND h = GDK_WINDOW_HWND(top->window);
+static void fontname_to_win32 (const char *src, int flag,
+			       char *name, int *pts)
+{
+    int sz;
 
-	SetWindowPos(h, HWND_TOP, 0, 0, 0, 0, /* was HWND_TOPMOST */
-		     SWP_NOMOVE | SWP_NOSIZE);
-	gtk_widget_grab_focus(w);
+    if (sscanf(src, "%31[^0123456789]%d", name, &sz) == 2) {
+	size_t i, n = strlen(name);
+
+	for (i=n-1; i>0; i--) {
+	    if (name[i] == ' ') name[i] = 0;
+	    else break;
+	}
+	*pts = sz * 10; /* measured in tenths of a point */
+    } else {
+	*name = 0;
+	strncat(name, src, 31);
+	if (flag == FIXED_FONT_SELECTION) *pts = 100;
+	else *pts = 80;
     }
 }
 
+void win32_font_selector (char *fontname, int flag)
+{
+    CHOOSEFONT cf;            /* common dialog box structure */
+    LOGFONT lf;               /* logical font structure */
+
+    ZeroMemory(&cf, sizeof cf);
+    cf.lStructSize = sizeof cf;
+    cf.Flags = CF_SCREENFONTS | CF_TTONLY | CF_LIMITSIZE | CF_INITTOLOGFONTSTRUCT;
+    cf.nSizeMin = 6;
+    cf.nSizeMax = 24;
+
+    ZeroMemory(&lf, sizeof lf);
+
+    cf.Flags |= CF_NOSCRIPTSEL;
+    
+    lf.lfWeight = FW_REGULAR;
+    lf.lfCharSet = DEFAULT_CHARSET;
+
+    if (flag == FIXED_FONT_SELECTION) {
+	cf.Flags |= CF_FIXEDPITCHONLY;
+    } 
+
+    fontname_to_win32(fontname, flag, lf.lfFaceName, &(cf.iPointSize));
+
+    cf.lpLogFont = &lf;
+
+    if (ChooseFont(&cf) == TRUE && *(cf.lpLogFont->lfFaceName)) {
+	sprintf(fontname, "%s%s %d", cf.lpLogFont->lfFaceName, 
+		font_weight_string(cf.lpLogFont->lfWeight), 
+		cf.iPointSize / 10);
+    } else {
+	/* signal cancellation */
+	*fontname = '\0';
+    }
+}
