@@ -24,27 +24,37 @@ void helpfile_init (void);
 
 void context_help (GtkWidget *widget, gpointer data);
 
-void plain_text_cmdref (gpointer p, guint cmdnum, GtkWidget *w);
+void command_help_callback (int cmdnum, int en);
 
-void genr_funcs_ref (gpointer p, guint fnum, GtkWidget *w);
+int command_help_index (const char *word);
 
-gint edit_script_help (GtkWidget *widget, GdkEventButton *b,
-		       windata_t *vwin);
+void function_help_callback (int fnum);
 
-void display_pdf_help (gpointer p, guint uguide, GtkWidget *w);
+void plain_text_cmdref (GtkAction *action);
 
-void datafile_find (GtkWidget *widget, gpointer data);
+void genr_funcs_ref (GtkAction *action);
 
-void menu_find (gpointer data, guint dbfind, GtkWidget *widget);
+gint interactive_script_help (GtkWidget *widget, GdkEventButton *b,
+			      windata_t *vwin);
 
-void text_find_callback (GtkWidget *w, gpointer data);
+void display_pdf_help (GtkAction *action);
+
+void display_gnuplot_help (void);
+
+void listbox_find (gpointer unused, gpointer data);
+
+void text_find (gpointer unused, gpointer data);
+
+void vwin_add_finder (windata_t *vwin);
+
+int add_help_navigator (windata_t *vwin, GtkWidget *hp);
 
 char *quoted_help_string (const char *s);
 
-void gretl_tooltips_init (void);
-
-void gretl_tooltips_add (GtkWidget *w, const gchar *str);
-
 int function_help_index_from_word (const char *s);
 
+int gui_console_help (const char *param);
+
+void set_up_helpview_menu (windata_t *hwin);
+
 #endif /* HELPFILES_H */
