@@ -18,6 +18,7 @@
  */
 
 #include "gretl.h"
+#include "dlgutils.h"
 #include "varprint.h"
 #include "textutil.h"
 #include "textbuf.h"
@@ -28,7 +29,9 @@
 #include "texprint.h"
 #include "system.h"
 
-/* find-and-replace related materials */
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 7
+# include "dlgutils.h"
+#endif
 
 struct search_replace {
     GtkWidget *w;
@@ -38,13 +41,13 @@ struct search_replace {
     gchar *replace;
 };
 
-static void replace_string_callback (GtkWidget *widget, 
-				     struct search_replace *s)
+/* Find/Replace functions */
+
+static void replace_string_setup (GtkWidget *widget, 
+				  struct search_replace *s)
 {
-    s->find = 
-	gtk_editable_get_chars(GTK_EDITABLE(s->f_entry), 0, -1);
-    s->replace = 
-	gtk_editable_get_chars(GTK_EDITABLE(s->r_entry), 0, -1);
+    s->find = gtk_editable_get_chars(GTK_EDITABLE(s->f_entry), 0, -1);
+    s->replace = gtk_editable_get_chars(GTK_EDITABLE(s->r_entry), 0, -1);
     gtk_widget_destroy(s->w);
 }
 
@@ -58,14 +61,22 @@ static void trash_replace (GtkWidget *widget,
 
 static void replace_string_dialog (struct search_replace *s)
 {
-    GtkWidget *label, *button, *hbox;
+    GtkWidget *label, *button;
+    GtkWidget *vbox, *hbox, *abox;
 
     s->w = gtk_dialog_new();
 
     gtk_window_set_title(GTK_WINDOW(s->w), _("gretl: replace"));
     gtk_container_set_border_width(GTK_CONTAINER(s->w), 5);
 
-    /* Find part */
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 7)
+    g_signal_connect(G_OBJECT(s->w), "key-press-event", 
+		     G_CALLBACK(esc_kills_window), NULL);
+#endif
+
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(s->w));
+
+    /* 'Find' part */
     hbox = gtk_hbox_new(TRUE, TRUE);
     label = gtk_label_new(_("Find:"));
     gtk_widget_show(label);
@@ -74,27 +85,26 @@ static void replace_string_dialog (struct search_replace *s)
     gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), s->f_entry, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(s->w)->vbox), 
-		       hbox, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
 
-    /* Replace part */
+    /* 'Replace' part */
     hbox = gtk_hbox_new(TRUE, TRUE);
     label = gtk_label_new(_("Replace with:"));
     gtk_widget_show(label);
     s->r_entry = gtk_entry_new();
     g_signal_connect(G_OBJECT(s->r_entry), "activate", 
-		     G_CALLBACK(replace_string_callback), s);
+		     G_CALLBACK(replace_string_setup), s);
 
     gtk_widget_show(s->r_entry);
     gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(hbox), s->r_entry, TRUE, TRUE, 0);
     gtk_widget_show(hbox);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(s->w)->vbox), 
-		       hbox, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
 
-    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(s->w)->action_area), 15);
-    gtk_box_set_homogeneous(GTK_BOX 
-			    (GTK_DIALOG(s->w)->action_area), TRUE);
+    abox = gtk_dialog_get_action_area(GTK_DIALOG(s->w));
+
+    gtk_box_set_spacing(GTK_BOX(abox), 15);
+    gtk_box_set_homogeneous(GTK_BOX(abox), TRUE);
     gtk_window_set_position(GTK_WINDOW(s->w), GTK_WIN_POS_MOUSE);
 
     g_signal_connect(G_OBJECT(s->w), "destroy",
@@ -103,21 +113,18 @@ static void replace_string_dialog (struct search_replace *s)
     /* replace button -- make this the default */
     button = gtk_button_new_with_label(_("Replace all"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(s->w)->action_area), 
-		       button, TRUE, TRUE, FALSE);
+    gtk_box_pack_start(GTK_BOX(abox), button, TRUE, TRUE, 0);
     g_signal_connect(G_OBJECT(button), "clicked",
-		     G_CALLBACK(replace_string_callback), s);
+		     G_CALLBACK(replace_string_setup), s);
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
 
     /* cancel button */
     button = gtk_button_new_with_label(_("Cancel"));
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(s->w)->action_area), 
-		       button, TRUE, TRUE, FALSE);
+    gtk_box_pack_start(GTK_BOX(abox), button, TRUE, TRUE, 0);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(trash_replace), s);
-
     gtk_widget_show(button);
 
     gtk_widget_grab_focus(s->f_entry);
@@ -126,7 +133,7 @@ static void replace_string_dialog (struct search_replace *s)
     gtk_main();
 }
 
-void text_replace (windata_t *vwin, guint u, GtkWidget *w)
+void text_replace (GtkWidget *w, windata_t *vwin)
 {
     gchar *buf = NULL;
     gchar *fullbuf = NULL;
@@ -141,17 +148,22 @@ void text_replace (windata_t *vwin, guint u, GtkWidget *w)
     GtkTextIter sel_start, sel_end, start, end;
     gboolean selected = FALSE;
 
+    s.find = NULL;
+    s.replace = NULL;
+
     replace_string_dialog(&s);
 
     if (s.find == NULL || s.replace == NULL || *s.find == '\0') {
 	return;
     }
 
-    gedit = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
+    gedit = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
 
     gtk_text_buffer_get_start_iter(gedit, &start);
     gtk_text_buffer_get_end_iter(gedit, &end);
 
+    /* grab full buffer for possible undo, even if we're
+       actually going to work on a selection */
     fullbuf = gtk_text_buffer_get_text(gedit, &start, &end, FALSE);
 
     if (gtk_text_buffer_get_selection_bounds(gedit, &sel_start, &sel_end)) {
@@ -197,6 +209,8 @@ void text_replace (windata_t *vwin, guint u, GtkWidget *w)
     *modbuf = '\0';
 
     if (selected) {
+	/* copy unmodified the portion of the original buffer
+	   before the selection */
 	tmp = gtk_text_buffer_get_text(gedit, &start, &sel_start, FALSE);
 	if (tmp != NULL) {
 	    strcat(modbuf, tmp);
@@ -205,19 +219,22 @@ void text_replace (windata_t *vwin, guint u, GtkWidget *w)
     }
 
     p = buf;
-    while (*p && (size_t) (p - buf) <= fullsz) {
+    while (*p) {
 	q = strstr(p, s.find);
 	if (q != NULL) {
 	    strncat(modbuf, p, q - p);
 	    strcat(modbuf, s.replace);
 	    p = q + len;
 	} else {
+	    /* no more replacements needed */
 	    strcat(modbuf, p);
 	    break;
 	}
     }
 
     if (selected) {
+	/* copy unmodified the portion of the original buffer
+	   after the selection */
 	tmp = gtk_text_buffer_get_text(gedit, &sel_end, &end, FALSE);
 	if (tmp != NULL) {
 	    strcat(modbuf, tmp);
@@ -227,11 +244,11 @@ void text_replace (windata_t *vwin, guint u, GtkWidget *w)
 
     if (vwin->sbuf == NULL) {
 	/* replace copy of original buffer for "undo" */
-	tmp = g_object_steal_data(G_OBJECT(vwin->w), "undo");
+	tmp = g_object_steal_data(G_OBJECT(vwin->text), "undo");
 	if (tmp != NULL) {
 	    g_free(tmp);
 	}
-	g_object_set_data(G_OBJECT(vwin->w), "undo", fullbuf);
+	g_object_set_data(G_OBJECT(vwin->text), "undo", fullbuf);
 	fullbuf = NULL;
     } 
 
@@ -284,26 +301,23 @@ static int special_text_handler (windata_t *vwin, guint fmt, int what)
 	MODEL *pmod = (MODEL *) vwin->data;
 
 	if (pmod->errcode) { 
-	    errbox("Couldn't format model");
-	    err = 1;
+	    err = pmod->errcode;
+	} else if (tex_format(prn)) {
+	    err = tex_print_model(pmod, datainfo, 
+				  get_tex_eqn_opt(), 
+				  prn);
 	} else {
-	    if (tex_format(prn)) {
-		tex_print_model(pmod, datainfo, 
-				get_tex_eqn_opt(), 
-				prn);
-	    } else {
-		printmodel(pmod, datainfo, OPT_NONE, prn);
-	    }
+	    err = printmodel(pmod, datainfo, OPT_NONE, prn);
 	}
     } else if (cmd == VAR || cmd == VECM) {
 	GRETL_VAR *var = (GRETL_VAR *) vwin->data;
 
-	gretl_VAR_print(var, datainfo, OPT_NONE, prn);
+	err = gretl_VAR_print(var, datainfo, OPT_NONE, prn);
     } else if (cmd == VAR_IRF || cmd == VAR_DECOMP) {
 	windata_t *parent = vwin->gretl_parent;
 
 	if (parent == NULL) {
-	    errbox("Couldn't find the VAR");
+	    err = E_DATA;
 	} else {
 	    GRETL_VAR *var = (GRETL_VAR *) parent->data;
 	    int h = vwin->active_var; /* here records preferred horizon */
@@ -317,12 +331,15 @@ static int special_text_handler (windata_t *vwin, guint fmt, int what)
     } else if (cmd == SYSTEM) {
 	equation_system *sys = (equation_system *) vwin->data;
 
-	gretl_system_print(sys, (const double **) Z, datainfo, OPT_NONE, prn);
+	err = gretl_system_print(sys, (const double **) Z, datainfo, 
+				 OPT_NONE, prn);
     } else if (cmd == VIEW_MODELTABLE) {
 	err = special_print_model_table(prn);
     } 
 
-    if (!err) {
+    if (err) {
+	gui_errmsg(err);
+    } else {
 	if (what == W_PREVIEW) {
 	    /* there's no RTF preview option */
 	    view_latex(prn);
@@ -331,7 +348,8 @@ static int special_text_handler (windata_t *vwin, guint fmt, int what)
 	} else if (what == W_SAVE) {
 	    int action = (fmt & GRETL_FORMAT_TEX)? SAVE_TEX : SAVE_RTF;
 
-	    file_selector(_("Save"), action, FSEL_DATA_PRN, prn);
+	    file_selector_with_parent(action, FSEL_DATA_PRN, 
+				      prn, vwin->main);
 	}
     }
 
@@ -347,7 +365,17 @@ void window_tex_callback (GtkWidget *w, windata_t *vwin)
 	N_("Copy"),
 	N_("Save")
     };
-    int opt = radio_dialog("gretl: LaTeX", NULL, opts, 3, 0, 0);
+    int opt;
+
+    if (vwin->role == VAR_IRF || vwin->role == VAR_DECOMP) {
+	if (vwin->gretl_parent == NULL) {
+	    warnbox(_("Not available"));
+	    gtk_widget_set_sensitive(w, FALSE);
+	    return;
+	}
+    }
+
+    opt = radio_dialog("gretl: LaTeX", NULL, opts, 3, 0, 0);
 
     if (opt >= 0) {
 	int fmt = GRETL_FORMAT_TEX;
@@ -364,52 +392,63 @@ void window_tex_callback (GtkWidget *w, windata_t *vwin)
     }
 }
 
-void model_tex_view (gpointer data, guint fmt, GtkWidget *w)
+static int tex_format_code (GtkAction *action)
+{
+    const gchar *s = gtk_action_get_name(action);
+    int fmt = GRETL_FORMAT_TEX;
+
+    if (strstr(s, "Eqn")) {
+	fmt |= GRETL_FORMAT_EQN;
+    }
+
+    return fmt;
+}
+
+void model_tex_view (GtkAction *action, gpointer data)
 {
     windata_t *vwin = (windata_t *) data;
+    int fmt = tex_format_code(action);
 
     special_text_handler(vwin, fmt, W_PREVIEW);
 }
 
-void model_tex_save (gpointer data, guint fmt, GtkWidget *w)
+void model_tex_save (GtkAction *action, gpointer data)
 {
     windata_t *vwin = (windata_t *) data;
+    int fmt = tex_format_code(action);
 
     special_text_handler(vwin, fmt, W_SAVE);
 }
 
-void system_tex_callback (gpointer data, guint opt, GtkWidget *w)
+void model_tex_copy (GtkAction *action, gpointer data) 
 {
     windata_t *vwin = (windata_t *) data;
+    int fmt = tex_format_code(action);
 
-    special_text_handler(vwin, GRETL_FORMAT_TEX, opt);
+    special_text_handler(vwin, fmt, W_COPY);
 }
 
 static gchar *text_window_get_copy_buf (windata_t *vwin, int select)
 {
-    GtkTextBuffer *textbuf = 
-	gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
-    gchar *cpybuf = NULL;
+    gchar *cpy = NULL;
 
-    if (!select) {
-	cpybuf = textview_get_text(vwin->w); 
-    } else if (gtk_text_buffer_get_selection_bounds(textbuf, NULL, NULL)) {
-	GtkTextIter selstart, selend;
+    if (select) {
+	GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
+	GtkTextIter start, end;
 
-	gtk_text_buffer_get_selection_bounds(textbuf, &selstart, &selend);
-	cpybuf = gtk_text_buffer_get_text(textbuf, &selstart, &selend, FALSE);
-    } 
+	if (gtk_text_buffer_get_selection_bounds(buf, &start, &end)) {
+	    cpy = gtk_text_buffer_get_text(buf, &start, &end, FALSE);
+	}
+    } else {	
+	cpy = textview_get_text(vwin->text); 
+    }
 
-    return cpybuf;
+    return cpy;
 }
 
 static gchar *maybe_amend_buffer (gchar *inbuf, int fmt)
 {
-#ifdef ENABLE_NLS
     gchar *outbuf = my_locale_from_utf8(inbuf);
-#else
-    gchar *outbuf = g_strdup(inbuf);
-#endif
 
     free(inbuf);
     inbuf = outbuf;
@@ -424,6 +463,27 @@ static gchar *maybe_amend_buffer (gchar *inbuf, int fmt)
     return outbuf;
 }
 
+int multiple_formats_ok (windata_t *vwin)
+{
+    int r = vwin->role;
+
+    if (r == SUMMARY || r == VAR_SUMMARY || 
+	r == ALL_SUMMARY || r == AFR || 
+	r == CORR || r == ALL_CORR || 
+	r == FCAST || r == COEFFINT || 
+	r == COVAR || r == VIEW_MODELTABLE || 
+	r == VAR || r == VECM || 
+	r == VAR_IRF || r == VAR_DECOMP) {
+	return 1;
+    } else if (r == VIEW_MODEL) {
+	MODEL *pmod = (MODEL *) vwin->data;
+
+	return !RQ_SPECIAL_MODEL(pmod);
+    } else {
+	return 0;
+    }
+}
+
 /* copying text from gretl windows */
 
 #define SPECIAL_FORMAT(f) ((f & GRETL_FORMAT_TEX) || \
@@ -435,10 +495,11 @@ static void window_copy_or_save (windata_t *vwin, guint fmt, int action)
 
     if (vwin->role == VIEW_MODEL && fmt == GRETL_FORMAT_CSV) {
 	special_text_handler(vwin, fmt, action);
-    } else if (MULTI_FORMAT_ENABLED(vwin->role) && SPECIAL_FORMAT(fmt)) {
+    } else if (multiple_formats_ok(vwin) && SPECIAL_FORMAT(fmt)) {
 	special_text_handler(vwin, fmt, action);
-    } else if (fmt == GRETL_FORMAT_CSV || fmt == GRETL_FORMAT_TAB) {
-	csv_copy_listed_vars(vwin, fmt, action);
+    } else if (fmt == GRETL_FORMAT_CSV || fmt == GRETL_FORMAT_TAB ||
+	       fmt == GRETL_FORMAT_RTF) {
+	copy_vars_formatted(vwin, fmt, action);
     } else if (fmt == GRETL_FORMAT_TXT || fmt == GRETL_FORMAT_RTF_TXT) {
 	cpybuf = text_window_get_copy_buf(vwin, 0);
     } else if (fmt == GRETL_FORMAT_SELECTION) {
@@ -457,22 +518,22 @@ static void window_copy_or_save (windata_t *vwin, guint fmt, int action)
 	}
 
 	textprn = gretl_print_new_with_buffer(cpybuf);
+
 	if (action == W_COPY) {
 	    prn_to_clipboard(textprn, fmt);
 	} else {
 	    int fcode = (fmt == GRETL_FORMAT_RTF_TXT)? 
 		SAVE_RTF : SAVE_OUTPUT;
 
-	    file_selector(_("Save"), fcode, FSEL_DATA_PRN, textprn);
+	    file_selector_with_parent(fcode, FSEL_DATA_PRN, 
+				      textprn, vwin->main);
 	}
 	gretl_print_destroy(textprn);
     }	
 }
 
-void window_copy (gpointer data, guint fmt, GtkWidget *w) 
+void window_copy (windata_t *vwin, guint fmt) 
 {
-    windata_t *vwin = (windata_t *) data;
-
     window_copy_or_save(vwin, fmt, W_COPY);
 }
 
@@ -485,24 +546,77 @@ void window_save (windata_t *vwin, guint fmt)
 
 #ifdef NATIVE_PRINTING
 
-void window_print (windata_t *vwin, guint u, GtkWidget *w) 
+void window_print (GtkAction *action, windata_t *vwin) 
 {
-    char *buf, *selbuf = NULL;
+    gchar *buf, *selbuf = NULL;
+    const char *filename = NULL;
     GtkTextBuffer *tbuf;
     GtkTextIter start, end;
 
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->w));
-    buf = textview_get_text(vwin->w);
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
+    buf = textview_get_text(vwin->text);
 
     if (gtk_text_buffer_get_selection_bounds(tbuf, &start, &end)) {
 	selbuf = gtk_text_buffer_get_text(tbuf, &start, &end, FALSE);
     }
 
-    winprint(buf, selbuf);
+    if (vwin->role == EDIT_SCRIPT ||
+	vwin->role == VIEW_SCRIPT) {
+	const char *p = strrchr(vwin->fname, SLASH);
+	
+	if (p != NULL) {
+	    filename = p + 1;
+	} else {
+	    filename = vwin->fname;
+	}
+    }
+
+    print_window_content(buf, selbuf, filename);
 }
 
 #endif
 
+#define UTF_MINUS(u,i) (u[i] == 0xE2 && u[i+1] == 0x88 && u[i+2] == 0x92)
+
+/* check @s for UTF-8 minus signs (U+2212), and if any are found do an
+   in-place replacement with ASCII dashes
+*/
+
+char *strip_utf_minus (char *s)
+{
+    unsigned char *u = (unsigned char *) s;
+    int i, n = strlen(s);
+    int got_minus = 0;
+
+    for (i=0; i<n-3; i++) {
+	if (UTF_MINUS(u, i)) {
+	    got_minus = 1;
+	    break;
+	}
+    }
+
+    if (got_minus) {
+	char *tmp = calloc(n, 1);
+
+	if (tmp != NULL) {
+	    int j = 0;
+
+	    for (i=0; i<n; i++) {
+		if (i < n - 3 && UTF_MINUS(u, i)) {
+		    tmp[j++] = '-';
+		    i += 2;
+		} else {
+		    tmp[j++] = u[i];
+		}
+	    }
+	    strcpy(s, tmp);
+	    free(tmp);
+	} 
+    } 
+
+    return s;
+}
+
 /* print buf to file, trying to ensure it's not messed up */
 
 void system_print_buf (const gchar *buf, FILE *fp)
@@ -513,10 +627,10 @@ void system_print_buf (const gchar *buf, FILE *fp)
     while (*p) {
 	if (*p == '\r') {
 	    if (*(p+1) != '\n') {
-		putc('\n', fp);
+		fputc('\n', fp);
 	    } 
 	} else {
-	    putc(*p, fp);
+	    fputc(*p, fp);
 	}
 	cbak = *p;
 	p++;
@@ -524,7 +638,7 @@ void system_print_buf (const gchar *buf, FILE *fp)
 
     /* ensure file ends with newline */
     if (cbak != '\n') {
-	putc('\n', fp);
+	fputc('\n', fp);
     }
 }
 
