@@ -21,7 +21,6 @@
 
 #include "genparse.h"
 #include "usermat.h"
-#include "loop_private.h"
 #include "gretl_func.h"
 #include "gretl_string_table.h"
 
@@ -57,6 +56,7 @@ struct str_table consts[] = {
     { CONST_PI,    "pi" },
     { CONST_NA,    "NA" },
     { CONST_WIN32, "WIN32" },
+    { CONST_EPS,   "macheps" },
     { 0,        NULL }
 };
 
@@ -71,15 +71,23 @@ struct str_table dvars[] = {
     { R_NOBS,      "$nobs" },
     { R_NVARS,     "$nvars" },
     { R_PD,        "$pd" },
+    { R_DATATYPE,  "$datatype" },
     { R_TEST_STAT, "$test" },
     { R_TEST_PVAL, "$pvalue" },
     { R_TEST_LNL,  "$rlnl" },
-    { R_INDEX,     "t" },
+    { R_KLNL,      "$kalman_lnl" },
+    { R_KS2,       "$kalman_s2" },
+    { R_KSTEP,     "$kalman_t" },
     { R_INDEX,     "obs" },
     { R_T1,        "$t1" },
     { R_T2,        "$t2" },
     { R_STOPWATCH, "$stopwatch" },
     { R_NSCAN,     "$nscan" },
+    { R_PUNIT,     "$unit" },
+    { R_WINDOWS,   "$windows" },
+    { R_VERSION,   "$version" },
+    { R_ERRNO,     "$error" },
+    { R_SWITCH,    "$switch" },
     { 0,           NULL },
 };
 
@@ -96,21 +104,28 @@ struct str_table mvars[] = {
     { M_BIC,     "$bic" },
     { M_HQC,     "$hqc" },
     { M_TRSQ,    "$trsq" },
+    { M_DWPVAL,  "$dwpval" },
+    { M_FSTT,    "$Fstat" },
+    { M_CHISQ,   "$chisq" },
     { M_UHAT,    "$uhat" },
     { M_YHAT,    "$yhat" },
+    { M_LLT,     "$llt" },
     { M_AHAT,    "$ahat" },
+    { M_SAMPLE,  "$sample" },
     { M_H,       "$h" },
     { M_COEFF,   "$coeff" },
     { M_SE,      "$stderr" },
     { M_VCV,     "$vcv" },
     { M_RHO,     "$rho" },
     { M_COMPAN,  "$compan" },
+    { M_XTXINV,  "$xtxinv" },
     { M_JALPHA,  "$jalpha" }, 
     { M_JBETA,   "$jbeta" },
     { M_JVBETA,  "$jvbeta" },
     { M_JS00,    "$s00" },
     { M_JS11,    "$s11" },
     { M_JS01,    "$s01" },
+    { M_EC,      "$ec" },
     { M_HAUSMAN, "$hausman" },
     { M_SARGAN,  "$sargan" },
     { M_SYSGAM,  "$sysGamma" },
@@ -118,6 +133,10 @@ struct str_table mvars[] = {
     { M_SYSB,    "$sysB" },
     { M_FCAST,   "$fcast" },
     { M_FCERR,   "$fcerr" },
+    { M_COEFF_CI,"$coeff_ci" },
+    { M_KLLT,    "$kalman_llt" },
+    { M_KUHAT,   "$kalman_uhat" },
+    { M_XLIST,   "$xlist" },
     { 0,         NULL }
 };
 
@@ -129,6 +148,12 @@ struct str_table funcs[] = {
     { F_ASIN,     "asin" },
     { F_ACOS,     "acos" },
     { F_ATAN,     "atan" },
+    { F_SINH,     "sinh" },
+    { F_COSH,     "cosh" },
+    { F_TANH,     "tanh" },
+    { F_ASINH,    "asinh" },
+    { F_ACOSH,    "acosh" },
+    { F_ATANH,    "atanh" },
     { F_LOG,      "log" },
     { F_LOG,      "ln" },
     { F_LOG10,    "log10" },
@@ -153,7 +178,6 @@ struct str_table funcs[] = {
     { F_T2,       "lastobs" },
     { F_RUNIFORM, "uniform" }, 
     { F_RNORMAL,  "normal" }, 
-    { F_RPOISSON, "genpois" },
     { F_CUM,      "cum" }, 
     { F_MISSING,  "missing" },
     { F_DATAOK,   "ok" },        /* opposite of missing */
@@ -175,12 +199,17 @@ struct str_table funcs[] = {
     { F_QNORM,    "qnorm" },
     { F_GAMMA,    "gammafun" },
     { F_LNGAMMA,  "lngamma" },
+    { F_DIGAMMA,  "digamma" },
     { F_RESAMPLE, "resample" },
+    { F_PNOBS,    "pnobs" },     /* per-unit nobs in panels */
+    { F_PMIN,     "pmin" },      /* panel min */
+    { F_PMAX,     "pmax" },      /* panel max */
     { F_PMEAN,    "pmean" },     /* panel mean */
     { F_PSD,      "psd" },       /* panel std dev */
     { F_HPFILT,   "hpfilt" },    /* Hodrick-Prescott filter */
     { F_BKFILT,   "bkfilt" },    /* Baxter-King filter */
     { F_FRACDIFF, "fracdiff" },  /* fractional difference */
+    { F_BOXCOX,   "boxcox" },    /* Box-Cox transformation */
     { F_COV,      "cov" },
     { F_COR,      "corr" },
     { F_MOVAVG,   "movavg" },
@@ -188,12 +217,14 @@ struct str_table funcs[] = {
     { F_ZEROS,    "zeros" },
     { F_ONES,     "ones" },
     { F_SEQ,      "seq" },
+    { F_REPLACE,  "replace" },
     { F_MUNIF,    "muniform" },
     { F_MNORM,    "mnormal" },
-    { F_SUMR,     "sumr" },
     { F_SUMC,     "sumc" },
-    { F_MEANR,    "meanr" },
+    { F_SUMR,     "sumr" },
     { F_MEANC,    "meanc" },
+    { F_MEANR,    "meanr" },
+    { F_SDC,      "sdc" },
     { F_MINC,     "minc" },
     { F_MAXC,     "maxc" },
     { F_MINR,     "minr" },
@@ -211,6 +242,7 @@ struct str_table funcs[] = {
     { F_MXTAB,    "mxtab" },
     { F_CDEMEAN,  "cdemean" },
     { F_CHOL,     "cholesky" },
+    { F_PSDROOT,  "psdroot" },
     { F_INV,      "inv" },
     { F_INVPD,    "invpd" },
     { F_GINV,     "ginv" },
@@ -242,24 +274,28 @@ struct str_table funcs[] = {
     { F_BFGSMAX,  "BFGSmax" },
     { F_OBSNUM,   "obsnum" },
     { F_ISSERIES, "isseries" },
+    { F_ISSCALAR, "isscalar" },
     { F_ISLIST,   "islist" },
     { F_ISSTRING, "isstring" },
     { F_ISNULL,   "isnull" },
     { F_LISTLEN,  "nelem" },
+    { F_PDF,      "pdf" },
     { F_CDF,      "cdf" },
     { F_INVCDF,   "invcdf" },
     { F_PVAL,     "pvalue" },
     { F_CRIT,     "critical" },
     { F_RANDGEN,  "randgen" },
+    { F_URCPVAL,  "urcpval" },
     { F_MAKEMASK, "makemask" },
     { F_VALUES,   "values" },
     { F_MSHAPE,   "mshape" },
     { F_SVD,      "svd" },
     { F_MOLS,     "mols" },
+    { F_MPOLS,    "mpols" },
     { F_MREAD,    "mread" },
     { F_MWRITE,   "mwrite" },
-    { F_MRSEL,    "selifr" },
     { F_MCSEL,    "selifc" },
+    { F_MRSEL,    "selifr" },
     { F_POLROOTS, "polroots" },
     { F_DUMIFY,   "dummify" },
     { F_WMEAN,    "wmean" },
@@ -267,6 +303,9 @@ struct str_table funcs[] = {
     { F_WSD,      "wsd" },
     { F_XPX,      "xpx" },
     { F_FILTER,   "filter" },
+    { F_KFILTER,  "kfilter" },
+    { F_KSMOOTH,  "ksmooth" },
+    { F_KSIMUL,   "ksimul" },
     { F_TRIMR,    "trimr" },
     { F_GETENV,   "getenv" },
     { F_ARGNAME,  "argname" },
@@ -274,17 +313,44 @@ struct str_table funcs[] = {
     { F_READFILE, "readfile" },
     { F_BACKTICK, "grab" },
     { F_STRSTR,   "strstr" },
+    { F_STRNCMP,  "strncmp" },
     { F_STRLEN,   "strlen" },
+    { F_SSCANF,   "sscanf" },
     { F_VARNAME,  "varname" },
+    { F_VARNUM,   "varnum" },
+    { F_TOLOWER,  "tolower" },
     { F_COLNAMES, "colnames" },
     { F_LJUNGBOX, "ljungbox" },
-    { 0,        NULL }
+    { F_MSORTBY,  "msortby" },
+    { F_LINCOMB,  "lincomb" },
+    { F_IMHOF,    "imhof" },
+    { F_TOEPSOLV, "toepsolv" },
+    { F_DSUM,     "diagcat" },
+    { F_XMIN,     "xmin" },
+    { F_XMAX,     "xmax" },
+    { F_CORRGM,   "corrgm" },
+    { F_MCOVG,    "mcovg" },
+    { F_FCSTATS,  "fcstats" },
+    { F_BESSEL,   "bessel" },
+    { F_FRACLAG,  "fraclag" },
+    { F_MREVERSE, "mreverse" },
+    { F_DESEAS,   "deseas" },
+    { F_PERGM,    "pergm" },
+    { F_IRR,      "irr" },
+    { F_NPV,      "npv" },
+    { F_LOGISTIC, "logistic" },
+    { F_WEEKDAY,  "weekday" },
+    { F_KDENSITY, "kdensity" },
+    { F_MONTHLEN, "monthlen" },
+    { F_EPOCHDAY, "epochday" },
+    { F_HASHGET,  "bundleget" },
+    { F_HASHSET,  "bundleset" },
+    { 0,          NULL }
 };
 
 struct str_table func_alias[] = {
     { F_GAMMA,     "gammafunc" },
     { F_GAMMA,     "gamma" },
-    { F_RPOISSON,  "poisson" },
     { F_PVAL,      "pval" },
     { F_LOG,       "logs" },
     { F_OBSLABEL,  "date" },
@@ -449,6 +515,20 @@ const char *data_var_name (int i)
     return dvars[i].str;
 }
 
+const char *gretl_function_complete (const char *s)
+{
+    size_t n = strlen(s);
+    int i;
+
+    for (i=0; funcs[i].str != NULL; i++) {
+	if (!strncmp(s, funcs[i].str, n)) {
+	    return funcs[i].str;
+	}
+    }
+
+    return NULL;
+}
+
 /* end external stuff */
 
 static int dummy_lookup (const char *s)
@@ -552,30 +632,47 @@ int genr_function_word (const char *s)
 static void undefined_symbol_error (const char *s, parser *p)
 {
     parser_print_input(p);
-    pprintf(p->prn, _("The symbol '%s' is undefined\n"), s);
-    sprintf(gretl_errmsg, _("The symbol '%s' is undefined\n"), s);
+
+    if (p->ch == '.') {
+	gretl_errmsg_sprintf(_("%s: no such object\n"), s);
+    } else {
+	gretl_errmsg_sprintf(_("The symbol '%s' is undefined\n"), s);
+    }
+
     p->err = E_UNKVAR;
 }
 
 static void function_noargs_error (const char *s, parser *p)
 {
     parser_print_input(p);
+
     pprintf(p->prn, _("'%s': no argument was given\n"), s);
-    sprintf(gretl_errmsg, _("'%s': no argument was given\n"), s);
-    p->err = 1;
+    gretl_errmsg_sprintf(_("'%s': no argument was given\n"), s);
+
+    p->err = E_ARGS;
 }
 
 void context_error (int c, parser *p)
 {
-    parser_print_input(p);
     if (c != 0) {
+	parser_print_input(p);
 	pprintf(p->prn, _("The symbol '%c' is not valid in this context\n"), c);
+	if (c == '&') {
+	    pputs(p->prn, _("(for logical AND, use '&&')\n"));
+	} else if (c == '|') {
+	    pputs(p->prn, _("(for logical OR, use '||')\n"));
+	}
     } else {
-	pprintf(p->prn, _("The symbol '%s' is not valid in this context\n"), 
-		getsymb(p->sym, p));
+	const char *s = getsymb(p->sym, p);
+
+	if (s != NULL && *s != '\0') {
+	    pprintf(p->prn, _("The symbol '%s' is not valid in this context\n"), 
+		    getsymb(p->sym, p));
+	}
     }
-    if (p->err == 0) {
-	p->err = 1;
+
+    if (!p->err) {
+	p->err = E_PARSE;
     }
 }
 
@@ -593,6 +690,16 @@ static char *get_quoted_string (parser *p)
 	p->err = E_PARSE;
     }
 
+    if (!p->err) {
+	if (p->ch == '.' && *p->point == '$') {
+	    /* maybe quoted name of saved object followed by 
+	       dollar variable? */
+	    p->sym = OVAR;
+	} else {
+	    p->sym = STR;
+	}
+    }
+
     return s;
 }
 
@@ -635,7 +742,7 @@ NODE *obs_node (parser *p)
     int special = 0;
     int t = -1;
 
-    close = haschar(']', s);
+    close = charpos(']', s);
 
 #if LDEBUG
     fprintf(stderr, "obs_node: s='%s', ch='%c', close=%d\n", 
@@ -649,7 +756,7 @@ NODE *obs_node (parser *p)
 	pprintf(p->prn, _("Unmatched '%c'\n"), '[');
 	p->err = E_PARSE;
     } else if (*s == '"' && close < OBSLEN + 2 &&
-	       haschar('"', s+1) == close - 2) {
+	       charpos('"', s+1) == close - 2) {
 	/* quoted observation label? */
 	strncat(word, s, close);
 	special = 1;
@@ -687,11 +794,11 @@ static void look_up_string_variable (const char *s, parser *p)
     const char *val = get_string_by_name(s + 1);
 
     if (val != NULL) {
-	p->idstr = gretl_strdup(val);
+	p->idstr = gretl_strdup(s);
 	if (p->idstr == NULL) {
 	    p->err = E_ALLOC;
 	} else {
-	    p->sym = STR;
+	    p->sym = VSTR;
 	}
     } else {
 	undefined_symbol_error(s, p);
@@ -713,6 +820,22 @@ static void look_up_dollar_word (const char *s, parser *p)
     }
 }
 
+#ifdef USE_RLIB
+# include "gretl_foreign.h"
+
+static int maybe_get_R_function (const char *s)
+{
+    if (strlen(s) >= 3 && !strncmp(s, "R.", 2)) {
+	return get_R_function_by_name(s + 2);
+    } else {
+	return 0;
+    }
+}
+
+#else /* !USE_RLIB */
+# define maybe_get_R_function(s) (0)
+#endif
+
 static void look_up_word (const char *s, parser *p)
 {
     int fsym, err = 0;
@@ -728,16 +851,20 @@ static void look_up_word (const char *s, parser *p)
 	    if (p->idnum > 0) {
 		p->sym = DUM;
 	    } else {
-		p->idnum = varindex(p->dinfo, s);
-		if (p->idnum < p->dinfo->v) {
-		    p->sym = (var_is_scalar(p->dinfo, p->idnum))?
-			USCALAR : USERIES;
+		if ((p->idnum = current_series_index(p->dinfo, s)) >= 0) {
+		    p->sym = USERIES;
 		} else if (!strcmp(s, "time")) {
 		    p->sym = DUM;
 		    p->idnum = DUM_TREND;
+		} else if (gretl_is_scalar(s)) {
+		    p->sym = USCALAR;
+		    p->idstr = gretl_strdup(s);
 		} else if (get_matrix_by_name(s)) {
 		    p->sym = UMAT;
 		    p->idstr = gretl_strdup(s);
+		} else if (gretl_is_bundle(s)) {
+		    p->sym = BUNDLE;
+		    p->idstr = gretl_strdup(s);
 		} else if (gretl_get_object_by_name(s)) {
 		    p->sym = UOBJ;
 		    p->idstr = gretl_strdup(s);
@@ -748,14 +875,24 @@ static void look_up_word (const char *s, parser *p)
 		    p->sym = UFUN;
 		    p->idstr = gretl_strdup(s);
 		} else if (string_is_defined(s)) {
-		    p->sym = STR;
-		    p->idstr = gretl_strdup(get_string_by_name(s));
+		    p->sym = VSTR;
+		    p->idstr = gretl_strdup(s);
 		} else if (p->targ == LIST &&
 			   varname_match_any(p->dinfo, s)) {
 		    p->sym = LIST;
 		    p->idstr = gretl_strdup(s);
+		} else if (!strcmp(s, "t")) {
+		    /* if "t" has not been otherwise defined, treat it
+		       as an alias for "obs"
+		    */
+		    p->sym = DVAR;
+		    p->idnum = R_INDEX;
+		} else if (maybe_get_R_function(s)) {
+		    /* note: all "native" types take precedence over this */
+		    p->sym = RFUN;
+		    p->idstr = gretl_strdup(s + 2);
 		} else {
-		    err = 1;
+		    err = E_UNKVAR;
 		}
 	    }
 	}
@@ -789,9 +926,13 @@ static void word_check_next_char (const char *s, parser *p)
 	} else if (p->sym == MVAR && model_data_matrix(p->idnum)) {
 	    /* old-style "$coeff(x1)" etc. */
 	    p->sym = DMSTR;
-	} else if (!func1_symb(p->sym) && !func2_symb(p->sym) &&
-		   !funcn_symb(p->sym) && p->sym != UFUN) {
-	    p->err = 1;
+	} else if (!func1_symb(p->sym) && 
+		   !func2_symb(p->sym) &&
+		   !func3_symb(p->sym) &&
+		   !funcn_symb(p->sym) && 
+		   p->sym != UFUN && 
+		   p->sym != RFUN) {
+	    p->err = E_PARSE;
 	} 
     } else if (p->ch == '[') {
 	if (p->sym == UMAT) {
@@ -804,15 +945,25 @@ static void word_check_next_char (const char *s, parser *p)
 	    /* observation from series */
 	    p->sym = OBS;
 	} else {
-	    p->err = 1;
+	    p->err = E_PARSE;
 	} 
-    } else if (p->ch == '.' && parser_charpos(p, '$') == 0) {
+    } else if (p->ch == '.' && *p->point == '$') {
 	if (p->sym == UOBJ) {
 	    /* name of saved object followed by dollar variable? */
 	    p->sym = OVAR;
+	} else if (p->sym == STR) {
+	    /* maybe quoted name of saved object followed by 
+	       dollar variable? */
+	    p->sym = OVAR;
 	} else {
-	    p->err = 1;
+	    p->err = E_PARSE;
 	}	    
+    } else if (p->ch == '.' && isalpha(*p->point)) {
+	if (p->sym == LIST) {
+	    p->sym = LISTVAR;
+	} else {
+	    p->err = E_PARSE;
+	}
     }
 
     if (p->err) {
@@ -826,7 +977,7 @@ static int is_word_char (parser *p)
 	return 1;
     } else if (p->targ == LIST && p->ch == '*') {
 	return 1;
-    }
+    } 
 
     return 0;
 }
@@ -840,6 +991,14 @@ static void getword (parser *p)
     word[i++] = p->ch;
     parser_getc(p);
 
+#ifdef USE_RLIB
+    /* allow for R.foo function namespace */
+    if (*word == 'R' && p->ch == '.' && *p->point != '$') {
+	word[i++] = p->ch;
+	parser_getc(p);
+    }
+#endif
+
     while (p->ch != 0 && is_word_char(p) && i < 31) {
 	word[i++] = p->ch;
 	parser_getc(p);
@@ -856,22 +1015,15 @@ static void getword (parser *p)
 	parser_getc(p);
     }
 
-    if (p->getstr) {
+    if (p->flags & P_GETSTR) {
 	/* uninterpreted string wanted */
 	p->sym = STR;
 	p->idstr = gretl_strdup(word);
-	p->getstr = 0;
+	p->flags ^= P_GETSTR;
 	return;
     }
 
-    /* handle loop index scalar */
-    if (word[1] == '\0' && is_active_index_loop_char(word[0])) {
-	p->sym = LOOPIDX;
-	p->idstr = gretl_strdup(word);
-	return;
-    }
-
-    if ((*word == '$' && word[1]) || !strcmp(word, "t") || !strcmp(word, "obs")) {
+    if ((*word == '$' && word[1]) || !strcmp(word, "obs")) {
 	look_up_dollar_word(word, p);
     } else if (*word == '@') {
 	look_up_string_variable(word, p);
@@ -900,6 +1052,18 @@ void set_matrix_slice_off (void)
     doing_matrix_slice = 0;
 }
 
+static int doing_lag_parse;
+
+void set_lag_parse_on (void)
+{
+    doing_lag_parse = 1;
+}
+
+void set_lag_parse_off (void)
+{
+    doing_lag_parse = 0;
+}
+
 static int colon_ok (char *s, int n)
 {
     int i;
@@ -964,10 +1128,14 @@ static double getdbl (parser *p)
 {
     char xstr[NUMLEN] = {0};
     double d = NADBL;
+    int gotcol = 0;
     int i = 0;
 
     while (ok_dbl_char(p->ch, xstr, i - 1) && i < NUMLEN - 1) {
 	xstr[i++] = p->ch;
+	if (p->ch == ':') {
+	    gotcol = 1;
+	}
 	parser_getc(p);
     }  
 
@@ -980,7 +1148,7 @@ static double getdbl (parser *p)
     fprintf(stderr, "getdbl: xstr = '%s'\n", xstr);
 #endif
     
-    if (strchr(xstr, ':')) {
+    if (gotcol) {
 	if (p->dinfo->pd == 1) {
 	    p->err = E_PDWRONG;
 	} else {
@@ -994,29 +1162,18 @@ static double getdbl (parser *p)
 	}
     } else {
 	d = dot_atof(xstr);
+#if LDEBUG
+	fprintf(stderr, "getdbl: dot_atof gave %g\n", d);
+#endif
     }
     
     return d;
 }
 
-#if 0 /* later? */
-static void deprecation_note (parser *p)
-{
-    if (p->sym == B_AND) {
-	pprintf(p->prn, "Note: the recommended form for logical "
-		"AND is '&&'\n");
-    } else if (p->sym == B_OR) {
-	pprintf(p->prn, "Note: the recommended form for logical "
-		"OR is '||'\n");
-    }
-}
-#endif
+#define word_start_special(c) (c == '$' || c == '@' || c == '_')
 
-#define matrix_gen(p) (p->lh.t == MAT || p->targ == MAT)
-
-#define unary_context(p) (p->sym < F2_MAX || p->sym == COM)
-
-#define lag_range_sym(p) (p->ch == 't' && *p->point == 'o' && \
+#define lag_range_sym(p) (doing_lag_parse && p->ch == 't' && \
+                          *p->point == 'o' && \
 			  *(p->point + 1) == ' ')
 
 void lex (parser *p)
@@ -1078,34 +1235,21 @@ void lex (parser *p)
 	    parser_getc(p);
 	    return;
         case '&': 
-#if 0 /* for later, but modified */
-	    if (unary_context(p)) {
-		p->sym = U_ADDR;
-		parser_getc(p);
-		return;
-	    }
-	    p->sym = B_AND;
 	    parser_getc(p);
 	    if (p->ch == '&') {
+		p->sym = B_AND;
 		parser_getc(p);
 	    } else {
-		deprecation_note(p);
+		p->sym = U_ADDR;
 	    }
-#else
-	    p->sym = B_AND;
-	    parser_getc(p);
-	    if (p->ch == '&') {
-		/* make "&&" equal to "&" */
-		parser_getc(p);
-	    }
-#endif
 	    return;
         case '|': 
-	    p->sym = (matrix_gen(p))? B_MRCAT : B_OR;
 	    parser_getc(p);
 	    if (p->ch == '|') {
 		p->sym = B_OR;
 		parser_getc(p);
+	    } else {
+		p->sym = B_VCAT;
 	    }
 	    return;
         case '!': 
@@ -1120,6 +1264,7 @@ void lex (parser *p)
         case '=': 
 	    parser_getc(p);
 	    if (p->ch == '=') {
+		/* allow "==" as synonym for "=" */
 		parser_getc(p);
 	    }
 	    p->sym = B_EQ;
@@ -1170,11 +1315,7 @@ void lex (parser *p)
 	    parser_getc(p);
 	    return;
         case '~':
-	    p->sym = B_MCCAT;
-	    parser_getc(p);
-	    return;
-        case '`': 
-	    p->sym = B_MRCAT;
+	    p->sym = B_HCAT;
 	    parser_getc(p);
 	    return;
         case ',': 
@@ -1233,7 +1374,7 @@ void lex (parser *p)
 		parser_getc(p);
 		return;
 	    } else {
-		/* not a "dot operator", back up */
+		/* not a "dot operator", so back up */
 		parser_ungetc(p);
 	    }
         default: 
@@ -1253,17 +1394,16 @@ void lex (parser *p)
 		p->sym = NUM;
 		return;
 	    } else if (islower(p->ch) || isupper(p->ch) || 
-		       p->ch == '$' || p->ch == '@') {
+		       word_start_special(p->ch)) {
 		getword(p);
 		return;
 	    } else if (p->ch == '"') {
 		p->idstr = get_quoted_string(p);
-		p->sym = STR;
 		return;
 	    } else {
 		parser_print_input(p);
 		pprintf(p->prn, _("Invalid character '%c'\n"), p->ch);
-		p->err = 1;
+		p->err = E_PARSE;
 		return;
 	    }
 	} /* end ch switch */
@@ -1305,15 +1445,24 @@ const char *getsymb (int t, const parser *p)
 	return "LIST";
     } else if (t == OVAR) {
 	return "OVAR";
-    }
+    } else if (t == VSTR) {
+	return "VSTR";
+    } else if (t == EMPTY) {
+	return "EMPTY";
+    } else if (t == LISTVAR) {
+	return "LISTVAR";
+    } 
 
     if (p != NULL) {
 	if (t == NUM) {
 	    return fromdbl(p->xval); 
-	} else if (t == USCALAR || t == USERIES) {
+	} else if (t == USERIES) {
 	    return p->dinfo->varname[p->idnum];
-	} else if (t == UMAT || t == UOBJ ||
-		   t == LOOPIDX) {
+	} else if (t == USCALAR) {
+	    return p->idstr;
+	} else if (t == BUNDLE) {
+	    return p->idstr;
+	} else if (t == UMAT || t == UOBJ) {
 	    return p->idstr;
 	} else if (t == CON) {
 	    return constname(p->idnum);
@@ -1323,7 +1472,7 @@ const char *getsymb (int t, const parser *p)
 	    return dvarname(p->idnum);
 	} else if (t == MVAR) {
 	    return mvarname(p->idnum);
-	} else if (t == UFUN) {
+	} else if (t == UFUN || t == RFUN) {
 	    return p->idstr;
 	} else if (t == STR) {
 	    return p->idstr;
@@ -1401,11 +1550,12 @@ const char *getsymb (int t, const parser *p)
 	return ".<";
     case B_KRON: 
 	return "**";
-    case B_MCCAT: 
+    case B_HCAT: 
 	return "~";
-    case B_MRCAT: 
-    case B_LCAT:
+    case B_VCAT: 
 	return "|";
+    case B_LCAT:
+	return " ";
     case P_COM: 
 	return ",";
     case P_DOT: 
