@@ -29,6 +29,25 @@
 # define GDEBUG 0
 #endif
 
+# define settings_obs_value(p) (p->lh.obs >= 0)
+
+static void write_scalar_message (const parser *p, int oldv, PRN *prn)
+{
+    double x = gretl_scalar_get_value(p->lh.name);
+
+    if (p->flags & P_LHSCAL) {
+	pprintf(prn, _("Replaced scalar %s"), p->lh.name);
+    } else {
+	pprintf(prn, _("Generated scalar %s"), p->lh.name);
+    }
+
+    if (na(x)) {
+	pputs(prn, " = NA");
+    } else {
+	pprintf(prn, " = %g", x);
+    }
+}
+
 static void gen_write_message (const parser *p, int oldv, PRN *prn)
 {
     if (prn == NULL || !gretl_messages_on()) {
@@ -36,24 +55,12 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
     }
 
     if (p->targ == NUM) {
-	if (var_is_series(p->dinfo, p->lh.v)) {
+	if (settings_obs_value(p)) {
+	    /* set specific observation in series */
 	    pprintf(prn, _("Modified series %s (ID %d)"),
 		    p->lh.name, p->lh.v);
 	} else {
-	    double x = (*p->Z)[p->lh.v][p->lh.obs];
-
-	    if (p->lh.v < oldv) {
-		pprintf(prn, _("Replaced scalar %s (ID %d)"),
-			p->lh.name, p->lh.v);
-	    } else {
-		pprintf(prn, _("Generated scalar %s (ID %d)"),
-			p->lh.name, p->lh.v);
-	    }
-	    if (na(x)) {
-		pputs(prn, " = NA");
-	    } else {
-		pprintf(prn, " = %g", x);
-	    }
+	    write_scalar_message(p, oldv, prn);
 	}
     } else if (p->targ == VEC) {
 	if (p->lh.v < oldv) {
@@ -64,10 +71,10 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 		    p->lh.name, p->lh.v);
 	}
     } else if (p->targ == MAT) {
-	if (p->lh.m0 != NULL && p->lh.substr != NULL && 
+	if ((p->flags & P_LHMAT) && p->lh.substr != NULL && 
 	    *p->lh.substr != '\0') {
 	    pprintf(prn, _("Modified matrix %s"), p->lh.name);
-	} else if (p->lh.m0 != NULL) {
+	} else if (p->flags & P_LHMAT) {
 	    pprintf(prn, _("Replaced matrix %s"), p->lh.name);
 	} else {
 	    pprintf(prn, _("Generated matrix %s"), p->lh.name);
@@ -91,19 +98,6 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
     pputc(prn, '\n');
 }
 
-static void gen_write_warning (const parser *p, PRN *prn)
-{
-    if (prn != NULL && !repeating_function_exec()) {
-	if (*p->warning != '\0') {
-	    pprintf(prn, "%s: %s: %s\n", _("Warning"),
-		    p->warning, _("missing values were generated"));
-	} else {
-	    pprintf(prn, "%s: %s\n", _("Warning"),
-		    _("missing values were generated"));
-	}
-    }
-}
-
 static int maybe_record_lag_info (parser *p)
 {
     const char *s = p->input;
@@ -129,7 +123,7 @@ static int maybe_record_lag_info (parser *p)
     if (sscanf(s, "%15[^ ()](%d)", vname, &lag) == 2) {
 	s = strchr(s, ')');
 	if (s != NULL && string_is_blank(s + 1)) {
-	    int pv = varindex(p->dinfo, vname);
+	    int pv = series_index(p->dinfo, vname);
 
 	    if (pv < p->dinfo->v) {
 		strcpy(p->dinfo->varinfo[p->lh.v]->parent, 
@@ -149,8 +143,8 @@ static void gen_write_label (parser *p, int oldv)
     const char *src = "";
     size_t len = 0;
 
-    if (p->targ != NUM && p->targ != VEC) {
-	/* not relevant for matrices, lists */
+    if (p->targ != VEC) {
+	/* this is relevant only for series */
 	return;
     }
 
@@ -230,66 +224,133 @@ int function_from_string (const char *s)
     return 0;
 }
 
-/* "reserved word" data */
+/**
+ * is_gretl_function_call:
+ * @s: the string to check.
+ *
+ * Returns: 1 if @s starts with the name of a built-in 
+ * gretl function, otherwise 0.
+ */
 
-static const char *res1[] = {
+int is_gretl_function_call (const char *s)
+{
+    char c, word[12];
+
+    sscanf(s, "%10[^ (]", word);
+    c = s[strlen(word)];
+    if (c == ' ' || c == '(') {
+	return function_lookup(word);
+    } else {
+	return 0;
+    }
+}
+
+static const char *reswords[] = {
+    /* constants */
     "const",
     "CONST",
     "pi",
     "NA",
-    "null"
-};
-
-static const char *res2[] = {
-    "t",
-    "obs",
+    "null",
+    "obs", /* not exactly a constant, but hey */
+    /* types */
     "scalar",
     "series",
-    "matrix"
+    "matrix",
+    "string",
+    "list",
+    "bundle",
+    "kalman",
+    "void",
+    /* debugging instructions, etc. */
+    "continue",
+    "next",
+    "to"
 };
 
 /**
  * gretl_reserved_word:
  * @str: string to be tested.
  *
- * Returns non-zero if @str is a reserved word that cannot 
+ * Returns: non-zero if @str is a reserved word that cannot 
  * figure as the name of a user-defined variable, otherwise 0.
  */
 
 int gretl_reserved_word (const char *str)
 {
-    const char *ruses[] = {
-	N_("a constant"),
-	N_("an internal variable")
-    };
-    static int n1 = sizeof res1 / sizeof res1[0];
-    static int n2 = sizeof res2 / sizeof res2[0];
-    int i, ret = 0;
+    static int n = sizeof reswords / sizeof reswords[0];
+    int i, ret = gretl_command_number(str);
 
-    for (i=0; i<n1 && !ret; i++) {
-	if (!strcmp(str, res1[i])) {
+    for (i=0; i<n && !ret; i++) {
+	if (!strcmp(str, reswords[i])) {
 	    ret = 1;
 	}
     }
 
-    for (i=0; i<n2 && !ret; i++) {
-	if (!strcmp(str, res2[i])) {
-	    ret = 2;
+    if (ret) {
+	gretl_errmsg_sprintf(_("'%s' may not be used as a "
+			       "variable name"), str);
+    }	
+
+    return ret;
+}
+
+/**
+ * extract_varname:
+ * @targ: target string into which to write name.
+ * @src: source string.
+ * @len: location to receive the length of the extracted portion.
+ * 
+ * Writes up to #VNAMELEN - 1 characters from @s into @vname.
+ * 
+ * Returns: 0 on success, non-zero if the number of valid varname
+ * characters in @s is greater than #VNAMELEN - 1.
+ */
+
+int extract_varname (char *targ, const char *src, int *len)
+{
+    int err = 0;
+
+    *targ = '\0';
+    *len = gretl_namechar_spn(src);
+
+    if (*len >= VNAMELEN) {
+	/* too long to be a valid variable name */
+	err = E_UNKVAR;
+    } else {
+	strncat(targ, src, *len);
+    }
+
+    return err;
+}
+
+static int try_for_listvar (const DATAINFO *pdinfo, const char *s)
+{
+    char vname[VNAMELEN];
+    char lname[VNAMELEN];
+
+    if (sscanf(s, "%15[^.].%15s", lname, vname) == 2) {
+	int *list = get_list_by_name(lname);
+
+	if (list != NULL) {
+	    int i, vi;
+
+	    for (i=1; i<=list[0]; i++) {
+		vi = list[i];
+		if (!strcmp(vname, pdinfo->varname[vi])) {
+		    return vi;
+		}
+	    }
 	}
     }
 
-    if (ret > 0) {
-	sprintf(gretl_errmsg, _("'%s' refers to %s and may not be used as a "
-			    "variable name"), str, _(ruses[ret - 1])); 
-    }
- 
-    return ret;
+    return pdinfo->v;
 }
 
 #define GEN_LEVEL_DEBUG 0
 
 /**
- * varindex:
+ * series_index:
  * @pdinfo: data information struct.
  * @varname: name of variable to test.
  *
@@ -298,17 +359,18 @@ int gretl_reserved_word (const char *str)
  * that name.
  */
 
-int varindex (const DATAINFO *pdinfo, const char *varname)
+int series_index (const DATAINFO *pdinfo, const char *varname)
 {
     const char *s = varname;
-    int fsd = 0;
-    int i, ret = pdinfo->v;
+    int i, fd, ret;
 
-    if (s == NULL || *s == 0) {
-	return ret;
+    if (pdinfo == NULL) {
+	return -1;
     }
 
-    if (isdigit(*s)) {
+    ret = pdinfo->v;
+
+    if (s == NULL || *s == 0 || isdigit(*s)) {
 	return ret;
     }
 
@@ -316,21 +378,31 @@ int varindex (const DATAINFO *pdinfo, const char *varname)
 	return 0;
     }
 
-    fsd = gretl_function_depth();
+    if (strchr(s, '.')) {
+	return try_for_listvar(pdinfo, s);
+    }
 
-    if (fsd > 0) {
-	/* inside a function: see only vars at that level */
+    fd = gretl_function_depth();
+
+    if (fd == 0) {
+	/* not inside a user function: easy */
 	for (i=1; i<pdinfo->v; i++) { 
-	    if (STACK_LEVEL(pdinfo, i) == fsd && 
-		!strcmp(pdinfo->varname[i], s)) {
+	    if (strcmp(pdinfo->varname[i], s) == 0) {
 		ret = i;
 		break;
 	    }
 	}
     } else {
-	/* see all vars */
+	/* The condition for recognizing a series by name, if we're
+	   inside a user function: it must exist at the current level
+	   of function execution, and its tenure at that level must
+	   not just be the result of its being a member of a list
+	   that was passed as an argument.
+	*/
 	for (i=1; i<pdinfo->v; i++) { 
-	    if (!strcmp(pdinfo->varname[i], s)) { 
+	    if (fd == STACK_LEVEL(pdinfo, i) &&
+		!var_is_listarg(pdinfo, i) && 
+		strcmp(pdinfo->varname[i], s) == 0) {
 		ret = i;
 		break;
 	    }
@@ -338,13 +410,38 @@ int varindex (const DATAINFO *pdinfo, const char *varname)
     }
 
 #if GEN_LEVEL_DEBUG
-    fprintf(stderr, "varindex for '%s', fsd = %d: got %d (pdinfo->v = %d)\n", 
-	    s, fsd, ret, pdinfo->v);
+    fprintf(stderr, "series_index for '%s', fd = %d: got %d (pdinfo->v = %d)\n", 
+	    s, fd, ret, pdinfo->v);
 #endif 
 
     return ret;
 }
 
+int current_series_index (const DATAINFO *pdinfo, const char *vname)
+{
+    int v = -1;
+
+    if (pdinfo != NULL) {
+	v = series_index(pdinfo, vname);
+	if (v >= pdinfo->v) {
+	    v = -1;
+	}
+    }
+
+    return v;
+}
+
+int gretl_is_series (const char *name, const DATAINFO *pdinfo)
+{
+    if (pdinfo == NULL) {
+	return 0;
+    } else {
+	int v = series_index(pdinfo, name);
+
+	return (v >= 0 && v < pdinfo->v);
+    }
+}
+
 int genr_special_word (const char *s)
 {
     if (!strcmp(s, "dummy") ||
@@ -364,8 +461,9 @@ static int gen_special (const char *s, const char *line,
 			double ***pZ, DATAINFO *pdinfo, 
 			PRN *prn, parser *p)
 {
+    const char *msg = NULL;
     int orig_v = pdinfo->v;
-    int msg = 0;
+    int write_label = 0;
     int err = 0;
 
     if (!strcmp(s, "markers")) {
@@ -375,43 +473,49 @@ static int gen_special (const char *s, const char *line,
 
 	if (di0 == 0) {
 	    err = 1;
-	} else if (di0 == orig_v) {
-	    pputs(prn, _("Periodic dummy variables generated.\n"));
-	} else {
-	    pputs(prn, _("Periodic dummy variables already present.\n"));
+	} else { 
+	    if (di0 == orig_v) {
+		msg = N_("Periodic dummy variables generated.\n");
+	    } else {
+		msg = N_("Periodic dummy variables already present.\n");
+	    }
 	}
     } else if (!strcmp(s, "timedum")) {
 	err = panel_dummies(pZ, pdinfo, OPT_T);
 	if (!err) {
-	    pputs(prn, _("Panel dummy variables generated.\n"));
+	    msg = N_("Panel dummy variables generated.\n");
 	}
     } else if (!strcmp(s, "unitdum")) {
 	err = panel_dummies(pZ, pdinfo, OPT_NONE);
 	if (!err) {
-	    pputs(prn, _("Panel dummy variables generated.\n"));
+	    msg = N_("Panel dummy variables generated.\n");
 	}
     } else if (!strcmp(s, "time")) {
 	err = gen_time(pZ, pdinfo, 1);
-	msg = 1;
+	write_label = 1;
     } else if (!strcmp(s, "index")) {
 	err = gen_time(pZ, pdinfo, 0);
-	msg = 1;
+	write_label = 1;
     } else if (!strcmp(s, "unit")) {
 	err = gen_unit(pZ, pdinfo);
-	msg = 1;
+	write_label = 1;
     } else if (!strcmp(s, "weekday")) {
 	err = gen_wkday(pZ, pdinfo);
-	msg = 1;
+	write_label = 1;
     } 
 
-    if (!err && msg) {
+    if (msg != NULL && gretl_messages_on()) {
+	pputs(prn, _(msg));
+    }
+
+    if (!err && write_label) {
 	strcpy(p->lh.name, s);
-	p->lh.v = varindex(pdinfo, s);
+	p->lh.v = series_index(pdinfo, s);
 	p->Z = pZ;
 	p->dinfo = pdinfo;
 	p->targ = VEC;
 	p->flags = 0;
-	p->err = p->warn = 0;
+	p->err = 0;
 	p->prn = prn;
 	gen_write_message(p, orig_v, prn);
     }	    
@@ -482,10 +586,18 @@ static int is_gen_special (const char *s, char *spec, const char **rem)
     return 0;
 }
 
+static int genr_last_type;
+
+int genr_get_last_output_type (void)
+{
+    return genr_last_type;
+}
+
 #define gen_verbose(f) (!(f & P_PRINT) && \
                         !(f & P_DISCARD) && \
                         !(f & P_PRIVATE) && \
                         !(f & P_UFUN) && \
+                        !(f & P_QUIET) && \
                         !(f & P_DECL))
 
 int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
@@ -493,8 +605,7 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 {
     char vname[VNAMELEN];
     const char *subline = NULL;
-    int oldv = pdinfo->v;
-    int flags = 0;
+    int oldv, flags = 0;
     parser p;
 
     if (opt & OPT_P) {
@@ -505,6 +616,12 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 	flags |= P_UFUN;
     }
 
+    if (opt & OPT_Q) {
+	flags |= P_QUIET;
+    }
+
+    oldv = (pdinfo != NULL)? pdinfo->v : 0;
+
 #if GDEBUG
     fprintf(stderr, "\n*** generate: line = '%s'\n", line);
 #endif
@@ -521,10 +638,6 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 	gen_save_or_print(&p, prn);
     }
 
-    if (!p.err) {
-	maybe_pick_up_sorted_markers(&p);
-    }
-
     if (!p.err && gen_verbose(p.flags)) {
 	gen_write_label(&p, oldv);
 	if (!(opt & OPT_Q)) {
@@ -532,8 +645,11 @@ int generate (const char *line, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (!p.err && p.warn) {
-	gen_write_warning(&p, prn);
+    genr_last_type = genr_get_output_type(&p);
+
+    if (p.err == 1) {
+	/* a fairly good guess? */
+	p.err = E_PARSE;
     }
 
     gen_cleanup(&p);
@@ -563,6 +679,8 @@ double generate_scalar (const char *s, double ***pZ,
 	} else {
 	    *err = E_TYPES;
 	}
+    } else if (*err == 1) {
+	*err = E_PARSE;
     }
 
     gen_cleanup(&p);
@@ -573,12 +691,13 @@ double generate_scalar (const char *s, double ***pZ,
 /* retrieve a series result directly */
 
 double *generate_series (const char *s, double ***pZ, 
-			 DATAINFO *pdinfo, int *err)
+			 DATAINFO *pdinfo, PRN *prn,
+			 int *err)
 {
     parser p;
     double *x = NULL;
 
-    *err = realgen(s, &p, pZ, pdinfo, NULL, P_SERIES | P_PRIVATE);
+    *err = realgen(s, &p, pZ, pdinfo, prn, P_SERIES | P_PRIVATE);
 
     if (!*err) {
 	NODE *n = p.ret;
@@ -594,6 +713,8 @@ double *generate_series (const char *s, double ***pZ,
 	} else {
 	    *err = E_TYPES;
 	}
+    } else if (*err == 1) {
+	*err = E_PARSE;
     }
 
     gen_cleanup(&p);
@@ -601,6 +722,53 @@ double *generate_series (const char *s, double ***pZ,
     return x;
 }
 
+/* retrieve a matrix result directly */
+
+gretl_matrix *generate_matrix (const char *s, double ***pZ, 
+			       DATAINFO *pdinfo, int *err)
+{
+    gretl_matrix *m = NULL;
+    parser p;
+
+    *err = realgen(s, &p, pZ, pdinfo, NULL, P_MATRIX | P_PRIVATE);
+
+    if (!*err) {
+	NODE *n = p.ret;
+
+	if (n->t == MAT) {
+	    if (n->flags & TMP_NODE) {
+		/* steal the generated matrix */
+		m = n->v.m;
+		n->v.m = NULL;
+	    } else {
+		m = gretl_matrix_copy(n->v.m);
+		if (m == NULL) {
+		    *err = E_ALLOC;
+		}
+	    }
+	} else if (n->t == NUM) {
+	    if (xna(n->v.xval)) {
+		*err = E_NAN;
+	    } else {
+		m = gretl_matrix_alloc(1, 1);
+		if (m == NULL) {
+		    *err = E_ALLOC;
+		} else {
+		    m->val[0] = n->v.xval;
+		}
+	    }
+	} else {
+	    *err = E_TYPES;
+	}
+    } else if (*err == 1) {
+	*err = E_PARSE;
+    }
+
+    gen_cleanup(&p);
+
+    return m;
+}
+
 /* retrieve a string result directly */
 
 char *generate_string (const char *s, double ***pZ, 
@@ -617,6 +785,8 @@ char *generate_string (const char *s, double ***pZ,
 	} else {
 	    *err = E_TYPES;
 	}
+    } else if (*err == 1) {
+	*err = E_PARSE;
     }
 
     gen_cleanup(&p);
@@ -683,9 +853,10 @@ int print_object_var (const char *oname, const char *param,
    probably multiple times */
 
 parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo, 
-		      int *err)
+		      gretlopt opt, int *err)
 {
     parser *p = malloc(sizeof *p);
+    int flags = P_COMPILE;
 
 #if GDEBUG
     fprintf(stderr, "\n*** genr_compile: s = '%s'\n", s);
@@ -696,7 +867,25 @@ parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo,
 	return NULL;
     }
 
-    *err = realgen(s, p, pZ, pdinfo, NULL, P_COMPILE | P_PRIVATE);
+    if (opt & OPT_P) {
+	flags |= P_PRIVATE;
+    }
+
+    if (opt & OPT_U) {
+	flags |= P_UFUN;
+    }
+
+    *err = realgen(s, p, pZ, pdinfo, NULL, flags);
+
+    if (*err) {
+	gen_cleanup(p);
+	free(p);
+	p = NULL;
+    }
+
+#if GDEBUG
+    fprintf(stderr, "genr_compile: err = %d\n", *err);
+#endif
 
     return p;
 }
@@ -704,7 +893,7 @@ parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo,
 /* run a previously compiled generator */
 
 int execute_genr (parser *p, double ***pZ, DATAINFO *pdinfo,
-		  PRN *prn)
+		  gretlopt opt, PRN *prn)
 {
 #if GDEBUG
     fprintf(stderr, "\n*** execute_genr: p=%p, LHS='%s'\n", 
@@ -712,11 +901,12 @@ int execute_genr (parser *p, double ***pZ, DATAINFO *pdinfo,
 #endif
 
     realgen(NULL, p, pZ, pdinfo, prn, P_EXEC);
-    gen_save_or_print(p, prn);
+
+    if (!p->err && !(p->flags & P_UFUN)) {
+	gen_save_or_print(p, prn);
+    } 
+
     gen_cleanup(p);
-#if PRESERVE_AUX_NODES
-    p->ecount += 1;
-#endif
 
 #if GDEBUG
     fprintf(stderr, "execute_genr: returning %d\n", p->err);
@@ -744,6 +934,37 @@ void destroy_genr (parser *p)
     }
 }
 
+void genr_set_loopline (parser *p, int i)
+{
+    p->loopline = i;
+}
+
+int genr_get_loopline (parser *p)
+{
+    if (p == NULL) {
+	return -1;
+    } else {
+	return p->loopline;
+    }
+}
+
+int genr_get_output_type (const parser *p)
+{
+    int t = GRETL_TYPE_NONE;
+
+    if (!p->err) {
+	if (p->targ == NUM) {
+	    t = GRETL_TYPE_DOUBLE;
+	} else if (p->targ == VEC) {
+	    t = GRETL_TYPE_SERIES;
+	} else if (p->targ == MAT) {
+	    t = GRETL_TYPE_MATRIX;
+	} 
+    }
+
+    return t;
+}
+
 int genr_get_output_varnum (const parser *p)
 {
     return p->lh.v;
@@ -758,11 +979,25 @@ gretl_matrix *genr_get_output_matrix (const parser *p)
     }
 }
 
+double genr_get_output_scalar (const parser *p)
+{
+    if (p->targ == NUM) {
+	return gretl_scalar_get_value(p->lh.name);
+    } else {
+	return NADBL;
+    }
+}
+
 int genr_is_print (const parser *p)
 {
     return (p->flags & P_PRINT);
 }
 
+int genr_is_autoregressive (const parser *p)
+{
+    return (p->flags & P_AUTOREG);
+}
+
 void genr_set_na_check (parser *p)
 {
     p->flags |= P_NATEST;
@@ -773,5 +1008,11 @@ void genr_unset_na_check (parser *p)
     p->flags &= ~P_NATEST;
 }
 
+int genr_get_series_max (parser *p)
+{
+    /* dummy: to be added later */
+    return 0;
+}
+
 
 
