@@ -20,6 +20,7 @@
 /* parser module for 'genr' and related commands */
 
 #include "genparse.h"
+#include "gretl_string_table.h"
 
 #if GENDEBUG
 # define SDEBUG 1
@@ -36,8 +37,8 @@ static NODE *powterm (parser *p);
 #if SDEBUG
 static void notify (const char *s, NODE *t, parser *p)
 {
-    fprintf(stderr, "%-8s: returning node at %p, err = %d\n", 
-	    s, (void *) t, p->err);
+    fprintf(stderr, "%-8s: returning node at %p (type %d), err = %d\n", 
+	    s, (void *) t, (t != NULL)? t->t : 0, p->err);
 }
 #endif
 
@@ -53,7 +54,7 @@ static NODE *newempty (int t)
 	n->t = t;
 	n->v.idnum = 0;
 	n->flags = 0;
-	n->aux = -1;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -69,22 +70,19 @@ static NODE *newref (parser *p, int t)
 #endif
 
     if (n != NULL) {
-	if (t == USCALAR) {
-	    n->aux = p->idnum;
-	    n->t = NUM;
-	    n->v.xval = (*p->Z)[n->aux][0];
-	} else if (t == USERIES) {
-	    n->aux = p->idnum;
+	if (t == USERIES) {
+	    n->vnum = p->idnum;
 	    n->t = VEC;
-	    n->v.xvec = (*p->Z)[n->aux];
+	    n->v.xvec = (*p->Z)[n->vnum];
 	} else {
 	    n->t = t;
-	    if (t == UMAT || t == UOBJ || t == LOOPIDX || t == LIST) {
+	    if (t == USCALAR || t == UMAT || t == UOBJ || 
+		t == LIST || t == BUNDLE) {
 		n->v.str = p->idstr;
 	    } else {
 		n->v.idnum = p->idnum;
 	    }
-	    n->aux = -1;
+	    n->vnum = NO_VNUM;
 	}
 	n->flags = 0;
     }
@@ -92,11 +90,11 @@ static NODE *newref (parser *p, int t)
     return n;
 }
 
-NODE *newstr (parser *p)
+static NODE *newstr (parser *p, int t)
 {  
     NODE *n;
 
-    if (p != NULL && p->idstr == NULL) {
+    if (p->idstr == NULL) {
 	fprintf(stderr, "newstr: input is NULL\n");
 	return NULL;
     }
@@ -105,14 +103,14 @@ NODE *newstr (parser *p)
 
 #if MDEBUG
     fprintf(stderr, "newstr: allocated node at %p (s = '%s')\n", 
-	    (void *) n, (p != NULL)? p->idstr : "null");
+	    (void *) n, p->idstr);
 #endif
 
     if (n != NULL) {
-	n->t = STR;
-	n->v.str = (p != NULL)? p->idstr : NULL;
+	n->t = t;
+	n->v.str = p->idstr;
 	n->flags = 0;
-	n->aux = -1;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -133,7 +131,7 @@ NODE *newdbl (double x)
 	n->t = NUM;
 	n->v.xval = x;
 	n->flags = 0;
-	n->aux = -1;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -154,7 +152,7 @@ static NODE *newb1 (int t, NODE *b)
 	n->t = t;
 	n->v.b1.b = b;
 	n->flags = 0;
-	n->aux = -1;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -176,7 +174,7 @@ static NODE *newb2 (int t, NODE *l, NODE *r)
 	n->v.b2.l = l;
 	n->v.b2.r = r;
 	n->flags = 0;
-	n->aux = -1;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -184,7 +182,7 @@ static NODE *newb2 (int t, NODE *l, NODE *r)
 
 /* node for ternary operator */
 
-static NODE *newb3 (int t, NODE *l, NODE *m, NODE *r)
+static NODE *newb3 (int t, NODE *l)
 {  
     NODE *n = malloc(sizeof *n);
 
@@ -196,10 +194,10 @@ static NODE *newb3 (int t, NODE *l, NODE *m, NODE *r)
     if (n != NULL) {
 	n->t = t;
 	n->v.b3.l = l;
-	n->v.b3.m = m;
-	n->v.b3.r = r;
+	n->v.b3.m = NULL;
+	n->v.b3.r = NULL;
 	n->flags = 0;
-	n->aux = -1;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -221,7 +219,7 @@ static NODE *newbn (int t)
 	n->v.bn.n_nodes = 0;
 	n->v.bn.n = NULL;
 	n->flags = 0;
-	n->aux = -1;
+	n->vnum = NO_VNUM;
     }
 
     return n;
@@ -243,8 +241,8 @@ static int push_bn_node (NODE *t, NODE *n)
 
 #if SDEBUG
     fprintf(stderr, "push_bn_node: n_nodes now = %d, "
-	    "added node of type %d\n", t->v.bn.n_nodes,
-	    n->t);
+	    "added node of type %d (vnum = %d)\n", 
+	    t->v.bn.n_nodes, n->t, n->vnum);
 #endif
 
     return 0;
@@ -252,10 +250,24 @@ static int push_bn_node (NODE *t, NODE *n)
 
 static void expected_symbol_error (int c, parser *p)
 {
+    const char *found = getsymb(p->sym, p);
+
     parser_print_input(p);
-    pprintf(p->prn, _("Expected '%c' but found '%s'\n"), c, 
-	    getsymb(p->sym, p));
-    p->err = 1;
+
+    if (*found == '\0') {
+	pprintf(p->prn, _("Expected '%c' but formula ended\n"), c);
+    } else {
+	pprintf(p->prn, _("Expected '%c' but found '%s'\n"), c, 
+		found);
+    }
+
+    if (!strcmp(found, "&")) {
+	pputs(p->prn, "(for logical AND, please use \"&&\")\n");
+    } else if (!strcmp(found, "|")) {
+	pputs(p->prn, "(for logical OR, please use \"||\")\n");
+    }
+
+    p->err = E_PARSE;
 }
 
 static void unmatched_symbol_error (int c, parser *p)
@@ -274,8 +286,8 @@ static void unmatched_symbol_error (int c, parser *p)
 static int unary_apost (parser *p) 
 {
     if (p->ch == '\'') {
-	/* peek at the next non-space character */
-	int c = parser_next_char(p);
+	/* peek ahead */
+	int c = parser_next_nonspace_char(p);
 
 	if (isalpha(c) || c == '$' || c == '(') {
 	    /* next symbol is variable or expression: the
@@ -318,7 +330,7 @@ static NODE *base (parser *p, NODE *up)
     case MVAR:
     case OBS:
     case LIST:
-    case LOOPIDX:
+    case BUNDLE:
 	t = newref(p, p->sym);
 	if (matrix_ref_node(p) && unary_apost(p)) {
 	    set_transpose(t);
@@ -386,6 +398,153 @@ static NODE *base (parser *p, NODE *up)
     return t;
 }
 
+/* construct a node that contains a reference to a data series, based
+   on the name of a list (already captured in p->idstr) and the name
+   of a variable that is supposed to be in the list, which is separated
+   from the listname by '.'.
+*/
+
+static NODE *listvar_node (parser *p)
+{
+    int *list = get_list_by_name(p->idstr);
+    NODE *ret = NULL;
+    char vname[VNAMELEN] = {0};
+    int i, n, v, ok = 0;
+
+    if (list == NULL) {
+	p->err = E_UNKVAR;
+	return NULL;
+    }
+
+    n = gretl_namechar_spn(p->point);
+    if (n >= VNAMELEN) {
+	/* too long -- can't be a valid varname */
+	p->err = E_UNKVAR;
+	return NULL;
+    }
+
+    for (i=0; i<n; i++) {
+	parser_getc(p);
+	vname[i] = p->ch;
+    }
+
+    for (i=1; i<=list[0]; i++) {
+	v = list[i];
+	if (!strcmp(vname, p->dinfo->varname[v])) {
+	    /* found the variable */
+	    free(p->idstr);
+	    p->idstr = NULL;
+	    p->idnum = v;
+	    ret = newref(p, USERIES);
+	    if (ret == NULL) {
+		p->err = E_ALLOC;
+	    } else {
+		ok = 1;
+	    }
+	    break;
+	}
+    }
+
+    if (!ok && !p->err) {
+	p->err = E_UNKVAR;
+    }
+
+    if (!p->err) {
+	parser_getc(p);
+	lex(p);
+    }
+
+    return ret;    
+}
+
+/* here we grab all the arguments to sscanf() as one big string,
+   which will be parsed later */
+
+static NODE *get_sscanf_args (parser *p)
+{
+    const char *src = NULL;
+    int wrapped = 0;
+
+    while (p->ch == ' ') {
+	parser_getc(p);
+    }
+
+    if (p->ch == '"') {
+	wrapped = 1;
+	/* don't lose the leading quote */
+	src = p->point - 1;
+	parser_getc(p);
+    }
+
+    if (p->ch == ')') {
+	p->err = (wrapped)? E_PARSE : E_ARGS;
+	return NULL;
+    } else {
+	const char *s = p->point;
+	int i, paren = 1, quoted = wrapped;
+	int len, close = -1;
+
+	/* find length of string to closing paren */
+	i = 0;
+	while (*s) {
+	    if (!quoted && *s == ')') paren--;
+	    if (paren == 0) {
+		close = i;
+		break;
+	    }
+	    if (*s == '"') {
+		quoted = !quoted;
+	    } else if (!quoted && *s == '(') {
+		paren++;
+	    } 
+	    s++;
+	    i++;
+	}
+
+	if (close < 0) {
+	    unmatched_symbol_error('(', p);
+	    return NULL;
+	}
+
+	if (src == NULL) {
+	    src = p->point - 1;
+	    len = close + 1;
+	} else {
+	    len = close + 2;
+	}
+
+	p->idstr = gretl_strndup(src, len);
+
+	if (p->idstr == NULL) {
+	    p->err = E_ALLOC;
+	    return NULL;
+	}
+
+	tailstrip(p->idstr);
+	parser_advance(p, close);
+    }
+
+    parser_getc(p);
+    lex(p);
+
+#if SDEBUG
+    fprintf(stderr, "get_sscanf_args: '%s'\n", p->idstr);
+#endif
+
+    return newstr(p, STR);
+}
+
+static void unwrap_string_arg (parser *p)
+{
+    int n = strlen(p->idstr);
+
+    if (p->idstr[n-1] == '"') {
+	p->idstr[n-1] = '\0';
+    } else {
+	unmatched_symbol_error('"', p);
+    }
+}
+
 /* Grab a string argument.  Note: we have a mechanism in genlex.c for
    retrieving arguments that take the form of quoted string literals
    or names of string variables.  The special use of this function is
@@ -394,10 +553,15 @@ static NODE *base (parser *p, NODE *up)
    is a string.  This function assumes that the argument in question
    is the last (or only) argument to a function: we look for a closing
    parenthesis and flag an error if we don't find one.
+
+   Depending on the context we may or may not want to "consume" the
+   trailing right paren that follows the string of interest; that's
+   controlled by the second argument.
 */
 
-static NODE *get_string_arg (parser *p)
+static NODE *get_final_string_arg (parser *p, int sym, int eat_last)
 {
+    const char *src = NULL;
     int wrapped = 0;
 
     while (p->ch == ' ') {
@@ -410,7 +574,11 @@ static NODE *get_string_arg (parser *p)
     }
 
     if (p->ch == ')') {
-	/* allow empty arg string "()" */
+	if (wrapped) {
+	    p->err = E_PARSE;
+	    return NULL;
+	} 
+	/* handle empty arg */
 	p->idstr = gretl_strdup("");
     } else {
 	int i, paren = 1, quoted = wrapped;
@@ -429,7 +597,7 @@ static NODE *get_string_arg (parser *p)
 		quoted = !quoted;
 	    } else if (!quoted && *s == '(') {
 		paren++;
-	    }
+	    } 
 	    s++;
 	    i++;
 	}
@@ -441,7 +609,8 @@ static NODE *get_string_arg (parser *p)
 
 	for (i=0; i<=close; i++) {
 	    if (!started && !isspace(p->ch)) {
-		p->idstr = gretl_strndup(p->point - 1, close - i + 1);
+		src = p->point - 1;
+		p->idstr = gretl_strndup(src, close - i + 1);
 		started = 1;
 	    }
 	    parser_getc(p);
@@ -453,81 +622,130 @@ static NODE *get_string_arg (parser *p)
 	return NULL;
     }
 
-    parser_getc(p);
+    tailstrip(p->idstr);
+    if (eat_last) {
+	/* consume trailing right paren */
+	parser_getc(p);
+    }
     lex(p);
 
-    tailstrip(p->idstr);
-
     if (wrapped) {
-	int n = strlen(p->idstr);
+	unwrap_string_arg(p);
+    } else if (sym != F_ISSTRING) {
+	/* not quoted: give priority to string variables */
+	const char *s = get_string_by_name(p->idstr);
 
-	if (p->idstr[n-1] == '"') {
-	    p->idstr[n-1] = '\0';
-	} else {
-	    unmatched_symbol_error('"', p);
+	if (s != NULL) {
+	    free(p->idstr);
+	    p->idstr = gretl_strdup(s);
+	    if (p->idstr == NULL) {
+		p->err = E_ALLOC;
+	    }
 	}
     }
 
 #if SDEBUG
-    fprintf(stderr, "get_string_arg: '%s'\n", p->idstr);
+    fprintf(stderr, "get_final_string_arg: '%s'\n", p->idstr);
 #endif
 
-    return newstr(p);
+    return (p->err)? NULL : newstr(p, STR);
+}
+
+static NODE *get_middle_string_arg (parser *p)
+{
+    const char *src = NULL;
+    const char *s;
+    int gotparen = 0, close = 0;
+    int i, quoted, paren = 0;
+
+    while (p->ch == ' ') {
+	parser_getc(p);
+    }
+
+    if (p->ch == '"') {
+	/* special: arg is wrapped in quotes */
+	close = parser_charpos(p, '"');
+	if (close < 0) {
+	    unmatched_symbol_error('"', p);
+	    return NULL;
+	}
+	p->idstr = gretl_strndup(p->point, close);
+	close++;
+	goto post_process;
+    }
+
+    /* find length of string to bare comma (unquoted, not in parens) or
+       matched right paren */
+
+    quoted = 0;
+    s = p->point;
+    i = 0;
+
+    while (*s) {
+	if (*s == '"') {
+	    quoted = !quoted;
+	} 
+	if (!quoted) {
+	    if (*s == '(') {
+		gotparen++;
+		paren++;
+	    } else if (*s == ')') {
+		paren--;
+	    }
+	    if (paren == 0) {
+		if (gotparen) {
+		    /* include right paren */
+		    close = i + 1;
+		    break;
+		} else if (*s == ',') {
+		    /* leave comma */
+		    close = i;
+		    break;
+		} 
+	    } 
+	}
+	s++;
+	i++;
+    }
+
+    if (paren > 0) {
+	unmatched_symbol_error('(', p);
+    } else if (paren < 0) {
+	unmatched_symbol_error(')', p);
+    } else if (quoted) {
+	unmatched_symbol_error('"', p);
+    }
+
+    if (p->err) {
+	return NULL;
+    }
+
+    src = p->point - 1;
+    p->idstr = gretl_strndup(src, close + 1);
+
+ post_process:
+
+    if (p->idstr == NULL) {
+	p->err = E_ALLOC;
+	return NULL;
+    }  
+
+    tailstrip(p->idstr);
+    parser_advance(p, close);
+    lex(p);
+
+#if SDEBUG
+    fprintf(stderr, "get_middle_string_arg: '%s'\n", p->idstr);
+#endif
+
+    return (p->err)? NULL : newstr(p, STR);
 }
 
 enum {
-    BOTH_OPT = 1,
-    RIGHT_OPT,
-    RIGHT_STR
+    RIGHT_STR = 1 << 0,
+    MID_STR   = 1 << 1
 };
 
-/* gather an unknown number of comma-separated arguments
-   for a (possibly user-defined) function */
-
-static void get_multi_args (NODE *t, parser *p)
-{
-    NODE *n;
-    char cexp = 0;
-
-#if SDEBUG
-    fprintf(stderr, "get_multi_args, p->sym = %d\n", p->sym);
-#endif    
-
-    if (p->sym == G_LPR) {
-	lex(p);
-	while (p->ch != 0 && !p->err) {
-	    n = expr(p);
-	    if (p->err) {
-		break;
-	    } else {
-		p->err = push_bn_node(t, n);
-	    }
-	    if (p->sym == P_COM) {
-		/* in case acceptance of plain strings was set,
-		   turn it off after the first arg */
-		p->getstr = 0;
-		lex(p);
-	    } else if (p->sym == G_RPR) {
-		break;
-	    }
-	}
-    } else {
-	cexp = '(';
-    }
-
-    if (cexp == 0 && !p->err) {
-	if (p->sym == G_RPR) {
-	    lex(p);
-	} else {
-	    unmatched_symbol_error('(', p);
-	}
-    }
-	    
-    if (cexp && p->err == 0) {
-	expected_symbol_error(cexp, p);
-    }
-}
-
 static void get_matrix_def (NODE *t, parser *p, int *sub)
 {
     NODE *n;
@@ -575,7 +793,7 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
 	}
     }
 	    
-    if (cexp && p->err == 0) {
+    if (cexp && !p->err) {
 	expected_symbol_error(cexp, p);
     }
 }	
@@ -599,10 +817,15 @@ static void get_slice_parts (NODE *t, parser *p)
 	    t->v.b2.l = expr(p);
 	}
 	if (p->sym == P_COL) {
-	    /* need second part of colon-separated range */
+	    /* second part of colon-separated range */
 	    t->v.b2.l = newb2(SUBSL, t->v.b2.l, NULL);
 	    lex(p);
-	    t->v.b2.l->v.b2.r = expr(p);
+	    if (p->sym == P_COM || p->sym == G_RBR) {
+		/* reached end: second part implicitly empty */
+		t->v.b2.l->v.b2.r = newempty(EMPTY);
+	    } else {
+		t->v.b2.l->v.b2.r = expr(p);
+	    }
 	}
 	if (p->sym == G_RBR) {
 	    /* no comma, no second arg string: may be OK */
@@ -620,10 +843,15 @@ static void get_slice_parts (NODE *t, parser *p)
 		t->v.b2.r = expr(p);
 	    }
 	    if (p->sym == P_COL) {
-		/* need second part of colon-separated range */
+		/* second part of colon-separated range */
 		t->v.b2.r = newb2(SUBSL, t->v.b2.r, NULL);
 		lex(p);
-		t->v.b2.r->v.b2.r = expr(p);
+		if (p->sym == G_RBR) {
+		    /* reached end: second part implicitly empty */
+		    t->v.b2.r->v.b2.r = newempty(EMPTY);
+		} else {
+		    t->v.b2.r->v.b2.r = expr(p);
+		}
 	    }
 	    if (p->sym == G_RBR) {
 		if (p->ch == '\'') {
@@ -641,69 +869,145 @@ static void get_slice_parts (NODE *t, parser *p)
 	cexp = '[';
     }
 	    
-    if (cexp && p->err == 0) {
+    if (cexp && !p->err) {
 	expected_symbol_error(cexp, p);
     }
 
     set_matrix_slice_off();
 }
 
-/* get up to two comma-separated arguments 
-   (possibly optional) */
-
-static void get_args (NODE *t, parser *p, int opt)
+static void attach_child (NODE *parent, NODE *child, int k, int i,
+			  parser *p)
 {
-    char cexp = 0;
+    if (p->err) {
+	return;
+    }
 
 #if SDEBUG
-    fprintf(stderr, "get_args...\n");
-#endif    
+    fprintf(stderr, "attach_child: i=%d, type=%d\n", i, child->t);
+#endif
 
-    if (p->sym == G_LPR) {
-	lex(p);
-	if (p->sym == G_RPR && opt == BOTH_OPT) {
-	    /* no args, but it's OK */
-	    t->v.b2.l = newempty(EMPTY);
-	    t->v.b2.r = newempty(EMPTY);
-	    lex(p);
-	    return;
-	}
-	t->v.b2.l = expr(p);
-	if (p->sym == G_RPR && opt == RIGHT_OPT) {
-	    /* no second arg, but it's OK */
-	    t->v.b2.r = newempty(EMPTY);
-	    lex(p);
-	    return;
-	}	    
-	if (p->sym == P_COM) {
-	    if (opt == RIGHT_STR) {
-		/* the second arg must be a literal string */
-		t->v.b2.r = get_string_arg(p);
-	    } else {
-		lex(p);
-		t->v.b2.r = expr(p);
-		if (p->sym == G_RPR) {
-		    if (p->ch == '\'') {
-			set_transpose(t);
-			parser_getc(p);
-		    }
-		    lex(p);
-		} else {
-		    cexp = ')';
-		}
-	    }
+    if (k == 2) {
+	/* 2-place node */
+	if (i == 0) {
+	    parent->v.b2.l = child;
 	} else {
-	    cexp = ',';
+	    parent->v.b2.r = child;
+	}
+    } else if (k == 3) {
+	/* 3-place node */
+	if (i == 0) {
+	    parent->v.b3.l = child;
+	} else if (i == 1) {
+	    parent->v.b3.m = child;
+	} else {
+	    parent->v.b3.r = child;
 	}
     } else {
-	cexp = '(';
+	/* n-place node */
+	p->err = push_bn_node(parent, child);
     }
-	    
-    if (cexp && p->err == 0) {
-	expected_symbol_error(cexp, p);
+}
+
+static void pad_parent (NODE *parent, int k, int i, parser *p)
+{
+    NODE *n;
+    int j;
+
+    for (j=i; j<k; j++) {
+	n = newempty(EMPTY);
+	attach_child(parent, n, k, j, p);
     }
 }
 
+#define next_arg_is_string(i,k,o) ((i>0 && i<k-1 && (o & MID_STR)) || \
+                                   (i==k-1 && (o & RIGHT_STR)))
+
+/* Get up to k comma-separated arguments (possibly optional).
+   However, if k < 0 this is a signal for get as many arguments as we
+   can find (the number is unknown in advance).
+*/
+
+static void get_args (NODE *t, parser *p, int f, int k, int opt, int *next)
+{
+    NODE *child;
+    char cexp = 0;
+    int i = 0;
+
+#if SDEBUG
+    fprintf(stderr, "get_args: k = %d...\n", k);
+#endif    
+
+    if (p->sym != G_LPR) {
+	expected_symbol_error('(', p);
+	return;
+    }	
+
+    lex(p);
+
+    while (((k > 0 && i < k) || p->ch) && !p->err) {
+	if (p->sym == G_RPR) {
+	    /* right paren: got to the end */
+	    break;
+	}
+
+	if (k > 0 && i == k) {
+	    gretl_errmsg_sprintf("%s: %s", getsymb(f, p), _("too many arguments"));
+	    p->err = E_ARGS;
+	    break;
+	}
+
+	/* get the next argument */
+	if (i > 0 && i < k - 1 && (opt & MID_STR)) {
+	    child = get_middle_string_arg(p);
+	} else if (i == k - 1 && (opt & RIGHT_STR)) {
+	    child = get_final_string_arg(p, 0, 0);
+	} else {
+	    child = expr(p);
+	}
+
+	attach_child(t, child, k, i++, p);
+
+	if (p->err || p->sym == G_RPR) {
+	    break;
+	} else if (p->sym == P_COM) {
+	    /* turn off flag for accepting string as first arg */
+	    p->flags &= ~P_GETSTR;
+	    /* lex unless next arg needs special handling */
+	    if (!next_arg_is_string(i, k, opt)) {
+		lex(p);
+	    }
+	} else {
+	    /* arg-separating comma was expected */
+	    cexp = ',';
+	    break;
+	}
+    }
+
+    if (!p->err) {
+	if (cexp) {
+	    expected_symbol_error(cexp, p);
+	} else if (p->sym == G_RPR) {
+	    if (i < k) {
+		pad_parent(t, k, i, p);
+	    } 
+	    /* matrix functions: handle trailing transpose */
+	    if (p->ch == '\'') {
+		set_transpose(t);
+		parser_getc(p);
+	    }
+	    lex(p);
+	    if (p->sym == G_LBR) {
+		*next = G_LBR;
+	    }
+	}
+    }
+
+#if SDEBUG
+    fprintf(stderr, "get_args: returning with p->err=%d\n", p->err);
+#endif    
+}
+
 static void get_ovar_ref (NODE *t, parser *p)
 {
     if (p->ch != '.' || parser_charpos(p, '$') != 0) {
@@ -721,8 +1025,8 @@ static void get_ovar_ref (NODE *t, parser *p)
 
     if (p->idnum == 0) {
 	p->err = E_PARSE;
-    } else if (p->sym == DMSL) {
-	/* followed by '[' slice mechanism? */
+    } else if (p->sym == DMSL || p->sym == DMSTR) {
+	/* followed by '[' or '(' matrix subspec? */
 	t->v.b2.r = powterm(p);
     } else {
 	t->v.b2.r = newref(p, p->sym);
@@ -731,8 +1035,12 @@ static void get_ovar_ref (NODE *t, parser *p)
 }
 
 static NODE *powterm (parser *p)
-{  
-    int opt = 0;
+{ 
+    /* watch out for unary operators */
+    int sym = p->sym == B_SUB ? U_NEG : 
+	p->sym == B_ADD ? U_POS : p->sym;
+    int opt = OPT_NONE;
+    int next = 0;
     NODE *t;
 
     if (p->err) {
@@ -744,58 +1052,98 @@ static NODE *powterm (parser *p)
 	    p->sym, p->ch? p->ch : '0', p->ch);
 #endif
 
-    if (p->sym == F_RUNIFORM || p->sym == F_RNORMAL) {
-	opt = BOTH_OPT;
-    } else if (p->sym == F_FDJAC || p->sym == F_BFGSMAX) {
-	opt = RIGHT_STR;
-    }
+    if (string_last_func(sym)) {
+	opt |= RIGHT_STR;
+    } 
 
-    if (func2_symb(p->sym)) {
-	t = newb2(p->sym, NULL, NULL);
+    if (string_mid_func(sym)) {
+	opt |= MID_STR;
+    }    
+
+    if (unary_op(sym)) {
+	if (p->ch == 0) {
+	    context_error(0, p);
+	    return NULL;
+	}
+        t = newb1(sym, NULL);
+        if (t != NULL) {
+            lex(p);
+            t->v.b1.b = powterm(p);
+        }
+    } else if (func2_symb(sym)) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    lex(p);
-	    get_args(t, p, opt);
+	    get_args(t, p, sym, 2, opt, &next);
 	}
-    } else if (string0_func(p->sym)) {
-	t = newb1(p->sym, NULL);
+    } else if (func3_symb(sym)) {
+	t = newb3(sym, NULL);
+	if (t != NULL) {
+	    lex(p);
+	    if (char0_func(sym)) {
+		p->flags |= P_GETSTR;
+	    }
+	    get_args(t, p, sym, 3, opt, &next);
+	}
+    } else if (char0_func(sym)) {
+	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
-		p->getstr = 1;
-		get_multi_args(t->v.b1.b, p);
+		p->flags |= P_GETSTR;
+		get_args(t->v.b1.b, p, sym, -1, opt, &next);
 	    }
-	}	
-    } else if (string_arg_func(p->sym)) {
-	t = newb1(p->sym, NULL);
+	}
+    } else if (sym == F_URCPVAL) {
+	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
-	    t->v.b1.b = get_string_arg(p);
+	    t->v.b1.b = newbn(FARGS);
+	    if (t != NULL) {
+		get_args(t->v.b1.b, p, sym, -1, opt, &next);
+	    }
+	}
+    } else if (sym == F_SSCANF) {
+	/* string arg is handled specially */
+	t = newb1(sym, NULL);
+	if (t != NULL) {
+	    lex(p);
+	    t->v.b1.b = get_sscanf_args(p);
 	}	
-    } else if (func1_symb(p->sym)) {
-	t = newb1(p->sym, NULL);
+    } else if (string_arg_func(sym)) {
+	t = newb1(sym, NULL);
+	if (t != NULL) {
+	    lex(p);
+	    t->v.b1.b = get_final_string_arg(p, sym, 1);
+	}	
+    } else if (func1_symb(sym)) {
+	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = base(p, t);
+	    if (p->sym == G_LBR) {
+		next = G_LBR;
+	    }
 	}
-    } else if (p->sym == LAG || p->sym == OBS) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == LAG || sym == OBS) {
+	if (sym == LAG) {
+	    set_lag_parse_on();
+	}
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    t->v.b2.l = newref(p, USERIES); 
 	    lex(p);
 	    t->v.b2.r = base(p, t);
 	}
-    } else if (func2_symb(p->sym)) {
-	t = newb2(p->sym, NULL, NULL);
-	if (t != NULL) {
-	    lex(p);
-	    get_args(t, p, opt);
-	}
-    } else if (p->sym == MSL || p->sym == DMSL) {
-	t = newb2(p->sym, NULL, NULL);
+	if (sym == LAG) {
+	    set_lag_parse_off();
+	}	
+    } else if (sym == MSL || sym == DMSL) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    if (p->sym == MSL) {
-		t->v.b2.l = newstr(p);
+		t->v.b2.l = newstr(p, STR);
 	    } else {
 		t->v.b2.l = newref(p, MVAR);
 	    }
@@ -805,27 +1153,41 @@ static NODE *powterm (parser *p)
 		get_slice_parts(t->v.b2.r, p);
 	    }
 	}
-    } else if (p->sym == DMSTR) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == DMSTR) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
 	    t->v.b2.l = newref(p, MVAR);
 	    lex(p);
-	    t->v.b2.r = get_string_arg(p);
+	    t->v.b2.r = get_final_string_arg(p, sym, 1);
 	}
-    } else if (p->sym == OVAR) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == OVAR) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newstr(p);
+	    t->v.b2.l = newstr(p, STR);
 	    get_ovar_ref(t, p);
 	}
-    } else if (p->sym == G_LPR) {
+    } else if (sym == LISTVAR) {
+	t = listvar_node(p);
+	if (t != NULL && (p->sym == G_LPR || p->sym == G_LBR)) {
+	    /* list.series node may be "inflected" as lag or obs */
+	    if (p->sym == G_LPR) 
+		fprintf(stderr, "*** listvar: setting LAG\n");
+	    p->sym = (p->sym == G_LPR)? LAG : OBS;
+	    t = newb2(p->sym, t, NULL);
+	    if (t != NULL) {
+		parser_ungetc(p);
+		lex(p);
+		t->v.b2.r = base(p, t);
+	    }
+	}
+    } else if (sym == G_LPR) {
 	/* dummy root for parenthesized expressions, to facilitate
 	   taking the transpose of matrix stuff, e.g. (A*B)' */
 	t = newb1(EROOT, NULL);
 	if (t != NULL) {
 	    t->v.b1.b = base(p, t);
 	}
-    } else if (p->sym == G_LCB) {
+    } else if (sym == G_LCB) {
 	/* explicit matrix definition, possibly followed by
 	   a "subslice" specification */
 	int sub = 0;
@@ -844,32 +1206,43 @@ static NODE *powterm (parser *p)
 		}		
 	    }
 	}
-    } else if (p->sym == UFUN) {
-	t = newb2(p->sym, NULL, NULL);
+    } else if (sym == UFUN || sym == RFUN) {
+	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newstr(p);
+	    t->v.b2.l = newstr(p, STR);
 	    lex(p);
 	    t->v.b2.r = newbn(FARGS);
 	    if (t != NULL) {
-		get_multi_args(t->v.b2.r, p);
+		get_args(t->v.b2.r, p, sym, -1, opt, &next);
 	    }
 	}
-    } else if (funcn_symb(p->sym)) {
-	t = newb1(p->sym, NULL);
+    } else if (funcn_symb(sym)) {
+	t = newb1(sym, NULL);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
-		get_multi_args(t->v.b1.b, p);
+		get_args(t->v.b1.b, p, sym, -1, opt, &next);
 	    }
 	}
-    } else if (p->sym == STR) {
-	t = newstr(p);
+    } else if (sym == STR || sym == VSTR) {
+	t = newstr(p, sym);
 	lex(p);
     } else {
 	t = base(p, NULL);
     }
 
+    if (next == G_LBR) {
+	/* support func(args)[slice] */
+	t = newb2(MSL, t, NULL);
+	if (t != NULL) {
+	    t->v.b2.r = newb2(MSL2, NULL, NULL);
+	    if (t->v.b2.r != NULL) {
+		get_slice_parts(t->v.b2.r, p);
+	    }
+	}	
+    }
+
 #if SDEBUG
     notify("powterm", t, p);
 #endif
@@ -877,12 +1250,37 @@ static NODE *powterm (parser *p)
     return t;
 }
 
+/* convert B_POW to right associativity: that is,
+   
+       pow           pow
+      L   R         L   R
+      |   |   ->    |   |
+     pow  c         a  pow
+     | |               | |
+     a b               b c
+
+*/
+
+static void convert_pow_term (NODE *n)
+{
+    NODE *L = n->v.b2.l;
+    NODE *a = L->v.b2.l;
+    NODE *b = L->v.b2.r;
+    NODE *c = n->v.b2.r;
+
+    n->v.b2.l = a;
+    n->v.b2.r = L;
+    L->v.b2.l = b;
+    L->v.b2.r = c;
+}
+
+#define pow_sym(t)     (t == B_POW || t == B_DOTPOW)
+#define pow_pow_node(n) (pow_sym(n->t) && pow_sym(n->v.b2.l->t))
+
 static NODE *factor (parser *p)
 {  
     int sym = p->sym == B_SUB ? U_NEG : 
-	p->sym == B_ADD ? U_POS : 
-	p->sym == B_AND ? U_ADDR :
-	p->sym;
+	p->sym == B_ADD ? U_POS : p->sym;
     NODE *t;
 
     if (p->err) {
@@ -914,6 +1312,10 @@ static NODE *factor (parser *p)
 			lex(p);
 			t->v.b2.r = powterm(p);
 		    }
+		    if (!p->err && pow_pow_node(t)) {
+			/* make exponentiation associate rightward */
+			convert_pow_term(t);
+		    }
 		}
 	    }
 	}
@@ -961,7 +1363,7 @@ static NODE *expr4 (parser *p)
 
     while (!p->err && (p->sym == B_ADD || p->sym == B_SUB || 
 		       p->sym == B_DOTADD || p->sym == B_DOTSUB ||
-		       p->sym == B_MCCAT || p->sym == B_MRCAT ||
+		       p->sym == B_HCAT || p->sym == B_VCAT ||
 		       p->sym == B_LCAT || p->sym == B_RANGE)) {
 	t = newb2(p->sym, t, NULL);
 	if (t != NULL) {
@@ -1081,7 +1483,7 @@ NODE *expr (parser *p)
     }
 
     while (!p->err && p->sym == QUERY) {
-	t = newb3(p->sym, t, NULL, NULL);
+	t = newb3(p->sym, t);
 	if (t != NULL) {
 	    lex(p);
 	    t->v.b3.m = expr(p);
