@@ -22,10 +22,13 @@
 #include "nlspec.h"
 #include "libset.h"
 #include "qr_estimate.h"
+#include "gretl_bfgs.h"
 
 #include "gretl_f2c.h"
 #include "../../minpack/minpack.h"
 
+#include <errno.h>
+
 #define GMM_DEBUG 0
 
 typedef struct colsrc_ colsrc;
@@ -39,22 +42,27 @@ struct colsrc_ {
 
 struct hac_info_ {
     int kern;              /* kernel type */
-    int h;                 /* integer bandwidth (Bartlett, Parzen) */                
+    int h;                 /* integer bandwidth (Bartlett, Parzen) */
     double bt;             /* QS bandwidth */
     int whiten;            /* pre-whiten (1) or not (0) */
 };
 
 struct ocset_ {
-    gretl_matrix *e;     /* GMM residual, or LHS term in O.C. */
-    gretl_matrix *Z;     /* instruments, or RHS terms in O.C. */
-    gretl_matrix *W;     /* matrix of weights */
-    gretl_matrix *tmp;   /* holds columnwise product of e and Z */
-    gretl_matrix *sum;   /* holds column sums of tmp */
-    gretl_matrix *S;     /* selector matrix for computing tmp */
-    colsrc *ecols;       /* info on provenance of columns of 'e' */
-    int noc;             /* total number of orthogonality conds. */
-    int step;            /* number of estimation steps */
-    hac_info hinfo;      /* HAC characteristics */
+    gretl_matrix *e;      /* GMM residual, or LHS term in O.C. */
+    gretl_matrix *Z;      /* instruments, or RHS terms in O.C. */
+    gretl_matrix *W;      /* matrix of weights */
+    gretl_matrix *tmp;    /* holds columnwise product of e and Z */
+    gretl_matrix *sum;    /* holds column sums of tmp */
+    gretl_matrix *S;      /* selector matrix for computing tmp */
+    colsrc *ecols;        /* info on provenance of columns of 'e' */
+    int noc;              /* total number of orthogonality conds. */
+    int step;             /* number of estimation steps */
+    int userwts;          /* got weights matrix from user (1/0) */
+    hac_info hinfo;       /* HAC characteristics */
+    char Wname[VNAMELEN]; /* name of weighting matrix */
+    char **lnames;        /* names of LHS terms in O.C.s */
+    char **rnames;        /* names of RHS terms in O.C.s */
+    int n_names;          /* number of right/left name pairs */
 };
 
 #define using_HAC(s) (s->oc->hinfo.kern >= KERNEL_BARTLETT)
@@ -74,6 +82,19 @@ void oc_set_destroy (ocset *oc)
     gretl_matrix_free(oc->S);
 
     free(oc->ecols);
+
+    if (oc->lnames != NULL) {
+	free_strings_array(oc->lnames, oc->n_names);
+    }
+
+    if (oc->rnames != NULL) {
+	free_strings_array(oc->rnames, oc->n_names);
+    }
+
+    if (!oc->userwts) {
+	/* we used auto-generated weights */
+	gretl_matrix_free(oc->W);
+    }
 	
     free(oc);
 }
@@ -94,6 +115,11 @@ static ocset *oc_set_new (void)
 	oc->ecols = NULL;
 	oc->noc = 0;
 	oc->step = 0;
+	oc->userwts = 0;
+	*oc->Wname = '\0';
+	oc->lnames = NULL;
+	oc->rnames = NULL;
+	oc->n_names = 0;
     }
 
     return oc;
@@ -101,63 +127,48 @@ static ocset *oc_set_new (void)
 
 static int gmm_unkvar (const char *s)
 {
-    sprintf(gretl_errmsg, _("Unknown variable '%s'"), s);
+    gretl_errmsg_sprintf(_("Unknown variable '%s'"), s);
     return E_UNKVAR;
 }
 
-static int matrix_is_dated (const gretl_matrix *m)
-{
-    return !(m->t1 == 0 && m->t2 == 0);
-}
+#define matrix_is_dated(m) (!(m->t1 == 0 && m->t2 == 0))
 
-/* Determine the type of an element in an "orthog" specification */
+/* Determine the type of an element in an "orthog" specification:
+   series, matrix or list of series
+*/
 
-static int oc_get_type (const char *name, const DATAINFO *pdinfo,
-			int rhs, int *err)
+static int 
+oc_get_type (const char *name, const DATAINFO *pdinfo, int *err)
 {
     int *list = NULL;
-    int v = varindex(pdinfo, name);
+    int j, v = current_series_index(pdinfo, name);
+    int ret = GRETL_TYPE_NONE;
 
 #if GMM_DEBUG
     fprintf(stderr, "oc_get_type: looking at '%s' (v = %d, pdinfo->v = %d)\n", 
 	    name, v, pdinfo->v);
 #endif
 
-    /* try for a series first */
-    if (v >= 0 && v < pdinfo->v) {
-	if (var_is_scalar(pdinfo, v)) {
-	    *err = E_TYPES;
-	    return GRETL_TYPE_NONE;
-	} else {
-	    return GRETL_TYPE_SERIES;
-	}
-    }
-
-    /* then a matrix */
-    if (get_matrix_by_name(name) != NULL) {
-	return GRETL_TYPE_MATRIX;
-    }
-
-    /* failing that, a list */
-    list = get_list_by_name(name);
-    if (list != NULL) {
-	int j;
-
+    if (v >= 0) {
+	/* try for a series first */
+	ret = GRETL_TYPE_SERIES;
+    } else if (get_matrix_by_name(name) != NULL) {
+	/* then a matrix */
+	ret = GRETL_TYPE_MATRIX;
+    } else if ((list = get_list_by_name(name)) != NULL) {
+	/* failing that, a list */
 	for (j=1; j<=list[0]; j++) {
 	    if (list[j] < 0 || list[j] >= pdinfo->v) {
 		*err = E_DATA;
 		return GRETL_TYPE_NONE;
-	    } else if (var_is_scalar(pdinfo, list[j])) {
-		*err = E_TYPES;
-		return GRETL_TYPE_NONE;
-	    }
+	    } 
 	}
-	return GRETL_TYPE_LIST;
+	ret = GRETL_TYPE_LIST;
+    } else {
+	*err = gmm_unkvar(name);
     }
 
-    *err = gmm_unkvar(name);
-
-    return GRETL_TYPE_NONE;
+    return ret;
 }
 
 /* See if column j in matrix b is present anywhere in matrix a.  If
@@ -384,6 +395,69 @@ push_column_source (nlspec *s, int v, const char *mname)
     return 0;
 }
 
+int nlspec_add_ivreg_oc (nlspec *s, int lhv, const int *rlist,
+			 const double **Z)
+{
+    gretl_matrix *e = NULL;
+    gretl_matrix *M = NULL;
+    int i, k, t, v;
+    double x;
+    int err = 0;
+
+    s->oc = oc_set_new();
+    if (s->oc == NULL) {
+	return E_ALLOC;
+    }
+
+    /* the left-hand side: series -> vector */
+
+    e = gretl_column_vector_alloc(s->nobs);
+    if (e == NULL) {
+	err = E_ALLOC;
+    } else {
+	e->t1 = s->t1;
+	e->t2 = s->t2;
+	for (t=0; t<s->nobs; t++) {
+	    x = Z[lhv][t + s->t1];
+	    gretl_vector_set(e, t, x);
+	}
+	err = push_column_source(s, lhv, NULL);
+    }
+
+    if (err) {
+	return err;
+    }
+
+    /* the right-hand side: list -> matrix */
+
+    k = rlist[0];
+    M = gretl_matrix_alloc(s->nobs, k);
+    if (M == NULL) {
+	err = E_ALLOC;
+    } else {
+	M->t1 = s->t1;
+	M->t2 = s->t2;
+	for (i=0; i<k && !err; i++) {
+	    v = rlist[i + 1];
+	    for (t=0; t<s->nobs; t++) {
+		x = Z[v][t + s->t1];
+		gretl_matrix_set(M, t, i, x);
+	    }
+	}
+    }
+
+    if (!err) {
+	s->oc->e = e;
+	s->oc->Z = M;
+	s->oc->noc = k;
+    } else {
+	oc_set_destroy(s->oc);
+	s->oc = NULL;
+    } 
+
+    return err;
+}
+
 static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 			    int rtype, const char *rname,
 			    const double **Z, const DATAINFO *pdinfo)
@@ -432,7 +506,7 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 	    }
 	}	
     } else if (ltype == GRETL_TYPE_SERIES) {
-	v = varindex(pdinfo, lname);
+	v = series_index(pdinfo, lname);
 	e = gretl_column_vector_alloc(s->nobs);
 	if (e == NULL) {
 	    err = E_ALLOC;
@@ -488,7 +562,7 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 	}
     } else {
 	/* name of single series */
-	v = varindex(pdinfo, rname);
+	v = series_index(pdinfo, rname);
 	k = 1;
 	M = gretl_column_vector_alloc(s->nobs);
 	if (M == NULL) {
@@ -551,18 +625,17 @@ int
 nlspec_add_orthcond (nlspec *s, const char *str,
 		     const double **Z, const DATAINFO *pdinfo)
 {
-    char lname[VNAMELEN];
-    char rname[VNAMELEN];
+    char lname[VNAMELEN], rname[VNAMELEN];
     int ltype = GRETL_TYPE_NONE, rtype = GRETL_TYPE_NONE;
-    int err = 0;
+    int n, err = 0;
 
     if (s->ci != GMM) {
 	return E_TYPES;
     }
 
     if (s->oc != NULL && s->oc->W != NULL) {
-	strcpy(gretl_errmsg, _("Orthogonality conditions must come before the "
-	       "weights matrix"));
+	gretl_errmsg_set(_("Orthogonality conditions must come before the "
+			   "weights matrix"));
 	return E_DATA;
     }
 
@@ -583,10 +656,10 @@ nlspec_add_orthcond (nlspec *s, const char *str,
 	}
     }
 
-    ltype = oc_get_type(lname, pdinfo, 0, &err);
+    ltype = oc_get_type(lname, pdinfo, &err);
 
     if (!err) {
-	rtype = oc_get_type(rname, pdinfo, 1, &err);
+	rtype = oc_get_type(rname, pdinfo, &err);
     }
 
     if (!err) {
@@ -594,6 +667,12 @@ nlspec_add_orthcond (nlspec *s, const char *str,
 			      Z, pdinfo);
     }
 
+    if (!err && gretl_in_gui_mode()) {
+	n = s->oc->n_names;
+	strings_array_add(&s->oc->lnames, &s->oc->n_names, lname);
+	strings_array_add(&s->oc->rnames, &n, rname);
+    } 
+
     if (err) {
 	nlspec_destroy_arrays(s);
 	oc_set_destroy(s->oc);
@@ -603,14 +682,7 @@ nlspec_add_orthcond (nlspec *s, const char *str,
     return err;
 }
 
-static int matrix_t1 (const gretl_matrix *m, int t1)
-{
-    if (m->t1 == 0 && m->t2 == 0) {
-	return t1;
-    } else {
-	return m->t1;
-    }
-}
+#define matrix_t1(m,t) ((m->t1 == 0 && m->t2 == 0)? t : m->t1)
 
 static int gmm_matrix_resize (gretl_matrix **pA, nlspec *s, int oldt1)
 {
@@ -638,13 +710,11 @@ static int gmm_matrix_resize (gretl_matrix **pA, nlspec *s, int oldt1)
     B->t1 = s->t1;
     B->t2 = s->t2;
 
-    gretl_matrix_free(A);
-    *pA = B;
+    gretl_matrix_replace(pA, B);
 
     return 0;
 }
 
-
 #define intmax3(a,b,c) (((a)>(b))? (((c)>(a))? (c) : (a)) : (((c)>(b))? (c) : (b)))
 #define intmin3(a,b,c) (((a)<(b))? (((c)<(a))? (c) : (a)) : (((c)<(b))? (c) : (b)))
 #define intdiff3(a,b,c) (a != b || a != c)
@@ -684,14 +754,28 @@ static int gmm_fix_datarows (nlspec *s)
     return err;
 }
 
+static int gmm_add_workspace (nlspec *s)
+{
+    int k = s->oc->noc;
+    int err = 0;
+
+    s->oc->tmp = gretl_matrix_alloc(s->nobs, k);
+    s->oc->sum = gretl_column_vector_alloc(k);
+
+    if (s->oc->tmp == NULL || s->oc->sum == NULL) {
+	err = E_ALLOC;
+    }
+
+    return err;
+}
+
 /* Add the matrix of weights to the GMM specification: this must
    come after all the O.C.s are given, so that we can check the
-   dimensions of the given matrix.
+   dimensions of the supplied matrix.
 */
 
 int nlspec_add_weights (nlspec *s, const char *str)
 {
-    char name[VNAMELEN];
     int k, err = 0;
 
     if (s->ci != GMM) {
@@ -699,12 +783,12 @@ int nlspec_add_weights (nlspec *s, const char *str)
     }
 
     if (s->oc == NULL) {
-	strcpy(gretl_errmsg, _("Weights must come after orthogonality conditions"));
+	gretl_errmsg_set(_("Weights must come after orthogonality conditions"));
 	return E_DATA;
     }
 
     if (s->oc->W != NULL) {
-	strcpy(gretl_errmsg, _("Weights are already defined"));
+	gretl_errmsg_set(_("Weights are already defined"));
 	return E_DATA;
     }
 
@@ -712,21 +796,21 @@ int nlspec_add_weights (nlspec *s, const char *str)
     fprintf(stderr, "nlspec_add_weights: line = '%s'\n", str);
 #endif
 
-    if (sscanf(str, "%15s", name) != 1) {
+    if (sscanf(str, "%15s", s->oc->Wname) != 1) {
 	return E_PARSE;
     } 
 
-    s->oc->W = get_matrix_by_name(name);
+    s->oc->W = get_matrix_by_name(s->oc->Wname);
     if (s->oc->W == NULL) {
-	return gmm_unkvar(name);
+	return gmm_unkvar(s->oc->Wname);
     }
 
     k = s->oc->noc;
 
     /* is the weight matrix of the correct dimensions? */
     if (s->oc->W->rows != k || s->oc->W->cols != k) {
-	sprintf(gretl_errmsg, _("Weight matrix is of wrong size: should be "
-				"%d x %d"), k, k);
+	gretl_errmsg_sprintf(_("Weight matrix is of wrong size: should be "
+			       "%d x %d"), k, k);
 	err = E_DATA;
     }
 
@@ -747,31 +831,77 @@ int nlspec_add_weights (nlspec *s, const char *str)
     /* now we're ready to add the workspace matrices */
 
     if (!err) {
-	s->oc->tmp = gretl_matrix_alloc(s->nobs, k);
-	s->oc->sum = gretl_column_vector_alloc(k);
-	if (s->oc->tmp == NULL || s->oc->sum == NULL) {
-	    err = E_ALLOC;
-	}
+	err = gmm_add_workspace(s);
     }
 
     return err;
 }
 
+void maybe_add_gmm_residual (MODEL *pmod, const nlspec *spec, 
+			     const DATAINFO *pdinfo)
+{
+    if (spec->oc != NULL && spec->oc->e != NULL && spec->oc->e->cols == 1) {
+	int t, s = 0;
+
+	if (pmod->uhat != NULL) {
+	    free(pmod->uhat);
+	}
+
+	pmod->uhat = malloc(pdinfo->n * sizeof *pmod->uhat);
+
+	if (pmod->uhat != NULL) {
+	    for (t=0; t<pdinfo->n; t++) {
+		if (t >= spec->t1 && t <= spec->t2) {
+		    pmod->uhat[t] = spec->oc->e->val[s++];
+		} else {
+		    pmod->uhat[t] = NADBL;
+		}
+	    }
+	    pmod->full_n = pdinfo->n;
+	}
+    }
+}
+
+void nlspec_print_gmm_info (const nlspec *spec, PRN *prn)
+{
+    int i;
+
+    if (spec->oc == NULL || spec->oc->lnames == NULL ||
+	spec->oc->rnames == NULL) {
+	return;
+    }
+
+    for (i=0; i<spec->oc->n_names; i++) {
+	pprintf(prn, "orthog %s ; %s\n", spec->oc->lnames[i], 
+		spec->oc->rnames[i]);
+    }
+
+    pprintf(prn, "weights %s\n", spec->oc->Wname);
+}
+
 /* sanity check before proceeding with GMM estimation */
 
 int check_gmm_requirements (nlspec *spec)
 {
+    int err = 0;
+
     if (spec->oc == NULL) {
-	strcpy(gretl_errmsg, _("No orthogonality conditions have been specified"));
-	return 1;
+	gretl_errmsg_set(_("No orthogonality conditions have been specified"));
+	err = E_DATA;
+    } else if (spec->oc->W == NULL) {
+	/* add automatic weights matrix */
+	spec->oc->W = gretl_identity_matrix_new(spec->oc->noc);
+	if (spec->oc->W == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    strcpy(spec->oc->Wname, "auto");
+	    err = gmm_add_workspace(spec);
+	}
+    } else {
+	spec->oc->userwts = 1;
     }
 
-    if (spec->oc->W == NULL) {
-	strcpy(gretl_errmsg, _("No weights have been specified"));
-	return 1;
-    }
-
-    return 0;
+    return err;
 }
 
 /* Update the column(s) of the LHS matrix in the O.C. set,
@@ -829,6 +959,10 @@ static int gmm_multiply_ocs (nlspec *s)
 	err = gretl_matrix_columnwise_product(s->oc->e,
 					      s->oc->Z,
 					      s->oc->tmp);
+	if (err) {
+	    fprintf(stderr, "gmm_multiply_ocs: err = %d from "
+		    "gretl_matrix_columnwise_product\n", err);
+	}
     } else {
 	/* select the wanted i, j pairs */
 	int i, j, k, m, t, p = 0;
@@ -865,6 +999,8 @@ static int gmm_multiply_ocs (nlspec *s)
     return err;
 }
 
+#define CRIT_DEBUG 0
+
 /* calculate the value of the GMM criterion given the current
    parameter values */
 
@@ -876,15 +1012,24 @@ static double get_gmm_crit (const double *b, void *p)
 
     update_coeff_values(b, s);
 
-    if (nl_calculate_fvec(s)) {
+    if ((err = nl_calculate_fvec(s))) {
+#if CRIT_DEBUG
+	fprintf(stderr, "get_gmm_crit: err on nl_calculate_fvec = %d\n", err);
+#endif
 	return NADBL;
     }
 
-    if (gmm_update_e(s)) {
+    if ((err = gmm_update_e(s))) {
+#if CRIT_DEBUG
+	fprintf(stderr, "get_gmm_crit: err on gmm_update_e = %d\n", err);
+#endif
 	return NADBL;
     }
 
-    if (gmm_multiply_ocs(s)) {
+    if ((err = gmm_multiply_ocs(s))) {
+#if CRIT_DEBUG
+	fprintf(stderr, "get_gmm_crit: err on gmm_multiply_ocs = %d\n", err);
+#endif
 	return NADBL;
     }
     
@@ -903,7 +1048,7 @@ static double get_gmm_crit (const double *b, void *p)
     }
 
 #if GMM_DEBUG > 2
-    gretl_matrix_print(s->oc->sum, "s->oc->sum");
+    gretl_matrix_print(s->oc->sum, "GMM: s->oc->sum");
     fprintf(stderr, "GMM: s->crit = %g\n", s->crit);
 #endif
 
@@ -957,27 +1102,23 @@ gmm_jacobian_calc (integer *m, integer *n, double *x, double *f,
 
 static int HAC_prewhiten (gretl_matrix *E, gretl_matrix *A)
 {
-    gretl_matrix *y = NULL;
-    gretl_matrix *X = NULL;
-    gretl_matrix *XTX = NULL;
-    gretl_matrix *b = NULL;
-    gretl_matrix *e = NULL;
+    gretl_matrix_block *B;
+    gretl_matrix *y, *X, *XTX;
+    gretl_matrix *b, *e;
     int T = E->rows;
     int k = E->cols;
     double xtj, eti;
     int i, j, t;
     int err = 0;
 
-    y = gretl_column_vector_alloc(T - 1);
-    X = gretl_matrix_alloc(T - 1, k);
-    XTX = gretl_matrix_alloc(k, k);
-    b = gretl_column_vector_alloc(k);
-    e = gretl_column_vector_alloc(k);
-
-    if (y == NULL || X == NULL || XTX == NULL ||
-	b == NULL || e == NULL) {
-	err = E_ALLOC;
-	goto bailout;
+    B = gretl_matrix_block_new(&y, T-1, 1,
+			       &X, T-1, k,
+			       &XTX, k, k,
+			       &b, k, 1,
+			       &e, k, 1,
+			       NULL);
+    if (B == NULL) {
+	return E_ALLOC;
     }
 
     /* make matrix of RHS vars */
@@ -1071,11 +1212,7 @@ static int HAC_prewhiten (gretl_matrix *E, gretl_matrix *A)
 
  bailout:
 
-    gretl_matrix_free(y);
-    gretl_matrix_free(X);
-    gretl_matrix_free(XTX);
-    gretl_matrix_free(b);
-    gretl_matrix_free(e);
+    gretl_matrix_block_destroy(B);
 
     return err;
 }
@@ -1086,7 +1223,6 @@ static int gmm_HAC (gretl_matrix *E, gretl_matrix *V, hac_info *hinfo)
     static gretl_matrix *Tmp;
     static gretl_matrix *A;
     static gretl_matrix *E2;
-
     int T, k;
     double w;
     int i, err = 0;
@@ -1202,21 +1338,16 @@ static void gmm_HAC_cleanup (void)
 
 int gmm_add_vcv (MODEL *pmod, nlspec *s)
 {
-    gretl_matrix *V = NULL;
-    gretl_matrix *J = NULL;
-    gretl_matrix *S = NULL;
-    gretl_matrix *m1 = NULL;
-    gretl_matrix *m2 = NULL;
-    gretl_matrix *m3 = NULL;
+    gretl_matrix_block *B;
+    gretl_matrix *V, *J, *S;
+    gretl_matrix *m1, *m2, *m3;
     int i, j, k = s->ncoeff;
     int T = s->nobs;
-
-    doublereal *wa4 = NULL;
+    doublereal *wa4;
     doublereal epsfcn = 0.0; /* could be > 0.0 */
     integer m, n, ldjac;
     integer iflag = 0;
-
-    double x, *f;
+    double *f;
     int err = 0;
 
     m = gretl_matrix_cols(s->oc->tmp);
@@ -1224,18 +1355,19 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
     ldjac = m; /* leading dimension of jac array */
 
     wa4 = malloc(m * sizeof *wa4);
-    V = gretl_matrix_alloc(k, k);
-    J = gretl_matrix_alloc(m, k);
-    S = gretl_matrix_alloc(m, m);
-    m1 = gretl_matrix_alloc(k, m);
-    m2 = gretl_matrix_alloc(k, k);
-    m3 = gretl_matrix_alloc(k, k);
+    if (wa4 == NULL) {
+	return E_ALLOC;
+    }
 
-    if (V == NULL || J == NULL || S == NULL ||
-	m1 == NULL || m2 == NULL || m3 == NULL ||
-	wa4 == NULL) {
-	err = E_ALLOC;
-	goto bailout;
+    B = gretl_matrix_block_new(&V, k, k,
+			       &J, m, k,
+			       &S, m, m,
+			       &m1, k, m,
+			       &m2, k, k,
+			       &m3, k, k,
+			       NULL);
+    if (B == NULL) {
+	return E_ALLOC;
     }
 
     if (using_HAC(s)) {
@@ -1296,26 +1428,18 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
     }
 
     if (!err) {
-	for (i=0; i<k; i++) {
-	    for (j=0; j<=i; j++) {
-		x = gretl_matrix_get(V, i, j);
-		pmod->vcv[ijton(i, j, k)] = x;
-		if (j == i) {
-		   pmod->sderr[i] = sqrt(x);
-		} 
-	    }
-	}
-	if (using_HAC(s)) {
-	    gretl_model_set_int(pmod, "using_hac", 1);
-	    gretl_model_set_int(pmod, "hac_kernel", s->oc->hinfo.kern);
-	    if (s->oc->hinfo.kern == KERNEL_QS) {
-		gretl_model_set_double(pmod, "qs_bandwidth", s->oc->hinfo.bt);
-	    } else {
-		gretl_model_set_int(pmod, "hac_lag", s->oc->hinfo.h);
-	    }
-	    if (s->oc->hinfo.whiten) {
-		gretl_model_set_int(pmod, "hac_prewhiten", 1);
-	    }
+	err = gretl_model_write_vcv(pmod, V);
+    }
+
+    if (!err && using_HAC(s)) {
+	if (s->oc->hinfo.kern == KERNEL_QS) {
+	    gretl_model_set_full_vcv_info(pmod, VCV_HAC, s->oc->hinfo.kern,
+					  0, s->oc->hinfo.whiten, 
+					  s->oc->hinfo.bt);
+	} else {
+	    gretl_model_set_full_vcv_info(pmod, VCV_HAC, s->oc->hinfo.kern,
+					  s->oc->hinfo.h, s->oc->hinfo.whiten, 
+					  NADBL);
 	}
     }
 
@@ -1336,20 +1460,13 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
 	}
 
 	if (s->opt & OPT_T) {
-	    gretl_model_set_int(pmod, "two-step", 1);
+	    pmod->opt |= OPT_T;
 	} else if (s->opt & OPT_I) {
-	    gretl_model_set_int(pmod, "iterated", 1);
+	    pmod->opt |= OPT_I;
 	}
     }
 
- bailout:
-
-    gretl_matrix_free(V);
-    gretl_matrix_free(J);
-    gretl_matrix_free(S);
-    gretl_matrix_free(m1);
-    gretl_matrix_free(m2);
-    gretl_matrix_free(m3);
+    gretl_matrix_block_destroy(B);
     free(wa4);
 
     return err;
@@ -1437,19 +1554,81 @@ static void gmm_print_oc (nlspec *s, PRN *prn)
     gretl_matrix_free(V);
 }
 
-/* Driver for BFGS, case of GMM estimation.  We may have to handle
+static double gmm_log_10 (double x)
+{
+    double y;
+
+    errno = 0;
+
+    y = log(fabs(x));
+
+    if (errno) {
+	y = NADBL;
+    } else {
+	y /= log(10.0);
+    }
+
+    return y;
+}
+
+/* The user didn't specify a matrix of initial weights in
+   GMM, and gretl assigned an identity matrix; here we try
+   to see if we should scale the weights for the sake of
+   keeping computer arithmetic in bounds.
+*/
+
+static int maybe_prescale_weights (nlspec *s)
+{
+    double *coeff;
+    double crit;
+ 
+    coeff = copyvec(s->coeff, s->ncoeff);
+    if (coeff == NULL) {
+	return E_ALLOC;
+    }
+
+    crit = -1 * get_gmm_crit(coeff, s);
+
+    if (crit > 0 && !na(crit)) {
+	double m, lc = gmm_log_10(crit);
+
+#if 0
+	fprintf(stderr, "maybe_preadjust_weights: crit=%g, lc=%g\n", 
+		crit, lc);
+	gretl_matrix_print(s->oc->sum, "s->oc->sum");
+#endif
+
+	if (!na(lc) && (lc > 5 || lc < -5)) {
+	    /* FIXME ? */
+	    if (lc > 0) {
+		m = floor(lc/2);
+	    } else {
+		m = ceil(lc/3);
+	    }
+	    m = pow(10, -m);
+	    fprintf(stderr, "GMM weights matrix: scaling by %g\n", m);
+	    gretl_matrix_multiply_by_scalar(s->oc->W, m);
+	}
+    }
+
+    free(coeff);
+
+    return 0;
+}
+
+/* BFGS driver, for the case of GMM estimation.  We may have to handle
    both "inner" (BFGS) and "outer" iterations here: the "outer"
    iterations (if applicable) are re-runs of BFGS using an updated
    weights matrix.
 */
 
-int gmm_calculate (nlspec *s, double *fvec, double *jac, PRN *prn)
+int gmm_calculate (nlspec *s, PRN *prn)
 {
     int full_fncount = 0;
     int full_grcount = 0;
     double itol = 1.0e-12, icrit = 1;
     double *oldcoeff = NULL;
-    int maxit = libset_get_int(BFGS_MAXITER);
+    int maxit = 500;
     gretlopt iopt = s->opt;
     int outer_iters = 0;
     int outer_max = 1;
@@ -1469,6 +1648,11 @@ int gmm_calculate (nlspec *s, double *fvec, double *jac, PRN *prn)
 	outer_max = 2;
     }
 
+    /* experimental, 2010-04-22 */
+    if (!s->oc->userwts) {
+	maybe_prescale_weights(s);
+    }
+
     while (!err && outer_iters < outer_max && !converged) {
 
 #if GMM_DEBUG
@@ -1481,7 +1665,7 @@ int gmm_calculate (nlspec *s, double *fvec, double *jac, PRN *prn)
 	err = BFGS_max(s->coeff, s->ncoeff, maxit, s->tol, 
 		       &s->fncount, &s->grcount, 
 		       get_gmm_crit, C_GMM, NULL, s,
-		       iopt, s->prn);
+		       NULL, iopt, s->prn);
 
 	/* don't keep displaying certain things */
 	iopt |= OPT_Q;
@@ -1631,6 +1815,20 @@ int gmm_missval_check_etc (nlspec *s)
 	    s->t1, s->t2);
 #endif
 
+#if 0
+    /* thought: allow matrix version of GMM for vectors longer
+       then the dataset -- but not ready yet */
+    if (!matrix_is_dated(s->oc->e) && !matrix_is_dated(s->oc->Z)) {
+	int nr1 = s->oc->e->rows;
+	int nr2 = s->oc->Z->rows;
+	
+	if (nr1 == nr2) {
+	    s->t1 = 0;
+	    s->t2 = nr1 - 1;
+	}
+    }
+#endif
+
     if (matrix_is_dated(s->oc->e)) {
 	if (s->oc->e->t1 > s->t1) {
 	    s->t1 = s->oc->e->t1;
