@@ -36,6 +36,14 @@ typedef enum {
 
 #define FN_NAMELEN 32
 
+#define ok_function_return_type(r) (r == GRETL_TYPE_DOUBLE || \
+				    r == GRETL_TYPE_SERIES || \
+				    r == GRETL_TYPE_MATRIX || \
+				    r == GRETL_TYPE_LIST ||   \
+				    r == GRETL_TYPE_STRING || \
+				    r == GRETL_TYPE_BUNDLE || \
+				    r == GRETL_TYPE_VOID)
+
 typedef struct ufunc_ ufunc;
 typedef struct fnpkg_ fnpkg;
 typedef struct fnargs_ fnargs;
@@ -52,6 +60,8 @@ int fn_param_type (const ufunc *fun, int i);
 
 const char *fn_param_name (const ufunc *fun, int i);
 
+const char *fn_param_descrip (const ufunc *fun, int i);
+
 double fn_param_default (const ufunc *fun, int i);
 
 double fn_param_minval (const ufunc *fun, int i);
@@ -64,18 +74,19 @@ int user_func_get_return_type (const ufunc *fun);
 
 const char *user_function_name_by_index (int i);
 
-int user_function_index_by_name (const char *name);
+int user_function_index_by_name (const char *name, 
+				 fnpkg *pkg);
+
+int user_function_set_debug (const char *name, int debug);
 
 void function_names_init (void);
 
-const char *next_free_function_name (void);
+const char *next_available_function_name (int *idxp);
 
 int gretl_compiling_function (void);
 
 int gretl_function_depth (void);
 
-int repeating_function_exec (void);
-
 int gretl_start_compiling_function (const char *line, PRN *prn);
 
 int gretl_function_append_line (const char *line);
@@ -89,61 +100,63 @@ int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
 
 char *gretl_func_get_arg_name (const char *argvar, int *err);
 
-void set_drop_function_vars (int s);
+int object_is_const (const char *name);
+
+void sample_range_get_extrema (const DATAINFO *pdinfo, int *t1, int *t2);
 
 const char *get_funcerr_message (void);
 
-int gretl_function_set_info (int i, const char *help);
+int function_return_type_from_string (const char *s);
 
-int gretl_function_get_info (int i, const char *key, char const **value);
+int gretl_function_print_code (ufunc *u, PRN *prn);
 
-int gretl_function_print_code (int i, PRN *prn);
+fnpkg *function_package_new (const char *fname, 
+			     char **pubnames, int n_pub,
+			     char **privnames, int n_priv, 
+			     int *err);
 
-void gretl_function_set_private (int i, int priv);
+int function_package_connect_funcs (fnpkg *pkg, 
+				    char **pubnames, int n_pub,
+				    char **privnames, int n_priv);
 
-int write_function_package (fnpkg *pkg,
-			    const char *fname,
-			    int pub, 
-			    const int *privlist, 
-			    const char *author,
-			    const char *version,
-			    const char *date,
-			    const char *descrip,
-			    FuncDataReq dreq,
-			    float minver);
+int function_package_set_properties (fnpkg *pkg, ...);
 
-int function_package_get_info (const char *fname,
-			       fnpkg **ppkg,
-			       int *pub, 
-			       int **privlist,
-			       char **author,
-			       char **version,
-			       char **date,
-			       char **descrip,
-			       FuncDataReq *dreq,
-			       float *minver);
+int function_package_get_properties (fnpkg *pkg, ...);
+
+const char *function_package_get_name (fnpkg *pkg);
+
+int function_package_write_file (fnpkg *pkg);
+
+int create_and_write_function_package (const char *fname, PRN *prn);
 
 int check_function_needs (const DATAINFO *pdinfo, FuncDataReq dreq,
-			  float minver);
+			  int minver);
 
-int write_user_function_file (const char *fname);
-
-int function_package_is_loaded (const char *fname);
-
-const char *function_package_description (const char *fname);
+int write_session_functions_file (const char *fname);
 
 int read_session_functions_file (const char *fname);
 
-int load_user_function_file (const char *fname);
+fnpkg *get_function_package_by_filename (const char *fname);
 
-int get_function_file_info (const char *fname, PRN *prn, char **pname);
+int load_function_package_from_file (const char *fname);
 
-int get_function_file_code (const char *fname, PRN *prn, char **pname);
+void function_package_unload_by_filename (const char *fname);
 
-char *get_function_file_header (const char *fname, char **pver,
-				int *err);
+void function_package_unload_full_by_filename (const char *fname);
 
-int update_function_from_script (const char *fname, int idx);
+int print_function_package_info (const char *fname, PRN *prn);
+
+int print_function_package_code (const char *fname, PRN *prn);
+
+ufunc *get_packaged_function_by_name (const char *name,
+				      fnpkg *pkg);
+
+int get_function_file_header (const char *fname, char **pdesc, char **pver);
+
+char *get_function_package_name (const char *path);
+
+int update_function_from_script (const char *funname, const char *path,
+				 fnpkg *pkg);
 
 int user_function_help (const char *fnname, PRN *prn);
 
@@ -158,5 +171,4 @@ int push_fn_arg (fnargs *args, int type, void *p);
 void adjust_indent (const char *line, int *this_indent,
 		    int *next_indent);
 
-
 #endif /* GRETL_FUNC_H */
