@@ -86,7 +86,7 @@ void gretl_pop_c_numeric_locale (void)
     return;
 }
 
-#endif /* ENABLE_NLS */
+#endif /* ENABLE_NLS or not */
 
 /**
  * doing_nls:
@@ -96,24 +96,21 @@ void gretl_pop_c_numeric_locale (void)
 
 int doing_nls (void)
 {
-#ifdef ENABLE_NLS
     static int called, nls;
 
     if (!called) {
-	nls = (strcmp("/File/_Open data", _("/File/_Open data")) ||
+	nls = (strcmp("_Open data", _("_Open data")) ||
 	       strcmp("Test statistic", _("Test statistic")) ||
 	       strcmp("annual", _("annual")));
 	called = 1;
     }
+
     return nls;
-#else
-    return 0;
-#endif
 }
 
 #ifdef ENABLE_NLS
+
 static int decpoint;
-#endif
 
 /**
  * reset_local_decpoint:
@@ -126,15 +123,12 @@ static int decpoint;
 
 int reset_local_decpoint (void)
 {
-#ifdef ENABLE_NLS
     struct lconv *lc;
 
     lc = localeconv();
     decpoint = *lc->decimal_point;
+    set_atof_point(decpoint);
     return decpoint;
-#else
-    return '.';
-#endif
 }
 
 /**
@@ -145,24 +139,32 @@ int reset_local_decpoint (void)
 
 int get_local_decpoint (void)
 {
-#ifdef ENABLE_NLS
     if (decpoint == 0) {
 	decpoint = reset_local_decpoint();
     }
     return decpoint;
-#else
-    return '.';
-#endif
 }
 
+#else
+
+int reset_local_decpoint (void)
+{
+    return '.';
+}
+
+int get_local_decpoint (void)
+{
+    return '.';
+}
+
+#endif
+
 /* fudges for strings that should not be in utf-8 under some
    conditions: under GTK translations always come out in utf-8 in the
    GUI, but when we're sending stuff to stderr we may have to put it
    into ISO-8859-N.
 */
 
-#ifdef ENABLE_NLS
-
 static int gretl_cset_maj;
 static int gretl_cset_min;
 # ifdef WIN32
@@ -186,7 +188,7 @@ void set_gretl_charset (const char *s)
 
     if (using_utf8) {
 	set_tex_use_utf(1);
-	set_fopen_use_utf8();
+	set_stdio_use_utf8();
     }
 
     *gretl_charset = '\0';
@@ -236,15 +238,26 @@ void set_gretl_charset (const char *s)
 #endif
 }
 
+#ifdef WIN32
+
+static void set_cp_from_locale (const char *loc)
+{
+    const char *p = strrchr(loc, '.');
+
+    if (p != NULL && strlen(p) > 3 && isdigit(p[1])) {
+        gretl_cpage = atoi(p + 1);
+        fprintf(stderr, "set_cp_from_locale: CP = %d\n", gretl_cpage);
+    }
+}
+
+#endif
+
+#ifdef ENABLE_NLS
+
 static const char *get_gretl_charset (void)
 {
     static char cset[12];
 
-    if (gretl_cset_maj > 0 && gretl_cset_min > 0) {
-	sprintf(cset, "ISO-%d-%d", gretl_cset_maj, gretl_cset_min);
-	return cset;
-    } 
-
 # ifdef WIN32
     if (gretl_cpage > 0) {
 	sprintf(cset, "CP%d", gretl_cpage);
@@ -252,9 +265,16 @@ static const char *get_gretl_charset (void)
     }
 # endif
 
+    if (gretl_cset_maj > 0 && gretl_cset_min > 0) {
+	sprintf(cset, "ISO-%d-%d", gretl_cset_maj, gretl_cset_min);
+	return cset;
+    } 
+
     return NULL;
 }
 
+#endif
+
 int iso_latin_version (void)
 {
     char *lang = NULL;
@@ -262,26 +282,31 @@ int iso_latin_version (void)
     if (gretl_cset_maj == 8859 &&
 	(gretl_cset_min == 1 || 
 	 gretl_cset_min == 2 ||
+	 gretl_cset_min == 5 ||
 	 gretl_cset_min == 9 ||
 	 gretl_cset_min == 15)) {
 	return gretl_cset_min;
     }
 
-# ifdef WIN32
+#ifdef WIN32
     if (gretl_cpage == 1252) {
 	return 1;
     } else if (gretl_cpage == 1250) {
 	return 2;
+    } else if (gretl_cpage == 1251) {
+	return 5;
     } else if (gretl_cpage == 1254) {
 	return 9;
     }
-# endif
+#endif
 
-    /* Polish, Turkish: UTF-8 locale? */
+    /* Polish, Russian, Turkish: UTF-8 locale? */
     lang = getenv("LANG");
     if (lang != NULL) {
 	if (!strncmp(lang, "pl", 2)) {
 	    return 2;
+	} else if (!strncmp(lang, "ru", 2)) {
+	    return 5;
 	} else if (!strncmp(lang, "tr", 2)) {
 	    return 9;
 	}
@@ -290,10 +315,30 @@ int iso_latin_version (void)
     return 1;
 }
 
-char *gp_gettext (const char *msgid)
+int chinese_locale (void)
 {
-    return gettext(msgid);
-} 
+#ifdef WIN32
+    return (gretl_cpage == 950);
+#else
+    char *lang = getenv("LANG");
+
+    return (lang != NULL && !strncmp(lang, "zh", 2));
+#endif
+}
+
+static int gui_native_printing;
+
+void set_gui_native_printing (void)
+{
+    gui_native_printing = 1;
+}
+
+void unset_gui_native_printing (void)
+{
+    gui_native_printing = 0;
+}
+
+#ifdef ENABLE_NLS
 
 char *iso_gettext (const char *msgid)
 {
@@ -304,8 +349,7 @@ char *iso_gettext (const char *msgid)
 
     /* command line program: switching of codesets should not be 
        required, since unlike the GUI program there's no need
-       to force UTF-8 as the default.  Though there might be 
-       a problem with gnuplot, perhaps?
+       to force UTF-8 as the default.  
     */
 
     if (!strcmp(msgid, "@CLI_INIT")) {
@@ -318,25 +362,28 @@ char *iso_gettext (const char *msgid)
     }
 
     /* iso_switch: we'll reckon that if the system character set is
-       not UTF-8, and is an ISO-9958-N or Windows CP12NN 8-bit set,
+       not UTF-8, and is an ISO-8859-N or Windows CP12NN 8-bit set,
        then we should probably recode when printing translated strings
        in the context of writing TeX, CSV and RTF files.  If
        iso_switch is non-zero (once it's determinate) this makes the
        I_() gettext macro use the system encoding, otherwise I_() is
        equivalent to plain _(), which always spits out UTF-8.  There's
-       a possible FIXME here: perhaps we should always recode for RTF
-       files -- is a UTF-8 encoded RTF file kosher?  But this seems
-       like a corner case.
+       a possible gotcha here: I suspect that a UTF-8 encoded RTF file 
+       is never kosher, regardless of the system.  
     */
 
     if (iso_switch < 0) {
 	/* not yet determined */
 	cset = get_gretl_charset();
-	fprintf(stderr, "get_gretl_charset gave %s\n", cset);
+	if (cset == NULL) {
+	    fprintf(stderr, "get_gretl_charset: using UTF-8\n");
+	} else {
+	    fprintf(stderr, "get_gretl_charset gave %s\n", cset);
+	}
 	iso_switch = (cset != NULL);
     }
 
-    if (iso_switch) {
+    if (iso_switch && !gui_native_printing) {
 	bind_textdomain_codeset(PACKAGE, cset);
 	ret = gettext(msgid);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -347,7 +394,321 @@ char *iso_gettext (const char *msgid)
     return ret;
 } 
 
-#endif  /* ENABLE_NLS */
+#endif /* ENABLE_NLS */
+
+#ifdef WIN32
+
+struct localeinfo {
+    int id;
+    const char *code;
+};
+
+/* the following are the strings that are accepted by setlocale()
+   on win32 */
+
+static struct localeinfo locales[] = {
+    { LANG_AUTO,  NULL },
+    { LANG_C,     "english" },
+    { LANG_EU,    "basque" },
+    { LANG_DE,    "german" },
+    { LANG_ES,    "spanish" },
+    { LANG_FR,    "french" },
+    { LANG_IT,    "italian" },
+    { LANG_PL,    "polish" },
+    { LANG_TR,    "turkish" },
+    { LANG_PT,    "portuguese" },
+    { LANG_PT_BR, "portuguese-brazilian" },
+    { LANG_RU,    "russian" },
+    { LANG_ZH_TW, "chinese-traditional" },
+    { LANG_CS,    "czech" },
+    { LANG_MAX,    NULL }
+};
+
+const char *locale_code_from_id (int langid)
+{
+    int i;
+
+    for (i=0; i<LANG_MAX; i++) {
+	if (langid == locales[i].id) {
+	    return locales[i].code;
+	}
+    }
+
+    return NULL;
+}
+
+#endif /* WIN32 */
+
+struct langinfo {
+    int id;
+    const char *name;
+    const char *code;
+};
+
+static struct langinfo langs[] = {
+    { LANG_AUTO,  "Automatic",            NULL    },
+    { LANG_C,     "English",              "C"     },
+    { LANG_EU,    "Basque",               "eu_ES" },
+    { LANG_ZH_TW, "Chinese (Taiwan)",     "zh_TW" },
+    { LANG_CS,    "Czech",                "cs_CZ" },
+    { LANG_FR,    "French",               "fr_FR" },
+    { LANG_DE,    "German",               "de_DE" },
+    { LANG_IT,    "Italian",              "it_IT" },
+    { LANG_PL,    "Polish",               "pl_PL" },
+    { LANG_PT,    "Portuguese",           "pt_PT" },
+    { LANG_PT_BR, "Portuguese (Brazil)",  "pt_BR" },
+    { LANG_RU,    "Russian",              "ru_RU" },
+    { LANG_ES,    "Spanish",              "es_ES" },
+    { LANG_TR,    "Turkish",              "tr_TR" },
+    { LANG_MAX,    NULL,                   NULL   }
+};
+
+const char *lang_string_from_id (int langid)
+{
+    int i;
+
+    for (i=0; i<LANG_MAX; i++) {
+	if (langid == langs[i].id) {
+	    return langs[i].name;
+	}
+    }
+
+    return NULL;
+}
+
+int lang_id_from_name (const char *s)
+{
+    int i;
+
+    if (s == NULL || *s == '\0') {
+	return 0;
+    }
+
+    for (i=0; i<LANG_MAX; i++) {
+	if (!strcmp(s, langs[i].name)) {
+	    return langs[i].id;
+	}
+    }
+
+    return 0;
+}
+
+const char *lang_code_from_id (int langid)
+{
+    int i;
+
+    for (i=0; i<LANG_MAX; i++) {
+	if (langid == langs[i].id) {
+	    return langs[i].code;
+	}
+    }
+
+    return NULL;
+}
+
+#ifdef WIN32
+
+static char *win32_set_numeric (const char *lang)
+{
+    char *set = NULL;
+    int i;
+
+    for (i=LANG_EU; i<LANG_MAX; i++) {
+	if (!strcmp(lang, langs[i].code) ||
+	    !strncmp(lang, langs[i].code, 2)) {
+	    set = setlocale(LC_NUMERIC, locales[i].code);
+	    if (set != NULL) {
+		break;
+	    }
+	}
+    }
+
+    return set;
+}
+
+#else /* !WIN32 */
+
+# ifdef ENABLE_NLS
+
+static char *other_set_numeric (const char *lang)
+{
+    char *set = setlocale(LC_NUMERIC, lang);
+
+    if (set == NULL) {
+	char lfix[32];
+
+	sprintf(lfix, "%s.UTF-8", lang);
+	set = setlocale(LC_NUMERIC, lfix);
+    }
+
+    return set;
+}
+
+# endif /* ENABLE_NLS */
+
+#endif /* WIN32 or not */
+
+void set_lcnumeric (int langid, int lcnumeric)
+{
+#ifndef ENABLE_NLS
+    return;
+#else
+    if (lcnumeric && langid != LANG_C) {
+	const char *lang;
+	char *set = NULL;
+
+	if (langid == LANG_AUTO) {
+	    /* respect the system LANG setting */
+	    lang = getenv("LANG");
+	} else {
+	    /* fake it from user preference */
+	    lang = lang_code_from_id(langid);
+	}
+
+	if (lang != NULL) {
+# ifdef WIN32
+	    set = win32_set_numeric(lang);
+# else
+	    set = other_set_numeric(lang);
+# endif
+	} 
+	if (set == NULL) {
+	    setlocale(LC_NUMERIC, "");
+	    putenv("LC_NUMERIC=");
+	}
+    } else {
+	/* either lcnumeric is not chosen, or we're in LANG_C */
+	setlocale(LC_NUMERIC, "C");
+	putenv("LC_NUMERIC=C");
+    }
+
+    reset_local_decpoint();
+#endif
+}
+
+#ifdef ENABLE_NLS
+
+static int set_locale_with_workaround (const char *lcode)
+{
+    char *test = setlocale(LC_ALL, lcode);
+
+# ifndef WIN32    
+    if (test == NULL) {
+	char lfix[32];
+
+	sprintf(lfix, "%s.UTF-8", lcode); /* Ubuntu, grr */
+	test = setlocale(LC_ALL, lfix);
+    }
+# endif
+
+    if (test != NULL) {
+	fprintf(stderr, "setlocale: '%s' -> '%s'\n", lcode, test);
+    }
+
+    return test == NULL;
+}
+
+# ifdef WIN32
+# define get_setlocale_string(i) (locale_code_from_id(i))
+# else
+# define get_setlocale_string(i) (lang_code_from_id(i))
+# endif
+
+#endif /* ENABLE_NLS */
+
+/* @langstr should be the English name of the selected language
+   as displayed in the GUI (e.g. "German", "French") 
+*/
+
+int test_locale (const char *langstr)
+{
+#ifndef ENABLE_NLS
+    return 1;
+#else
+    const char *lcode;
+    char *orig, ocpy[64];
+    int langid, err = 0;
+
+    langid = lang_id_from_name(langstr);
+    lcode = get_setlocale_string(langid);
+    orig = setlocale(LC_ALL, NULL);
+
+    gretl_error_clear();
+
+    *ocpy = '\0';
+    strncat(ocpy, orig, 63);
+
+    err = set_locale_with_workaround(lcode);
+
+    if (err) {
+	gretl_errmsg_sprintf(_("%s: locale is not supported "
+			       "on this system"), lcode);
+    } else {
+	setlocale(LC_ALL, ocpy); /* restore the original locale */
+    } 
+
+    return err;
+#endif
+}
+
+int force_language (int langid)
+{
+#ifndef ENABLE_NLS
+    return 1;
+#else
+    const char *lcode = NULL;
+    int err = 0;
+
+    if (langid == LANG_C) {
+	putenv("LANGUAGE=english");
+	putenv("LANG=C");
+	setlocale(LC_ALL, "C");
+    } else {
+	lcode = get_setlocale_string(langid);
+	if (lcode != NULL) {  
+# ifdef WIN32
+	    char *newloc = setlocale(LC_ALL, lcode);
+
+            fprintf(stderr, "lcode='%s', newloc='%s'\n", lcode, newloc);
+	    if (newloc != NULL) {
+		set_cp_from_locale(newloc);
+	    } else {
+		err = 1;
+	    }
+# else
+	    err = set_locale_with_workaround(lcode);
+# endif
+	}
+    }
+
+# if defined(WIN32)
+    if (langid == LANG_C) {
+	SetEnvironmentVariable("LC_ALL", "C");
+	putenv("LC_ALL=C");
+	textdomain("none");
+    } else if (lcode != NULL) {
+        char estr[32];
+
+        lcode = lang_code_from_id(langid);
+	SetEnvironmentVariable("LC_ALL", lcode);
+	sprintf(estr, "LC_ALL=%s", lcode);
+	putenv(estr);
+	SetEnvironmentVariable("LANG", lcode);
+	sprintf(estr, "LANG=%s", lcode);
+	putenv(estr);
+    }
+# elif defined(OSX_BUILD)
+    if (langid != LANG_C) {
+	char estr[64];
+
+	sprintf(estr, "LANGUAGE=%s", lang_code_from_id(langid));
+	putenv(estr);
+    }
+# endif
+
+    return err;
+#endif /* ENABLE_NLS */
+}
 
 static void 
 iso_to_ascii_translate (char *targ, const char *src, int latin)
@@ -593,29 +954,6 @@ char *iso_to_ascii (char *s)
     return real_iso_to_ascii(s, 1);
 }
 
-char *get_month_name (char *mname, int m)
-{
-    struct tm mt;
-
-    mt.tm_sec = 0;
-    mt.tm_min = 0;
-    mt.tm_hour = 0;
-    mt.tm_mday = 1;
-    mt.tm_mon = m - 1;
-    mt.tm_year = 100;
-
-    strftime(mname, 7, "%b", &mt);
-    *mname = tolower(*mname);
-
-#ifdef ENABLE_NLS
-    real_iso_to_ascii(mname, gretl_cset_min);
-#endif
-
-    return mname;
-}
-
-#ifdef ENABLE_NLS
-
 char *sprint_l2_to_ascii (char *targ, const char *s, size_t len)
 {
     iso_to_ascii_translate(targ, s, 2);
@@ -644,6 +982,8 @@ static char *get_gp_encoding_set (char *s, int targ)
 	strcpy(s, "ISO-8859-");
 	if (latin == 2) {
 	    strcat(s, "2");
+	} else if (latin == 5) {
+	    strcat(s, "5");
 	} else if (latin == 9) {
 	    strcat(s, "9");
 	} else if (latin == 15) {
@@ -656,6 +996,8 @@ static char *get_gp_encoding_set (char *s, int targ)
 	strcpy(s, "CP125");
 	if (latin == 2) {
 	    strcat(s, "0");
+	} else if (latin == 5) {
+	    strcat(s, "1");
 	} else if (latin == 9) {
 	    strcat(s, "4");
 	} else {
@@ -689,7 +1031,7 @@ char *utf8_to_latin (const char *s)
     return ret;
 }
 
-/* convert from UTF-8 to Windows codepage, for EMF graphs */
+/* convert from UTF-8 to Windows codepage, e.g. for EMF graphs */
 
 char *utf8_to_cp (const char *s)
 {
@@ -729,11 +1071,18 @@ int gretl_is_ascii (const char *buf)
     return 1;
 }
 
+/* We want to print @str in a field of @width (visible) characters,
+   but @str may contain multi-byte characters. In that case, determine 
+   the adjustment to @width that is needed to avoid underrun and
+   return the adjusted value.
+*/
+
 int get_utf_width (const char *str, int width)
 {
-    width += strlen(str) - g_utf8_strlen(str, -1);
+    /* the number of "invisible" bytes */
+    int invis = strlen(str) - g_utf8_strlen(str, -1);
 
-    return width;
+    return width + invis;
 }
 
 int get_translated_width (const char *str)
@@ -747,6 +1096,8 @@ int get_translated_width (const char *str)
 
 static int printing_to_console;
 
+#ifdef ENABLE_NLS
+
 char *maybe_iso_gettext (const char *msgid)
 {
    if (printing_to_console) {
@@ -756,6 +1107,8 @@ char *maybe_iso_gettext (const char *msgid)
    }
 } 
 
+#endif
+
 void check_for_console (PRN *prn)
 {
     if (prn != NULL) {
@@ -769,11 +1122,4 @@ void console_off (void)
     printing_to_console = 0;
 }
 
-#else
 
-int gretl_is_ascii (const char *buf)
-{
-    return 1;
-}
-
-#endif /* !ENABLE_NLS */
