@@ -17,20 +17,39 @@
  * 
  */
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "gretl_list.h"
 #include "gretl_func.h"
 #include "libset.h"
 #include "gretl_xml.h"
 
+#include <errno.h>
+#include <glib.h>
+
 #define LDEBUG 0
 
-#define LNAMELEN 32
+/**
+ * SECTION:gretl_list
+ * @short_description: handling of lists of integers
+ * @title: Lists
+ * @include: libgretl.h
+ *
+ * Lists of integers are used in many contexts in libgretl, e.g.
+ * for holding the ID numbers of variables in a regression
+ * specification. A gretl "list" is simply an array of ints
+ * following a definite convention: the value at position 0
+ * gives the number of elements that follow. The total number
+ * of ints in the list foo is therefore foo[0] + 1, and reading
+ * the substantive members of foo involves looping from
+ * position 1 to position foo[0].
+ */
 
 typedef struct saved_list_ saved_list;
 
 struct saved_list_ {
-    char name[LNAMELEN];
+    char name[VNAMELEN];
     int *list;
     int level;
 };
@@ -62,7 +81,7 @@ static saved_list *saved_list_new (const int *list, const char *name)
 	    sl = NULL;
 	} else {
 	    *sl->name = 0;
-	    strncat(sl->name, name, LNAMELEN - 1);
+	    strncat(sl->name, name, VNAMELEN - 1);
 	}
     }
 
@@ -79,17 +98,19 @@ static void free_saved_list (saved_list *sl)
 
 static saved_list *get_saved_list_by_name (const char *name)
 {
+    saved_list *sl = NULL;
     int fsd = gretl_function_depth();
     int i;
 
     for (i=0; i<n_lists; i++) {
 	if (!strcmp(name, list_stack[i]->name) && 
 	    fsd == list_stack[i]->level) {
-	    return list_stack[i];
+	    sl = list_stack[i];
+	    break;
 	}
     }
 
-    return NULL;
+    return sl;
 }
 
 /**
@@ -148,8 +169,8 @@ const char *get_list_name_by_index (int idx)
  * get_list_by_name:
  * @name: the name of the list to be found.
  *
- * Looks up @name in the stack of saved lists (if any) and
- * retrieves the associated list.
+ * Looks up @name in the stack of saved lists, at the current level
+ * of function execution, and retrieves the associated list.
  *
  * Returns: the list, or %NULL if the lookup fails. 
  */
@@ -239,15 +260,15 @@ int subtract_from_list_by_name (const char *targ, const int *sub)
 /**
  * replace_list_by_name:
  * @targ: the name of the target list.
- * @new: replacement list
+ * @src: replacement list
  *
  * If @targ is the name of a saved list, replace the
- * list of that name with @new.
+ * list of that name with @src.
  *
  * Returns: 0 on success, non-zero code on failure.
  */
 
-int replace_list_by_name (const char *targ, const int *new)
+int replace_list_by_name (const char *targ, const int *src)
 {
     saved_list *sl = get_saved_list_by_name(targ);
     int err = 0;
@@ -255,7 +276,7 @@ int replace_list_by_name (const char *targ, const int *new)
     if (sl == NULL) {
 	err = E_UNKVAR;
     } else {
-	int *tmp = gretl_list_copy(new);
+	int *tmp = gretl_list_copy(src);
 
 	if (tmp == NULL) {
 	    err = E_ALLOC;
@@ -283,6 +304,13 @@ static int real_remember_list (const int *list, const char *name,
 	    " n_lists=%d\n", name, force_new, n_lists);
 #endif
 
+    /* Note: 'force_new' means that we'll add a new list even if it
+       has the same name as an existing one.  This makes sense only if
+       we're copying a list in the context of running a user-defined
+       function, since in that case the new list will exist at a
+       different "stack level" from any prior list.
+    */
+
     if (!force_new) {
 	orig = get_saved_list_by_name(name);
     }
@@ -322,7 +350,7 @@ static int real_remember_list (const int *list, const char *name,
 		}
 		if (strlen(realname) < strlen(name)) {
 		    pprintf(prn, _("Warning: the name was truncated to %d characters\n"), 
-			    LNAMELEN - 1);
+			    VNAMELEN - 1);
 		}
 	    }
 	    n_lists++;
@@ -345,8 +373,8 @@ static int real_remember_list (const int *list, const char *name,
  * @name: name to be given to the list.
  * @prn: printing struct.
  *
- * Adds @list to the stack of saved lists and associates it
- * with @name, unless there is already a list with the given
+ * Adds a copy of @list to the stack of saved lists and associates
+ * it with @name, unless there is already a list with the given
  * name in which case the original list is replaced.  A status
  * message is printed to @prn.
  *
@@ -402,15 +430,15 @@ static int destroy_saved_list (saved_list *sl)
 /**
  * rename_saved_list:
  * @orig: the original name of the list.
- * @new: the new name to be given.
+ * @newname: the new name to be given.
  *
  * Renames a saved list from @orig to @new.  If there is
- * already a list called @new, it is destroyed.
+ * already a list called @newname, it is destroyed.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int rename_saved_list (const char *orig, const char *new)
+int rename_saved_list (const char *orig, const char *newname)
 {
     saved_list *sl0, *sl1;
     int err = 0;
@@ -419,13 +447,13 @@ int rename_saved_list (const char *orig, const char *new)
     if (sl0 == NULL) {
 	err = 1;
     } else {
-	/* is there already a list called @new? */
-	sl1 = get_saved_list_by_name(new);
+	/* is there already a list called @newname? */
+	sl1 = get_saved_list_by_name(newname);
 	if (sl1 != NULL) {
 	    err = destroy_saved_list(sl1);
 	}
 	*sl0->name = '\0';
-	strncat(sl0->name, new, LNAMELEN - 1);
+	strncat(sl0->name, newname, VNAMELEN - 1);
     } 
 
     return err;
@@ -434,18 +462,18 @@ int rename_saved_list (const char *orig, const char *new)
 /**
  * copy_named_list_as:
  * @orig: the name of the original list.
- * @new: the name to be given to the copy.
+ * @newname: the name to be given to the copy.
  *
  * If a saved list is found by the name @orig, a copy of
  * this list is added to the stack of saved lists under the
- * name @new.  This is intended for use when a list is given
+ * name @newname.  This is intended for use when a list is given
  * as the argument to a user-defined function: it is copied
  * under the name assigned by the function's parameter list.
  *
  * Returns: 0 on success, non-zero on error.
  */
 
-int copy_named_list_as (const char *orig, const char *new)
+int copy_named_list_as (const char *orig, const char *newname)
 {
     saved_list *sl;
     int err = 0;
@@ -454,7 +482,7 @@ int copy_named_list_as (const char *orig, const char *new)
     if (sl == NULL) {
 	err = 1;
     } else {
-	err = real_remember_list(sl->list, new, 1, NULL);
+	err = real_remember_list(sl->list, newname, 1, NULL);
 	if (!err) {
 	    sl = list_stack[n_lists - 1];
 	    sl->level += 1;
@@ -526,6 +554,48 @@ int create_named_null_list (const char *name)
     return err;
 }
 
+/**
+ * create_named_signgleton_list:
+ * @varnum: ID number of the series to use.
+ * @name: the name to be given to the list.
+ *
+ * Creates a one-element list under the given @name and adds it
+ * to the stack of saved lists at the next level of
+ * function execution.  This is intended for use when a single 
+ * series is given as an argument to a user-defined function in
+ * a 'slot' where a list is required.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int create_named_singleton_list (int varnum, const char *name)
+{
+    saved_list *sl = NULL;
+    int *list = NULL;
+    int err;
+
+    list = gretl_list_new(1);
+    if (list == NULL) {
+	return E_ALLOC;
+    }
+
+    list[1] = varnum;
+
+    err = real_remember_list(list, name, 1, NULL);
+    if (!err) {
+	sl = list_stack[n_lists - 1];
+	sl->level += 1;
+    } 
+
+#if LDEBUG
+    fprintf(stderr, "create_named_singleton_list: sl at %p\n", (void *) sl);
+#endif
+
+    free(list);
+
+    return err;
+}
+
 /**
  * destroy_saved_lists_at_level:
  * @level: stack level of function execution.
@@ -578,6 +648,17 @@ int destroy_saved_lists_at_level (int level)
     return err;
 }
 
+int delete_list_by_name (const char *name)
+{
+    saved_list *sl = get_saved_list_by_name(name);
+
+    if (sl == NULL) {
+	return E_UNKVAR;
+    } else {
+	return destroy_saved_list(sl);
+    }
+}
+
 static int var_is_deleted (const int *dlist, int dmin, int i)
 {
     int v = dmin + i - 1;
@@ -610,6 +691,11 @@ int gretl_lists_revise (const int *dlist, int dmin)
     int lmax = 0;
     int i, j, k;
 
+#if 0
+    fprintf(stderr, "gretl_lists_revise: dlist = %p, dmin = %d\n", 
+	    (void *) dlist, dmin);
+#endif
+
     if (dlist != NULL) {
 	/* determine lowest deleted ID */
 	dmin = dlist[1];
@@ -655,11 +741,11 @@ int gretl_lists_revise (const int *dlist, int dmin)
     /* use mapping to revise saved lists */
     for (j=0; j<n_lists; j++) {
 	list = list_stack[j]->list;
-	for (i=1; i<=list[0]; i++) {
+	for (i=list[0]; i>0; i--) {
 	    k = list[i] - dmin + 1;
 	    if (k >= 1) {
 		if (maplist[k] == -1) {
-		    gretl_list_delete_at_pos(list, i--);
+		    gretl_list_delete_at_pos(list, i);
 		} else {
 		    list[i] = maplist[k];
 		}
@@ -749,10 +835,9 @@ int *gretl_consecutive_list_new (int lmin, int lmax)
 	return NULL;
     }
 
-    list = malloc((n + 1) * sizeof *list);
+    list = gretl_list_new(n);
 
     if (list != NULL) {
-	list[0] = n;
 	for (i=0; i<n; i++) {
 	    list[i+1] = lmin + i;
 	}
@@ -886,6 +971,14 @@ int gretl_list_cmp (const int *list1, const int *list2)
 {
     int i;
 
+    if (list1 == NULL && list2 != NULL) {
+	return 1;
+    } else if (list1 != NULL && list2 == NULL) {
+	return 1;
+    } else if (list1 == NULL && list2 == NULL) {
+	return 0;
+    }
+
     for (i=0; i<=list1[0]; i++) {
 	if (list2[i] != list1[i]) {
 	    return 1;
@@ -940,59 +1033,172 @@ int *gretl_list_copy (const int *src)
     return targ;
 }
 
+/**
+ * gretl_list_copy_from_pos:
+ * @src: an array of integers, the first element of which holds
+ * a count of the number of elements following.
+ *
+ * Returns: an allocated copy @src from position @pos onward
+ * (or %NULL on failure).
+ */
+
+int *gretl_list_copy_from_pos (const int *src, int pos)
+{
+    int *targ = NULL;
+    int i, n;
+
+    if (src != NULL && ((n = src[0] - pos + 1) > 0)) {
+	targ = gretl_list_new(n);
+	if (targ != NULL) {
+	    for (i=1; i<=n; i++) {
+		targ[i] = src[i+pos-1];
+	    }
+	}
+    }
+
+    return targ;
+}
+
 /**
  * gretl_list_from_string:
- * @liststr: string representation of list of integers.
+ * @str: string representation of list of integers.
+ * @err: location to receive error code.
  *
  * Reads a string containing a list of integers, separated by
  * spaces and/or commas and possibly wrapped in parentheses,
  * and constructs an array of these integers.  The first 
  * element is the number of integers that follow.
+ * This function supports an abbreviation for consecutive
+ * (increasing) integers in the list, using the notation, e.g., 
+ * "1-4" as shorthand for "1 2 3 4".
  *
  * Returns: the allocated array, or %NULL on failure.
  */
 
-int *gretl_list_from_string (const char *liststr)
+int *gretl_list_from_string (const char *str, int *err)
 {
-    const char *s = liststr;
-    char numstr[8];
+    char *p, *q, *s, *next;
+    int i, r1, r2, rg;
     int *list;
     int n = 0;
 
-    if (liststr == NULL) {
+    if (str == NULL) {
+	*err = E_DATA;
 	return NULL;
     }
 
-    while (*s) {
-	s += strspn(s, " ,()");
-	if (*s && sscanf(s, "%7[^ ,()]", numstr)) {
+    /* 'p' marks the memory to be freed */
+    p = s = gretl_strdup(str);
+    if (s == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    *err = 0;
+
+    /* strip white space at both ends */
+    while (isspace(*s)) s++;
+    tailstrip(s);
+
+    /* strip parentheses, if present */
+    if (*s == '(') {
+	n = strlen(s);
+	if (s[n-1] != ')') {
+	    /* got opening paren but no close */
+	    *err = E_PARSE;
+	    return NULL;
+	}
+	s[n-1] = '\0';
+	s++;
+	while (isspace(*s)) s++;
+	tailstrip(s);
+    }
+
+    q = s; /* copy relevant starting point */
+
+    charsub(s, ',', ' ');
+
+    errno = 0;
+
+    /* first pass: figure out the number of values
+       in the list, checking for errors as we go
+    */
+
+    n = 0; /* value counter */
+
+    while (*s && !*err) {
+	s += strspn(s, " ");
+	if (n > 0 && *s == ';') {
+	    /* list separator */
 	    n++;
-	    s += strlen(numstr);
+	    s++;
+	    continue;
+	}
+	r1 = strtol(s, &next, 10);
+	if (errno || next == s) {
+	    fprintf(stderr, "gretl_list_from_string: '%s'\n", s);
+	    *err = E_PARSE;
+	} else {
+	    s = next;
+	    if (*s == '-') {
+		/* hyphen indicating range? */
+		s++;
+		r2 = strtol(s, &next, 10);
+		if (errno || next == s) {
+		    *err = E_PARSE;
+		} else if (r2 < r1) {
+		    *err = E_PARSE;
+		} else {
+		    n += r2 - r1 + 1;
+		}
+		s = next;
+	    } else {
+		/* single numerical value */
+		n++;
+	    }
 	}
     }
 
-    if (n == 0) {
+    if (*err || n == 0) {
+	free(p);
 	return NULL;
     }
 
     list = gretl_list_new(n);
     if (list == NULL) {
+	*err = E_ALLOC;
+	free(p);
 	return NULL;
     }
 
-    s = liststr;
-    n = 1;
+    /* second pass: fill out the list (no error
+       checking should be needed at this stage) 
+    */
+
+    s = q; /* back to start of string */
+    n = 1; /* list position indicator */
+
     while (*s) {
-	s += strspn(s, " ,()");
-	if (*s && sscanf(s, "%7[^ ,()]", numstr)) {
-	    if (*numstr == ';') {
-		list[n++] = LISTSEP;
-	    } else {
-		list[n++] = atoi(numstr);
-	    }
-	    s += strlen(numstr);
+	s += strspn(s, " ");
+	if (*s == ';') {
+	    list[n++] = LISTSEP;
+	    s++;
+	    continue;
 	}
-    }    
+	r1 = strtol(s, &s, 10);
+	if (*s == '-') {
+	    s++;
+	    r2 = strtol(s, &s, 10);
+	    rg = r2 - r1 + 1;
+	    for (i=0; i<rg; i++) {
+		list[n++] = r1 + i;
+	    }
+	} else {
+	    list[n++] = r1;
+	}
+    } 
+
+    free(p);
 
     return list;
 }
@@ -1007,7 +1213,7 @@ int *gretl_list_from_string (const char *liststr)
  * positive integers in a range that is sensible for ID 
  * numbers of variables, typically with three digits or less, 
  * and will fail if the list contains any numbers greater 
- * than 999.
+ * than 9998.
  *
  * Returns: The string representation of the list on success,
  * or %NULL on failure.
@@ -1019,7 +1225,7 @@ char *gretl_list_to_string (const int *list)
     char numstr[8];
     int len, i, err = 0;
 
-    len = 4 * (list[0] + 1);
+    len = 6 * (list[0] + 1);
     if (len > MAXLINE - 32) {
 	return NULL;
     }
@@ -1030,11 +1236,8 @@ char *gretl_list_to_string (const int *list)
     }
 
     *buf = '\0';
+
     for (i=1; i<=list[0]; i++) {
-	if (abs(list[i] > 999)) {
-	    err = 1;
-	    break;
-	}
 	if (list[i] == LISTSEP) {
 	    strcat(buf, " ;");
 	} else {
@@ -1051,6 +1254,62 @@ char *gretl_list_to_string (const int *list)
     return buf;
 }
 
+/**
+ * gretl_list_to_lags_string:
+ * @list: array of integers.
+ * @err: location to receive error code.
+ * 
+ * Prints the given @list of integers into a newly
+ * allocated string, separated by commas.  Will fail
+ * if the list contains any numbers greater than 998.
+ *
+ * Returns: The string representation of the list on success,
+ * or %NULL on failure.
+ */
+
+char *gretl_list_to_lags_string (const int *list, int *err)
+{
+    char *buf;
+    char numstr[8];
+    int len, i;
+
+    len = 4 * (list[0] + 1);
+
+    if (len > MAXLINE - 32) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    buf = malloc(len);
+    if (buf == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    *buf = '\0';
+
+    for (i=1; i<=list[0]; i++) {
+	if (abs(list[i] >= 999)) {
+	    *err = E_DATA;
+	    break;
+	} else {
+	    if (i == 1) {
+		sprintf(numstr, "%d", list[i]);
+	    } else {
+		sprintf(numstr, ",%d", list[i]);
+	    }
+	    strcat(buf, numstr);
+	}
+    }
+
+    if (*err) {
+	free(buf);
+	buf = NULL;
+    }
+
+    return buf;
+}
+
 /**
  * in_gretl_list:
  * @list: an array of integers, the first element of which holds
@@ -1066,24 +1325,24 @@ char *gretl_list_to_string (const int *list)
 
 int in_gretl_list (const int *list, int k)
 {
-    int i, ret = 0;
+    int i;
 
     if (list != NULL) {
-	for (i=1; i<=list[0] && !ret; i++) {
+	for (i=1; i<=list[0]; i++) {
 	    if (list[i] == k) {
-		ret = i;
+		return i;
 	    }
 	}
     }
 
-    return ret;
+    return 0;
 }
 
-static void reglist_move_const (int *list, int from)
+static void reglist_move_const (int *list, int k)
 {
-    int i, cnum = list[from];
+    int i, cnum = list[k];
 
-    for (i=from; i>2; i--) {
+    for (i=k; i>2; i--) {
 	list[i] = list[i-1];
     }
 
@@ -1414,14 +1673,15 @@ static int *real_gretl_list_omit (const int *orig, const int *omit,
 
     *err = 0;
 
-    /* check for spurious "omissions" */
+    /* check how many terms we're omitting */
     for (i=1; i<=nomit; i++) {
 	int pos = in_gretl_list(orig, omit[i]);
 
 	if (pos < minpos) {
 	    if (mode == LIST_OMIT_TIGHT) {
-		sprintf(gretl_errmsg, _("Variable %d was not in the original list"),
-			omit[i]);
+		/* "omission" is spurious */
+		gretl_errmsg_sprintf(_("Variable %d was not in the original list"),
+				     omit[i]);
 		*err = 1;
 		return NULL;
 	    } else {
@@ -1487,7 +1747,8 @@ static int *real_gretl_list_omit (const int *orig, const int *omit,
  * @err: pointer to receive error code.
  *
  * Creates a list containing the elements of @orig that are not
- * present in @omit.
+ * present in @omit.  It is an error if the @omit list contains
+ * members that are not present in @orig.
  *
  * Returns: new list on success, %NULL on error.
  */
@@ -1497,6 +1758,26 @@ int *gretl_list_omit (const int *orig, const int *omit, int minpos, int *err)
     return real_gretl_list_omit(orig, omit, minpos, LIST_OMIT_TIGHT, err);
 }
 
+/**
+ * gretl_list_drop:
+ * @orig: an array of integers, the first element of which holds
+ * a count of the number of elements following.
+ * @drop: list of variables to drop.
+ * @err: pointer to receive error code.
+ *
+ * Creates a list containing the elements of @orig that are not
+ * present in @drop.  Unlike gretl_list_omit(), processing always
+ * starts from position 1 in @orig, and it is not an error if
+ * some members of @drop are not present in @orig.  
+ *
+ * Returns: new list on success, %NULL on error.
+ */
+
+int *gretl_list_drop (const int *orig, const int *drop, int *err)
+{
+    return real_gretl_list_omit(orig, drop, 1, LIST_OMIT_SLOPPY, err);
+}
+
 /**
  * gretl_list_diff:
  * @targ: target list (must be pre-allocated).
@@ -1554,7 +1835,9 @@ int gretl_list_diff (int *targ, const int *biglist, const int *sublist)
  *
  * Returns: a newly allocated list including the elements of @biglist,
  * from position @minpos onwards, that are not present in @sublist, 
- * again from @minpos onwards, or %NULL on failure.  
+ * again from @minpos onwards, or %NULL on failure.  Note that
+ * comparison stops whenever a list separator is found; i.e. only
+ * the pre-separator portions of the lists are compared.
  */
 
 int *gretl_list_diff_new (const int *biglist, const int *sublist,
@@ -1575,9 +1858,14 @@ int *gretl_list_diff_new (const int *biglist, const int *sublist,
 
     for (i=minpos; i<=biglist[0]; i++) {
 	bi = biglist[i];
+	if (bi == LISTSEP) {
+	    break;
+	}
 	match = 0;
 	for (j=minpos; j<=sublist[0]; j++) {
-	    if (sublist[j] == bi) {
+	    if (sublist[j] == LISTSEP) {
+		break;
+	    } else if (sublist[j] == bi) {
 		match = 1;
 		break;
 	    }
@@ -1749,7 +2037,7 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 			    "model have been redefined");
     const char *label;
     char rword[16];
-    int j, mc, repl, err = 0;
+    int j, mc, repl;
 
     if (ref_id == 0) {
 	mc = get_model_count();
@@ -1761,6 +2049,10 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 	if (list[j] == LISTSEP) {
 	    continue;
 	}
+	if (list[j] >= pdinfo->v) {
+	    gretl_errmsg_set(_(errmsg));
+	    return E_DATA;
+	}
 	label = VARLABEL(pdinfo, list[j]);
 	*rword = '\0';
 	sscanf(label, "%15s", rword);
@@ -1768,14 +2060,13 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
 	    repl = 0;
 	    sscanf(label, "%*s %*s %*s %d", &repl);
 	    if (repl >= mc) {
-		strcpy(gretl_errmsg, _(errmsg));
-		err = E_DATA;
-		break;
+		gretl_errmsg_set(_(errmsg));
+		return E_DATA;
 	    }
 	}
     }
 
-    return err;
+    return 0;
 }
 
 /**
@@ -1783,6 +2074,8 @@ int list_members_replaced (const int *list, const DATAINFO *pdinfo,
  * @list: an array of integer variable ID numbers, the first element
  * of which holds a count of the number of elements following.
  * @minpos: position in @list at which to start the search (>= 1).
+ * @Z: data array.
+ * @pdinfo: dataset information.
  *
  * Checks @list for the presence, in position @minpos or higher, of
  * a variable whose valid values in sample all equal 1.  This usually
@@ -1833,9 +2126,11 @@ int gretl_list_separator_position (const int *list)
 {
     int i;
 
-    for (i=1; i<=list[0]; i++) {
-        if (list[i] == LISTSEP) {
-	    return i;
+    if (list != NULL) {
+	for (i=1; i<=list[0]; i++) {
+	    if (list[i] == LISTSEP) {
+		return i;
+	    }
 	}
     }
 
@@ -1878,9 +2173,10 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
     int i, n = 0;
     int err = 0;
 
-    for (i=1; i<=list[0] && n==0; i++) {
+    for (i=1; i<=list[0]; i++) {
 	if (list[i] == LISTSEP) {
 	    n = i;
+	    break;
 	}
     }
 
@@ -1916,6 +2212,56 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
     return err;
 }
 
+/**
+ * gretl_lists_join_with_separator:
+ * @list1: first sub-list.
+ * @list2: second sub-list.
+ *
+ * Concatenates the content of @list2 onto @list1, after first
+ * appending the list separator.  It is acceptable that @list1
+ * be %NULL, in which case the returned list is just @list2
+ * with the separator prepended.  But it is not acceptable that
+ * @list2 be null; in that this function returns %NULL.
+ *
+ * Returns: alllcated list  on success or %NULL on failure.
+ */
+
+int *gretl_lists_join_with_separator (const int *list1, const int *list2)
+{
+    int *biglist;
+    int i, j, n;
+
+    if (list2 == NULL) {
+	return NULL;
+    }
+
+    n = (list1 != NULL)? list1[0] : 0;
+
+
+    n += list2[0] + 1;
+    biglist = gretl_list_new(n);
+
+    if (biglist == NULL) {
+	return NULL;
+    }
+
+    j = 1;
+
+    if (list1 != NULL) {
+	for (i=1; i<=list1[0]; i++) {
+	    biglist[j++] = list1[i];
+	}
+    }
+
+    biglist[j++] = LISTSEP;
+
+    for (i=1; i<=list2[0]; i++) {
+	biglist[j++] = list2[i];
+    }    
+
+    return biglist;
+}
+
 static int real_list_dup (const int *list, int start, int stop)
 {
     int i, j, ret = -1;
@@ -1951,7 +2297,7 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
     int start = 2;
     int i, ret = -1;
 
-    if (ci == COINT) {
+    if (ci == COINT || ci == ANOVA) {
 	start = 1;
     } else if (ci == ARCH) {
 	start = 3;
@@ -1971,7 +2317,7 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
 		break;
 	    }
 	}
-    } else if (ci == TSLS || ci == HECKIT || ci == EQUATION) {
+    } else if (ci == IVREG || ci == HECKIT || ci == EQUATION) {
 	multi = 1;
 	for (i=2; i<list[0]; i++) {
 	    if (list[i] == LISTSEP) {
@@ -2061,9 +2407,9 @@ int gretl_lists_share_members (const int *list1, const int *list2)
  * @pdinfo: dataset information.
  * @nvars: location for return of number of elements in full list.
  *
- * Creates a newly allocated list including all variables in the
- * dataset that are not scalars, are not hidden variables, and
- * are accessible at the current level of function execution.
+ * Creates a newly allocated list including all series in the
+ * dataset that are not hidden variables, and are accessible
+ * at the current level of function execution.
  * The return value is %NULL in case either (a) allocation of
  * memory failed, or (b) the resulting list would be empty.
  * The caller can distinguish between these possibilities by
@@ -2081,7 +2427,7 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
     int *list = NULL;
 
     for (i=1; i<pdinfo->v; i++) {
-	if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i) &&
+	if (!var_is_hidden(pdinfo, i) &&
 	    STACK_LEVEL(pdinfo, i) == fsd) {
 	    nv++;
 	}
@@ -2098,7 +2444,7 @@ int *full_var_list (const DATAINFO *pdinfo, int *nvars)
     if (list != NULL) {
 	j = 1;
 	for (i=1; i<pdinfo->v; i++) {
-	    if (var_is_series(pdinfo, i) && !var_is_hidden(pdinfo, i) &&
+	    if (!var_is_hidden(pdinfo, i) &&
 		STACK_LEVEL(pdinfo, i) == fsd) {
 		list[j++] = i;
 	    }
@@ -2209,7 +2555,7 @@ int *gretl_list_build (const char *s, const DATAINFO *pdinfo, int *err)
 		    *err = E_PARSE;
 		}
 	    } else {
-		v = varindex(pdinfo, test);
+		v = series_index(pdinfo, test);
 		if (v < pdinfo->v) {
 		    list = gretl_list_append_term(&list, v);
 		} else {
@@ -2288,7 +2634,7 @@ int load_user_lists_file (const char *fname)
 		if (!gretl_xml_get_prop_as_string(cur, "name", &lname)) {
 		    err = E_DATA;
 		} else {
-		    strncat(list_stack[i]->name, lname, LNAMELEN - 1);
+		    strncat(list_stack[i]->name, lname, VNAMELEN - 1);
 		    free(lname);
 		    list_stack[i]->list = 
 			gretl_xml_node_get_list(cur, doc, &err);
@@ -2356,8 +2702,8 @@ int gretl_serialize_lists (const char *fname)
 /**
  * gretl_list_print:
  * @lname: name of list.
- * @pdinfo:
- * @prn:
+ * @pdinfo: dataset information.
+ * @prn: gretl printing struct.
  * 
  * Prints to @prn the given @list of variables, by name.
  */
@@ -2392,3 +2738,88 @@ void gretl_list_print (const char *lname, const DATAINFO *pdinfo,
 	pputc(prn, '\n');
     }
 }
+
+/**
+ * varname_match_list:
+ * @pdinfo: pointer to dataset information.
+ * @pattern: pattern to be matched.
+ *
+ * Returns: a list of ID numbers of variables whose names
+ * match @pattern, or %NULL if there are no matches.
+ */
+
+int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
+{
+    GPatternSpec *pspec;
+    int *list = NULL;
+    int i, fd, n = 0;
+
+    if (pdinfo == NULL || pdinfo->v == 0) {
+	return NULL;
+    }
+
+    fd = gretl_function_depth();
+
+    pspec = g_pattern_spec_new(pattern);
+
+    for (i=1; i<pdinfo->v; i++) { 
+	if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
+	    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+		n++;
+	    }
+	}
+    }
+
+    if (n > 0) {
+	list = gretl_list_new(n);
+	if (list != NULL) {
+	    int j = 1;
+
+	    for (i=1; i<pdinfo->v; i++) { 
+		if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
+		    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+			list[j++] = i;
+		    }
+		}
+	    }
+	}
+    }
+
+    g_pattern_spec_free(pspec);
+
+    return list;
+}
+
+/**
+ * varname_match_any:
+ * @pdinfo: pointer to dataset information.
+ * @pattern: pattern to be matched.
+ *
+ * Returns: 1 if at least one variable in the dataset has a
+ * name that matches @pattern, otherwise 0.
+ */
+
+int varname_match_any (const DATAINFO *pdinfo, const char *pattern)
+{
+    GPatternSpec *pspec;
+    int i, fd, ret = 0;
+
+    fd = gretl_function_depth();
+
+    pspec = g_pattern_spec_new(pattern);
+
+    for (i=1; i<pdinfo->v; i++) { 
+	if (fd == 0 || fd == STACK_LEVEL(pdinfo, i)) {
+	    if (g_pattern_match_string(pspec, pdinfo->varname[i])) {
+		ret = 1;
+		break;
+	    }
+	}
+    }
+
+    g_pattern_spec_free(pspec);
+
+    return ret;
+}
+
+
