@@ -23,16 +23,45 @@
 #include <errno.h>
 #include <glib.h>
 
+/**
+ * SECTION:gretl_prn
+ * @short_description: gretl printing struct
+ * @title: PRN 
+ * @include: libgretl.h
+ *
+ * Most libgretl functions that print output call for a
+ * pointer-to-PRN as one of their arguments. The PRN type is
+ * an opaque structure, constructed and manipulated by the
+ * functions listed here. It is used as a means of generalizing
+ * the printing operation, which may be to a regular file, a
+ * temporary file or a buffer.
+ *
+ * To get hold of a PRN use gretl_print_new() or one of the more
+ * specific constructors, and to free it use gretl_print_destroy().
+ * If you want to use a PRN dirctly for printing in your own code, use
+ * the functions pprintf(), pputs() and pputc(). These are
+ * counterparts to the standard C functions fprintf, fputs and
+ * fputc, but note that with pputs and pputc the PRN argument
+ * must be given first (unlike fputs and fputc in which the
+ * FILE argument goes last). 
+ *
+ * Note that whenever a PRN appears as a function parameter
+ * in libgretl it is OK to give a NULL argument: in that case 
+ * printing is suppressed. (This obviously makes sense only if the
+ * function in question does something besides just printing.)
+ */
+
 struct PRN_ {
-    FILE *fp;
-    FILE *fpaux;    
-    char *buf;
-    size_t bufsize;
-    int savepos;
-    PrnFormat format;
-    int fixed;
-    char delim;
-    char *fname;
+    FILE *fp;          /* file to which to print, or NULL */
+    FILE *fpaux;       /* auxiliary file (output redirected) */
+    char *buf;         /* buffer to which to print, or NULL */
+    size_t bufsize;    /* allocated size of buffer */
+    size_t blen;       /* string length of buffer */
+    int savepos;       /* saved position in stream or buffer */
+    PrnFormat format;  /* plain, TeX, RTF */
+    int fixed;         /* = 1 for fixed-size buffer */
+    char delim;        /* CSV field delimiter */
+    char *fname;       /* file name, or NULL */
 };
 
 #undef PRN_DEBUG
@@ -59,7 +88,7 @@ void gretl_print_destroy (PRN *prn)
     }
 
     if (prn->fname != NULL) {
-	remove(prn->fname);
+	gretl_remove(prn->fname);
 	free(prn->fname);
     }
 
@@ -86,10 +115,10 @@ static int prn_add_tempfile (PRN *prn)
 
     errno = 0;
 
-    sprintf(fname, "%sprntmp.XXXXXX", gretl_dot_dir());
+    sprintf(fname, "%sprntmp.XXXXXX", gretl_dotdir());
 
 #ifdef WIN32
-    fd = g_mkstemp(fname);
+    fd = gretl_mkstemp(fname);
 #else
     fd = mkstemp(fname);
 #endif
@@ -109,8 +138,7 @@ static int prn_add_tempfile (PRN *prn)
     }
 
     if (errno != 0) {
-	gretl_errmsg_set_from_errno();
-	
+	gretl_errmsg_set_from_errno("prn_add_tempfile");
     }
 
     return err;
@@ -119,42 +147,47 @@ static int prn_add_tempfile (PRN *prn)
 static PRN *real_gretl_print_new (PrnType ptype, 
 				  const char *fname,
 				  char *buf,
-				  int *err)
+				  FILE *fp,
+				  int *perr)
 {
     PRN *prn = malloc(sizeof *prn);
+    int err = 0;
 
     if (prn == NULL) {
-	*err = E_ALLOC;
+	if (perr != NULL) {
+	    *perr = E_ALLOC;
+	}
 	return NULL;
     }
 
-    *err = 0;
-
 #if PRN_DEBUG
-    fprintf(stderr, "real_gretl_print_new: type=%d, fname='%s', buf=%p\n",
-	    ptype, fname, (void *) buf);
+    fprintf(stderr, "real_gretl_print_new: type=%d, fname='%s', buf=%p, fp=%d\n",
+	    ptype, fname, (void *) buf, (void *) fp);
 #endif    
 
     prn->fp = NULL;
     prn->fpaux = NULL;
     prn->buf = NULL;
     prn->bufsize = 0;
+    prn->blen = 0;
     prn->savepos = -1;
     prn->format = GRETL_FORMAT_TXT;
     prn->fixed = 0;
     prn->delim = ',';
     prn->fname = NULL;
 
-    if (ptype == GRETL_PRINT_FILE) {
+    if (ptype == GRETL_PRINT_STREAM) {
+	prn->fp = fp;
+    } else if (ptype == GRETL_PRINT_FILE) {
 	prn->fp = gretl_fopen(fname, "w");
 	if (prn->fp == NULL) {
-	    *err = E_FOPEN;
+	    err = E_FOPEN;
 	    free(prn);
 	    prn = NULL;
 	} 
     } else if (ptype == GRETL_PRINT_TEMPFILE) {
-	*err = prn_add_tempfile(prn);
-	if (*err) {
+	err = prn_add_tempfile(prn);
+	if (err) {
 	    free(prn);
 	    prn = NULL;
 	} else {
@@ -175,7 +208,7 @@ static PRN *real_gretl_print_new (PrnType ptype,
 	    int p = pprintf(prn, "@init");
 
 	    if (p < 0) {
-		*err = E_ALLOC;
+		err = E_ALLOC;
 		free(prn);
 		prn = NULL;
 	    }
@@ -185,6 +218,10 @@ static PRN *real_gretl_print_new (PrnType ptype,
 	}
     }
 
+    if (perr != NULL) {
+	*perr = err;
+    }
+
     return prn;
 }
 
@@ -209,20 +246,16 @@ static PRN *real_gretl_print_new (PrnType ptype,
 
 PRN *gretl_print_new (PrnType ptype, int *err)
 {
-    int myerr, *errp;
-
     if (ptype == GRETL_PRINT_FILE) {
 	fprintf(stderr, "gretl_print_new: needs a filename\n");
 	return NULL;
     }
 
-    errp = (err == NULL)? &myerr : err;
-
 #if PRN_DEBUG
     fprintf(stderr, "gretl_print_new() called, type = %d\n", ptype);
 #endif
 
-    return real_gretl_print_new(ptype, NULL, NULL, errp);
+    return real_gretl_print_new(ptype, NULL, NULL, NULL, err);
 }
 
 /**
@@ -243,7 +276,7 @@ PRN *gretl_print_new_with_filename (const char *fname, int *err)
 	return NULL;
     }
 
-    return real_gretl_print_new(GRETL_PRINT_FILE, fname, NULL, err);
+    return real_gretl_print_new(GRETL_PRINT_FILE, fname, NULL, NULL, err);
 }
 
 /**
@@ -259,7 +292,7 @@ PRN *gretl_print_new_with_filename (const char *fname, int *err)
 
 PRN *gretl_print_new_with_tempfile (int *err)
 {
-    return real_gretl_print_new(GRETL_PRINT_TEMPFILE, NULL, NULL, err);
+    return real_gretl_print_new(GRETL_PRINT_TEMPFILE, NULL, NULL, NULL, err);
 }
 
 /**
@@ -274,6 +307,23 @@ int gretl_print_has_tempfile (PRN *prn)
     return (prn != NULL && prn->fname != NULL && prn->fp != NULL);
 }
 
+/**
+ * gretl_print_get_tempfile_name:
+ * @prn: printing struct to test.
+ * 
+ * Returns: if @prn has a tempfile attached, return the name
+ * of that file, otherwise return %NULL.
+ */
+
+const char * gretl_print_get_tempfile_name (PRN *prn)
+{
+    if (prn != NULL) {
+	return prn->fname;
+    } else {
+	return NULL;
+    }
+}
+
 /**
  * gretl_print_new_with_buffer:
  * @buf: pre-allocated text buffer.
@@ -294,20 +344,108 @@ PRN *gretl_print_new_with_buffer (char *buf)
     if (buf == NULL) {
 	return NULL;
     } else {
-	int err;
-
-	return real_gretl_print_new(GRETL_PRINT_BUFFER, NULL, buf, &err);
+	return real_gretl_print_new(GRETL_PRINT_BUFFER, NULL, buf, NULL, NULL);
     }
 }
 
+/**
+ * gretl_print_new_with_stream:
+ * @fp: pre-opened stream.
+ * 
+ * Creates and initializes a gretl printing struct, with
+ * printing to @fp.
+ *
+ * Note that @fp will be closed if and when #gretl_print_destroy
+ * is called on the #PRN pointer obtained.
+ *
+ * Returns: pointer to newly created struct, or %NULL on failure.
+ */
+
+PRN *gretl_print_new_with_stream (FILE *fp)
+{
+    if (fp == NULL) {
+	return NULL;
+    } else {
+	return real_gretl_print_new(GRETL_PRINT_STREAM, NULL, NULL, fp, NULL);
+    }
+} 
+ 
+/**
+ * gretl_print_detach_stream
+ * @prn: printing struct to operate on.
+ * 
+ * Sets the stream member of @prn to %NULL so that @prn can
+ * be destroyed without closing the associated stream.  May be
+ * used in conjunction with gretl_print_new_with_stream().
+ */
+
+void gretl_print_detach_stream (PRN *prn)
+{
+    prn->fp = NULL;
+}
+
+/**
+ * gretl_print_rename_file:
+ * @prn: printing struct to operate on.
+ * @oldpath: name of current file (or %NULL if @prn was
+ * set up using #gretl_print_new_with_tempfile).
+ * @newpath: new name for file.
+ * 
+ * If @prn is printing to a %FILE pointer, rename the
+ * file to which it's printing and switch the stream
+ * to the new file.
+ *
+ * Returns: 0 on success, 1 on error.
+ */
+
+int gretl_print_rename_file (PRN *prn, const char *oldpath,
+			     const char *newpath)
+{
+    int err = 0;
+
+    if (prn == NULL) {
+	fprintf(stderr, "gretl_print_rename_file: prn is NULL\n");
+	return E_DATA;
+    }
+
+    if (prn->fp == NULL || prn->fpaux != NULL) {
+	return E_DATA;
+    }
+
+    fclose(prn->fp);
+
+    if (oldpath == NULL && prn->fname != NULL) {
+	/* renaming from tempfile */
+	err = gretl_rename(prn->fname, newpath);
+    } else {
+	err = gretl_rename(oldpath, newpath);
+    }
+
+    if (!err) {
+	prn->fp = gretl_fopen(newpath, "a");
+	if (prn->fname != NULL) {
+	    /* @prn originally used a tempfile: the record of
+	       the temporary filename should be deleted 
+	    */
+	    free(prn->fname);
+	    prn->fname = NULL;
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_print_reset_buffer:
  * @prn: printing struct to operate on.
  * 
- * Write a NUL byte to the start of the buffer associated
- * with @prn, if any.
+ * If @prn has an attached buffer, write a NUL byte to
+ * the start of the buffer and reset the count of bytes
+ * printed to zero.  The next call to @pprintf or
+ * similar will then overwite rather than cumulating
+ * the printed content.
  *
- * Returns: 0 on success, 1 on error.
+ * Returns: 0 on success, 1 if @prn has no buufer.
  */
 
 int gretl_print_reset_buffer (PRN *prn)
@@ -316,6 +454,7 @@ int gretl_print_reset_buffer (PRN *prn)
 
     if (prn != NULL && prn->buf != NULL) {
 	*prn->buf = '\0';
+	prn->blen = 0;
     } else {
 	err = 1;
     }
@@ -339,6 +478,83 @@ const char *gretl_print_get_buffer (PRN *prn)
     return (prn != NULL)? prn->buf : NULL;
 }
 
+/**
+ * gretl_print_get_trimmed_buffer:
+ * @prn: printing struct.
+ * 
+ * Obtain a pointer to the buffer associated
+ * with @prn, if any.  This pointer must not be
+ * modified in any way.  An opening newline
+ * is skipped.
+ *
+ * Returns: the buffer, or %NULL on failure.
+ */
+
+const char *gretl_print_get_trimmed_buffer (PRN *prn)
+{
+    char *buf = (prn != NULL)? prn->buf : NULL;
+
+    if (buf != NULL) {
+	int i, n;
+
+	if (*buf == '\n') {
+	    buf++;
+	}
+	n = strlen(buf);
+	for (i=n-1; i>0; i--) {
+	    if (buf[i] == '\n' && buf[i-1] == '\n') {
+		buf[i] = '\0';
+	    } else {
+		break;
+	    }
+	}
+    }
+
+    return buf;
+}
+
+/**
+ * gretl_print_get_size:
+ * @prn: printing struct.
+ * @width: location to receive width, or %NULL.
+ * @height: location to receive height, or %NULL.
+ * 
+ * If @prn has a non-null buffer attached, provide
+ * the width and/or height of the buffer, the width in
+ * characters and the height in lines.
+ */
+
+void gretl_print_get_size (PRN *prn, int *width, int *height)
+{
+    int w = 0, h = 0;
+
+    if (prn != NULL && prn->buf != NULL) {
+	const char *s = prn->buf;
+	int lw = 0;
+
+	while (*s) {
+	    if (*s == '\n') {
+		h++;
+		if (lw > w) {
+		    w = lw;
+		}
+		lw = 0;
+	    } else {
+		lw++;
+	    }
+	    s++;
+	}
+    }
+
+    if (width != NULL) {
+	*width = w;
+    }
+
+    if (height != NULL) {
+	*height = h;
+    }    
+}
+
 /**
  * gretl_print_steal_buffer:
  * @prn: printing struct.
@@ -395,6 +611,7 @@ FILE *gretl_print_read_tempfile (PRN *prn)
 /**
  * gretl_print_stop_tempfile_read:
  * @prn: printing struct.
+ * @fp: auxiliary stream.
  * 
  * For @prn with tempfile attached, stops reading
  * of the tempfile and closes @fp (recording the
@@ -514,7 +731,7 @@ void gretl_print_set_delim (PRN *prn, char delim)
 }
 
 /**
- * gretl_print_unset_doc_format:
+ * gretl_print_toggle_doc_flag:
  * @prn: printing struct.
  * 
  * Toggles the %GRETL_FORMAT_DOC flag on @prn.
@@ -527,22 +744,54 @@ void gretl_print_toggle_doc_flag (PRN *prn)
     }
 }
 
-static int realloc_prn_buffer (PRN *prn, size_t blen)
+/**
+ * gretl_print_set_utf_flag:
+ * @prn: printing struct.
+ * 
+ * Sets the %GRETL_FORMAT_UTF flag on @prn.
+ */
+
+void gretl_print_set_utf_flag (PRN *prn)
 {
+    if (prn != NULL) {
+	prn->format |= GRETL_FORMAT_UTF;
+    }
+}
+
+/**
+ * gretl_print_supports_utf:
+ * @prn: printing struct.
+ * 
+ * Returns: 1 if @prn supports UTF-8 characters, else 0.
+ */
+
+int gretl_print_supports_utf (PRN *prn)
+{
+    if (prn != NULL) {
+	return (prn->format & GRETL_FORMAT_UTF)? 1 : 0;
+    } else {
+	return 0;
+    }
+}
+
+static int realloc_prn_buffer (PRN *prn)
+{
+    size_t newlen;
     char *tmp;
     int err = 0;
 
 #if PRN_DEBUG
     fprintf(stderr, "%d bytes left\ndoing realloc(%p, %d)\n",
-	    prn->bufsize - blen, prn->buf, 2 * prn->bufsize);
+	    prn->bufsize - prn->blen, prn->buf, 2 * prn->bufsize);
 #endif
-    prn->bufsize *= 2; 
 
-    tmp = realloc(prn->buf, prn->bufsize); 
+    newlen = prn->bufsize * 2;
+    tmp = realloc(prn->buf, newlen); 
 
     if (tmp == NULL) {
 	err = 1;
     } else {
+	prn->bufsize = newlen;
 	prn->buf = tmp;
 #if PRN_DEBUG
 	fprintf(stderr, "realloc: prn->buf is %d bytes at %p\n",
@@ -550,15 +799,37 @@ static int realloc_prn_buffer (PRN *prn, size_t blen)
 #endif
     }
 
-    memset(prn->buf + blen, 0, 1);
+    memset(prn->buf + prn->blen, 0, 1);
 
     return err;
 }
 
+static int pprintf_init (PRN *prn)
+{
+    int ret = 0;
+
+    prn->bufsize = 2048;
+    prn->buf = malloc(prn->bufsize);
+
+#if PRN_DEBUG
+    fprintf(stderr, "pprintf: malloc'd %d bytes at %p\n", prn->bufsize,  
+	    (void *) prn->buf); 
+#endif
+
+    if (prn->buf == NULL) {
+	ret = -1;
+    } else {
+	memset(prn->buf, 0, 1);
+	prn->blen = 0;
+    }
+
+    return ret;
+}
+
 /**
  * pprintf:
  * @prn: gretl printing struct.
- * @template: as in printf().
+ * @format: as in printf().
  * @Varargs: arguments to be printed.
  *
  * Multi-purpose printing function: can output to stream, to buffer
@@ -570,10 +841,9 @@ static int realloc_prn_buffer (PRN *prn, size_t blen)
  * failure.
  */
 
-int pprintf (PRN *prn, const char *template, ...)
+int pprintf (PRN *prn, const char *format, ...)
 {
     va_list args;
-    size_t blen;
     int rem, plen = 0;
 
     if (prn == NULL || prn->fixed) {
@@ -581,52 +851,42 @@ int pprintf (PRN *prn, const char *template, ...)
     }
 
     if (prn->fp != NULL) {
-	va_start(args, template);
-	plen = vfprintf(prn->fp, template, args);
+	/* printing to stream: straightforward */
+	va_start(args, format);
+	plen = vfprintf(prn->fp, format, args);
 	va_end(args);
 	return plen;
     }
 
-    if (strncmp(template, "@init", 5) == 0) {
-	prn->bufsize = 2048;
-	prn->buf = malloc(prn->bufsize);
-#if PRN_DEBUG
-  	fprintf(stderr, "pprintf: malloc'd %d bytes at %p\n", prn->bufsize,  
-		(void *) prn->buf); 
-#endif
-	if (prn->buf == NULL) {
-	    return -1;
-	}
-	memset(prn->buf, 0, 1);
-	return 0;
+    if (strncmp(format, "@init", 5) == 0) {
+	return pprintf_init(prn);
     }
 
     if (prn->buf == NULL) {
 	return 0;
     }
 
-    blen = strlen(prn->buf);
-
-    if (prn->bufsize - blen < 1024) {
-	if (realloc_prn_buffer(prn, blen)) {
+    if (prn->bufsize - prn->blen < 1024) {
+	if (realloc_prn_buffer(prn)) {
 	    return -1;
 	}
     }
 
-    rem = prn->bufsize - blen - 1;
+#if PRN_DEBUG
+    fprintf(stderr, "printing at %p\n", (void *) (prn->buf + prn->blen));
+#endif
 
-    va_start(args, template);
-#if PRN_DEBUG
-    fprintf(stderr, "printing at %p\n", (void *) (prn->buf + blen));
-#endif
-    plen = vsnprintf(prn->buf + blen, rem, template, args);
+    /* printing to buffer: be careful not to overrun */
+    rem = prn->bufsize - prn->blen - 1;
+    va_start(args, format);
+    plen = vsnprintf(prn->buf + prn->blen, rem, format, args);
     va_end(args);
-#if PRN_DEBUG
-    fprintf(stderr, "printed %d byte(s)\n", strlen(prn->buf) - blen);
-#endif
 
     if (plen >= rem) {
 	fputs("pprintf warning: string was truncated\n", stderr);
+	prn->blen += rem;
+    } else {
+	prn->blen += plen;
     }
 
     return plen;
@@ -643,10 +903,11 @@ int pprintf (PRN *prn, const char *template, ...)
 
 int pputs (PRN *prn, const char *s)
 {
-    size_t blen;
     int slen, bytesleft;
 
-    if (prn == NULL || prn->fixed) return 0;
+    if (prn == NULL || prn->fixed) {
+	return 0;
+    }
 
     slen = strlen(s);
 
@@ -655,17 +916,17 @@ int pputs (PRN *prn, const char *s)
 	return slen;
     }
 
-    if (prn->buf == NULL) return 0;
+    if (prn->buf == NULL) {
+	return 0;
+    }
 
-    blen = strlen(prn->buf);
+    bytesleft = prn->bufsize - prn->blen;
 
-    bytesleft = prn->bufsize - blen;
-
-    while (prn->bufsize - blen < 1024 || bytesleft <= slen) {
-	if (realloc_prn_buffer(prn, blen)) {
+    while (prn->bufsize - prn->blen < 1024 || bytesleft <= slen) {
+	if (realloc_prn_buffer(prn)) {
 	    return -1;
 	}
-	bytesleft = prn->bufsize - blen;
+	bytesleft = prn->bufsize - prn->blen;
     }	
 
 #if PRN_DEBUG
@@ -673,7 +934,8 @@ int pputs (PRN *prn, const char *s)
 	    (int) prn->bufsize, (int) blen, bytesleft, slen);
 #endif
 
-    strcpy(prn->buf + blen, s);
+    strcpy(prn->buf + prn->blen, s);
+    prn->blen += slen;
 
     return slen;
 }
@@ -689,51 +951,36 @@ int pputs (PRN *prn, const char *s)
 
 int pputc (PRN *prn, int c)
 {
-    size_t blen;
-
-    if (prn == NULL || prn->fixed) return 0;
+    if (prn == NULL || prn->fixed) {
+	return 0;
+    }
 
     if (prn->fp != NULL) {
 	fputc(c, prn->fp);
 	return 1;
     }
 
-    if (prn->buf == NULL) return 0;
+    if (prn->buf == NULL) {
+	return 0;
+    }
 
-    blen = strlen(prn->buf);
-
-    if (prn->bufsize - blen < 1024) {
-	if (realloc_prn_buffer(prn, blen)) {
+    if (prn->bufsize - prn->blen < 1024) {
+	if (realloc_prn_buffer(prn)) {
 	    return -1;
 	}
     }
 
 #if PRN_DEBUG
-    fprintf(stderr, "pputc: adding char at %p\n", (void *) (prn->buf + blen));
+    fprintf(stderr, "pputc: adding char at %p\n", (void *) (prn->buf + prn->blen));
 #endif
-    prn->buf[blen] = c;
-    prn->buf[blen + 1] = '\0';
+
+    prn->buf[prn->blen] = c;
+    prn->buf[prn->blen + 1] = '\0';
+    prn->blen += 1;
 
     return 1;
 }
 
-#if 0
-/**
- * pflush:
- * @prn: gretl printing struct.
- * 
- * Returns: the number of bytes printed, or -1 on memory allocation
- * failure.
- */
-
-void pflush (PRN *prn)
-{
-    if (prn != NULL && prn->fp != NULL) {
-	fflush(prn->fp);
-    }
-}
-#endif
-
 /**
  * gretl_prn_newline:
  * @prn: gretl printing struct.
@@ -883,7 +1130,7 @@ int print_end_redirection (PRN *prn)
 
 int plain_format (PRN *prn)
 {
-    return (prn != NULL && prn->format == GRETL_FORMAT_TXT);
+    return (prn != NULL && (prn->format & GRETL_FORMAT_TXT));
 }
 
 /**
@@ -993,7 +1240,66 @@ char prn_delim (PRN *prn)
 }
 
 /**
- * set_up_verbose_printer.
+ * gretl_print_has_buffer:
+ * @prn: gretl printing struct.
+ * 
+ * Returns: 1 if @prn is using a buffer for printing,
+ * otherwise 0 (e.g., if a file is being used).
+ */
+
+int gretl_print_has_buffer (PRN *prn)
+{
+    return (prn != NULL && prn->buf != NULL);
+}
+
+/**
+ * gretl_print_alloc:
+ * @prn: gretl printing struct.
+ * @s: desired size in bytes.
+ *
+ * If @prn is using a buffer for printing, try to
+ * ensure that at least @s bytes are available for
+ * further printing.  May speed things up if we
+ * know there is a large amount of text to be
+ * printed.
+ * 
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int gretl_print_alloc (PRN *prn, size_t s)
+{
+    size_t rem;
+    int err = 0;
+
+    if (prn == NULL || prn->buf == NULL || prn->fixed) {
+	return E_DATA;
+    }
+
+    rem = prn->bufsize - prn->blen;
+
+    if (rem <= s) {
+	size_t newlen = prn->blen + s + 1;
+	char *tmp;
+
+	tmp = realloc(prn->buf, newlen); 
+	if (tmp == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    prn->buf = tmp;
+	    prn->bufsize = newlen;
+	    memset(prn->buf + prn->blen, 0, 1);
+#if PRN_DEBUG
+	    fprintf(stderr, "gretl_print_alloc: realloc'd prn->buf "
+		    "to %d bytes\n", newlen);
+#endif
+	}
+    }
+
+    return err;
+}
+
+/**
+ * set_up_verbose_printer:
  * @opt: %OPT_V for verbosity.
  * @prn: currently active printing struct.
  *
@@ -1005,6 +1311,7 @@ PRN *set_up_verbose_printer (gretlopt opt, PRN *prn)
     PRN *vprn = NULL;
 
     if (opt & OPT_V) {
+	/* verbose printing is wanted */
 	if (iter_print_func_installed()) {
 	    int err;
 
@@ -1017,6 +1324,14 @@ PRN *set_up_verbose_printer (gretlopt opt, PRN *prn)
     return vprn;
 }
 
+/**
+ * close_down_verbose_printer:
+ * @vprn: prn pointer obtained via set_up_verbose_printer().
+ *
+ * Turns off verbose printing and frees the resources 
+ * associated with @vprn.
+ */
+
 void close_down_verbose_printer (PRN *vprn)
 {
     if (vprn != NULL) {
