@@ -27,15 +27,16 @@
 typedef struct _col_table col_table;
 
 struct _col_table {
-    int idx;
-    int n_strs;
-    char **strs;
-    GHashTable *hash;
+    int idx;          /* column index (variable number) */
+    int n_strs;       /* number of strings in table */
+    char **strs;      /* saved strings */
+    GHashTable *hash; /* hash table for quick lookup */
 };
 
 struct _gretl_string_table {
-    int n_cols;
-    col_table **cols;
+    int n_cols;       /* number of columns (variables) included */
+    col_table **cols; /* column tables (see above) */
+    char *extra;      /* extra information */
 };
 
 static void clean_up_codevars (void);
@@ -54,6 +55,13 @@ static col_table *col_table_new (int colnum)
     return ct;
 }
 
+/**
+ * gretl_string_table_new:
+ * @err: location to receive error code.
+ *
+ * Returns: a pointer to a newly allocated string table.
+ */
+
 gretl_string_table *gretl_string_table_new (int *err)
 {
     gretl_string_table *st = malloc(sizeof *st);
@@ -63,19 +71,36 @@ gretl_string_table *gretl_string_table_new (int *err)
     } else {
 	st->cols = NULL;
 	st->n_cols = 0;
+	st->extra = NULL;
     }
 
     return st;
 }
 
+/**
+ * gretl_string_table_new_from_cols_list:
+ * @list: list of 'columns' (variables) whose values are to be 
+ * given a string representation.  
+ * 
+ * These values in @list should correspond to the 0-based indices 
+ * of the variables in question within the dataset.  For example, 
+ * if strings are to be recorded for variables 2, 5 and 10 the
+ * @list argument would be {3, 2, 5, 10}.
+ *
+ * Returns: pointer to a newly allocated string table
+ * containing a column member for each ID in @list.
+ */
+
 gretl_string_table *string_table_new_from_cols_list (int *list)
 {
     gretl_string_table *st;
     int ncols = list[0];
-    int i, j;
+    int i, j, err = 0;
 
-    st = malloc(sizeof *st);
-    if (st == NULL) return NULL;
+    st = gretl_string_table_new(&err);
+    if (st == NULL) {
+	return NULL;
+    }
 
     st->cols = malloc(ncols * sizeof *st->cols);
     if (st->cols == NULL) {
@@ -111,8 +136,7 @@ static int col_table_get_index (const col_table *ct, const char *s)
     return ret;
 }
 
-static int 
-col_table_add_string (col_table *ct, const char *s)
+static int col_table_add_string (col_table *ct, const char *s)
 {
     char **strs;
     int n = ct->n_strs + 1;
@@ -155,6 +179,26 @@ gretl_string_table_add_column (gretl_string_table *st, int colnum)
     return cols[n-1];
 }
 
+/**
+ * gretl_string_table_index:
+ * @st: a gretl string table.
+ * @s: the string to look up or add.
+ * @col: index of the column to be accessed or created.
+ * @addcol: non-zero to indicate that a column should be
+ * added if it's not already present.
+ * @prn: gretl printer (or %NULL).
+ *
+ * This function has two main uses: for lookup in the context of
+ * a completed string table, or for constructing such a
+ * table (with @addcol non-zero).  The returned index reflects 
+ * any additions to the table that may be required (if column
+ * @col does not already exist, or if string @s is not already
+ * stored for column @col).
+ *
+ * Returns: the 1-based index of @s within the column of
+ * @st that has index @col, if available, otherwise 0.
+ */
+
 int 
 gretl_string_table_index (gretl_string_table *st, const char *s, int col,
 			  int addcol, PRN *prn)
@@ -165,7 +209,7 @@ gretl_string_table_index (gretl_string_table *st, const char *s, int col,
     if (st == NULL) return idx;
 
     for (i=0; i<st->n_cols; i++) {
-	if ((st->cols[i])->idx == col) {
+	if (st->cols[i]->idx == col) {
 	    ct = st->cols[i];
 	    break;
 	}
@@ -179,8 +223,7 @@ gretl_string_table_index (gretl_string_table *st, const char *s, int col,
 	ct = gretl_string_table_add_column(st, col);
 	if (ct != NULL) {
 	    pprintf(prn, M_("variable %d: translating from strings to "
-			    "code numbers\n"), 
-		    col);
+			    "code numbers\n"), col);
 	}
     }
 
@@ -191,6 +234,29 @@ gretl_string_table_index (gretl_string_table *st, const char *s, int col,
     return idx;
 }
 
+/* Used in the context of deletion of "empty" variables from
+   an imported dataset: the index of a given "column" in 
+   a string table is adjusted to match the new position of
+   the variable in question. 
+*/
+
+int gretl_string_table_reset_column_id (gretl_string_table *st, 
+					int oldid, int newid)
+{
+    if (st != NULL) {
+	int i;
+
+	for (i=0; i<st->n_cols; i++) {
+	    if (st->cols[i]->idx == oldid) {
+		st->cols[i]->idx = newid;
+		return 0;
+	    }
+	}
+    }
+
+    return E_DATA;
+}
+
 static void col_table_destroy (col_table *ct)
 {
     int i;
@@ -209,6 +275,13 @@ static void col_table_destroy (col_table *ct)
     free(ct);
 }
 
+/**
+ * gretl_string_table_destroy:
+ * @st: gretl string table.
+ *
+ * Frees all resources associated with @st.
+ */
+
 void gretl_string_table_destroy (gretl_string_table *st)
 {
     int i;
@@ -220,23 +293,41 @@ void gretl_string_table_destroy (gretl_string_table *st)
     }
     free(st->cols);
 
+    if (st->extra != NULL) {
+	free(st->extra);
+    }
+
     free(st);
 }
 
+/**
+ * gretl_string_table_print:
+ * @st: gretl string table.
+ * @pdinfo: dataset information (for names of variables).
+ * @fname: name of file to which to print.
+ * @prn: gretl printer (or %NULL).
+ *
+ * Prints table @st to a file called @fname.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
 int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
 			      const char *fname, PRN *prn)
 {
-    int i, j;
     const col_table *ct;
     const char *fshort;
     char stname[MAXLEN];
     FILE *fp;
+    int i, j;
     int err = 0;
 
-    if (st == NULL) return E_DATA;
+    if (st == NULL) {
+	return E_DATA;
+    }
 
     strcpy(stname, "string_table.txt");
-    gretl_path_prepend(stname, gretl_work_dir());
+    gretl_path_prepend(stname, gretl_workdir());
 
     fp = gretl_fopen(stname, "w");
     if (fp == NULL) {
@@ -245,42 +336,60 @@ int gretl_string_table_print (gretl_string_table *st, DATAINFO *pdinfo,
 
     fshort = strrchr(fname, SLASH);
     if (fshort != NULL) {
-	fprintf(fp, "%s\n\n", fshort + 1);
+	fprintf(fp, "%s\n", fshort + 1);
     } else {
-	fprintf(fp, "%s\n\n", fname);
+	fprintf(fp, "%s\n", fname);
     }
 
-    fputs(M_("One or more non-numeric variables were found.\n"
-	     "Gretl cannot handle such variables directly, so they\n"
-	     "have been given numeric codes as follows.\n\n"), fp);
-
+    if (st->n_cols > 0) {
+	fputc('\n', fp);
+	fputs(M_("One or more non-numeric variables were found.\n"
+		 "Gretl cannot handle such variables directly, so they\n"
+		 "have been given numeric codes as follows.\n\n"), fp);
+	if (st->extra != NULL) {
+	    fputs(M_("In addition, some mappings from numerical values to string\n"
+		     "labels were found, and are printed below.\n\n"), fp);
+	}
+    }
+    
     for (i=0; i<st->n_cols; i++) {
 	ct = st->cols[i];
-	if (!err) {
-	    fprintf(fp, M_("String code table for variable %d (%s):\n"), 
-		    ct->idx, pdinfo->varname[ct->idx]);
-	} else {
-	    pprintf(prn, M_("String code table for variable %d (%s):\n"), 
-		    ct->idx, pdinfo->varname[ct->idx]);
-	}
+	fprintf(fp, M_("String code table for variable %d (%s):\n"), 
+		ct->idx, pdinfo->varname[ct->idx]);
 	for (j=0; j<ct->n_strs; j++) {
-	    if (!err) {
-		fprintf(fp, "%3d = '%s'\n", j+1, ct->strs[j]);
-	    } else {
-		pprintf(prn, "%3d = '%s'\n", j+1, ct->strs[j]);
-	    }
+	    fprintf(fp, "%3d = '%s'\n", j+1, ct->strs[j]);
 	}
     }
 
-    if (fp != NULL) {
-	pprintf(prn, M_("String code table written to\n %s\n"), stname);
-	fclose(fp);
-	set_string_table_written();
+    if (st->extra != NULL) {
+	fputs(st->extra, fp);
     }
 
+    pprintf(prn, M_("String code table written to\n %s\n"), stname);
+
+    fclose(fp);
+    set_string_table_written();
+
     return err;
 }
 
+/**
+ * gretl_string_table_add_extra:
+ * @st: gretl string table.
+ * @prn: gretl printer.
+ *
+ * Steals the printing buffer from @prn and adds it to @st.
+ * The buffer will be appended when @st is printed via
+ * gretl_string_table_print().
+ */
+
+void gretl_string_table_add_extra (gretl_string_table *st, PRN *prn)
+{
+    if (st != NULL && prn != NULL) {
+	st->extra = gretl_print_steal_buffer(prn);
+    }
+}
+
 /* below: saving of user-defined strings */
 
 typedef struct saved_string_ saved_string;
@@ -304,7 +413,9 @@ static saved_string built_ins[] = {
     { "tramo",    0, NULL },
     { "tramodir", 0, NULL },
     { "seats",    0, NULL },
-    { "shelldir", 0, NULL }
+    { "shelldir", 0, NULL },
+    { "Rbin",     0, NULL },
+    { "Rlib",     0, NULL },
 };
 
 #ifdef WIN32
@@ -313,6 +424,15 @@ static saved_string dsep = { "dirsep", 0, "\\" };
 static saved_string dsep = { "dirsep", 0, "/" };
 #endif
 
+/**
+ * gretl_insert_builtin_string:
+ * @name: the name of the string to be added or replaced.
+ * @s: the value for this string variable.
+ *
+ * Inserts value @s for string @name in gretl's table
+ * of built-in string variables.
+ */
+
 void gretl_insert_builtin_string (const char *name, const char *s)
 {
     int n = sizeof built_ins / sizeof built_ins[0];
@@ -375,7 +495,13 @@ static saved_string *get_saved_string_by_name (const char *name,
     return NULL;
 }
 
-/* access a saved string by name */
+/**
+ * get_string_by_name:
+ * @name: the name of the string variable to access.
+ *
+ * Returns: the value of string variable @name, or %NULL
+ * if there is no such variable.
+ */
 
 char *get_string_by_name (const char *name)
 {
@@ -424,6 +550,17 @@ static saved_string *add_named_string (const char *name)
     return &S[n];
 }
 
+/**
+ * add_string_as:
+ * @s: string value to be added.
+ * @name: the name of the string variable to add.
+ *
+ * Adds @s to the saved array of string variables 
+ * under the name @name.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
 int add_string_as (const char *s, const char *name)
 {
     int n = n_saved_strings;
@@ -486,6 +623,17 @@ static int destroy_saved_string (saved_string *S)
     return err;
 }
 
+/**
+ * delete_saved_string:
+ * @name: the name of the string variable to delete.
+ * @prn: gretl printer (or %NULL).
+ *
+ * Deletes the string variable called @name.
+ *
+ * Returns: 0 on success, non-zero on failure (e.g.
+ * attempting to delete a built-in string variable).
+ */
+
 int delete_saved_string (const char *name, PRN *prn)
 {
     saved_string *S;
@@ -495,7 +643,7 @@ int delete_saved_string (const char *name, PRN *prn)
     if (S == NULL) {
 	err = E_UNKVAR;
     } else if (builtin) {
-	sprintf(gretl_errmsg, _("You cannot delete '%s'"), name);
+	gretl_errmsg_sprintf(_("You cannot delete '%s'"), name);
 	err = E_DATA;
     } else {
 	err = destroy_saved_string(S);
@@ -608,7 +756,7 @@ static int shell_grab (const char *arg, char **sout)
     }
 
     if (!libset_get_bool(SHELL_OK)) {
-	strcpy(gretl_errmsg, _("The shell command is not activated."));
+	gretl_errmsg_set(_("The shell command is not activated."));
 	return 1;
     }
 
@@ -644,10 +792,9 @@ char *gretl_backtick (const char *arg, int *err)
 
 char *gretl_getenv (const char *key, int *err)
 {
-    char *test = NULL;
+    char *test = getenv(key);
     char *val = NULL;
 
-    test = getenv(key);
     if (test == NULL) {
 	val = gretl_strdup("");
     } else {
@@ -678,18 +825,24 @@ static char *strim (char *s)
 
 char *retrieve_date_string (int t, const DATAINFO *pdinfo, int *err)
 {
-    char datestr[OBSLEN] = {0};
     char *ret = NULL;
 
-    if (t > 0 && t <= pdinfo->n) {
+    if (t <= 0 || t > pdinfo->n) {
+	*err = E_DATA;
+    } else if (pdinfo->S != NULL) {
+	ret = gretl_strdup(pdinfo->S[t-1]);
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}	
+    } else {
+	char datestr[OBSLEN] = {0};
+
 	ntodate(datestr, t - 1, pdinfo);
 	ret = gretl_strdup(datestr);
 	if (ret == NULL) {
 	    *err = E_ALLOC;
 	}
-    } else {
-	*err = E_DATA;
-    }
+    } 
 
     return ret;
 }
@@ -704,11 +857,10 @@ char *retrieve_file_content (const char *fname, int *err)
 	char fullname[FILENAME_MAX];
 	GError *gerr = NULL;
 	gsize len = 0;
-	
 
 	*fullname = '\0';
 	strncat(fullname, fname, FILENAME_MAX - 1);
-	addpath(fullname, NULL, 0);
+	addpath(fullname, 0);
 
 	g_file_get_contents(fullname, &ret, &len, &gerr);
 
@@ -767,7 +919,7 @@ int save_named_string (const char *name, const char *s, PRN *prn)
 
     if (str != NULL && builtin) {
 	if (prn == NULL) {
-	    sprintf(gretl_errmsg, _("You cannot overwrite '%s'\n"), name);
+	    gretl_errmsg_sprintf(_("You cannot overwrite '%s'\n"), name);
 	} else {
 	    pprintf(prn, _("You cannot overwrite '%s'\n"), name);
 	}
@@ -790,7 +942,8 @@ int save_named_string (const char *name, const char *s, PRN *prn)
 	err = E_ALLOC;
     }
 
-    if (prn != NULL && !err && gretl_messages_on() && *s != '\0') {
+    if (prn != NULL && !err && gretl_messages_on() && 
+	!gretl_looping_quietly() && *s != '\0') {
 	if (add) {
 	    pprintf(prn, _("Generated string %s\n"), name); 
 	} else {
@@ -871,13 +1024,13 @@ static char *maybe_get_subst (char *name, int *n, int quoted,
 
 static void too_long (void)
 {
-    sprintf(gretl_errmsg, _("Maximum length of command line "
-			    "(%d bytes) exceeded\n"), MAXLINE);
+    gretl_errmsg_sprintf(_("Maximum length of command line "
+			   "(%d bytes) exceeded\n"), MAXLINE);
 }
 
 #define var_context(s,i) (i > 8 && !strncmp(s - 9, "isstring(", 9))
 
-int substitute_named_strings (char *line)
+int substitute_named_strings (char *line, int *subst)
 {
     char sname[VNAMELEN];
     int len = strlen(line);
@@ -890,14 +1043,8 @@ int substitute_named_strings (char *line)
 	return 0;
     }
 
-    /* when using the genr apparatus for a string, let @foo be handled 
-       as a variable (FIXME?) */
-    if (!strncmp(line, "string ", 7)) {
-	return 0;
-    }
-
     if (!strncmp(line, "sscanf", 6)) {
-	/* when scanning, let @foo be handled as a variable */
+	/* when scanning, let @foo be handled as a variable (FIXME?) */
 	return 0;
     }    
 
@@ -927,7 +1074,7 @@ int substitute_named_strings (char *line)
 	    }
 	}
 	if (*s == '@' && !var_context(s, i)) {
-	    n = gretl_varchar_spn(s + 1);
+	    n = gretl_namechar_spn(s + 1);
 	    if (n > 0) {
 		if (n >= VNAMELEN) {
 		    n = VNAMELEN - 1;
@@ -953,6 +1100,7 @@ int substitute_named_strings (char *line)
 			len += m - (n + 1);
 			s += m - 1;
 			i += m - 1;
+			*subst = 1;
 		    }
 		    if (freeit) {
 			free(sub);
