@@ -25,13 +25,17 @@
 #include "libset.h"
 #include "usermat.h"
 #include "gretl_panel.h"
-#include "loop_private.h"
-#include "texprint.h"
+#include "gretl_scalar.h"
+#include "gretl_bundle.h"
 #include "gretl_string_table.h"
 #include "forecast.h"
+#include "kalman.h"
+
+#ifdef USE_RLIB
+# include "gretl_foreign.h"
+#endif
 
 #include <errno.h>
-#include <time.h>
 
 #ifndef WIN32
 # include <signal.h>
@@ -105,41 +109,29 @@ int ijton (int i, int j, int nrows)
 }
 
 /**
- * ztox:
- * @i: index number of variable to extract.
- * @px: array into which to write the series.
- * @Z: data matrix.
+ * transcribe_array:
+ * @targ: arrat to which to write.
+ * @src: array from which to read.
  * @pdinfo: data information struct.
  * 
- * Pull one series from data matrix and put it into @px.
+ * Copy from @src to @targ, skipping any missing values,
+ * over the sample range defined in @pdinfo.
  *
- * Returns: the number of valid observations put into @px.
+ * Returns: the number of valid observations put into @targ.
  */
 
-int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo) 
+int transcribe_array (double *targ, const double *src, 
+		      const DATAINFO *pdinfo) 
 {
-    int t, m = 0;
-    double xx;
+    int t, n = 0;
 
-    if (var_is_scalar(pdinfo, i)) {
-	px[0] = Z[i][0];
-	return 1;
-    }
-    
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	xx = Z[i][t];
-	if (na(xx)) {
-	    continue;
+	if (!na(src[t])) {
+	    targ[n++] = src[t];
 	}
-	else px[m++] = xx;
     }
 
-    if (m == 0) {
-	fprintf(stderr, "\nztox: No valid observations for variable %s\n", 
-		pdinfo->varname[i]);
-    } 
-
-    return m;
+    return n;
 }
 
 /**
@@ -148,7 +140,7 @@ int ztox (int i, double *px, const double **Z, const DATAINFO *pdinfo)
  * @t2: ending observation. 
  * @x: data series to examine.
  * 
- * Check whether variable @x has only 0 or 1 values over the
+ * Check whether series @x has only 0 or 1 values over the
  * given sample range (or possibly missing values).
  *
  * Returns: 0 if the variable is not a 0/1 dummy, otherwise the
@@ -185,7 +177,7 @@ int gretl_isdummy (int t1, int t2, const double *x)
  * @t1: starting observation.
  * @t2: ending observation. 
  * 
- * Check whether variable @x has only zero values over the
+ * Check whether series @x has only zero values over the
  * given sample range (or possibly missing values).
  *
  * Returns: 1 if the variable is all zeros, otherwise 0.
@@ -211,7 +203,7 @@ int gretl_iszero (int t1, int t2, const double *x)
  * @t1: starting observation.
  * @t2: ending observation. 
  * 
- * Check whether variable @x is constant over the
+ * Check whether series @x is constant over the
  * given sample range (aside from any missing values).
  *
  * Returns: 1 if the variable is constant, otherwise 0.
@@ -244,7 +236,7 @@ int gretl_isconst (int t1, int t2, const double *x)
  * @t1: starting observation.
  * @t2: ending observation. 
  * 
- * Check whether variable @x equals 1 over the
+ * Check whether series @x equals 1 over the
  * given sample range (aside from any missing values).
  *
  * Returns: 1 if so, otherwise 0.
@@ -270,8 +262,8 @@ int gretl_isunits (int t1, int t2, const double *x)
  * @t1: starting observation.
  * @t2: ending observation. 
  * 
- * Check whether variable @x contains only integer values over the
- * given sample range (aside from any missing values).
+ * Check whether series @x contains only integer values over
+ * the given sample range (aside from any missing values).
  *
  * Returns: 1 if so, otherwise 0.
  */
@@ -290,34 +282,81 @@ int gretl_isint (int t1, int t2, const double *x)
     return ret;
 }
 
+/**
+ * gretl_iscount:
+ * @x: data series to examine.
+ * @t1: starting observation.
+ * @t2: ending observation. 
+ * 
+ * Check whether series @x contains nothing but non-negative
+ * integer values (some of which are > 1) over the 
+ * given sample range.
+ *
+ * Returns: 1 if so, otherwise 0.
+ */
+
+int gretl_iscount (int t1, int t2, const double *x)
+{
+    int t, xi;
+    int g1 = 0;
+    int ret = 1;
+
+    for (t=t1; t<=t2; t++) {
+	if (na(x[t])) {
+	    continue;
+	}
+	if (x[t] < 0.0) {
+	    ret = 0;
+	    break;
+	}
+	xi = x[t];
+	if (x[t] != (double) xi) {
+	    ret = 0;
+	    break;
+	}
+	if (x[t] > 1.0) {
+	    g1 = 1;
+	}
+    }
+
+    if (g1 == 0) {
+	ret = 0;
+    }
+
+    return ret;
+}
+
 #define FEWVALS 32
 
-static int few_vals (int t1, int t2, const double *x)
+static int few_vals (int t1, int t2, const double *x, double *ratio)
 {
     double test[FEWVALS];
     int match;
-    int i, t, n = 0;
+    int i, t, n = 0, nv = 0;
 
     for (t=t1; t<=t2; t++) {
 	if (!na(x[t])) {
 	    match = 0;
-	    for (i=0; i<n; i++) {
+	    for (i=0; i<nv; i++) {
 		if (x[t] == test[i]) {
 		    match = 1;
 		    break;
 		}
 	    }
 	    if (!match) {
-		if (n == FEWVALS) {
-		    n++;
+		if (nv == FEWVALS) {
+		    nv++;
 		    break;
 		}
-		test[n++] = x[t];
+		test[nv++] = x[t];
 	    }
+	    n++;
 	}
     }
 
-    return n;
+    *ratio = (double) nv / n;
+
+    return nv;
 }
 
  /**
@@ -367,9 +406,15 @@ int gretl_isdiscrete (int t1, int t2, const double *x)
     }
 
     if (d) {
-	n = few_vals(t1, t2, x);
+	n = few_vals(t1, t2, x, &r);
 	if (n > FEWVALS) {
 	    d = 0;
+	} else if (r > 0.9) {
+	    /* somewhat arbitrary: but if r (= ratio of distinct
+	       values to number of cases) is "too high", perhaps we
+	       should not take the var as discrete
+	    */
+	    d = 0;
 	} else if (n < 5) {
 	    d = 2;
 	}
@@ -417,7 +462,7 @@ int gretl_is_oprobit_ok (int t1, int t2, const double *x)
  * @pdinfo: dataset information. 
  * 
  * Check whether variable Z[v] equals 1 over the sample
- * range given in @pdinfo, (aside from any missing values).
+ * range given in @pdinfo (aside from any missing values).
  *
  * Returns: 1 if so, otherwise 0.
  */
@@ -436,10 +481,10 @@ int true_const (int v, const double **Z, const DATAINFO *pdinfo)
  * @a: pointer to first element to compare.
  * @b: pointer to second element to compare.
  *
- * Comparison function for use with %qsort.  Sorts doubles in
+ * Comparison function for use with qsort.  Sorts doubles in
  * ascending order.
  * 
- * Returns: appropriate value for %qsort.
+ * Returns: appropriate value for qsort.
  */
 
 int gretl_compare_doubles (const void *a, const void *b)
@@ -455,10 +500,10 @@ int gretl_compare_doubles (const void *a, const void *b)
  * @a: pointer to first element to compare.
  * @b: pointer to second element to compare.
  * 
- * Comparison function for use with %qsort.  Sorts doubles in
+ * Comparison function for use with qsort.  Sorts doubles in
  * descending order.
  * 
- * Returns: appropriate value for %qsort.
+ * Returns: appropriate value for qsort.
  */
 
 int gretl_inverse_compare_doubles (const void *a, const void *b)
@@ -474,10 +519,10 @@ int gretl_inverse_compare_doubles (const void *a, const void *b)
  * @a: pointer to first element to compare.
  * @b: pointer to second element to compare.
  *
- * Comparison function for use with %qsort.  Sorts integers in
+ * Comparison function for use with qsort.  Sorts integers in
  * ascending order.
  * 
- * Returns: appropriate value for %qsort.
+ * Returns: appropriate value for qsort.
  */
 
 int gretl_compare_ints (const void *a, const void *b)
@@ -579,11 +624,15 @@ void printlist (const int *list, const char *msg)
     }
 
     if (list == NULL) {
-	fputs( "list is NULL", stderr);
+	fputs("list is NULL", stderr);
     } else {
 	fprintf(stderr, "%d : ", list[0]);
 	for (i=1; i<=list[0]; i++) {
-	    fprintf(stderr, "%d ", list[i]);
+	    if (list[i] == LISTSEP) {
+		fputs("; ", stderr);
+	    } else {
+		fprintf(stderr, "%d ", list[i]);
+	    }
 	}
     }
 
@@ -673,37 +722,6 @@ int ls_criteria (MODEL *pmod)
     return err;
 }
 
-/**
- * gretl_print_criteria:
- * @ess: error sum of squares.
- * @nobs: number of observations.
- * @ncoeff: number of parameters estimated.
- * @prn: printing struct.
- *
- * Prints the values of the model selection criteria AIC, 
- * BIC and HQC for the given parameters.
- *
- * Returns: 0 on success, non-zero on error.
- */
-
-int gretl_print_criteria (double ess, int nobs, int ncoeff, PRN *prn)
-{
-    double ll, aic, bic, hqc;
-    int err;
-
-    err = gretl_calculate_criteria(ess, nobs, ncoeff, &ll, &aic, &bic, &hqc);
-
-    if (err) {
-	pputs(prn, _("Error calculating model selection criteria\n"));
-    } else {
-	pprintf(prn, _("Using ess = %g, %d observations, %d coefficients\n"), 
-		ess, nobs, ncoeff);
-	pprintf(prn, "\nAIC = %g\nBIC = %g\nHQC = %g\n\n", aic, bic, hqc);
-    }
-
-    return err;
-}
-
 static char *
 real_format_obs (char *obs, int maj, int min, int pd, char sep)
 {
@@ -789,32 +807,31 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
 
     if (pd == 1) {
 	if (min > 0) {
-	    strcpy(gretl_errmsg, _("no ':' allowed in starting obs with "
-				   "frequency 1"));
+	    gretl_errmsg_set(_("no ':' allowed in starting obs with "
+			       "frequency 1"));
 	    err = 1;
 	} else if (opt == OPT_S || opt == OPT_C) {
-	    strcpy(gretl_errmsg, _("panel data must have frequency > 1"));
+	    gretl_errmsg_set(_("panel data must have frequency > 1"));
 	    err = 1;
 	}
     } else {
 	if (min == 0) {
-	    strcpy(gretl_errmsg, _("starting obs must contain a ':' with "
-				   "frequency > 1"));
+	    gretl_errmsg_set(_("starting obs must contain a ':' with "
+			       "frequency > 1"));
 	    err = 1;
 	} else if (min > pd) {
-	    sprintf(gretl_errmsg, 
-		    _("starting obs '%s' is incompatible with frequency"), 
-		    stobs);
+	    gretl_errmsg_sprintf(_("starting obs '%s' is incompatible with frequency"), 
+				 stobs);
 	    err = 1;
 	} else if (opt == OPT_X) {
-	    strcpy(gretl_errmsg, _("cross-sectional data: frequency must be 1"));
+	    gretl_errmsg_set(_("cross-sectional data: frequency must be 1"));
 	    err = 1;
 	} else if (n % pd != 0) {
 	    if (opt == OPT_S || opt == OPT_C) {
-		sprintf(gretl_errmsg, _("Panel datasets must be balanced.\n"
-					"The number of observations (%d) is not a multiple\n"
-					"of the number of %s (%d)."), 
-			n, ((opt == OPT_S)? _("periods") : _("units")), pd);
+		gretl_errmsg_sprintf(_("Panel datasets must be balanced.\n"
+				       "The number of observations (%d) is not a multiple\n"
+				       "of the number of %s (%d)."), 
+				     n, ((opt == OPT_S)? _("periods") : _("units")), pd);
 		err = 1;
 	    }
 	}
@@ -828,7 +845,7 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
 /**
  * set_obs:
  * @line: command line.
- * @Z: data array.
+ * @pZ: pointer to data array.
  * @pdinfo: data information struct.
  * @opt: %OPT_S for stacked time-series, %OPT_C for stacked cross-section,
  * %OPT_T for time series, %OPT_X for cross section, %OPT_P to set
@@ -840,20 +857,29 @@ catch_setobs_errors (const char *stobs, int pd, int n, int min, gretlopt opt)
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int set_obs (const char *line, double **Z, DATAINFO *pdinfo, 
+int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo, 
 	     gretlopt opt)
 {
+    char pdstr[VNAMELEN];
     char stobs[OBSLEN];
     int structure = STRUCTURE_UNKNOWN;
     double sd0 = pdinfo->sd0;
     int pd, dated = 0;
     int err = 0;
 
+    if (pZ == NULL || pdinfo == NULL) {
+	return E_NODATA;
+    }
+
+    if ((opt & (OPT_R | OPT_P)) && *pZ == NULL) {
+	return E_NODATA;
+    }
+
     gretl_error_clear();
 
     if (opt & OPT_R) {
 	/* restructure panel: "hidden" option */
-	return switch_panel_orientation(Z, pdinfo);
+	return switch_panel_orientation(*pZ, pdinfo);
     }    
 
     if (!strcmp(line, "setobs")) {
@@ -862,16 +888,21 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
     }
 
     if (opt & OPT_P) {
-	return set_panel_structure_from_line(line, Z, pdinfo);
+	return set_panel_structure_from_line(line, *pZ, pdinfo);
     }
 
     /* now we get down to business */
 
-    if (sscanf(line, "%*s %d %10s", &pd, stobs) != 2) {
-	strcpy(gretl_errmsg, _("Failed to parse line as frequency, startobs"));
+    if (sscanf(line, "%*s %15s %10s", pdstr, stobs) != 2) {
+	gretl_errmsg_set(_("Failed to parse line as frequency, startobs"));
 	return 1;
     }
 
+    pd = gretl_int_from_string(pdstr, &err);
+    if (err) {
+	return err;
+    }
+
     /* truncate stobs if not a calendar date */
     if (strchr(stobs, '/') != NULL) {
 	dated = 1;
@@ -881,8 +912,7 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
 
     /* does frequency make sense? */
     if (pd < 1 || (pdinfo->n > 0 && pd > pdinfo->n && opt != OPT_T)) {
-	sprintf(gretl_errmsg, 
-		_("frequency (%d) does not make seem to make sense"), pd);
+	gretl_errmsg_sprintf(_("frequency (%d) does not make seem to make sense"), pd);
 	return 1;
     }
 
@@ -901,7 +931,7 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
 
     if (dated) {
 	if (opt == OPT_X || opt == OPT_S || opt == OPT_C) {
-	    sprintf(gretl_errmsg, _("starting obs '%s' is invalid"), stobs);
+	    gretl_errmsg_sprintf(_("starting obs '%s' is invalid"), stobs);
 	    return 1;
 	}
 
@@ -910,7 +940,7 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
 	    double ed0 = get_epoch_day(stobs);
 
 	    if (ed0 < 0) {
-		sprintf(gretl_errmsg, _("starting obs '%s' is invalid"), stobs);
+		gretl_errmsg_sprintf(_("starting obs '%s' is invalid"), stobs);
 		return 1;
 	    }
 
@@ -920,7 +950,7 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
 	    /* replace any existing markers with date strings */
 	    dataset_destroy_obs_markers(pdinfo);
 	} else {
-	    sprintf(gretl_errmsg, _("starting obs '%s' is invalid"), stobs);
+	    gretl_errmsg_sprintf(_("starting obs '%s' is invalid"), stobs);
 	    return 1;
 	}
     } else if (structure == TIME_SERIES && pd == 10) {
@@ -930,7 +960,7 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
 	int maj = 0, min = 0;
 
 	if (get_stobs_maj_min(stobs, &maj, &min)) {
-	    sprintf(gretl_errmsg, _("starting obs '%s' is invalid"), stobs);
+	    gretl_errmsg_sprintf(_("starting obs '%s' is invalid"), stobs);
 	    return 1;
 	}
 
@@ -976,14 +1006,18 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
 	dataset_destroy_panel_info(pdinfo);
     }
 
-    ntodate_full(pdinfo->stobs, 0, pdinfo); 
-    ntodate_full(pdinfo->endobs, pdinfo->n - 1, pdinfo);
+    ntodate(pdinfo->stobs, 0, pdinfo); 
+    ntodate(pdinfo->endobs, pdinfo->n - 1, pdinfo);
 
     /* pre-process stacked cross-sectional panels: put into canonical
        stacked time series form
     */
     if (pdinfo->structure == STACKED_CROSS_SECTION) {
-	err = switch_panel_orientation(Z, pdinfo);
+	if (*pZ == NULL) {
+	    err = E_NODATA;
+	} else {
+	    err = switch_panel_orientation(*pZ, pdinfo);
+	}
     }	
 
     if (!err && pdinfo->structure == STACKED_TIME_SERIES) {
@@ -993,11 +1027,56 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
     return err;
 }
 
+/**
+ * gretl_double_from_string:
+ * @s: string to examine.
+ * @err: location to receive error code.
+ * 
+ * If @s is a valid string representation of a double,
+ * return that integer, otherwise if @s is the name of a
+ * scalar variable, return the value of that variable,
+ * otherwise set the content of @err to a non-zero value.
+ *
+ * Returns: double value.
+ */
+
+double gretl_double_from_string (const char *s, int *err)
+{
+    char *test;
+    double x;
+
+    if (s == NULL || *s == 0) {
+	*err = E_DATA;
+	return NADBL;
+    }
+
+    errno = 0;
+
+    x = strtod(s, &test);
+
+    if (errno == ERANGE) {
+	*err = E_DATA;
+	errno = 0;
+	return NADBL;
+    }
+
+    if (*test == '\0') {
+	return x;
+    }
+
+    if (gretl_is_scalar(s)) {
+	x = gretl_scalar_get_value(s);
+    } else {
+	*err = E_DATA;
+	x = NADBL;
+    } 
+
+    return x;    
+}
+
 /**
  * gretl_int_from_string:
  * @s: string to examine.
- * @Z: data array.
- * @pdinfo: data information struct.
  * @err: location to receive error code.
  * 
  * If @s is a valid string representation of an integer,
@@ -1008,10 +1087,10 @@ int set_obs (const char *line, double **Z, DATAINFO *pdinfo,
  * Returns: integer value.
  */
 
-int gretl_int_from_string (const char *s, const double **Z, 
-			   const DATAINFO *pdinfo, int *err)
+int gretl_int_from_string (const char *s, int *err)
 {
     char *test;
+    double x;
     int n = 0;
 
     if (s == NULL || *s == 0) {
@@ -1031,27 +1110,18 @@ int gretl_int_from_string (const char *s, const double **Z,
 
     if (*test == '\0') {
 	return n;
-    } else if (Z == NULL || pdinfo == NULL) {
-	*err = E_DATA;
-    } else if (test[1] == '\0' && is_active_index_loop_char(*test)) {
-	n = loop_scalar_read(*s);
-    } else {
-	int v = varindex(pdinfo, s);
-	double x;
-
-	if (v >= pdinfo->v) {
-	    *err = E_UNKVAR;
-	} else if (var_is_series(pdinfo, v)) {
-	    *err = E_TYPES;
+    } else if (gretl_is_scalar(s)) {
+	x = gretl_scalar_get_value(s);
+	if (na(x)) {
+	    *err = E_MISSDATA;
+	} else if (fabs(x) > INT_MAX) {
+	    *err = E_DATA;
 	} else {
-	    x = Z[v][0];
-	    if (na(x)) {
-		*err = E_MISSDATA;
-	    } else {
-		n = (int) x;
-	    }
-	} 
-    }
+	    n = (int) x;
+	}	
+    } else {
+	*err = E_DATA;
+    } 
 
     return n;    
 }
@@ -1104,35 +1174,32 @@ int varnum_from_string (const char *str, DATAINFO *pdinfo)
 }
 
 /**
- * rename_var_by_id:
- * @idstr: string representation of the ID number of the
- * variable to be renamed.
- * @vname: new name to give the variable.
+ * gretl_type_from_name:
+ * @s: the name to check.
  * @pdinfo: dataset information.
  * 
- * Returns: 0 on sucess, %E_DATA on error.
+ * Returns: non-zero if @s is the name of an existing
+ * series, matrix, scalar, list or string variable,
+ * otherwise 0.
  */
 
-int rename_var_by_id (const char *idstr, const char *vname, 
-		      DATAINFO *pdinfo)
+GretlType gretl_type_from_name (const char *s, const DATAINFO *pdinfo)
 {
-    int v = varnum_from_string(idstr, pdinfo);
-
-    if (v < 0) {
-	return E_DATA;
+    if (gretl_is_series(s, pdinfo)) {
+	return GRETL_TYPE_SERIES;
+    } else if (get_matrix_by_name(s) != NULL) {
+	return GRETL_TYPE_MATRIX;
+    } else if (gretl_is_scalar(s)) {
+	return GRETL_TYPE_DOUBLE;
+    } else if (get_list_by_name(s)) {
+	return GRETL_TYPE_LIST;
+    } else if (get_string_by_name(s)) {
+	return GRETL_TYPE_STRING;
+    } else if (gretl_is_bundle(s)) {
+	return GRETL_TYPE_BUNDLE;
+    } else {
+	return GRETL_TYPE_NONE;
     }
-
-    if (var_is_const(pdinfo, v)) {
-	return overwrite_err(pdinfo, v);
-    }
-
-    /* should be pre-checked for validity of varname and
-       non-duplication (see interact.c under RENAME)
-    */
-
-    strcpy(pdinfo->varname[v], vname);
-
-    return 0;
 }
 
 /**
@@ -1176,9 +1243,9 @@ double *copyvec (const double *src, int n)
 
 void doubles_array_free (double **X, int m)
 {
-    int i;
-
     if (X != NULL) {
+	int i;
+
 	for (i=0; i<m; i++) {
 	    free(X[i]);
 	}
@@ -1192,7 +1259,8 @@ void doubles_array_free (double **X, int m)
  * @n: length of each sub-array.
  *
  * Allocates a 2-dimensional array of doubles, that is,
- * @m arrays each containing @n elements.
+ * @m arrays each containing @n elements.  If @n is
+ * zero the sub-arrays are just set to %NULL.
  * 
  * Returns: the allocated array, or %NULL on failure.
  */
@@ -1202,8 +1270,11 @@ double **doubles_array_new (int m, int n)
     double **X;
     int i;
 
-    X = malloc(m * sizeof *X);
+    if (m == 0) {
+	return NULL;
+    }
 
+    X = malloc(m * sizeof *X);
     if (X == NULL) {
 	return X;
     }
@@ -1212,18 +1283,89 @@ double **doubles_array_new (int m, int n)
 	X[i] = NULL;
     }
 
-    for (i=0; i<m; i++) {
-	X[i] = malloc(n * sizeof **X);
-	if (X[i] == NULL) {
-	    doubles_array_free(X, m);
-	    X = NULL;
-	    break;
+    if (n > 0) {
+	for (i=0; i<m; i++) {
+	    X[i] = malloc(n * sizeof **X);
+	    if (X[i] == NULL) {
+		doubles_array_free(X, m);
+		X = NULL;
+		break;
+	    }
 	}
     }
 
     return X;
 }
 
+/**
+ * doubles_array_new0:
+ * @m: number of sub-arrays.
+ * @n: length of each sub-array.
+ *
+ * Works just as doubles_array_new(), except that on
+ * successful allocation all values in the arrays are
+ * set to zero.
+ * 
+ * Returns: the allocated array, or %NULL on failure.
+ */
+
+double **doubles_array_new0 (int m, int n)
+{
+    double **X = doubles_array_new(m, n);
+
+    if (X != NULL && n > 0) {
+	int i, j;
+
+	for (i=0; i<m; i++) {
+	    for (j=0; j<n; j++) {
+		X[i][j] = 0.0;
+	    }
+	}
+    }
+
+    return X;
+}
+
+/**
+ * doubles_array_adjust_length:
+ * @X: original two-dimensional array.
+ * @m: number of sub-arrays in @X.
+ * @new_n: new length of each sub-array.
+ * 
+ * For each of the @m sub-arrays in @X, reallocate to
+ * a length of @new_n.
+ * 
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int doubles_array_adjust_length (double **X, int m, int new_n)
+{
+    int err = 0;
+
+    if (X == NULL || m == 0) {
+	; /* no-op */
+    } else {
+	double *xi;
+	int i;
+
+	for (i=0; i<m && !err; i++) {
+	    if (new_n == 0) {
+		free(X[i]);
+		X[i] = NULL;
+	    } else {
+		xi = realloc(X[i], new_n * sizeof *xi);
+		if (xi == NULL) {
+		    err = E_ALLOC;
+		} else {
+		    X[i] = xi;
+		}
+	    }
+	}
+    }
+
+    return err;
+}
+
 /**
  * data_array_from_model:
  * @pmod: reference model.
@@ -1289,101 +1431,27 @@ double **data_array_from_model (const MODEL *pmod, double **Z, int missv)
     return X;
 }
 
-/**
- * re_estimate:
- * @model_spec: estimation command line.
- * @tmpmod: pointer to model to recieve results.
- * @pZ: pointer to data array.
- * @pdinfo: dataset information.
- *
- *
- * Returns: 0 on success, non-zero on failure.
- */
-
-int re_estimate (char *model_spec, MODEL *tmpmod, 
-		 double ***pZ, DATAINFO *pdinfo) 
-{
-    CMD cmd;
-    double rho = 0.0;
-    int err = 0;
-
-    if (gretl_cmd_init(&cmd)) {
-	return 1;
-    }
-
-    err = parse_command_line(model_spec, &cmd, pZ, pdinfo);
-    if (err) {
-	gretl_cmd_free(&cmd);
-	return err;
-    }
-
-    gretl_model_init(tmpmod);
-
-    switch (cmd.ci) {
-    case AR:
-	*tmpmod = ar_func(cmd.list, pZ, pdinfo, OPT_NONE, NULL);
-	break;
-    case AR1:
-	rho = estimate_rho(cmd.list, pZ, pdinfo, cmd.opt, NULL, &err);
-	if (!err) {
-	    *tmpmod = ar1_lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt, rho);
-	}
-	break;
-    case HSK:
-	*tmpmod = hsk_func(cmd.list, pZ, pdinfo);
-	break;
-    case LOGIT:
-    case PROBIT:
-	*tmpmod = logit_probit(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt, 
-			       NULL);
-	break;
-    case TOBIT:
-	*tmpmod = tobit_model(cmd.list, pZ, pdinfo, NULL);
-	break;
-    case POISSON:
-	*tmpmod = poisson_model(cmd.list, pZ, pdinfo, NULL);
-	break;
-    case HECKIT:
-	*tmpmod = heckit_model(cmd.list, pZ, pdinfo, cmd.opt, NULL);
-	break;
-    case OLS:
-    case WLS:
-    case HCCM:
-	*tmpmod = lsq(cmd.list, pZ, pdinfo, cmd.ci, cmd.opt);
-	break;
-    case TSLS:
-	break;
-    default:
-	break;
-    }
-
-    if (tmpmod->errcode) {
-	err = 1;
-	clear_model(tmpmod);
-    }
-
-    gretl_cmd_free(&cmd);
-
-    return err;
-}
-
 double get_xvalue (int i, const double **Z, const DATAINFO *pdinfo)
 {
-    if (var_is_series(pdinfo, i)) {
-	return Z[i][pdinfo->t1];
-    } else {
-	return Z[i][0];
-    }	
+    return Z[i][pdinfo->t1];
 }
 
 #ifndef WIN32
 
-static int font_not_found (const char *s)
+static int non_fatal (const char *s)
 {
-    /* "Could not find/open font when opening font X, using default" 
-       or "gnuplot_x11: Some character sets not available" */
+    /* 
+       "Could not find/open font when opening font X, using default" 
+       "gnuplot_x11: Some character sets not available" 
+       "Warning: empty y2 range..."
+       pango warning for, e.g., FreeSans font w/o GPOS table
+    */
 
-    if (strstr(s, "using default") || strstr(s, "character sets not available")) {
+    if (strstr(s, "using default") ||
+	strstr(s, "trying default") ||
+	strstr(s, "character sets not available") ||
+	strstr(s, "Warning: empty ") ||
+	strstr(s, "Pango-WARNING")) {
 	return 1;
     } else {
 	return 0;
@@ -1415,9 +1483,9 @@ int gretl_spawn (char *cmdline)
 	ret = 1;
     } else if (errout && *errout) {
 	fprintf(stderr, "stderr: '%s'\n", errout);
-	if (!font_not_found(errout)) {
+	if (!non_fatal(errout)) {
 	    gretl_errmsg_set(errout);
-	    fprintf(stderr, "gretl_errmsg: '%s'\n", gretl_errmsg);
+	    fprintf(stderr, "gretl_errmsg: '%s'\n", gretl_errmsg_get());
 	    ret = 1;
 	}
     } else if (status != 0) {
@@ -1425,7 +1493,7 @@ int gretl_spawn (char *cmdline)
 	    gretl_errmsg_set(sout);
 	    fprintf(stderr, "gretl_spawn: status = %d: '%s'\n", status, sout);
 	} else {
-	    strcpy(gretl_errmsg, _("Command failed"));
+	    gretl_errmsg_set(_("Command failed"));
 	    fprintf(stderr, "gretl_spawn: status = %d\n", status);
 	}
 	ret = 1;
@@ -1456,12 +1524,12 @@ int gretl_copy_file (const char *src, const char *dest)
     }
    
     if ((srcfd = gretl_fopen(src, "rb")) == NULL) {
-	sprintf(gretl_errmsg, _("Couldn't open %s"), src);
+	gretl_errmsg_sprintf(_("Couldn't open %s"), src);
 	return 1; 
     }
 
     if ((destfd = gretl_fopen(dest, "wb")) == NULL) {
-	sprintf(gretl_errmsg, _("Couldn't write to %s"), dest);
+	gretl_errmsg_sprintf(_("Couldn't write to %s"), dest);
 	fclose(srcfd);
 	return 1;
     }
@@ -1476,25 +1544,121 @@ int gretl_copy_file (const char *src, const char *dest)
     return 0;
 }  
 
-/* timer */  
+int gretl_delete_var_by_name (const char *s, PRN *prn)
+{
+    int err = 0;
 
-static clock_t tim0;
+    if (!strcmp(s, "kalman")) {
+	err = delete_kalman(prn);	
+    } else if (gretl_is_scalar(s)) {
+	err = gretl_scalar_delete(s, prn);
+    } else if (get_matrix_by_name(s)) {
+	err = user_matrix_destroy_by_name(s, prn);
+    } else if (get_string_by_name(s)) {
+	err = delete_saved_string(s, prn);
+    } else if (gretl_is_bundle(s)) {
+	err = gretl_bundle_delete(s, prn);
+    } else {
+	err = E_UNKVAR;
+    }
+
+    return err;
+}
+
+/* internal execution timer: on OpenMP use omp_get_wtime(); else
+   on Windows use use GetTickCount; else use times() if available,
+   otherwise fall back on clock()
+*/ 
+
+#if defined(_OPENMP)
+
+#include <omp.h>
+
+static double tim0;
 
 static void gretl_stopwatch_init (void)
 {
-    tim0 = clock();
+    tim0 = omp_get_wtime();
 }
 
 double gretl_stopwatch (void)
 {
-    clock_t tim1 = clock();
-    double x = (double) (tim1 - tim0) / CLOCKS_PER_SEC;
+    double tim1 = omp_get_wtime();
+    double x = tim1 - tim0;
 
     tim0 = tim1;
 
     return x;
 } 
 
+#elif defined(WIN32)
+
+#include <windows.h>
+
+static DWORD tim0;
+
+static void gretl_stopwatch_init (void)
+{
+    tim0 = GetTickCount();
+}
+
+double gretl_stopwatch (void)
+{
+    DWORD tim1 = GetTickCount();
+    double x;
+
+    x = (double) (tim1 - tim0) / 1000.0;
+    tim0 = tim1;
+
+    return x;
+} 
+
+#else /* !OPENMP, !WIN32 */
+
+static clock_t tim0;
+
+#ifdef HAVE_SYS_TIMES_H
+# include <sys/times.h>
+# include <unistd.h>
+static unsigned ticks_per_sec;
+#else
+# include <time.h>
+#endif
+
+static void gretl_stopwatch_init (void)
+{
+#ifdef HAVE_SYS_TIMES_H
+    struct tms timebuf;
+
+    ticks_per_sec = sysconf(_SC_CLK_TCK);
+    tim0 = times(&timebuf);
+#else
+    tim0 = clock();
+#endif
+}
+
+double gretl_stopwatch (void)
+{
+    clock_t tim1;
+    double x;
+
+#ifdef HAVE_SYS_TIMES_H
+    struct tms timebuf;
+
+    tim1 = times(&timebuf);
+    x = (double) (tim1 - tim0) / ticks_per_sec;
+#else
+    tim1 = clock();
+    x = (double) (tim1 - tim0) / CLOCKS_PER_SEC;
+#endif
+
+    tim0 = tim1;
+
+    return x;
+} 
+
+#endif /* stopwatch variants */
+
 /* library init and cleanup functions */
 
 void libgretl_init (void)
@@ -1502,7 +1666,6 @@ void libgretl_init (void)
     libset_init();
     gretl_rand_init();
     gretl_stopwatch_init();
-    set_gretl_tex_preamble(); 
 }
 
 void libgretl_session_cleanup (int mode)
@@ -1513,14 +1676,14 @@ void libgretl_session_cleanup (int mode)
     gretl_plotx(NULL);
 
     if (mode != SESSION_PRESERVE_MATRICES) {
+	destroy_user_scalars();
 	destroy_user_matrices();
+	destroy_user_bundles();
     }
 }
 
 void libgretl_cleanup (void)
 {
-    const char *p;
-
     libgretl_session_cleanup(SESSION_CLEAR_FULL);
 
     gretl_rand_free();
@@ -1531,15 +1694,13 @@ void libgretl_cleanup (void)
     saved_strings_cleanup();
     lapack_mem_free();
     forecast_matrix_cleanup();
+    option_flags_cleanup();
+    kalman_cleanup();
+    gnuplot_cleanup();
 
-    p = strstr(gretl_plotfile(), "gpttmp");
-    if (p != NULL) {
-	int pnum;
-
-	if (!sscanf(p, "gpttmp%d.plt", &pnum)) {
-	    remove(gretl_plotfile());
-	}
-    }
+#ifdef USE_RLIB
+    gretl_R_cleanup();
+#endif
 }
 
 /* record and retrieve hypothesis test results */
@@ -1621,3 +1782,5 @@ double get_last_lnl (char *blurb)
     return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_LNL);
 }
 
+
+
