@@ -19,9 +19,19 @@
 
 #include "libgretl.h"
 #include "usermat.h"
+#include "matrix_extra.h"
 
-/* functions that convert between gretl_matrix and other
-   datatypes; also printing of gretl_matrices */
+/**
+ * SECTION:matrix_extra
+ * @short_description: more matrix operations
+ * @title: Matrix extra
+ * @include: gretl/libgretl.h, gretl/matrix_extra.h
+ *
+ * Functions that convert between gretl_matrix and other
+ * datatypes; also printing of gretl_matrices and some
+ * functions relating to masking and cutting rows and/or
+ * columns of matrices.
+ */
 
 /**
  * gretl_vector_from_array:
@@ -31,7 +41,7 @@
  * to use the squares of the elements of @x.
  *
  * Returns: pointer to a newly allocated gretl_vector containing
- * the elements of x (or their squares), or %NULL on failure.  
+ * the elements of x (or their squares), or NULL on failure.  
  * Missing valies in @x are skipped.
  */
 
@@ -69,7 +79,7 @@ gretl_vector_from_array (const double *x, int n, GretlMatrixMod mod)
  * @t2: ending observation.
  *
  * Returns: a newly allocated gretl_vector containing the values
- * of the given data series for the given range, or %NULL on failure.  
+ * of the given data series for the given range, or NULL on failure.  
  */
 
 gretl_vector *gretl_vector_from_series (const double *x, 
@@ -100,7 +110,7 @@ gretl_vector *gretl_vector_from_series (const double *x,
  * @cols: number of columns in target matrix.
  *
  * Returns: allocated gretl_matrix, the elements of which are set to
- * the values in @X, or %NULL on allocation failure.
+ * the values in @X, or NULL on allocation failure.
  */
 
 gretl_matrix *gretl_matrix_from_2d_array (const double **X, 
@@ -126,20 +136,19 @@ gretl_matrix *gretl_matrix_from_2d_array (const double **X,
  * gretl_matrix_from_scalar:
  * @x: scalar to be "promoted".
  *
- * Returns: allocated 1x1 gretl_matrix, the single element
- * of which is set to @x, or %NULL on allocation failure
- * or if @x = #NADBL.
+ * Returns: allocated 1 x 1 gretl_matrix, the single element
+ * of which is set to @x, or NULL on allocation failure.
+ * If @x = %NADBL the matrix value is %M_NA.
  */
 
 gretl_matrix *gretl_matrix_from_scalar (double x) 
 {
-    gretl_matrix *m = NULL;
+    gretl_matrix *m = gretl_matrix_alloc(1, 1);
 
-    if (!na(x)) {
-	m = gretl_matrix_alloc(1, 1);
-	if (m != NULL) {
-	    m->val[0] = x;
-	}
+    if (na(x)) {
+	m->val[0] = M_NA;
+    } else {
+	m->val[0] = x;
     }
 
     return m;
@@ -160,19 +169,20 @@ static int count_selection (const char *s, int n)
  * gretl_vcv_matrix_from_model:
  * @pmod: pointer to model
  * @select: char array indicating which rows and colums to select
- * (or %NULL for the full matrix).
+ * (or NULL for the full matrix).
+ * @err: location to receive error code.
  *
  * Produces all or part of the covariance matrix for @pmod 
  * in the form of a gretl_matrix.  Storage is allocated, to be freed
- * by the caller.  If @select is not %NULL, it should be an array
+ * by the caller.  If @select is not NULL, it should be an array
  * with non-zero elements in positions corresponding to the
  * desired rows (and columns), and zero elements otherwise.
  * 
- * Returns: the covariance matrix, or %NULL on error.
+ * Returns: the covariance matrix, or NULL on error.
  */
 
 gretl_matrix *
-gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
+gretl_vcv_matrix_from_model (MODEL *pmod, const char *select, int *err)
 {
     gretl_matrix *vcv;
     int i, j, idx, nc;
@@ -180,7 +190,8 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
     int k = pmod->ncoeff;
 
     /* first ensure the model _has_ a vcv */
-    if (makevcv(pmod, pmod->sigma)) {
+    *err = makevcv(pmod, pmod->sigma);
+    if (*err) {
 	return NULL;
     }
 
@@ -191,11 +202,13 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
     }
     
     if (nc == 0) {
+	*err = E_DATA;
 	return NULL;
     }
 
     vcv = gretl_matrix_alloc(nc, nc);
     if (vcv == NULL) {
+	*err = E_ALLOC;
 	return NULL;
     }
 
@@ -226,19 +239,20 @@ gretl_vcv_matrix_from_model (MODEL *pmod, const char *select)
  * gretl_coeff_vector_from_model:
  * @pmod: pointer to model
  * @select: char array indicating which rows to select
- * (or %NULL for the full vector).
+ * (or NULL for the full vector).
+ * @err: location to receive error code.
  *
  * Produces all or part of the coefficient vector for @pmod  
  * in the form of a gretl column vector.  Storage is allocated, to be freed
- * by the caller.  If @select is non-%NULL, it should be an array
+ * by the caller.  If @select is non-NULL, it should be an array
  * with non-zero elements in positions corresponding to the
  * desired rows and zero elements otherwise.
  * 
- * Returns: the coefficient vector, or %NULL on error.
+ * Returns: the coefficient vector, or NULL on error.
  */
 
 gretl_vector *
-gretl_coeff_vector_from_model (const MODEL *pmod, const char *select)
+gretl_coeff_vector_from_model (const MODEL *pmod, const char *select, int *err)
 {
     gretl_vector *b;
     int i, j, nc;
@@ -251,11 +265,13 @@ gretl_coeff_vector_from_model (const MODEL *pmod, const char *select)
     }
     
     if (nc == 0) {
+	*err = E_DATA;
 	return NULL;
     }
 
     b = gretl_column_vector_alloc(nc);
     if (b == NULL) {
+	*err = E_ALLOC;
 	return NULL;
     }
 
@@ -275,12 +291,12 @@ gretl_coeff_vector_from_model (const MODEL *pmod, const char *select)
  * @list: list of variables by ID number.
  * @Z: data array.
  * @pdinfo: pointer to data information struct.
- * @means: pointer to pick up vector of means, or %NULL to discard.
+ * @means: pointer to pick up vector of means, or NULL to discard.
  * @errp: pointer to receive non-zero error code in case of
- * failure, or %NULL.
+ * failure, or NULL.
  *
  * Returns: the variance-covariance matrix of the listed variables
- * (over the currently defined data sample), or %NULL in case of
+ * (over the currently defined data sample), or NULL in case of
  * failure.
  */
 
@@ -366,7 +382,7 @@ gretl_covariance_matrix_from_varlist (const int *list, const double **Z,
  * Copies the values from row @i of matrix @m into the array
  * @x, which should already be allocated to the correct size.
  *
- * Returns: 0 on sucess, 1 if the row is out of bounds.
+ * Returns: 0 on sucess, non-zero if the row is out of bounds.
  */
 
 int gretl_matrix_row_to_array (const gretl_matrix *m, int i, double *x)
@@ -374,7 +390,7 @@ int gretl_matrix_row_to_array (const gretl_matrix *m, int i, double *x)
     int j, err = 0;
 
     if (i < 0 || i >= gretl_matrix_rows(m)) {
-	err = 1;
+	err = E_DATA;
     } else {
 	for (j=0; j<m->cols; j++) {
 	    x[j] = gretl_matrix_get(m, i, j);
@@ -384,6 +400,44 @@ int gretl_matrix_row_to_array (const gretl_matrix *m, int i, double *x)
     return err;
 }
 
+/**
+ * gretl_matrix_get_columns:
+ * @m: source matrix.
+ * @err: location to receive error code.
+ *
+ * Constructs a two-dimensional array in which each sub-array
+ * is a pointer to a column of @m. The content of these arrays
+ * belongs to @m and must not be freed; only the returned
+ * pointer itself should be freed.
+ *
+ * Returns: the constructed array on success or NULL on failure.
+ */
+
+double **gretl_matrix_get_columns (const gretl_matrix *m, int *err)
+{
+    double **X = NULL;
+
+    if (gretl_is_null_matrix(m)) {
+	*err = E_DATA;
+    } else {
+	double *val = m->val;
+	int j;
+
+	X = doubles_array_new(m->cols, 0);
+
+	if (X == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    for (j=0; j<m->cols; j++) {
+		X[j] = val;
+		val += m->rows;
+	    }
+	}
+    }
+
+    return X;
+}
+
 static int get_mask_count (const char *mask, int n)
 {
     int i, k = 0;
@@ -395,12 +449,6 @@ static int get_mask_count (const char *mask, int n)
     return k;
 }
 
-enum {
-    OBS_MISS_MASK,
-    OBS_MISS_ERR,
-    OBS_MISS_SKIP
-};
-
 static gretl_matrix *
 real_gretl_matrix_data_subset (const int *list, const double **Z,
 			       int t1, int t2, const char *mask,
@@ -408,26 +456,35 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
 {
     gretl_matrix *M;
     double x;
-    int T = t2 - t1 + 1;
+    int T, Tmax = t2 - t1 + 1;
     int k = list[0];
     int skip;
     int j, s, t;
 
-    if (k <= 0 || T <= 0) {
+    if (k <= 0 || Tmax <= 0) {
 	*err = E_DATA;
 	return NULL;
     }	
 
     *err = 0;
 
+    T = Tmax;
+
     if (mask != NULL) {
-	T -= get_mask_count(mask, T);
-    } else if (op == OBS_MISS_SKIP || op == OBS_MISS_ERR) {
+	T -= get_mask_count(mask, Tmax);
+    } else if (op == M_MISSING_TRIM) {
+	*err = check_for_missing_obs(list, &t1, &t2, Z);
+	if (*err) {
+	    return NULL;
+	} else {
+	    Tmax = T = t2 - t1 + 1;
+	}
+    } else if (op == M_MISSING_SKIP || op == M_MISSING_ERROR) {
 	for (t=t1; t<=t2 && !*err; t++) {
 	    for (j=0; j<k; j++) {
 		x = Z[list[j+1]][t];
 		if (na(x)) {
-		    if (op == OBS_MISS_SKIP) {
+		    if (op == M_MISSING_SKIP) {
 			T--;
 		    } else {
 			*err = E_MISSDATA;
@@ -455,7 +512,7 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
 	skip = 0;
 	if (mask != NULL) {
 	    skip = mask[t - t1];
-	} else if (op == OBS_MISS_SKIP) {
+	} else if (op == M_MISSING_SKIP) {
 	    for (j=0; j<k; j++) {
 		x = Z[list[j+1]][t];
 		if (na(x)) {
@@ -479,12 +536,17 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
 	}
     }
 
-    if (!*err) {
+    if (!*err && (mask != NULL || op == M_MISSING_OK)) {
 	for (j=0; j<k && !*err; j++) {
 	    for (t=0; t<T && !*err; t++) {
 		x = gretl_matrix_get(M, t, j);
 		if (na(x)) {
-		    *err = E_MISSDATA;
+		    if (op == M_MISSING_OK) {
+			/* convert NA to NAN */
+			gretl_matrix_set(M, t, j, M_NA);
+		    } else {
+			*err = E_MISSDATA;
+		    }
 		}
 	    }
 	}
@@ -493,7 +555,10 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
     if (*err) {
 	gretl_matrix_free(M);
 	M = NULL;
-    } 
+    } else if (T == Tmax) {
+	M->t1 = t1;
+	M->t2 = t2;
+    }
 
     return M;
 }
@@ -504,97 +569,147 @@ real_gretl_matrix_data_subset (const int *list, const double **Z,
  * @Z: data array.
  * @t1: starting observation.
  * @t2: ending observation.
- * @mask: missing observations mask, or %NULL.
- * @err: location to recieve error code.
+ * @missop: how to handle missing observations.
+ * @err: location to receive error code.
  *
  * Creates a gretl matrix holding the subset of variables from
  * @Z specified by @list, over the sample range @t1 to @t2,
- * inclusive.  Variables are in columns.  If @mask is not
- * %NULL then it should be an array of char of length (@t2 - @t1
- * + 1) with 1s in the positions of observations to exclude
- * from the subset and zeros elsewhere. This apparatus can be
- * used to exclude missing observations.
+ * inclusive.  Variables are in columns. The @missop flag
+ * can be %M_MISSING_OK to indicate that it's OK to include
+ * missing values in the matrix (these become NaNs),
+ * %M_MISSING_ERROR (it's an error of any missing values are
+ * found), or %M_MISSING_SKIP (observations with any missing
+ * values are omitted from the matrix).
  *
- * Returns: allocated matrix or %NULL on failure. 
+ * Returns: allocated matrix or NULL on failure. 
  */
 
 gretl_matrix *gretl_matrix_data_subset (const int *list, const double **Z,
-					int t1, int t2, const char *mask,
+					int t1, int t2, int missop, 
 					int *err)
+{
+    return real_gretl_matrix_data_subset(list, Z, t1, t2, NULL, 
+					 missop, err);
+}
+
+/**
+ * gretl_matrix_data_subset_masked:
+ * @list: list of variable to process.
+ * @Z: data array.
+ * @t1: starting observation.
+ * @t2: ending observation.
+ * @mask: missing observations mask, or NULL.
+ * @err: location to receive error code.
+ *
+ * Creates a gretl matrix holding the subset of variables from
+ * @Z specified by @list, over the sample range @t1 to @t2,
+ * inclusive.  Variables are in columns.  @mask should be an
+ * array of char of length (@t2 - @t1 + 1) with 1s in the positions 
+ * of observations to exclude from the subset and zeros elsewhere. 
+ * This apparatus can be used to exclude missing observations.
+ *
+ * Returns: allocated matrix or NULL on failure. 
+ */
+
+gretl_matrix *
+gretl_matrix_data_subset_masked (const int *list, const double **Z,
+				 int t1, int t2, const char *mask,
+				 int *err)
 {
     return real_gretl_matrix_data_subset(list, Z, t1, t2, mask, 
-					 OBS_MISS_MASK, err);
+					 M_MISSING_ERROR, err);
 }
 
 /**
- * gretl_matrix_data_subset_no_missing:
- * @list: list of variable to process.
- * @Z: data array.
- * @t1: starting observation.
- * @t2: ending observation.
+ * gretl_dataset_from_matrix:
+ * @m: source matrix.
+ * @list: list of columns (1-based) to include, or NULL.
+ * @pZ: location to receive data array.
  * @err: location to receive error code.
  *
- * Creates a gretl matrix holding the subset of variables from
- * @Z specified by @list, over the sample range @t1 to @t2,
- * inclusive.  Variables are in columns.  If any missing 
- * values are encountered this constitutes an error.
+ * Creates a gretl dataset from matrix @m, either using the
+ * columns specified in @list or using all columns if @list
+ * is NULL.
  *
- * Returns: allocated matrix or %NULL on failure. 
+ * Returns: pointer to new dataset information struct on success, 
+ * or NULL on failure.
  */
 
-gretl_matrix *
-gretl_matrix_data_subset_no_missing (const int *list, const double **Z,
-				     int t1, int t2, int *err)
+DATAINFO *gretl_dataset_from_matrix (gretl_matrix *m, const int *list,
+				     double ***pZ, int *err)
 {
-    return real_gretl_matrix_data_subset(list, Z, t1, t2, NULL, 
-					 OBS_MISS_ERR, err);
-}
+    DATAINFO *dinfo = NULL;
+    const char **names;
+    int i, t, col, nv, T;
 
-/**
- * gretl_matrix_data_subset_skip_missing:
- * @list: list of variable to process.
- * @Z: data array.
- * @t1: starting observation.
- * @t2: ending observation.
- * @err: location to receive error code.
- *
- * Creates a gretl matrix holding the subset of variables from
- * @Z specified by @list, over the sample range @t1 to @t2,
- * inclusive.  Variables are in columns.  If there is a missing
- * value for any variable on a given row, that row is skipped.
- *
- * Returns: allocated matrix or %NULL on failure. 
- */
+    if (gretl_is_null_matrix(m)) {
+	*err = E_DATA;
+	return NULL;
+    }
 
-gretl_matrix *
-gretl_matrix_data_subset_skip_missing (const int *list, const double **Z,
-				       int t1, int t2, int *err)
-{
-    return real_gretl_matrix_data_subset(list, Z, t1, t2, NULL, 
-					 OBS_MISS_SKIP, err);
+    T = gretl_matrix_rows(m);
+    nv = gretl_matrix_cols(m);
+
+    if (list != NULL) {
+	for (i=1; i<=list[0]; i++) {
+	    col = list[i];
+	    if (col < 1 || col > nv) {
+		fprintf(stderr, "column %d: out of bounds\n", col);
+		*err = E_DATA;
+		break;
+	    } 
+	}
+	nv = list[0];
+    } 
+
+    if (!*err) {
+	dinfo = create_auxiliary_dataset(pZ, nv + 1, T);
+	if (dinfo == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (*err) {
+	return NULL;
+    }
+
+    names = user_matrix_get_column_names(m);
+
+    for (i=1; i<=nv; i++) {
+	col = (list != NULL)? list[i] - 1 : i - 1;
+	for (t=0; t<T; t++) {
+	    (*pZ)[i][t] = gretl_matrix_get(m, t, col);
+	}
+	if (names != NULL) {
+	    strcpy(dinfo->varname[i], names[col]);
+	} else {
+	    sprintf(dinfo->varname[i], "col%d", col + 1);
+	}
+    }
+
+    return dinfo;
 }
 
 /**
  * gretl_plotfit_matrices:
- * @yno: ID number of the y variable.
- * @xno: ID number of the y variable.
+ * @yvar: the y variable.
+ * @xvar: the x variable.
  * @fit: type of fit sought.
- * @Z: data array.
  * @t1: starting observation.
  * @t2: ending observation.
  * @py: location to receive y vector.
  * @pX: location to receive X matrix.
  *
- * Creates a vector y and matrix X based on the input @yno, @xno
- * and @fit, using the given sample range.  An observation is
- * skipped if any of the variables in @list are missing at that
- * observation.
+ * Creates a vector y and matrix X based on the input @yvar, 
+ * @xvar and @fit, using the given sample range.  An observation
+ * is skipped if either @yvar or @xvar is missing at that
+ * observation. 
  *
  * Returns: 0 on success, non-zero code on error.
  */
 
-int gretl_plotfit_matrices (int yno, int xno, FitType fit,
-			    const double **Z, int t1, int t2, 
+int gretl_plotfit_matrices (const double *yvar, const double *xvar,
+			    FitType fit, int t1, int t2, 
 			    gretl_matrix **py, gretl_matrix **pX)
 {
     gretl_matrix *y = NULL;
@@ -617,7 +732,7 @@ int gretl_plotfit_matrices (int yno, int xno, FitType fit,
 
     for (s=0; s<T; s++) {
 	t = s + t1;
-	if (na(Z[yno][t]) || na(Z[xno][t])) {
+	if (na(yvar[t]) || (xvar != NULL && na(xvar[t]))) {
 	    mask[s] = 1;
 	} else {
 	    n++;
@@ -649,11 +764,11 @@ int gretl_plotfit_matrices (int yno, int xno, FitType fit,
 	t = s + t1;
 	if (!mask[s]) {
 	    j = 0;
-	    y->val[i] = Z[yno][t];
+	    y->val[i] = yvar[t];
 	    if (fit != PLOT_FIT_LOESS) {
 		gretl_matrix_set(X, i, j++, 1.0);
 	    }
-	    xt = Z[xno][t];
+	    xt = (xvar != NULL)? xvar[t] : s;
 	    if (fit == PLOT_FIT_INVERSE) {
 		gretl_matrix_set(X, i, j++, 1.0 / xt);
 	    } else {
@@ -681,39 +796,6 @@ int gretl_plotfit_matrices (int yno, int xno, FitType fit,
     return err;
 }
 
-/* delete the columns of X specified in @list */
-
-int gretl_matrix_delete_columns (gretl_matrix *X, int *list)
-{
-    size_t csz = X->rows * sizeof *X->val;
-    void *dest, *src;
-    int i, j, n, col;
-
-    for (i=1; i<=list[0]; i++) {
-	col = list[i];
-	if (col < 0 || col >= X->cols) {
-	    return E_NONCONF;
-	}
-    }
-
-    for (i=1; i<=list[0]; i++) {
-	col = list[i];
-	dest = X->val + col * X->rows;
-	src = X->val + (col + 1) * X->rows;
-	n = X->cols - col - 1;
-	if (n > 0) {
-	    memmove(dest, src, n * csz);
-	}
-	for (j=i+1; j<=list[0]; j++) {
-	    list[j] -= 1;
-	}
-    }
-
-    X->cols -= list[0];
-
-    return 0;
-}
-
 /**
  * gretl_matrix_read_from_text:
  * @fname: name of text file.
@@ -725,49 +807,47 @@ int gretl_matrix_delete_columns (gretl_matrix *X, int *list)
  * of the csv file, so no heuristics are necessary. In case of error,
  * an empty matrix is returned and @err is filled appropriately.
  *
- * Returns: The matrix read from file, or %NULL.
+ * Returns: The matrix read from file, or NULL.
  */
 
 gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err)
 {
-    int r, c, i, j;
-    int ret;
+    int r, c, i, j, n;
     double x;
     gretl_matrix *A = NULL;
-    FILE *f;
+    FILE *fp;
 
-    f = fopen(fname, "r");
+    fp = gretl_read_user_file(fname);
 
-    if (f == NULL) {
+    if (fp == NULL) {
 	*err = E_FOPEN;
-	return gretl_null_matrix_new();
+	return NULL;
     }
 
-    ret = fscanf(f, "%d %d\n", &r, &c);
+    n = fscanf(fp, "%d %d\n", &r, &c);
 
-    if (ret < 2 || r <= 0 || c <= 0) {
+    if (n < 2 || r <= 0 || c <= 0) {
 	*err = E_DATA;
-	A = gretl_null_matrix_new();
-    } else {
-	A = gretl_matrix_alloc(r, c);
+	fclose(fp);
+	return NULL;
     }
 
+    A = gretl_matrix_alloc(r, c);
     if (A == NULL) {
 	*err = E_ALLOC;
-    }
-
-    if (*err) {
-	fclose(f);
-	return A;
+	fclose(fp);
+	return NULL;
     }
 
     gretl_push_c_numeric_locale();
 
     for (i=0; i<r && !*err; i++) {
 	for (j=0; j<c && !*err; j++) {
-	    if (fscanf(f, "%lf", &x) != 1) {
+	    if (fscanf(fp, "%lf", &x) != 1) {
 		*err = E_DATA;
 		fprintf(stderr, "error reading row %d, column %d\n", i+1, j+1);
+		gretl_matrix_free(A);
+		A = NULL;
 	    } else {
 		gretl_matrix_set(A, i, j, x);
 	    }
@@ -776,13 +856,13 @@ gretl_matrix *gretl_matrix_read_from_text (const char *fname, int *err)
 
     gretl_pop_c_numeric_locale();
     
-    fclose(f);
+    fclose(fp);
 
     return A;
 }
 
 /**
- * gretl_matrix_write_as text:
+ * gretl_matrix_write_as_text:
  * @A: matrix.
  * @fname: name of file to write.
  *
@@ -799,28 +879,30 @@ int gretl_matrix_write_as_text (gretl_matrix *A, const char *fname)
     int r = A->rows;
     int c = A->cols;
     int i, j, err = 0;
-    FILE *f;
+    FILE *fp;
 
-    f = fopen(fname, "w");
+    fname = gretl_maybe_switch_dir(fname);
 
-    if (f == NULL) {
+    fp = gretl_fopen(fname, "w");
+
+    if (fp == NULL) {
 	return E_FOPEN;
     }
 
-    fprintf(f, "%d\t%d\n", r, c);
+    fprintf(fp, "%d\t%d\n", r, c);
     
     gretl_push_c_numeric_locale();
 
     for (i=0; i<r; i++) {
 	for (j=0; j<c; j++) {
-	    fprintf(f, "%26.18E\t", gretl_matrix_get(A, i, j));
+	    fprintf(fp, "%26.18E\t", gretl_matrix_get(A, i, j));
 	}
-	fputc('\n', f);
+	fputc('\n', fp);
     }
 
     gretl_pop_c_numeric_locale();
     
-    fclose(f);
+    fclose(fp);
 
     return err;
 }
@@ -875,7 +957,7 @@ static int max_numchars (const gretl_matrix *m)
 
 static void 
 real_matrix_print_to_prn (const gretl_matrix *m, const char *msg, 
-			  int packed, int errout, 
+			  int packed, int errout, int plain,
 			  const char **heads, PRN *prn)
 {
     char numstr[32];
@@ -895,7 +977,7 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 	return;
     }
 
-    if (msg != NULL && *msg != '\0') {
+    if (msg != NULL && *msg != '\0' && !plain) {
 	pprintf(prn, "%s (%d x %d)", msg, m->rows, m->cols);
 	if (!(m->t1 == 0 && m->t2 == 0)) {
 	    pprintf(prn, " [t1 = %d, t2 = %d]\n\n", m->t1 + 1, m->t2 + 1);
@@ -965,7 +1047,7 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 /**
  * gretl_matrix_print_to_prn:
  * @m: matrix to print.
- * @msg: accompanying message text (or %NULL if no message is wanted).
+ * @msg: accompanying message text (or NULL if no message is wanted).
  * @prn: pointer to gretl printing struct.
  *
  * Prints the matrix @m to @prn.
@@ -974,14 +1056,14 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 void 
 gretl_matrix_print_to_prn (const gretl_matrix *m, const char *msg, PRN *prn)
 {
-    real_matrix_print_to_prn(m, msg, 0, 0, NULL, prn);
+    real_matrix_print_to_prn(m, msg, 0, 0, 0, NULL, prn);
 }
 
 /**
  * gretl_matrix_print_with_col_heads:
- * @m: matrix to print.
- * @title: accompanying title (or %NULL if no title is wanted).
- * @heads: array of strings to identify the columns.
+ * @m: T x k matrix to print.
+ * @title: accompanying title (or NULL if no title is wanted).
+ * @heads: array of k strings to identify the columns.
  * @prn: pointer to gretl printing struct.
  *
  * Prints the matrix @m to @prn, with column headings given
@@ -993,38 +1075,129 @@ void gretl_matrix_print_with_col_heads (const gretl_matrix *m,
 					const char **heads,
 					PRN *prn)
 {
-    real_matrix_print_to_prn(m, title, 0, 0, heads, prn);
+    real_matrix_print_to_prn(m, title, 0, 0, 0, heads, prn);
+}
+
+static void maybe_print_col_heads (const gretl_matrix *m, 
+				   const char *fmt,
+				   int wid, int prec,
+				   int icast, PRN *prn)
+{
+    const char **heads;
+
+    heads = user_matrix_get_column_names(m);
+    
+    if (heads != NULL) {
+	char wtest[32];
+	double x;
+	int j, n;
+
+	x = gretl_matrix_get(m, 0, 0);
+
+	if (icast) {
+	    if (wid >= 0 && prec >= 0) {
+		snprintf(wtest, 32, fmt, wid, prec, (int) x);
+	    } else if (wid >= 0 || prec >= 0) {
+		n = (wid >= 0)? wid : prec;
+		snprintf(wtest, 32, fmt, n, (int) x);
+	    } else {
+		snprintf(wtest, 32, fmt, (int) x);
+	    }
+	} else {
+	    if (wid >= 0 && prec >= 0) {
+		snprintf(wtest, 32, fmt, wid, prec, x);
+	    } else if (wid >= 0 || prec >= 0) {
+		n = (wid >= 0)? wid : prec;
+		snprintf(wtest, 32, fmt, n, x);
+	    } else {
+		snprintf(wtest, 32, fmt, x);
+	    }
+	}
+
+	n = strlen(wtest);
+	for (j=0; j<m->cols; j++) {
+	    pprintf(prn, "%*s", n, heads[j]);
+	}
+	pputc(prn, '\n');
+    }
 }
 
 /**
- * gretl_packed_matrix_print:
- * @m: packed matrix to print.
- * @msg: accompanying message text (or %NULL if no message is wanted).
+ * gretl_matrix_print_with_format:
+ * @m: matrix to print.
+ * @fmt: a printf-type format string.
+ * @wid: an integer width, or 0.
+ * @prec: an integer precision, or 0.
+ * @prn: pointer to gretl printing struct.
  *
- * Prints the symmetric matrix @m (packed as lower triangle)
- * to stderr.
+ * Prints the matrix @m to @prn, with the elements of @m
+ * formatted as specified by @fmt, @wid and @prec. 
+ * The arguments @wid and/or @prec are required only if 
+ * @fmt contains one or more placeholders ("*") to be filled out.
+ * For example, if @fmt is "\%14.6f" then neither @wid nor
+ * @prec is needed and both should be set to 0; if 
+ * @fmt is "\%*.6f" then a @wid value is needed but @prec
+ * should be 0.
  */
 
-void gretl_packed_matrix_print (const gretl_matrix *m, const char *msg)
+void gretl_matrix_print_with_format (const gretl_matrix *m, 
+				     const char *fmt,
+				     int wid, int prec,
+				     PRN *prn)
 {
-    PRN *prn;
-    int err = 0;
+    if (prn == NULL) {
+	return;
+    }
 
-    prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
+    if (gretl_is_null_matrix(m) || fmt == NULL || *fmt == '\0') {
+	real_matrix_print_to_prn(m, NULL, 0, 0, 1, NULL, prn);
+    } else {
+	int intcast = 0;
+	double x;
+	int i, j, c;
 
-    if (!err) {
-	real_matrix_print_to_prn(m, msg, 1, 0, NULL, prn);
-	gretl_print_destroy(prn);
+	c = fmt[strlen(fmt)-1];
+	if (c == 'd' || c == 'u' || c == 'x' || c == 'l') {
+	    intcast = 1;
+	}
+
+	maybe_print_col_heads(m, fmt, wid, prec, intcast, prn);
+
+	for (i=0; i<m->rows; i++) {
+	    for (j=0; j<m->cols; j++) {
+		x = gretl_matrix_get(m, i, j);
+		if (intcast) {
+		    if (wid >= 0 && prec >= 0) {
+			pprintf(prn, fmt, wid, prec, (int) x);
+		    } else if (wid >= 0 || prec >= 0) {
+			c = (wid >= 0)? wid : prec;
+			pprintf(prn, fmt, c, (int) x);
+		    } else {
+			pprintf(prn, fmt, (int) x);
+		    }
+		} else {
+		    if (wid >= 0 && prec >= 0) {
+			pprintf(prn, fmt, wid, prec, x);
+		    } else if (wid >= 0 || prec >= 0) {
+			c = (wid >= 0)? wid : prec;
+			pprintf(prn, fmt, c, x);
+		    } else {
+			pprintf(prn, fmt, x);
+		    }
+		}
+	    }
+	    pputc(prn, '\n');
+	}
     }
 }
 
 /**
  * debug_print_matrix:
  * @m: matrix to print.
- * @msg: accompanying message text (or %NULL if no message is wanted).
+ * @msg: accompanying message text (or NULL if no message is wanted).
  *
- * Prints the matrix @m to stderr, as with gretl_matrix_print(), but
- * appends the address of the matrix struct.
+ * Prints the matrix @m to stderr with high precision, along
+ * with the address of the matrix struct.
  */
 
 void debug_print_matrix (const gretl_matrix *m, const char *msg)
@@ -1058,3 +1231,315 @@ void debug_print_matrix (const gretl_matrix *m, const char *msg)
     }
 }
 
+static int count_unmasked_elements (const char *mask, int n)
+{
+    int i, c = 0;
+
+    for (i=0; i<n; i++) {
+	c += (mask[i] == 0);
+    }
+
+    return c;
+}
+
+/**
+ * gretl_matrix_cut_rows:
+ * @m: matrix to process.
+ * @mask: character array of length equal to the rows of @m,
+ * with 1s indicating rows to be cut, 0s for rows to be
+ * retained.
+ *
+ * In-place reduction of @m based on @mask: the masked rows
+ * are cut out of @m.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_matrix_cut_rows (gretl_matrix *m, const char *mask)
+{
+    int i, j, k, n;
+    double x;
+
+    if (m == NULL || mask == NULL) {
+	return E_DATA;
+    }
+
+    n = count_unmasked_elements(mask, m->rows);
+
+    for (j=0; j<m->cols; j++) {
+	k = 0;
+	for (i=0; i<m->rows; i++) {
+	    if (!mask[i]) {
+		x = gretl_matrix_get(m, i, j);
+		m->val[j * n + k] = x;
+		k++;
+	    }
+	}
+    }
+
+    m->rows = n;
+
+    return 0;
+}
+
+/**
+ * gretl_matrix_cut_cols:
+ * @m: matrix to process.
+ * @mask: character array of length equal to the cols of @m,
+ * with 1s indicating cols to be cut, 0s for cols to be
+ * retained.
+ *
+ * In-place reduction of @m based on @mask: the masked cols
+ * are cut out of @m.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_matrix_cut_cols (gretl_matrix *m, const char *mask)
+{
+    int i, j, k, n;
+    double x;
+
+    if (m == NULL || mask == NULL) {
+	return E_DATA;
+    }
+
+    n = count_unmasked_elements(mask, m->cols);
+
+    k = 0;
+    for (i=0; i<m->cols; i++) {
+	if (!mask[i]) {
+	    for (j=0; j<m->rows; j++) {
+		x = gretl_matrix_get(m, j, i);
+		m->val[k++] = x;
+	    }
+	}
+    }
+
+    m->cols = n;
+
+    return 0;
+}
+
+/**
+ * gretl_matrix_cut_rows_cols:
+ * @m: square matrix to process.
+ * @mask: character array of length equal to the dimension
+ * of @m, with 1s indicating rows and columns to be cut, 0s
+ * for rows/columns to be retained.
+ *
+ * In-place reduction of @m based on @mask: the masked rows
+ * and columns are cut out of @m.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int gretl_matrix_cut_rows_cols (gretl_matrix *m, const char *mask)
+{
+    gretl_matrix *tmp;
+    double x;
+    int i, j, k, l, n;
+
+    if (m == NULL || mask == NULL || m->rows != m->cols) {
+	return E_DATA;
+    }
+
+    n = count_unmasked_elements(mask, m->rows);
+
+    /* allocate smaller temp matrix */
+    tmp = gretl_matrix_alloc(n, n);
+    if (tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    /* copy unmasked values into temp matrix */
+    k = 0;
+    for (i=0; i<m->rows; i++) {
+	if (!mask[i]) {
+	    l = 0;
+	    for (j=0; j<m->cols; j++) {
+		if (!mask[j]) {
+		    x = gretl_matrix_get(m, i, j);
+		    gretl_matrix_set(tmp, k, l++, x);
+		}
+	    }
+	    k++;
+	}
+    }
+
+    /* resize the original matrix, copy the values back,
+       and free the temp matrix */
+    gretl_matrix_reuse(m, n, n);
+    gretl_matrix_copy_values(m, tmp);
+    gretl_matrix_free(tmp);
+
+    return 0;
+}
+
+/**
+ * gretl_matrix_zero_row_mask:
+ * @m: matrix to process.
+ * @err: location to receive error code.
+ *
+ * Checks matrix @m for rows that are all zero.  If there are
+ * any such rows, constructs a mask of length equal to the
+ * number of rows in @m, with 1s indicating zero rows, 0s
+ * elsewhere.  If there are no such rows, returns NULL.
+ * 
+ * E_ALLOC is written to @err in case a mask should have
+ * been constructed but allocation failed.
+ *
+ * Returns: allocated mask or NULL.
+ */
+
+char *gretl_matrix_zero_row_mask (const gretl_matrix *m, int *err)
+{
+    char *mask = NULL;
+    int row0, any0 = 0;
+    int i, j;
+
+    mask = calloc(m->rows, 1);
+    if (mask == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+    
+    for (i=0; i<m->rows; i++) {
+	row0 = 1;
+	for (j=0; j<m->cols; j++) {
+	    if (gretl_matrix_get(m, i, j) != 0.0) {
+		row0 = 0;
+		break;
+	    }
+	}
+	if (row0) {
+	    mask[i] = 1;
+	    any0 = 1;
+	}
+    }
+
+    if (!any0) {
+	free(mask);
+	mask = NULL;
+    }
+
+    return mask;
+}
+
+/**
+ * gretl_matrix_rank_mask:
+ * @m: matrix to process.
+ * @err: location to receive error code.
+ *
+ * Performs a QR decomposition of matrix @m and uses this
+ * to assess the rank of @m.  If @m is not of full rank,
+ * constructs a mask of length equal to the numbers of
+ * columns in @m, with 1s in positions corresponding 
+ * to diagonal elements of R that are effectively 0, and
+ * 0s elsewhere.  If @m is of full column rank, NULL is
+ * returned.
+ * 
+ * E_ALLOC is written to @err in case a mask should have
+ * been constructed but allocation failed.
+ *
+ * Returns: allocated mask or NULL.
+ */
+
+char *gretl_matrix_rank_mask (const gretl_matrix *m, int *err)
+{
+    gretl_matrix *Q = NULL;
+    gretl_matrix *R = NULL;
+    char *mask = NULL;
+    double Ri;
+    int fullrank = 1;
+    int i, n = m->cols;
+
+    Q = gretl_matrix_copy(m);
+    if (Q == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    R = gretl_matrix_alloc(n, n);
+    if (R == NULL) {
+	gretl_matrix_free(Q);
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    *err = gretl_matrix_QR_decomp(Q, R);
+
+    if (!*err) {
+	mask = calloc(n, 1);
+	if (mask == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	for (i=0; i<n; i++) {
+	    Ri = gretl_matrix_get(R, i, i);
+	    if (fabs(Ri) < R_DIAG_MIN) {
+		mask[i] = 1;
+		fullrank = 0;
+	    }
+	}
+    }
+
+    if (*err || fullrank) {
+	free(mask);
+	mask = NULL;
+    }
+
+    gretl_matrix_free(Q);
+    gretl_matrix_free(R);
+    
+    return mask;
+}
+
+/**
+ * gretl_matrix_mp_ols:
+ * @y: dependent variable vector.
+ * @X: matrix of independent variables.
+ * @b: vector to hold coefficient estimates.
+ * @vcv: matrix to hold the covariance matrix of the coefficients,
+ * or NULL if this is not needed.
+ * @uhat: vector to hold the regression residuals, or NULL if 
+ * these are not needed.
+ * @s2: pointer to receive residual variance, or NULL.  Note:
+ * if @s2 is NULL, the "vcv" estimate will be plain (X'X)^{-1}.
+ *
+ * Computes OLS estimates using Cholesky factorization, via
+ * the GMP multiple-precision library, and puts the
+ * coefficient estimates in @b.  Optionally, calculates the
+ * covariance matrix in @vcv and the residuals in @uhat.
+ * 
+ * Returns: 0 on success, non-zero error code on failure.
+ */
+
+int gretl_matrix_mp_ols (const gretl_vector *y, const gretl_matrix *X,
+			 gretl_vector *b, gretl_matrix *vcv, 
+			 gretl_vector *uhat, double *s2)
+{
+    void *handle = NULL;
+    int (*matrix_mp_ols) (const gretl_vector *,
+			  const gretl_matrix *,
+			  gretl_vector *,
+			  gretl_matrix *,
+			  gretl_vector *,
+			  double *);
+    int err;
+
+    matrix_mp_ols = get_plugin_function("matrix_mp_ols", &handle);
+    if (matrix_mp_ols == NULL) {
+	return 1;
+    }
+
+    err = (*matrix_mp_ols)(y, X, b, vcv, uhat, s2);
+
+    close_plugin(handle);
+
+    return err;
+}
+
+
