@@ -17,10 +17,19 @@
  * 
  */
 
-/* non-parametric stats for gretl */
-
 #include "libgretl.h"
 
+/**
+ * SECTION:nonparam
+ * @short_description: selected nonparametric routines
+ * @title: Nonparametric
+ * @include: libgretl.h
+ *
+ * Includes various nonparametric tests: rank correlation,
+ * runs (randomness), and differences between series.  Also
+ * "loess" regression a la William Cleveland.
+ */
+
 /* Spearman, one-tailed alpha = .005, .025, .05 */
 
 static double rhocrit[18][3] = { 
@@ -264,7 +273,7 @@ static void print_raw_and_ranked (int vx, int vy,
 }
 
 /**
- * spearman:
+ * spearman_rho:
  * @list: list of (two) variables to process.
  * @Z: data array.
  * @pdinfo: information on the data set.
@@ -277,8 +286,8 @@ static void print_raw_and_ranked (int vx, int vy,
  * Returns: 0 on successful completion, 1 on error.
  */
 
-int spearman (const int *list, const double **Z, const DATAINFO *pdinfo, 
-	      gretlopt opt, PRN *prn)
+int spearman_rho (const int *list, const double **Z, const DATAINFO *pdinfo, 
+		  gretlopt opt, PRN *prn)
 {
     int T = pdinfo->t2 - pdinfo->t1 + 1;
     double *rx = NULL, *ry = NULL;
@@ -308,6 +317,12 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
 
     pprintf(prn, _("\nFor the variables '%s' and '%s'\n"), pdinfo->varname[vx],
 	    pdinfo->varname[vy]);
+
+    if (na(rho)) {
+	pputs(prn, _("Spearman's rank correlation is undefined\n"));
+	return err;
+    }
+
     pprintf(prn, _("Spearman's rank correlation coefficient (rho) = %.8f\n"), rho);
 
     if (rho == 0.0) {
@@ -323,7 +338,7 @@ int spearman (const int *list, const double **Z, const DATAINFO *pdinfo,
 
 	pputs(prn, _("Under the null hypothesis of no correlation:\n "));
 	pprintf(prn, _("t(%d) = %g, with two-tailed p-value %.4f\n"), m - 2,
-		tval, student_pvalue_2(tval, m - 2));
+		tval, student_pvalue_2(m - 2, tval));
     } else if (m >= 7) {
 	double pval = spearman_signif(fabs(rho), m);
 
@@ -496,8 +511,22 @@ static int real_kendall_tau (const double *x, const double *y,
     return 0;
 }
 
-int kendall (const int *list, const double **Z, const DATAINFO *pdinfo, 
-	     gretlopt opt, PRN *prn)
+/**
+ * kendall_tau:
+ * @list: list of (two) variables to process.
+ * @Z: data array.
+ * @pdinfo: information on the data set.
+ * @opt: if includes %OPT_V, print both the "raw" and the ranked data.
+ * @prn: gretl printing struct.
+ *
+ * Calculates and prints Kendall's rank correlation tau statistic for
+ * the two variables specified in @list.
+ * 
+ * Returns: 0 on successful completion, 1 on error.
+ */
+
+int kendall_tau (const int *list, const double **Z, const DATAINFO *pdinfo, 
+		 gretlopt opt, PRN *prn)
 {
     struct xy_pair *xy = NULL;
     int T = pdinfo->t2 - pdinfo->t1 + 1;
@@ -541,7 +570,7 @@ int kendall (const int *list, const double **Z, const DATAINFO *pdinfo,
     if (!err) {
 	pprintf(prn, _("\nFor the variables '%s' and '%s'\n"), pdinfo->varname[vx],
 		pdinfo->varname[vy]);
-	pprintf(prn, "Kendall's tau = %.8f\n", tau);
+	pprintf(prn, "%s = %.8f\n", _("Kendall's tau"), tau);
 	pputs(prn, _("Under the null hypothesis of no correlation:\n "));
 	pprintf(prn, _("z-score = %g, with two-tailed p-value %.4f\n"), z,
 		normal_pvalue_2(z));
@@ -561,7 +590,6 @@ int kendall (const int *list, const double **Z, const DATAINFO *pdinfo,
 
     pputc(prn, '\n');
 
-    /* clean up */
     free(xy);
 
     return err;
@@ -713,7 +741,7 @@ double lockes_test (const double *x, int t1, int t2)
  * @prn: gretl printing struct.
  *
  * Performs, and prints the results of, the runs test for randomness
- * for the variable specified by @varno.  The normal approximation
+ * for the variable specified by @v.  The normal approximation
  * is that given in Gary Smith, Statistical Reasoning, 2e, p. 674.
  * 
  * Returns: 0 on successful completion, non-zero on error.
@@ -758,9 +786,8 @@ int runs_test (int v, const double **Z, const DATAINFO *pdinfo,
     }
 
     if (n <= 1) {
-	pputs(prn, _("\nInsufficient data for runs test\n"));
 	free(x);
-	return 1;
+	return E_TOOFEW;
     }
 
     Np = (x[0] > 0);
@@ -836,17 +863,17 @@ static void print_z_prob (double z, PRN *prn)
     if (z > 0) {
 	p = normal_pvalue_1(z);
 	if (!na(p)) {
-	    pprintf(prn, "  Prob(Z > %g) = %g\n", z, p);
+	    pprintf(prn, "  P(Z > %g) = %g\n", z, p);
 	}
     } else if (z < 0) {
 	p = normal_cdf(z);
 	if (!na(p)) {
-	    pprintf(prn, "  Prob(Z < %g) = %g\n", z, p);
+	    pprintf(prn, "  P(Z < %g) = %g\n", z, p);
 	}
     }
 
     if (!na(p) && p > 0) {
-	pprintf(prn, "  Two-tailed p-value = %g\n", 2 * p);
+	pprintf(prn, "  %s = %g\n", _("Two-tailed p-value"), 2 * p);
     }
 }
 
@@ -974,7 +1001,8 @@ signed_rank_test (const double *x, const double *y,
 
     pprintf(prn, "  n = %d\n", n);
     pprintf(prn, "  W+ = %g, W- = %g\n", wp, wm);
-    pprintf(prn, "  (zero differences: %d, non-zero ties: %d)\n", Z, k);
+    pprintf(prn, "  (%s: %d, %s: %d)\n", _("zero differences"), 
+	    _("non-zero ties"), Z, k);
 
     if (n > 8) {
 	double s2, x, num, z;
@@ -996,7 +1024,8 @@ signed_rank_test (const double *x, const double *y,
 	pprintf(prn, "  5%% critical values: %d (two-tailed), %d (one-tailed)\n",
 		rank5[n-6][0], rank5[n-6][1]);
     } else {
-	pprintf(prn, "  Sample too small for statistical significance\n");
+	pprintf(prn, "  %s\n", 
+		_("Sample too small for statistical significance"));
     }
 
     pputc(prn, '\n');
@@ -1119,7 +1148,8 @@ static int rank_sum_test (const double *x, const double *y,
 	    close_plugin(handle);
 	}
     } else {
-	pprintf(prn, "  Sample too small for statistical significance\n");
+	pprintf(prn, "  %s\n", 
+		_("Sample too small for statistical significance"));
     }
 
     pputc(prn, '\n');
@@ -1160,11 +1190,11 @@ static int sign_test (const double *x, const double *y,
     pprintf(prn, _("Under the null hypothesis of no difference, W "
 		   "follows B(%d, %.1f)\n"), n, 0.5);
     pprintf(prn, "  %s(W <= %d) = %g\n", _("Prob"), w, 
-	    binomial_cdf(w, n, 0.5));
+	    binomial_cdf(0.5, n, w));
     if (w == 0) {
 	pv = 1.0;
     } else {
-	pv = binomial_cdf_comp(w - 1, n, 0.5);
+	pv = binomial_cdf_comp(0.5, n, w - 1);
     }
     pprintf(prn, "  %s(W >= %d) = %g\n\n", _("Prob"), w, pv);
 
@@ -1416,7 +1446,7 @@ gretl_matrix *loess_fit (const gretl_matrix *x, const gretl_matrix *y,
     }
 
     if (!data_pre_sorted(x)) {
-	strcpy(gretl_errmsg, "loess: the data must be sorted by x");
+	gretl_errmsg_set("loess: the data must be sorted by x");
 	*err = E_DATA;
 	return NULL;
     }
