@@ -24,8 +24,8 @@
 #include "system.h"
 #include "objstack.h"
 #include "usermat.h"
-#include "modelspec.h"
 #include "forecast.h"
+#include "kalman.h"
 
 #define ODEBUG 0
 
@@ -53,9 +53,63 @@ static GretlObjType get_stacked_type_by_data (void *ptr)
 	}
     }
 
-    return GRETL_OBJ_UNKNOWN;
+    return GRETL_OBJ_NULL;
 }
 
+#if ODEBUG
+
+static int object_get_refcount (void *ptr, GretlObjType type)
+{
+    int rc = -999;
+
+    if (type == GRETL_OBJ_EQN) {
+	MODEL *pmod = (MODEL *) ptr;
+
+	if (pmod != NULL) {
+	    rc = pmod->refcount;
+	}
+	rc = pmod->refcount;
+    } else if (type == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = (GRETL_VAR *) ptr;
+
+	if (var != NULL) {
+	    rc = var->refcount;
+	}
+    } else if (type == GRETL_OBJ_SYS) {
+	equation_system *sys = (equation_system *) ptr;
+
+	if (sys != NULL) {
+	    rc = sys->refcount;
+	}
+    }
+
+    return rc;
+} 
+
+#endif
+
+#if ODEBUG > 1
+
+static void print_ostack (void)
+{
+    fprintf(stderr, "\n*** object stack: n_obj = %d\n", n_obj);
+
+    if (n_obj > 0) {
+	void *p;
+	int i, t;
+
+	for (i=0; i<n_obj; i++) {
+	    p = ostack[i].ptr;
+	    t = ostack[i].type;
+	    fprintf(stderr, " %d: %p (type=%d, refcount=%d)\n", i, p, 
+		    t, object_get_refcount(p, t));
+	}
+	fputc('\n', stderr);
+    }
+}
+
+#endif
+
 /**
  * gretl_object_ref:
  * @ptr: pointer to gretl obejct (e.g. #MODEL).
@@ -92,29 +146,62 @@ void gretl_object_ref (void *ptr, GretlObjType type)
     }
 }
 
-static void gretl_object_unstack (void *ptr)
+enum {
+    UNSTACK_REMOVE,
+    UNSTACK_DESTROY
+};
+
+static int unstack_replace;
+
+static void gretl_object_unstack (void *ptr, int action)
 {
     int i, pos = -1;
 
+#if ODEBUG
+    fprintf(stderr, "gretl_object_unstack: ptr=%p, action=%d\n", ptr, action);
+#endif
+
+    if (action == UNSTACK_DESTROY && ptr == last_model.ptr) {
+#if ODEBUG
+	fprintf(stderr, " %p is 'last_model'\n", ptr);
+#endif
+	/* avoid double-freeing */
+	last_model.ptr = NULL;
+	last_model.type = GRETL_OBJ_NULL;
+    }
+
+    if (unstack_replace) {
+	return;
+    }
+
     for (i=0; i<n_obj; i++) {
 	if (ptr == ostack[i].ptr) {
 	    pos = i;
 	    break;
 	}
     }
+
+#if ODEBUG
+    fprintf(stderr, " stack pos for %p = %d, n_obj = %d\n",
+	    ptr, pos, n_obj);
+#endif
     
     if (pos >= 0) {
-	stacker *new_stack;
-
-	for (i=pos; i<n_obj-1; i++) {
-	    ostack[i] = ostack[i+1];
-	}
-
 	n_obj--;
+	if (n_obj == 0) {
+	    free(ostack);
+	    ostack = NULL;
+	} else {
+	    stacker *new_stack;
 
-	new_stack = realloc(ostack, (n_obj - 1) * sizeof *new_stack);
-	if (new_stack != NULL) {
-	    ostack = new_stack;
+	    for (i=pos; i<n_obj; i++) {
+		ostack[i] = ostack[i+1];
+	    }
+
+	    new_stack = realloc(ostack, n_obj * sizeof *new_stack);
+	    if (new_stack != NULL) {
+		ostack = new_stack;
+	    }
 	}
     }
 }
@@ -126,7 +213,7 @@ static void gretl_object_destroy (void *ptr, GretlObjType type)
 	    ptr, type);
 #endif
 
-    gretl_object_unstack(ptr);
+    gretl_object_unstack(ptr, UNSTACK_DESTROY);
     
     if (type == GRETL_OBJ_EQN) {
 	gretl_model_free(ptr);
@@ -137,34 +224,6 @@ static void gretl_object_destroy (void *ptr, GretlObjType type)
     }
 }
 
-static int gretl_object_get_refcount (void *ptr, GretlObjType type)
-{
-    int rc = -999;
-
-    if (type == GRETL_OBJ_EQN) {
-	MODEL *pmod = (MODEL *) ptr;
-
-	if (pmod != NULL) {
-	    rc = pmod->refcount;
-	}
-	rc = pmod->refcount;
-    } else if (type == GRETL_OBJ_VAR) {
-	GRETL_VAR *var = (GRETL_VAR *) ptr;
-
-	if (var != NULL) {
-	    rc = var->refcount;
-	}
-    } else if (type == GRETL_OBJ_SYS) {
-	equation_system *sys = (equation_system *) ptr;
-
-	if (sys != NULL) {
-	    rc = sys->refcount;
-	}
-    }
-
-    return rc;
-}    
-
 /* The stuff below: Note that we can't "protect" a model (against
    deletion) simply by setting its refcount to some special value,
    when models are being reused, since the refcount will go to 0 every
@@ -265,17 +324,17 @@ static int model_is_protected (MODEL *pmod)
 
 void gretl_object_unref (void *ptr, GretlObjType type)
 {
-    int rc = 999;
+    int *rc = NULL;
+
+    if (ptr == NULL) {
+	/* no-op */
+	return;
+    }
 
     if (type == GRETL_OBJ_ANY) {
 	type = get_stacked_type_by_data(ptr);
     } 
 
-#if ODEBUG
-    fprintf(stderr, "gretl_object_unref: %p (incoming count = %d)\n", 
-	    ptr, gretl_object_get_refcount(ptr, type));
-#endif
-
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = (MODEL *) ptr;
 
@@ -283,28 +342,41 @@ void gretl_object_unref (void *ptr, GretlObjType type)
 	    if (model_is_protected(pmod)) {
 		return; /* note */
 	    }
-	    pmod->refcount -= 1;
-	    rc = pmod->refcount;
+	    rc = &pmod->refcount;
 	}
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) ptr;
 
 	if (var != NULL) {
-	    var->refcount -= 1;
-	    rc = var->refcount;
+	    rc = &var->refcount;
 	}
     } else if (type == GRETL_OBJ_SYS) {
 	equation_system *sys = (equation_system *) ptr;
 
 	if (sys != NULL) {
-	    sys->refcount -= 1;
-	    rc = sys->refcount;
+	    rc = &sys->refcount;
+	}
+    } else {
+	fprintf(stderr, "gretl_object_unref: %p: bad object type\n",
+		ptr);
+	return;
+    }
+
+#if ODEBUG
+    fprintf(stderr, "gretl_object_unref: %p (incoming count = %d)\n", 
+	    ptr, *rc);
+#endif
+
+    if (rc != NULL) {
+	*rc -= 1;
+	if (*rc <= 0) {
+	    gretl_object_destroy(ptr, type);
 	}
     }
 
-    if (rc <= 0) {
-	gretl_object_destroy(ptr, type);
-    }
+#if ODEBUG > 1
+    print_ostack();
+#endif
 }
 
 /**
@@ -325,7 +397,7 @@ void set_as_last_model (void *ptr, GretlObjType type)
 
     if (last_model.ptr != ptr && last_model.ptr != NULL) {
 #if ODEBUG
-	fprintf(stderr, " kicking old object at %p (type %d) off the stack\n",
+	fprintf(stderr, " unrefing old object at %p (type %d)\n",
 		last_model.ptr, last_model.type);
 #endif
 	gretl_object_unref(last_model.ptr, last_model.type);
@@ -334,12 +406,16 @@ void set_as_last_model (void *ptr, GretlObjType type)
     if (last_model.ptr != ptr || last_model.type != type) {
 	last_model.ptr = ptr;
 	last_model.type = type;
-	gretl_object_ref(ptr, type);
+	if (ptr != NULL) {
+	    gretl_object_ref(ptr, type);
+	}
     }
 
 #if ODEBUG
-    fprintf(stderr, " refcount on \"last_model\" = %d\n",
-	    gretl_object_get_refcount(last_model.ptr, last_model.type));
+    if (last_model.ptr != NULL) {
+	fprintf(stderr, " refcount on \"last_model\" = %d\n",
+		object_get_refcount(last_model.ptr, last_model.type));
+    }
 #endif
 }
 
@@ -367,21 +443,32 @@ void *get_last_model (GretlObjType *type)
  *
  * Returns: the name of the object of type @type with
  * location @p, or %NULL if the object is not found.
+ * The return value may be ovewritten (up to
+ * MAXSAVENAME-1 characters), but must not be freed.
  */
 
-const char *gretl_object_get_name (void *p, GretlObjType type)
+char *gretl_object_get_name (void *p, GretlObjType type)
 {
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = p;
 
+	if (pmod->name == NULL) {
+	    pmod->name = calloc(MAXSAVENAME, 1);
+	}
 	return pmod->name;
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = p;
 
+	if (var->name == NULL) {
+	    var->name = calloc(MAXSAVENAME, 1);
+	}
 	return var->name;
     } else if (type == GRETL_OBJ_SYS) {
 	equation_system *sys = p;
 
+	if (sys->name == NULL) {
+	    sys->name = calloc(MAXSAVENAME, 1);
+	}
 	return sys->name;
     }
 
@@ -569,6 +656,53 @@ int gretl_object_compose_name (void *p, GretlObjType type)
     return err;
 }
 
+int gretl_object_compose_unique_name (void *p, GretlObjType type)
+{
+    char name[32];
+    int id, err = 0;
+
+    if (type == GRETL_OBJ_EQN) {
+	MODEL *pmod = (MODEL *) p;
+	
+	id = pmod->ID;
+	sprintf(name, "%s %d", _("Model"), id);
+	while (get_model_by_name(name) != NULL) {
+	    sprintf(name, "%s %d", _("Model"), ++id);
+	}
+	gretl_model_set_name(pmod, name);
+    } else if (type == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = (GRETL_VAR *) p;
+
+	if (var->ci == VAR) {
+	    id = ++n_vars;
+	    sprintf(name, "%s %d", _("VAR"), id);
+	    while (get_VAR_by_name(name) != NULL) {
+		sprintf(name, "%s %d", _("VAR"), ++id);
+	    }
+	} else {
+	    id = gretl_VECM_id(var);
+	    sprintf(name, "%s %d", _("VECM"), id);
+	    while (get_VECM_by_name(name) != NULL) {
+		sprintf(name, "%s %d", _("VECM"), ++id);
+	    }
+	}
+	gretl_VAR_set_name(var, name);
+    } else if (type == GRETL_OBJ_SYS) {
+	equation_system *sys = (equation_system *) p;
+	
+	id = ++n_sys;
+	sprintf(name, "%s %d", _("System"), id);
+	while (get_equation_system_by_name(name) != NULL) {
+	    sprintf(name, "%s %d", _("System"), ++id);
+	}
+	equation_system_set_name(sys, name);
+    } else {
+	err = 1;
+    }
+
+    return err;
+}
+
 int gretl_object_rename (void *p, GretlObjType type, const char *oname)
 {
     int err = 0;
@@ -588,7 +722,7 @@ int gretl_object_rename (void *p, GretlObjType type, const char *oname)
 
 /* safety measure prior to freeing models on exit */
 
-void remove_model_from_stack (MODEL *pmod)
+void remove_model_from_stack_on_exit (MODEL *pmod)
 {
     int i;
 
@@ -608,6 +742,15 @@ void remove_model_from_stack (MODEL *pmod)
     gretl_model_unprotect(pmod);
 }
 
+void gretl_object_remove_from_stack (void *ptr, GretlObjType type)
+{
+#if ODEBUG
+    fprintf(stderr, "gretl_object_remove_from_stack\n");
+#endif
+    gretl_object_unstack(ptr, UNSTACK_REMOVE);
+    gretl_object_unref(ptr, type);
+}
+
 static int object_stack_index (const void *p)
 {
     int i, ret = -1;
@@ -650,10 +793,13 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
     }
 
     if (object_stack_index(p) >= 0) {
+#if ODEBUG
+	fprintf(stderr, " real_stack_object: done, no-op\n");
+#endif
 	return 0;
     }
 
-    if (name == NULL) {
+    if (name == NULL || *name == '\0') {
 	err = gretl_object_compose_name(p, type);
     } else {
 	err = gretl_object_rename(p, type, name);
@@ -668,9 +814,12 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
     if (orig != NULL) {
 	/* replace existing object of same name */
 #if ODEBUG
-	fprintf(stderr, "stack_object: replacing at %p\n", orig->ptr);
+	fprintf(stderr, "  replacing at %p (onum = %d, ptr = %p)\n", 
+		orig, onum, orig->ptr);
 #endif
+	unstack_replace = 1;
 	gretl_object_unref(orig->ptr, orig->type);
+	unstack_replace = 0;
 	ostack[onum].ptr = p;
 	ostack[onum].type = type;
 	pprintf(prn, "Replaced object '%s'\n", name);
@@ -688,20 +837,25 @@ real_stack_object (void *p, GretlObjType type, const char *name, PRN *prn)
 	gretl_object_ref(p, type);
 	n_obj++;
 	pprintf(prn, "Added object '%s'\n", name);
-
     }
 
 #if ODEBUG
-    fprintf(stderr, "stack_object: name='%s', type=%d, ptr=%p (n_obj=%d)\n",
+    fprintf(stderr, " real_stack_object, on exit: '%s', type=%d, ptr=%p (n_obj=%d)\n",
 	    name, type, (void *) p, n_obj);
-#endif  
+#endif 
+
+#if ODEBUG > 1
+    print_ostack();
+#endif
 
     return 0;
 }
 
 int gretl_stack_object (void *ptr, GretlObjType type)
 {
-    return real_stack_object(ptr, type, NULL, NULL);
+    char *name = gretl_object_get_name(ptr, type);
+
+    return real_stack_object(ptr, type, name, NULL);
 }
 
 int gretl_stack_object_as (void *ptr, GretlObjType type, const char *name)
@@ -709,36 +863,25 @@ int gretl_stack_object_as (void *ptr, GretlObjType type, const char *name)
     return real_stack_object(ptr, type, name, NULL);
 }
 
-int maybe_stack_var (GRETL_VAR *var, const CMD *cmd)
+int maybe_stack_var (GRETL_VAR *var, CMD *cmd)
 {
-    char vname[MAXSAVENAME];
-    int ret = 0;
+    const char *name;
+    int err = 0;
 
-    if (var == NULL) {
-	return 0;
+    if (var != NULL) {
+	set_as_last_model(var, GRETL_OBJ_VAR);
+
+	name = gretl_cmd_get_savename(cmd);
+
+	if (*name != '\0') {
+	    err = real_stack_object(var, GRETL_OBJ_VAR, name, NULL);
+	} 
     }
 
-    gretl_cmd_get_savename(vname);
-
-#if ODEBUG
-    fprintf(stderr, "\nmaybe_stack_var: initial refcount = %d\n", var->refcount);
-#endif
-
-    set_as_last_model(var, GRETL_OBJ_VAR);
-
-#if ODEBUG
-    fprintf(stderr, "maybe_stack_var: set %p as last model, refcount = %d\n", 
-	    (void *) var, var->refcount);
-#endif
-
-    if (*vname) {
-	ret = real_stack_object(var, GRETL_OBJ_VAR, vname, NULL);
-    } 
-
-    return ret;
+    return err;
 }
 
-/* Called in gretlcli.c, after sucessful estimation of a
+/* Called from interact.c, after sucessful estimation of a
    (single-equation) model.  We automatically put the model in place
    as the "last model" for reference purposes (e.g. in genr).  In
    addition, if the model has been assigned a "savename" (via
@@ -748,49 +891,41 @@ int maybe_stack_var (GRETL_VAR *var, const CMD *cmd)
    estimation command, we do not lose the saved (named) model content.
 
    Reference counting: the model that is passed in should have its
-   refcount raised to 2 on account of set_as_last_model.  The refcount of
+   refcount raised by 1 on account of set_as_last_model.  The refcount of
    the copy (if applicable) should be 1, since the only reference to
    this model pointer is the one on the stack of named objects.
+
+   We return a pointer to the model as stacked, in case any further
+   action has to be taken (e.g. in the GUI).
 */
 
-int maybe_stack_model (MODEL *pmod, const CMD *cmd, PRN *prn)
+MODEL *maybe_stack_model (MODEL *pmod, CMD *cmd, PRN *prn, int *err)
 {
-    char mname[MAXSAVENAME];
-    MODEL *cpy = NULL;
-    int err = 0;
+    const char *name = gretl_cmd_get_savename(cmd);
+    MODEL *smod = NULL;
 
-    gretl_cmd_get_savename(mname);
+    if (*name != '\0') {
+	MODEL *cpy = gretl_model_copy(pmod);
 
-#if ODEBUG
-    fprintf(stderr, "\nmaybe_stack_model: initial refcount = %d\n", 
-	    pmod->refcount);
-#endif
+	if (cpy == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    *err = real_stack_object(cpy, GRETL_OBJ_EQN, name, NULL);
+	}
 
-    set_as_last_model(pmod, GRETL_OBJ_EQN);
-
-#if ODEBUG
-    fprintf(stderr, "maybe_stack_model: set %p as last model, refcount %d\n",
-	    (void *) pmod, pmod->refcount);
-#endif
-
-    if (*mname == 0) {
-	return 0;
+	if (!*err) {
+	    set_as_last_model(cpy, GRETL_OBJ_EQN);
+	    pprintf(prn, _("%s saved\n"), name);
+	} else {
+	    errmsg(*err, prn);
+	}
+	smod = cpy;
+    } else {
+	set_as_last_model(pmod, GRETL_OBJ_EQN);
+	smod = pmod;
     }
 
-    cpy = gretl_model_copy(pmod);
-    if (cpy == NULL) {
-	err = E_ALLOC;
-    }
-
-    if (!err) {
-	err = real_stack_object(cpy, GRETL_OBJ_EQN, mname, NULL);
-    }
-
-    if (!err) {
-	pprintf(prn, _("%s saved\n"), mname);
-    }
-
-    return err;
+    return smod;
 }
 
 #define INVALID_STAT -999.999
@@ -798,7 +933,9 @@ int maybe_stack_model (MODEL *pmod, const CMD *cmd, PRN *prn)
 /* retrieve from an object some value that is stored on the object in
    the form of a simple scalar */
 
-static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
+static double real_get_obj_scalar (void *p, GretlObjType type, 
+				   double ***pZ, DATAINFO *pdinfo,
+				   int idx)
 {
     double x = INVALID_STAT;
     int err = 0;
@@ -810,7 +947,7 @@ static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
     if (type == GRETL_OBJ_EQN) {
 	MODEL *pmod = (MODEL *) p;
 
-	x = gretl_model_get_scalar(pmod, idx, &err);
+	x = gretl_model_get_scalar(pmod, idx, pZ, pdinfo, &err);
 	if (err) {
 	    x = INVALID_STAT;
 	}
@@ -823,12 +960,18 @@ static double real_get_obj_scalar (void *p, GretlObjType type, int idx)
 	    x = sys->ll;
 	} else if (idx == M_ESS) {
 	    x = sys->ess;
+	} else if (idx == M_DF) {
+	    x = sys->df;
 	}
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) p;
 
 	if (idx == M_T) {
 	    x = var->T;
+	} else if (idx == M_DF) {
+	    x = var->df;
+	} else if (idx == M_NCOEFF) {
+	    x = var->ncoeff;
 	} else if (idx == M_LNL) {
 	    x = var->ll;
 	} else if (idx == M_AIC) {
@@ -894,6 +1037,27 @@ real_get_obj_matrix (void *p, GretlObjType type, int idx, int *err)
     return M;
 }
 
+static int *
+real_get_obj_list (void *p, GretlObjType type, int idx, int *err)
+{
+    int *list = NULL;
+    
+    if (idx <= 0) {
+	*err = 1;
+	return list;
+    }
+
+    if (type == GRETL_OBJ_EQN && idx == M_XLIST) {
+	MODEL *pmod = (MODEL *) p;
+
+	list = gretl_model_get_x_list(pmod);
+    } else {
+	*err = E_BADSTAT;
+    }
+
+    return list;
+}
+
 static stacker genr_model;
 
 void set_genr_model (MODEL *pmod)
@@ -904,7 +1068,7 @@ void set_genr_model (MODEL *pmod)
 
 void unset_genr_model (void)
 {
-    genr_model.type = GRETL_OBJ_UNKNOWN;
+    genr_model.type = GRETL_OBJ_NULL;
     genr_model.ptr = NULL;
 }
 
@@ -943,21 +1107,50 @@ static stacker *find_smatch (const char *oname)
     return smatch;
 }
 
-/* retrieve the type of a named model, or of the last model
-   if @name is NULL */
+/* Retrieve the type of a named model, or of the last model if @name
+   is NULL.  If it's a single-equation model and @ci is non-NULL,
+   also retrieve the model's command index in @ci.
+*/
 
-GretlObjType gretl_model_get_type (const char *name)
+GretlObjType gretl_model_get_type_and_ci (const char *name,
+					  int *ci)
 {
     stacker *smatch = find_smatch(name);
+    GretlObjType ret = GRETL_OBJ_NULL;
 
-    if (smatch == NULL) {
-	return GRETL_OBJ_UNKNOWN;
-    } else {
-	return smatch->type;
+    if (ci != NULL) {
+	*ci = 0;
     }
+
+    if (smatch != NULL) {
+	ret = smatch->type;
+	if (ret == GRETL_OBJ_EQN && ci != NULL) {
+	    MODEL *pmod = smatch->ptr;
+
+	    *ci = pmod->ci;
+	}
+    }
+
+    return ret;
 }
 
-double saved_object_get_scalar (const char *oname, int idx, int *err)
+int *saved_object_get_list (const char *oname, int idx, int *err)
+{
+    int *ret = NULL;
+    stacker *smatch;
+
+    smatch = find_smatch(oname);
+
+    if (smatch != NULL) {
+	ret = real_get_obj_list(smatch->ptr, smatch->type, idx, err);
+    }
+
+    return ret;
+}
+
+double saved_object_get_scalar (const char *oname, int idx, 
+				double ***pZ, DATAINFO *pdinfo,
+				int *err)
 {
     double ret = INVALID_STAT;
     stacker *smatch;
@@ -965,7 +1158,8 @@ double saved_object_get_scalar (const char *oname, int idx, int *err)
     smatch = find_smatch(oname);
 
     if (smatch != NULL) {
-	ret = real_get_obj_scalar(smatch->ptr, smatch->type, idx);
+	ret = real_get_obj_scalar(smatch->ptr, smatch->type, 
+				  pZ, pdinfo, idx);
     }
 
     if (ret == INVALID_STAT) {
@@ -1009,6 +1203,8 @@ saved_object_get_matrix (const char *oname, int idx, int *err)
 
     if (idx == M_FCAST || idx == M_FCERR) {
 	M = get_forecast_matrix(idx, err);
+    } else if (idx == M_KLLT || idx == M_KUHAT) {
+	M = user_kalman_get_matrix(idx, err);
     } else {
 	stacker *smatch = find_smatch(oname);
 
@@ -1024,15 +1220,15 @@ saved_object_get_matrix (const char *oname, int idx, int *err)
     return M;
 }
 
-static int varchar_spn_with_space (const char *s)
+static int namechar_spn_with_space (const char *s)
 {
-    const char *varchars = "abcdefghijklmnopqrstuvwxyz"
+    const char *ok = "abcdefghijklmnopqrstuvwxyz"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 	"0123456789_ ";
     int ret = 0;
 
     if (isalpha(*s)) {
-	ret = strspn(s, varchars);
+	ret = strspn(s, ok);
     }
 
     return ret;
@@ -1064,9 +1260,9 @@ int parse_object_command (const char *s, char *name, char **cmd)
     }
 
     if (quoted) {
-	len = varchar_spn_with_space(s);
+	len = namechar_spn_with_space(s);
     } else {
-	len = gretl_varchar_spn(s);
+	len = gretl_namechar_spn(s);
     }
 
     if (len == 0) {
@@ -1159,6 +1355,8 @@ static void saved_object_free (stacker *s)
     }
 }
 
+#define sys_modtest_opt_ok(o) (o & (OPT_A | OPT_H | OPT_N))
+
 int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo, 
 			PRN *prn)
 {
@@ -1169,7 +1367,7 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
     ptr = get_last_model(&type);  
     if (ptr == NULL) {
 	pputs(prn, _("Can't do this: no model has been estimated yet\n"));
-	return 1;
+	return E_DATA;
     }
 
     if (type == GRETL_OBJ_EQN) {
@@ -1181,11 +1379,14 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
 	if (model_sample_problem(pmod, pdinfo)) {
 	    pputs(prn, _("Can't do: the current data set is different from "
 			 "the one on which\nthe reference model was estimated\n"));
-	    err = 1;
+	    err = E_DATA;
 	}
     } else if (type == GRETL_OBJ_SYS) {
-	if (ci != RESTRICT && ci != TESTUHAT && ci != FCAST) {
-	    err = E_NOTIMP;
+	err = E_NOTIMP;
+	if (ci == RESTRICT || ci == FCAST) {
+	    err = 0;
+	} else if (ci == MODTEST && sys_modtest_opt_ok(opt)) {
+	    err = 0;
 	}
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) ptr;
@@ -1195,22 +1396,18 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
 
 	if (ci == RESTRICT && r > 0) {
 	    err = 0;
-	} else if (ci == TESTUHAT || ci == FCAST) {
+	} else if (ci == FCAST) {
 	    err = 0;
-	} else if (ci == LMTEST && ((opt & OPT_A) || (opt & OPT_H))) {
+	} else if (ci == MODTEST && sys_modtest_opt_ok(opt)) {
 	    err = 0;
 	} 
     }
 
-    if (err == E_NOTIMP) {
-	pputs(prn, _("Sorry, command not available for this estimator"));
-	pputc(prn, '\n');
-    }
-
     return err;
 }
 
-int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn)
+int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, 
+			  gretlopt opt, PRN *prn)
 {
     GretlObjType type;
     void *ptr;
@@ -1228,7 +1425,7 @@ int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, PRN *prn)
 	    gretl_model_get_int(pmod, "ordered")) {
 	    err = E_NOTIMP;
 	} else if (!exact_fit_check(pmod, prn)) {
-	    err = model_error_dist(ptr, pZ, pdinfo, prn);
+	    err = model_error_dist(ptr, pZ, pdinfo, opt, prn);
 	}
     } else if (type == GRETL_OBJ_SYS) {
 	err = system_normality_test(ptr, prn);
@@ -1285,9 +1482,9 @@ int check_variable_deletion_list (int *list, const DATAINFO *pdinfo)
 
     vsave = highest_numbered_var_in_saved_object(pdinfo);
 
-    for (i=1; i<=list[0]; i++) {
+    for (i=list[0]; i>0; i--) {
 	if (list[i] <= vsave) {
-	    gretl_list_delete_at_pos(list, i--);
+	    gretl_list_delete_at_pos(list, i);
 	    pruned = 1;
 	}
     }
@@ -1297,18 +1494,22 @@ int check_variable_deletion_list (int *list, const DATAINFO *pdinfo)
 
 void gretl_saved_objects_cleanup (void)
 {
+    void *lmp = last_model.ptr;
+    int lmt = last_model.type;
     int i;
 
+#if ODEBUG
+    fprintf(stderr, "gretl_saved_objects_cleanup, n_obj = %d\n", n_obj);
+#endif
+
     for (i=0; i<n_obj; i++) {
-	if (ostack[i].ptr == last_model.ptr) {
-	    if (gretl_object_get_refcount(ostack[i].ptr, ostack[i].type) == 1) {
-		/* don't double-free! */
-		last_model.ptr = NULL;
-	    }
+	if (ostack[i].ptr == lmp) {
+	    /* don't double-free! */
+	    lmp = NULL;
 	}
 #if ODEBUG
-	fprintf(stderr, "gretl_saved_objects_cleanup:\n"
-		" calling saved_object_free on ostack[%d]\n", i);
+	fprintf(stderr, " calling saved_object_free on ostack[%d] (%p)\n", 
+		i, ostack[i].ptr);
 #endif
 	saved_object_free(&ostack[i]);
     }
@@ -1320,16 +1521,16 @@ void gretl_saved_objects_cleanup (void)
     n_sys = 0;
     n_vars = 0;
 
-    if (last_model.ptr != NULL) {
-	if (last_model.type != GRETL_OBJ_EQN || 
-	    !model_is_protected(last_model.ptr)) {
+    if (lmp != NULL) {
+	if (lmt != GRETL_OBJ_EQN || !model_is_protected(lmp)) {
 #if ODEBUG
 	    fprintf(stderr, "gretl_saved_objects_cleanup:\n"
-		    " calling gretl_object_destroy on last_model\n");
+		    " calling gretl_object_destroy on last_model (%p)\n", lmp);
 #endif
-	    gretl_object_destroy(last_model.ptr, last_model.type);
+	    gretl_object_destroy(lmp, lmt);
 	}
 	last_model.ptr = NULL;
 	last_model.type = 0;
     }
 }
+
