@@ -23,11 +23,52 @@
 #include "libset.h"
 #include "gretl_func.h"
 #include "gretl_string_table.h"
+#include "gretl_scalar.h"
+#include "matrix_extra.h"
 
 #include <errno.h>
 
 #define PSDEBUG 0
 
+/* For the point of "x *= (1 + 0x1p-52)" below, see
+   http://sourceware.org/bugzilla/show_bug.cgi?id=4943 ,
+   contribution from Eric Postpischil, 2007-10-05.
+*/
+
+static void printf_series (int v, const double **Z,
+			   const DATAINFO *pdinfo,
+			   const char *fmt, 
+			   int wid, int prec, 
+			   int wstar, int pstar,
+			   PRN *prn)
+{
+    char label[OBSLEN];
+    double x;
+    int n, t;
+
+    n = max_obs_label_length(pdinfo) + 1;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	get_obs_string(label, t, pdinfo);
+	pprintf(prn, "%*s ", n, label);
+	x = Z[v][t];
+	if (na(x)) {
+	    pputc(prn, '\n');
+	    continue;
+	}
+	x *= (1 + 0x1p-52);
+	if (wstar && pstar) {
+	    pprintf(prn, fmt, wid, prec, x);
+	} else if (wstar || pstar) {
+	    wid = (wstar)? wid : prec;
+	    pprintf(prn, fmt, wid, x);
+	} else {
+	    pprintf(prn, fmt, x);
+	}
+	pputc(prn, '\n');
+    }
+}
+
 static int printf_escape (int c, PRN *prn)
 {
     switch (c) {
@@ -59,75 +100,121 @@ static char *printf_get_string (char *s, double ***pZ,
 				DATAINFO *pdinfo, int t, 
 				int *err)
 {
-    const char *p = NULL;
-    const char *q = NULL;
     char *ret = NULL;
-    int len = 0;
 
-    /* special, not yet in genr */
-
-    if (!strncmp(s, "marker", 6) && pdinfo->S != NULL) {
-	/* observation label */
+    if (!strncmp(s, "marker", 6) && pdinfo != NULL && pdinfo->S != NULL) {
+	/* special, not in regular genr */
+	const char *p = pdinfo->S[t];
+	const char *q = s + 6;
+	int len = strlen(p);
 	int offset = 0;
 
-	p = pdinfo->S[t];
-	len = strlen(p);
-	q = s + 6;
 	while (isspace(*q)) q++;
+
 	if (*q == '+') {
-	    q++;
-	    offset = atoi(q);
+	    offset = atoi(++q);
 	    len -= offset;
 	}
 	if (len >= 0) {
 	    ret = gretl_strndup(p + offset, len);
+	    if (ret == NULL) {
+		*err = E_ALLOC;
+	    }
 	}
     } else {
 	ret = generate_string(s, pZ, pdinfo, err);
     }
 
-    if (ret == NULL) {
-	ret = gretl_strdup("");
-    }
-
-    if (ret == NULL) {
-	*err = E_ALLOC;
+    if (ret == NULL && !*err) {
+	*err = E_DATA;
     }
 
     return ret;
 }
 
-static double printf_get_scalar (char *s, double ***pZ,
-				 DATAINFO *pdinfo, int t, 
-				 int *err)
+/* here we're trying to get a field width or precision
+   specifier */
+
+static int printf_get_int (const char *s, double ***pZ,
+			   DATAINFO *pdinfo, int *err)
 {
-    double x = NADBL;
-    int v;
+    double x;
+    int ret = 0;
 
 #if PSDEBUG
-    fprintf(stderr, "printf_get_scalar: looking at '%s'\n", s);
+    fprintf(stderr, "printf_get_int: looking at '%s'\n", s);
 #endif
 
     if (numeric_string(s)) {
-	return atof(s);
-    }
-
-    v = varindex(pdinfo, s);
-
-    if (v < pdinfo->v && var_is_series(pdinfo, v)) {
-	char genstr[32];
-
-	sprintf(genstr, "%s[%d]", s, t + 1);
-	x = generate_scalar(genstr, pZ, pdinfo, err);
+	x = dot_atof(s);
+    } else if (gretl_is_scalar(s)) {
+	x = gretl_scalar_get_value(s);
     } else {
 	x = generate_scalar(s, pZ, pdinfo, err);
     }
 
+    if (!*err && (xna(x) || fabs(x) > 255)) {
+	*err = E_DATA;
+    } 
+
+    if (!*err) {
+	ret = (int) x;
+    }
+
 #if PSDEBUG
-    fprintf(stderr, "printf_get_scalar: returning %g\n", x);
+    fprintf(stderr, "printf_get_int: returning %g\n", x);
 #endif
 
-    return x;
+    return ret;
+}
+
+/* gen_arg_val: when this function is called we have already
+   determined that the argument string @s is not associated with a
+   string format specifier, is not a literal numerical value, and is
+   not the name of an existing scalar, matrix or series.  So now we
+   attempt to generate a value (of type unknown).
+*/
+
+static int gen_arg_val (const char *s, double ***pZ, DATAINFO *pdinfo, 
+			double *px, gretl_matrix **pm, int *pv,
+			int *scalar)
+{
+    const char *name = "$ptmp";
+    char *genstr;
+    int err = 0;
+
+    genstr = gretl_strdup_printf("%s=%s", name, s); 
+    if (genstr == NULL) {
+	return E_ALLOC;
+    }
+
+    err = generate(genstr, pZ, pdinfo, OPT_P, NULL);
+
+    if (!err) {
+	int type = genr_get_last_output_type();
+
+	if (type == GRETL_TYPE_DOUBLE) {
+	    *scalar = 1;
+	    *px = gretl_scalar_get_value(name);
+	    gretl_scalar_delete(name, NULL);
+	} else if (type == GRETL_TYPE_MATRIX) {
+	    gretl_matrix *m = get_matrix_by_name(name);
+
+	    *pm = gretl_matrix_copy(m);
+	    if (*pm == NULL) {
+		err = E_ALLOC;
+	    }
+	    user_matrix_destroy_by_name(name, NULL);
+	} else if (type == GRETL_TYPE_SERIES) {
+	    *pv = current_series_index(pdinfo, name);
+	} else {
+	    err = E_TYPES; /* FIXME cleanup? */
+	}
+    }
+
+    free(genstr);
+
+    return err;
 }
 
 /* dup argv (s) up to the next free comma */
@@ -193,7 +280,7 @@ get_printf_format_chunk (const char *s, int *fc,
 			 int *len, int *wstar, int *pstar,
 			 int *err)
 {
-    const char *cnvchars = "eEfgGxduxs";
+    const char *cnvchars = "eEfgGduxs";
     const char *numchars = "0123456789";
     char *chunk = NULL;
     const char *p = s;
@@ -280,8 +367,11 @@ get_printf_format_chunk (const char *s, int *fc,
     return chunk;
 }
 
-/* extract the next conversion spec from *pfmt, find and evaluate the
-   corresponding elements in *pargs, and print the result */
+/* Extract the next conversion spec from *pfmt, find and evaluate the
+   corresponding elements in *pargs, and print the result. Note
+   that @t should be negative (and therefore ignored) unless
+   we're doing the "genr markers" special thing.
+*/
 
 static int print_arg (char **pfmt, char **pargs, 
 		      double ***pZ, DATAINFO *pdinfo,
@@ -291,10 +381,15 @@ static int print_arg (char **pfmt, char **pargs,
     char *fmt = NULL;
     char *arg = NULL;
     char *str = NULL;
+    gretl_matrix *m = NULL;
+    int free_m = 0;
+    int series_v = -1;
+    int free_v = 0;
     double x = NADBL;
     int flen = 0, alen = 0;
     int wstar = 0, pstar = 0;
     int wid = 0, prec = 0;
+    int got_scalar = 0;
     int fc = 0;
     int err = 0;
 
@@ -315,10 +410,7 @@ static int print_arg (char **pfmt, char **pargs,
 	/* evaluate field width specifier */
 	arg = get_next_arg(*pargs, &alen, &err);
 	if (!err) {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && (na(x) || fabs(x) > 255)) {
-		err = E_DATA;
-	    }
+	    x = printf_get_int(arg, pZ, pdinfo, &err);
 	}
 	free(arg);
 	if (err) {
@@ -332,10 +424,7 @@ static int print_arg (char **pfmt, char **pargs,
 	/* evaluate precision specifier */
 	arg = get_next_arg(*pargs, &alen, &err);
 	if (!err) {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && (na(x) || fabs(x) > 255)) {
-		err = E_DATA;
-	    }
+	    x = printf_get_int(arg, pZ, pdinfo, &err);
 	}
 	free(arg);
 	if (err) {
@@ -347,27 +436,69 @@ static int print_arg (char **pfmt, char **pargs,
 
     /* get next substantive arg */
     arg = get_next_arg(*pargs, &alen, &err);
+
     if (!err) {
+	int v = -1;
+
 	if (fc == 's') {
+	    /* must be printing a string */
 	    str = printf_get_string(arg, pZ, pdinfo, t, &err);
-	} else {
-	    x = printf_get_scalar(arg, pZ, pdinfo, t, &err);
-	    if (!err && na(x)) {
-		fc = fmt[flen - 1] = 's';
-		str = gretl_strdup("NA");
+	} else if (numeric_string(arg)) {
+	    /* printing a scalar value */
+	    x = dot_atof(arg);
+	    got_scalar = 1;
+	} else if (gretl_is_scalar(arg)) {
+	    /* printing a named scalar */
+	    x = gretl_scalar_get_value(arg);
+	    got_scalar = 1;
+	} else if ((m = get_matrix_by_name(arg)) != NULL) {
+	    ; /* printing a named matrix */
+	} else if ((v = current_series_index(pdinfo, arg)) >= 0) {
+	    /* printing a named series, unless t >= 0 */
+	    if (t < 0) {
+		series_v = v; 
+	    } else {
+		x = (*pZ)[v][t];
+		got_scalar = 1;
 	    }
-	}
+	} else {
+	    /* try treating as generated value */
+	    err = gen_arg_val(arg, pZ, pdinfo, &x, &m, &v, &got_scalar);
+	    if (m != NULL) {
+		free_m = 1;
+	    } else if (v > 0) {
+		series_v = v;
+		free_v = 1;
+	    }
+	} 
 	*pargs += alen;
     }
+
     free(arg);
 
     if (err) {
 	goto bailout;
     } 
 
+    if (got_scalar && na(x)) {
+	fc = fmt[flen - 1] = 's';
+	str = gretl_strdup("NA");
+    }	
+
     /* do the actual printing */
 
-    if (fc == 's') {
+    if (m != NULL) {
+	/* printing a matrix */
+	if (!wstar) wid = -1;
+	if (!pstar) prec = -1;
+	gretl_matrix_print_with_format(m, fmt, wid, prec, prn);
+    } else if (series_v >= 0) {
+	/* printing a series */
+	printf_series(series_v, (const double **) *pZ,
+		      pdinfo, fmt, wid, prec, 
+		      wstar, pstar, prn);
+    } else if (fc == 's') {
+	/* printing a string */
 	if (wstar && pstar) {
 	    pprintf(prn, fmt, wid, prec, str);
 	} else if (wstar || pstar) {
@@ -377,6 +508,7 @@ static int print_arg (char **pfmt, char **pargs,
 	    pprintf(prn, fmt, str);
 	}
     } else if (strchr(intconv, fc)) {
+	/* printing a scalar as int */
 	if (wstar && pstar) {
 	    pprintf(prn, fmt, wid, prec, (int) x);
 	} else if (wstar || pstar) {
@@ -385,7 +517,11 @@ static int print_arg (char **pfmt, char **pargs,
 	} else {
 	    pprintf(prn, fmt, (int) x);
 	}
-    } else {
+    } else if (got_scalar) {
+	/* printing a (non-missing) scalar value */
+	if (!isnan(x) && !isinf(x)) {
+	    x *= (1 + 0x1p-52);
+	}
 	if (wstar && pstar) {
 	    pprintf(prn, fmt, wid, prec, x);
 	} else if (wstar || pstar) {
@@ -405,6 +541,12 @@ static int print_arg (char **pfmt, char **pargs,
     free(fmt);
     free(str);
     
+    if (free_m) {
+	gretl_matrix_free(m);
+    } else if (free_v) {
+	dataset_drop_last_variables(1, pZ, pdinfo);
+    }
+    
     return err;
 }
 
@@ -414,7 +556,7 @@ static int split_printf_line (const char *s, char *targ, int *sp,
 			      char **format, char **args)
 {
     const char *p;
-    int n;
+    int n, err = 0;
 
     *sp = 0;
 
@@ -428,14 +570,14 @@ static int split_printf_line (const char *s, char *targ, int *sp,
     if (*sp) {
 	/* need a target name */
 	s += strspn(s, " ");
-	n = gretl_varchar_spn(s);
-	if (n == 0 || n >= VNAMELEN) {
-	    return E_PARSE;
-	} else {
-	    *targ = '\0';
-	    strncat(targ, s, n);
-	    s += n;
+	err = extract_varname(targ, s, &n);
+	if (!err && n == 0) {
+	    err = E_PARSE;
 	}
+	if (err) {
+	    return err;
+	}
+	s += n;
 	/* allow comma after target */
 	s += strspn(s, " ");
 	if (*s == ',') {
@@ -462,7 +604,7 @@ static int split_printf_line (const char *s, char *targ, int *sp,
 
     if (n == 0) {
 	/* empty format string */
-	return 0; 
+	return 0;
     }
 
     *format = gretl_strndup(p, n);
@@ -495,7 +637,6 @@ static int real_do_printf (const char *line, double ***pZ,
 			   int t)
 {
     PRN *prn = inprn;
-    char *p, *q;
     char targ[VNAMELEN];
     char *format = NULL;
     char *args = NULL;
@@ -505,10 +646,6 @@ static int real_do_printf (const char *line, double ***pZ,
 
     *targ = '\0';
 
-    if (t < 0) {
-	t = pdinfo->t1;
-    }
-
     err = split_printf_line(line, targ, &sp, &format, &args);
     if (err) {
 	return err;
@@ -532,21 +669,28 @@ static int real_do_printf (const char *line, double ***pZ,
     fprintf(stderr, " args = '%s'\n", args);
 #endif
 
-    p = format;
-    q = args;
+    if (format != NULL) {
+	char *p = format;
+	char *q = args;
 
-    while (*p && !err) {
-	if (*p == '%' && *(p+1) == '%') {
-	    pputc(prn, '%');
-	    p += 2;
-	} else if (*p == '%') {
-	    err = print_arg(&p, &q, pZ, pdinfo, t, prn);
-	} else if (*p == '\\') {
-	    err = printf_escape(*(p+1), prn);
-	    p += 2;
-	} else {
-	    pputc(prn, *p);
-	    p++;
+	while (*p && !err) {
+	    if (*p == '%' && *(p+1) == '%') {
+		pputc(prn, '%');
+		p += 2;
+	    } else if (*p == '%') {
+		err = print_arg(&p, &q, pZ, pdinfo, t, prn);
+	    } else if (*p == '\\') {
+		err = printf_escape(*(p+1), prn);
+		p += 2;
+	    } else {
+		pputc(prn, *p);
+		p++;
+	    }
+	}
+
+	if (q != NULL && *q != '\0') {
+	    pprintf(prn, "\nunmatched argument '%s'", q);
+	    err = E_PARSE;
 	}
     }
 
@@ -573,7 +717,7 @@ static int real_do_printf (const char *line, double ***pZ,
  * @pdinfo: dataset information.
  * @prn: printing struct.
  *
- * Implement a somewhat limited version of C's printf
+ * Implements a somewhat limited version of C's printf()
  * for use in gretl scripts.
  *
  * Returns: 0 on success, non-zero on error.
@@ -590,16 +734,13 @@ int do_printf (const char *line, double ***pZ,
 static int 
 sscanf_target_var (const char *vname, DATAINFO *pdinfo, int *err)
 {
-    int v = varindex(pdinfo, vname);
-
-    if (v <= 0 || v >= pdinfo->v) {
-	*err = E_UNKVAR;
-    } else if (!var_is_scalar(pdinfo, v)) {
-	strcpy(gretl_errmsg, _("sscanf: numerical target must be scalar"));
+    if (gretl_is_scalar(vname)) {
+	return 0;
+    } else {
+	gretl_errmsg_set(_("sscanf: numerical target must be scalar"));
 	*err = E_DATA;
+	return 0;
     }
-
-    return v;
 }
 
 struct bracket_scan {
@@ -607,6 +748,27 @@ struct bracket_scan {
     char *chrs;
 };
 
+static int bscan_escape (char *s)
+{
+    int i, n = strlen(s);
+
+    for (i=0; i<n; i++) {
+	if (s[i] == '\\' && (i == 0 || s[i-1] != '\\')) {
+	    if (s[i+1] == 'n') {
+		s[i] = '\n';
+		shift_string_left(s + i + 1, 1);
+		i++;
+	    } else if (s[i+1] == 't') {
+		s[i] = '\t';
+		shift_string_left(s + i + 1, 1);
+		i++;
+	    }		
+	}
+    }
+
+    return 0;
+}
+
 static int 
 parse_bracket_scanner (const char **pfmt, struct bracket_scan *bscan)
 {
@@ -645,6 +807,8 @@ parse_bracket_scanner (const char **pfmt, struct bracket_scan *bscan)
 	return E_ALLOC;
     }
 
+    bscan_escape(bscan->chrs);
+
 #if PSDEBUG
     fprintf(stderr, "bracket_scan: reverse = %d, chars = '%s'\n",
 	    bscan->reverse, bscan->chrs);
@@ -740,13 +904,19 @@ scan_string (const char *targ, char **psrc, int width,
 	n = strcspn(*psrc, " \t\n");
     }
 
+#if PSDEBUG
+    fprintf(stderr, "scan_string: *psrc='%s', n=%d\n", *psrc, n);
+    int m = strcspn(*psrc, "\n");
+    fprintf(stderr, "m=%d (chrs='%s')\n", m, bscan->chrs);
+#endif
+
     if (width != WIDTH_UNSPEC && n > width) {
 	n = width;
     }
 
     if (targ != NULL) {
 	if (!is_user_string(targ)) {
-	    sprintf(gretl_errmsg, _("%s: not a string variable"), targ);
+	    gretl_errmsg_sprintf(_("%s: not a string variable"), targ);
 	    err = E_UNKVAR;
 	} else if (n > 0) {
 	    char *conv = gretl_strndup(*psrc, n);
@@ -813,8 +983,12 @@ static int scan_scalar (const char *targ, char **psrc,
 	}
     }
 
-    if (v > 0 && !errno && endp != *psrc) {
-	Z[v][0] = (fc == 'd')? k : x;
+    if (!errno && endp != *psrc) {
+	if (v == 0) {
+	    gretl_scalar_set_value(targ, (fc == 'd')? k : x);
+	} else if (v > 0) {
+	    Z[v][0] = (fc == 'd')? k : x;
+	}
 	*ns += 1;
     }
 
@@ -947,6 +1121,9 @@ static int scan_arg (char **psrc, char **pfmt, char **pargs,
     /* do the actual scanning */
 
     if (!err && fc == 's') {
+#if PSDEBUG
+	fprintf(stderr, "calling scan_string\n");
+#endif
 	err = scan_string(targ, psrc, wid, &bscan, ns);
     } else if (!err && fc == 'm') {
 	err = scan_matrix(targ, psrc, wid, ns);
@@ -977,18 +1154,89 @@ static int get_literal_length (const char *s)
     return n;
 }
 
+static char *get_literal_or_stringvar (const char **src, int *err)
+{
+    const char *s = *src;
+    char *ret = NULL;
+    char *tmp = NULL;
+    int literal = 0;
+    int n = 0;
+
+    if (*s == '"') {
+	literal = 1;
+	s++;
+	n = get_literal_length(s);
+    } else {
+	if (*s == '@') s++;
+	n = gretl_namechar_spn(s);
+    } 
+
+    if (n == 0) {
+	/* nothing there */
+	*err = E_PARSE;
+	return NULL;
+    }
+
+    tmp = gretl_strndup(s, n);
+    if (tmp == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    if (literal) {
+	ret = tmp;
+	s++;
+    } else {
+	char *svar = get_string_by_name(tmp);
+
+	if (svar == NULL) {
+	    *err = E_UNKVAR;
+	} else {
+	    ret = gretl_strdup(svar);
+	    if (ret == NULL) {
+		*err = E_ALLOC;
+	    }
+	}
+	free(tmp);
+    }
+
+    /* advance the caller's pointer */
+    *src = s + n;
+
+    return ret;
+}
+
+/* Eat white space; detect comma and eat if found; eat more white
+   space.  Return 1 if comma was found else 0.
+*/
+
+int eat_comma (const char **src) 
+{
+    const char *s = *src;
+    int ret = 0;
+
+    s += strspn(s, " ");
+
+    if (*s == ',') {
+	ret = 1;
+	s++;
+	s += strspn(s, " ");
+    }
+
+    *src = s;
+
+    return ret;
+}
+
 /* split line into source string, format and arguments: 
    acceptable sources are string literals or string variables
 */
 
 static int 
-split_scanf_line (const char *s, char **src, char **format, 
-		  char **args, int *literal)
+split_scanf_line (const char *s, char **psrc, char **pfmt, 
+		  char **args)
 {
-    const char *p;
-    char *srcname;
-    int gotcom = 0;
-    int n = 0;
+    int gotcom, err = 0;
 
     if (!strncmp(s, "sscanf ", 7)) {
 	s += 7;
@@ -996,88 +1244,32 @@ split_scanf_line (const char *s, char **src, char **format,
 
     s += strspn(s, " ");
 
-    /* string literal or string variable as source */
-    if (*s == '"') {
-	*literal = 1;
-	s++;
-	n = get_literal_length(s);
-    } else {
-	if (*s == '@') s++;
-	n = gretl_varchar_spn(s);
-    } 
-
-    if (n == 0) {
+    /* get the source string */
+    *psrc = get_literal_or_stringvar(&s, &err);
+    if (err) {
+	return err;
+    }
+    
+    /* skip to format string */
+    gotcom = eat_comma(&s);
+    if (!gotcom) {
+	/* empty format */
 	return E_PARSE;
     }
 
-    srcname = gretl_strndup(s, n);
-    if (srcname == NULL) {
-	return E_ALLOC;
-    }
-
-    if (*literal) {
-	*src = srcname;
-	s++;
-    } else {
-	*src = get_string_by_name(srcname);
-	free(srcname);
-	if (*src == NULL) {
-	    return E_UNKVAR;
-	}
-    }
-
-    s += n;
-    if (*s == ',') {
-	gotcom = 1;
-	s++;
-    }
-
-    /* skip to next field */
-    s += strspn(s, " ");
-    if (!gotcom && *s == ',') {
-	s++;
-	s += strspn(s, " ");
-    }
-
     /* get format string */
-    if (*s != '"' || *(s+1) == '\0') {
+    *pfmt = get_literal_or_stringvar(&s, &err);
+    if (err) {
+	return err;
+    }
+
+    /* skip to first arg */
+    gotcom = eat_comma(&s);
+    if (!gotcom) {
+	/* empty args */
 	return E_PARSE;
     }
 
-    s++;
-    p = s;
-
-    n = 0;
-    while (*s) {
-	if (*s == '"' && *(s-1) != '\\') {
-	    break;
-	}
-	n++;
-	s++;
-    }
-
-    if (n == 0) {
-	/* empty format string */
-	return 0; 
-    }
-
-    *format = gretl_strndup(p, n);
-    if (*format == NULL) {
-	return E_ALLOC;
-    }
-
-    s++;
-    s += strspn(s, " ");
-
-    if (*s != ',') {
-	/* empty args */
-	*args = NULL;
-	return 0;
-    }
-
-    s++;
-    s += strspn(s, " ");
-
     *args = gretl_strdup(s);
     if (*args == NULL) {
 	return E_ALLOC;
@@ -1100,7 +1292,7 @@ int n_scanned_items (void)
  * @pdinfo: dataset information.
  * @prn: printing struct.
  *
- * Implement a somewhat limited version of C's sscanf
+ * Implements a somewhat limited version of C's sscanf()
  * for use in gretl scripts.
  *
  * Returns: 0 on success, non-zero on error.
@@ -1113,19 +1305,21 @@ int do_sscanf (const char *line, double ***pZ,
     char *src = NULL;
     char *format = NULL;
     char *args = NULL;
-    int literal = 0;
     int err = 0;
 
     nscan = 0;
     gretl_error_clear();
 
-    err = split_scanf_line(line, &src, &format, &args, &literal);
+#if PSDEBUG
+    fprintf(stderr, "do_sscanf: line = '%s'\n", line);
+#endif
+
+    err = split_scanf_line(line, &src, &format, &args);
     if (err) {
 	return err;
     }
 
 #if PSDEBUG
-    fprintf(stderr, "do_sscanf: line = '%s'\n", line);
     fprintf(stderr, " src = '%s'\n", src);
     fprintf(stderr, " format = '%s'\n", format);
     fprintf(stderr, " args = '%s'\n", args);
@@ -1156,14 +1350,11 @@ int do_sscanf (const char *line, double ***pZ,
 	}
     }
 
-    if (literal) {
-	free(src);
-    }
-
+    free(src);
     free(format);
     free(args);
 
-    if (!err && gretl_messages_on()) {
+    if (!err && gretl_messages_on() && !gretl_looping_quietly()) {
 	pprintf(prn, "Number of items successfully scanned = %d\n", 
 		nscan);
     }
@@ -1176,7 +1367,7 @@ int do_sscanf (const char *line, double ***pZ,
      genr markers = format, arg1,...
 
    We're assuming that we're just getting the part after
-   the equals sign here, in the variable s.
+   the equals sign here, in the variable @s.
 */
 
 int generate_obs_markers (const char *s, double ***pZ, DATAINFO *pdinfo)
