@@ -25,6 +25,16 @@
 #include <time.h>
 #include <glib.h>
 
+/**
+ * SECTION:strutils
+ * @short_description: miscellaneous string-handling utilities
+ * @title: Strings
+ * @include: libgretl.h
+ *
+ * Various functions for creating, testing and manipulating
+ * strings and arrays of strings.
+ */
+
 /**
  * string_is_blank:
  * @s: the string to examine.
@@ -50,6 +60,13 @@ int string_is_blank (const char *s)
     return ret;
 }
 
+static int atof_point;
+
+void set_atof_point (char c)
+{
+    atof_point = c;
+}
+
 /**
  * dot_atof:
  * @s: the string to convert.
@@ -61,13 +78,26 @@ int string_is_blank (const char *s)
 
 double dot_atof (const char *s)
 {
+#ifndef ENABLE_NLS
+    return atof(s);
+#else
     double x;
 
-    gretl_push_c_numeric_locale();
-    x = atof(s);
-    gretl_pop_c_numeric_locale();
+    if (atof_point == 0) {
+	struct lconv *lc = localeconv();
+
+	atof_point = *lc->decimal_point;
+    }
+    if (atof_point == '.') {
+	x = atof(s);
+    } else {
+	gretl_push_c_numeric_locale();
+	x = atof(s);
+	gretl_pop_c_numeric_locale();
+    }
 
     return x;
+#endif
 }
 
 /**
@@ -154,7 +184,7 @@ char *delchar (int c, char *str)
  * @idx: the starting point for deleting characters.
  * @count: the number of characters to delete.
  *
- * Deletes @count characters from @str, starting at position @indx.
+ * Deletes @count characters from @str, starting at position @idx.
  *
  * Returns: the modified string.
  */
@@ -171,7 +201,7 @@ char *gretl_delete (char *str, int idx, int count)
 }
 
 /**
- * haschar:
+ * charpos:
  * @c: the character to look for.
  * @s: the string to examine.
  *
@@ -179,7 +209,7 @@ char *gretl_delete (char *str, int idx, int count)
  * found.
  */
 
-int haschar (char c, const char *s)
+int charpos (char c, const char *s)
 {
     int i = 0;
 
@@ -237,30 +267,66 @@ char *charsub (char *str, char find, char repl)
     return str;
 }
 
+/**
+ * comma_separate_numbers:
+ * @s: the string to operate on.
+ *
+ * Given a string which contains two or more numbers
+ * separated by spaces and/or commas, revise the
+ * string to ensure that all the numbers are comma-separated.
+ *
+ * Returns: the (possibly modified) string.
+ */
+
+char *comma_separate_numbers (char *s) 
+{
+    const char *numstart = "+-.0123456789";
+    char *p = s;
+    int i, n, done;
+
+    while (*s) {
+	n = strspn(s, " ,");
+	if (n > 0 && s[n] != '\0' && strchr(numstart, s[n])) {
+	    done = 0;
+	    for (i=0; i<n && !done; i++) {
+		if (s[i] == ',') {
+		    done = 1;
+		}
+	    }
+	    if (!done) {
+		*s = ',';
+	    }
+	}
+	s += (n > 0)? n : 1;
+    }
+
+    return p;
+}
+
 /**
  * has_suffix:
  * @str: the string to check.
- * @sfx: the suffix to check for.
+ * @sfx: the suffix to check for, including the leading '.'
  *
  * Returns: 1 if @str ends with @sfx (on a case-insensitive
- * comparison), 0 otherwise.
+ * comparison), 0 otherwise.  
  */
 
 int has_suffix (const char *str, const char *sfx)
 {
-    int diff, ret = 0;
+    const char *p;
+    int ret = 0;
 
     if (str != NULL && sfx != NULL) {
-	diff = strlen(str) - strlen(sfx);
-	if (diff >= 0) {
+	p = strrchr(str, '.');
+	if (p != NULL && strlen(p) == strlen(sfx)) {
 	    ret = 1;
-	    str += diff;
-	    while (*str) {
-		if (*str != *sfx && *str != toupper(*sfx)) {
+	    while (*p) {
+		if (*p != *sfx && *p != toupper(*sfx)) {
 		    ret = 0;
 		    break;
 		}
-		str++;
+		p++;
 		sfx++;
 	    }
 	}
@@ -304,6 +370,32 @@ int numeric_string (const char *str)
     return ret;
 }
 
+/**
+ * integer_string:
+ * @str: the string to examine.
+ *
+ * Returns: 1 if the given @str represents an integer, otherwise 0.
+ */
+
+int integer_string (const char *str)
+{
+    char *test;
+    int ret = 1;
+
+    if (str == NULL || *str == '\0') {
+	return 0;
+    }
+
+    errno = 0;
+
+    strtol(str, &test, 10);
+    if (*test != '\0' || errno != 0) {
+	ret = 0;
+    }
+
+    return ret;
+}
+
 /**
  * ends_with_backslash:
  * @s: the string to examine.
@@ -356,7 +448,7 @@ char *lower (char *str)
  * gretl_strdup:
  * @src: the string to duplicate.
  *
- * Returns: an allocated copy of @src, or %NULL on error.
+ * Returns: an allocated copy of @src, or NULL on error.
  */
 
 char *gretl_strdup (const char *src)
@@ -379,7 +471,7 @@ char *gretl_strdup (const char *src)
  * @n: the maximum number of characters to copy.
  *
  * Returns: an allocated copy of at most @n characters from 
- * @src, or %NULL on error.
+ * @src, or NULL on error.
  */
 
 char *gretl_strndup (const char *src, size_t n)
@@ -405,15 +497,15 @@ char *gretl_strndup (const char *src, size_t n)
 
 /**
  * gretl_strdup_printf:
- * @template: as in printf().
+ * @format: as in printf().
  * @Varargs: arguments to be printed.
  *
  * Print the arguments according to @format.
  * 
- * Returns: allocated result of the printing, or %NULL on failure.
+ * Returns: allocated result of the printing, or NULL on failure.
  */
 
-char *gretl_strdup_printf (const char *template, ...)
+char *gretl_strdup_printf (const char *format, ...)
 {
     va_list args;
     int plen, bsize = 2048;
@@ -426,8 +518,8 @@ char *gretl_strdup_printf (const char *template, ...)
 
     memset(buf, 0, 1);
 
-    va_start(args, template);
-    plen = vsnprintf(buf, bsize, template, args);
+    va_start(args, format);
+    plen = vsnprintf(buf, bsize, format, args);
     va_end(args);
 
     if (plen >= bsize) {
@@ -442,16 +534,16 @@ char *gretl_strdup_printf (const char *template, ...)
  * gretl_str_expand:
  * @orig: pointer to the base string.
  * @add: the string to be added.
- * @sep: string to be interpolated, or %NULL.
+ * @sep: string to be interpolated, or NULL.
  *
  * Creates a newly allocated string built by concatenating
  * @orig and @add, with @sep interpolated unless @sep is
- * %NULL, and replaces the content of @orig with the new string.
- * As a special case, if @orig is %NULL, or if the content of
- * @orig is %NULL, we just duplicate @add.
+ * NULL, and replaces the content of @orig with the new string.
+ * As a special case, if @orig is NULL, or if the content of
+ * @orig is NULL, we just duplicate @add.
  *
- * Returns: the reallocated string, or %NULL on failure.  In case
- * of failure the content of @orig is freed, if @orig is not %NULL,
+ * Returns: the reallocated string, or NULL on failure.  In case
+ * of failure the content of @orig is freed, if @orig is not NULL,
  * to avoid memory leakage.
  */
 
@@ -495,15 +587,15 @@ char *gretl_str_expand (char **orig, const char *add, const char *sep)
 /**
  * gretl_word_strdup:
  * @src: the source string.
- * @ptr: location to receive end of word pointer, or %NULL.
+ * @ptr: location to receive end of word pointer, or NULL.
  *
  * Copies the first 'word' found in @src, where a word
  * is defined as consisting of alphanumeric characters
- * and the underscore.  If @ptr is not %NULL, on exit it
+ * and the underscore.  If @ptr is not NULL, on exit it
  * points at the next position in @src after the copied
  * word.
  *
- * Returns: the allocated word or %NULL in case no word is
+ * Returns: the allocated word or NULL in case no word is
  * found, or if allocation fails.
  */
 
@@ -546,6 +638,58 @@ char *gretl_word_strdup (const char *src, const char **ptr)
     return targ;
 }
 
+/**
+ * gretl_quoted_string_strdup:
+ * @s: the source string.
+ * @ptr: location to receive end pointer, or NULL.
+ *
+ * If @s starts with a quote (double or single), return a copy of  
+ * the portion of @s that is enclosed in quotes.  That is, 
+ * from @s + 1 up to but not including the next matching quote.
+ * If @ptr is not NULL, on output it receives a pointer to
+ * the next byte in @s after the closing quote.
+ *
+ * Returns: the allocated string or NULL on failure.
+ */
+
+char *gretl_quoted_string_strdup (const char *s, const char **ptr)
+{
+    char q, *ret = NULL;
+    const char *p = NULL;
+
+    if (s != NULL && (*s == '"' || *s == '\'')) {
+	int gotit = 0;
+
+	q = *s;
+	s++;
+	p = s;
+	while (*p && !gotit) {
+	    if (*p == q && *(p-1) != '\\') {
+		/* found non-escaped matching quote */
+		gotit = 1;
+	    } else {
+		p++;
+	    }
+	}
+	if (!gotit) {
+	    p = NULL;
+	}
+    }
+
+    if (p == NULL) {
+	if (ptr != NULL) {
+	    *ptr = NULL;
+	}
+    } else {
+	if (ptr != NULL) {
+	    *ptr = p + 1;
+	}
+	ret = gretl_strndup(s, p - s);
+    }
+
+    return ret;
+}
+
 /**
  * gretl_string_split:
  * @s: the source string.
@@ -555,7 +699,7 @@ char *gretl_word_strdup (const char *src, const char **ptr)
  * by one or more spaces, and creates an array of those substrings. 
  * On sucessful exit, @n holds the number of substrings. 
  *
- * Returns: the allocated array or %NULL in case of failure.
+ * Returns: the allocated array or NULL in case of failure.
  */
 
 char **gretl_string_split (const char *s, int *n)
@@ -592,52 +736,6 @@ char **gretl_string_split (const char *s, int *n)
     return S;
 }
 
-/**
- * gretl_double_from_string:
- * @s: the source string.
- * @ptr: location to receive end of field pointer, or %NULL.
- *
- * Tries to parse the first 'word' found in @s as a 
- * floating-point number, and returns either the number or
- * #NADBL on failure.  If the parsing was successful and if
- * @ptr is not %NULL, on exit it points at the next position 
- * in @s after the just-parsed numeric portion.
- *
- * Returns: floating-point value or #NADBL.
- */
-
-double gretl_double_from_string (const char *s, const char **ptr)
-{
-    double x = NADBL;
-
-    if (s == NULL) {
-	if (ptr != NULL) {
-	    *ptr = NULL;
-	}
-    } else if (*s == '\0') {
-	if (ptr != NULL) {
-	    *ptr = s;
-	}
-    } else {
-	int n = strspn(s, " ");
-
-	s += n;
-	n = strcspn(s, " ");
-	if (n <= 31) {
-	    if (sscanf(s, "%lf", &x) != 1) {
-		x = NADBL;
-		n = 0;
-	    } 
-	}
-
-	if (n > 0 && ptr != NULL) {
-	    *ptr = s + n;
-	}
-    }
-
-    return x;
-}
-
 /**
  * gretl_trunc:
  * @str: the string to truncate.
@@ -658,7 +756,7 @@ char *gretl_trunc (char *str, size_t n)
 }
 
 /**
- * gretl_varchar_spn:
+ * gretl_namechar_spn:
  * @s: the string to examine.
  *
  * Returns: the length of the intial segment of @s which
@@ -667,15 +765,15 @@ char *gretl_trunc (char *str, size_t n)
  * starting with a letter.
  */
 
-int gretl_varchar_spn (const char *s)
+int gretl_namechar_spn (const char *s)
 {
-    const char *varchars = "abcdefghijklmnopqrstuvwxyz"
+    const char *ok = "abcdefghijklmnopqrstuvwxyz"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 	"0123456789_";
     int ret = 0;
 
     if (isalpha(*s)) {
-	ret = strspn(s, varchars);
+	ret = strspn(s, ok);
     }
 
     return ret;
@@ -778,7 +876,7 @@ char *chopstr (char *str)
 
     n = strlen(str);
 
-    for (i = n - 1; i >= 0; i--) {
+    for (i=n-1; i>=0; i--) {
 	if (isspace(str[i]) || str[i] == '\r') {
 	    str[i] = '\0';
 	} else {
@@ -851,25 +949,35 @@ int get_base (char *targ, const char *src, char c)
 /**
  * top_n_tail:
  * @str: the string to process.
+ * @maxlen: maximum length of string, including NUL termination.
+ * @err: location to receive error code, or NULL.
  *
  * Drop leading space and trailing space and newline from string,
  * then replace a trailing backslash (if any) with a space.
+ * If @str does not end with a newline within the limit set by
+ * @maxlen, and @err is not NULL, then E_TOOLONG is written 
+ * to @err.
  * 
- * Returns: 1 if a trailing backslash was found, otherwise 0.
+ * Returns: 1 if a trailing backslash or comma was found, 
+ * otherwise 0.
  */
 
-int top_n_tail (char *str)
+int top_n_tail (char *str, size_t maxlen, int *err)
 {
-    int i, len, bs = 0;
+    int i, n, cont = 0;
 
     if (str == NULL || *str == 0 || *str == '\n' || *str == '\r') {
 	return 0;
     }
 
-    len = strlen(str);
+    n = strlen(str) - 1;
+
+    if (err != NULL && n > maxlen - 2 && str[n] != '\n') {
+	*err = E_TOOLONG;
+    }
 
     /* chop any trailing space */
-    for (i=len-1; i>=0; i--) {
+    for (i=n; i>=0; i--) {
 	if (isspace((unsigned char) str[i])) {
 	    str[i] = 0;
 	} else {
@@ -878,9 +986,9 @@ int top_n_tail (char *str)
     }
 
     if (*str != 0) {
-	/* drop any leading spaces, also possible questionmark
-	   2007-11-07: try to catch non-breaking spaces too */
-#if 1
+	/* Drop any leading spaces, also possible questionmark.  Try
+	   to catch non-breaking spaces too -- ugh, Windows!
+	*/
 	i = 0;
 	while (isspace((unsigned char) str[i]) || 
 	       str[i] == '?' ||
@@ -888,22 +996,28 @@ int top_n_tail (char *str)
 	       str[i] == (char) 0xA0) {
 	    i++;
 	}
-#else
-	i = strspn(str, " \t?");
-#endif
 	if (i > 0) {
 	    shift_string_left(str, i);
 	}
 
-	/* replace backslash, if present */
-	len = strlen(str);
-	if (str[len - 1] == '\\') {
-	    str[len - 1] = ' ';
-	    bs = 1;
+	/* does this line start a comment? */
+	if (strchr(str, '#') || !strncmp(str, "/*", 2)) {
+	    ; /* leave well alone */
+	} else {
+	    /* replace backslash, if present */
+	    n = strlen(str) - 1;
+	    if (n >= 0) {
+		if (str[n] == '\\') {
+		    str[n] = ' ';
+		    cont = 1;
+		} else if (str[n] == ',') {
+		    cont = 1;
+		}
+	    }
 	}
     }
 
-    return bs;
+    return cont;
 }  
 
 /**
@@ -1104,9 +1218,9 @@ char *safecpy (char *targ, const char *src, int n)
  * @nstrs: number of strings in array.
  *
  * Allocates storage for @nstrs strings and initalizes all 
- * to %NULL.
+ * to NULL.
  * 
- * Returns: the allocated array, or %NULL on failure.
+ * Returns: the allocated array, or NULL on failure.
  */
 
 char **strings_array_new (int nstrs)
@@ -1152,14 +1266,19 @@ int strings_array_add (char ***pS, int *n, const char *p)
     }
 
     *pS = Tmp;
-    Tmp[m] = gretl_strdup(p);
 
-    if (Tmp[m] == NULL) {
-	return E_ALLOC;
+    if (p != NULL) {
+	Tmp[m] = gretl_strdup(p);
+	if (Tmp[m] == NULL) {
+	    return E_ALLOC;
+	}
     } else {
-	*n += 1;
-	return 0;
+	Tmp[m] = NULL;
     }
+    
+    *n += 1;
+
+    return 0;
 }
 
 /**
@@ -1171,7 +1290,7 @@ int strings_array_add (char ***pS, int *n, const char *p)
  * @len bytes long.  The first byte of each string is
  * initialized to 0.
  * 
- * Returns: the allocated array, or %NULL on failure.
+ * Returns: the allocated array, or NULL on failure.
  */
 
 char **strings_array_new_with_length (int nstrs, int len)
@@ -1201,12 +1320,84 @@ char **strings_array_new_with_length (int nstrs, int len)
     return S;
 }
 
+/**
+ * strings_array_realloc_with_length:
+ * @pS: existing array to reallocate.
+ * @oldn: original number of strings in the array.
+ * @newn: new number of strings in array.
+ * @len: number of bytes per string.
+ *
+ * Adjusts the storage in @pS to a size of @newn
+ * strings, each of them @len bytes long.  The first 
+ * byte of any additional strings is initialized to 0.
+ * This function may be used either to expand or to
+ * shrink an existing array of strings.
+ * 
+ * Returns: the new array, or NULL on failure.
+ */
+
+char **strings_array_realloc_with_length (char ***pS, 
+					  int oldn, 
+					  int newn,
+					  int len)
+{
+    char **S;
+    int i, j;
+
+    if (pS == NULL) {
+	/* huh? */
+	return NULL;
+    }
+
+    if (newn == oldn) {
+	/* no-op */
+	return *pS;
+    }
+
+    if (newn <= 0) {
+	free_strings_array(*pS, oldn);
+	*pS = NULL;
+	return NULL;
+    }
+
+    /* in case we're shrinking the array */
+    for (i=newn; i<oldn; i++) {
+	free((*pS)[i]);
+	(*pS)[i] = NULL;
+    }
+
+    S = realloc(*pS, newn * sizeof *S);
+    if (S == NULL) {
+	free_strings_array(*pS, oldn);
+	*pS = NULL;
+	return NULL;
+    }
+
+    *pS = S;
+
+    /* in case we're expanding the array */
+    for (i=oldn; i<newn; i++) {
+	S[i] = malloc(len);
+	if (S[i] == NULL) {
+	    for (j=0; j<i; j++) {
+		free(S[j]);
+	    }
+	    free(*pS);
+	    *pS = NULL;
+	    return NULL;
+	}
+	S[i][0] = '\0';
+    }
+
+    return *pS;
+}
+
 /**
  * strings_array_dup:
  * @strs: array of strings to be copied.
  * @n: number of strings in array.
  *
- * Returns: an allocated copy of @strs, or %NULL on failure.
+ * Returns: an allocated copy of @strs, or NULL on failure.
  */
 
 char **strings_array_dup (char **strs, int n)
@@ -1239,13 +1430,40 @@ char **strings_array_dup (char **strs, int n)
     return S;
 }
 
+/**
+ * strings_array_cmp:
+ * @strs1: first array of strings.
+ * @strs2: second array of strings.
+ * @n: number of strings to examine.
+ *
+ * Compares for equality two arrays of strings, each of
+ * which must contain at least @n elements.  Equality
+ * of the arrays means that strcmp returns 0 for
+ * each pair of strings @strs1[i], @strs2[i], for i
+ * equals 0 to @n - 1.
+ *
+ * Returns: 0 if the arrays compare equal, non-zero
+ * otherwise.
+ */
+
+int strings_array_cmp (char **strs1, char **strs2, int n)
+{
+    int i, ret = 0;
+
+    for (i=0; i<n && !ret; i++) {
+	ret = strcmp(strs1[i], strs2[i]);
+    }
+
+    return ret;
+}
+
 /**
  * free_strings_array:
  * @strs: array of allocated strings.
  * @nstrs: number of strings in array.
  *
  * Frees each allocated string in @strs, then frees @strs itself.
- * Checks that @strs is not %NULL before proceeding.
+ * Checks that @strs is not NULL before proceeding.
  */
 
 void free_strings_array (char **strs, int nstrs)
@@ -1260,38 +1478,6 @@ void free_strings_array (char **strs, int nstrs)
     }
 }
 
-static char *
-real_get_obs_string (char *obs, int t, const DATAINFO *pdinfo, int full)
-{
-    if (pdinfo->markers && pdinfo->S != NULL) { 
-	/* data marker strings present */
-	strcpy(obs, pdinfo->S[t]);
-    } else {
-	if (full) {
-	    ntodate_full(obs, t, pdinfo);
-	} else {
-	    ntodate(obs, t, pdinfo);
-	}
-    }
-
-    if (!full) {
-	if (strlen(obs) > 8) { 
-	    char tmp[9];
-
-	    if (isdigit(*obs) && isdigit(*(obs + 1))) {
-		strcpy(tmp, obs + 2);
-		strcpy(obs, tmp);
-	    } else {
-		*tmp = '\0';
-		strncat(tmp, obs, 8);
-		strcpy(obs, tmp);
-	    }
-	} 
-    }
-
-    return obs;
-}
-
 /**
  * get_obs_string:
  * @obs: char array big enough to hold the observation (#OBSLEN).
@@ -1303,22 +1489,13 @@ real_get_obs_string (char *obs, int t, const DATAINFO *pdinfo, int full)
 
 char *get_obs_string (char *obs, int t, const DATAINFO *pdinfo)
 {
-    return real_get_obs_string(obs, t, pdinfo, 0);
-}
+    if (dataset_has_markers(pdinfo)) { 
+	strcpy(obs, pdinfo->S[t]);
+    } else {
+	ntodate(obs, t, pdinfo);
+    }
 
-/**
- * get_full_obs_string:
- * @obs: char array big enough to hold the observation (#OBSLEN).
- * @t: zero-based observation number.
- * @pdinfo: pointer to dataset information.
- *
- * Returns: the observation string corresponding to @t, using
- * a four-digit year in case of dated daily data.
- */
-
-char *get_full_obs_string (char *obs, int t, const DATAINFO *pdinfo)
-{
-    return real_get_obs_string(obs, t, pdinfo, 1);
+    return obs;
 }
 
 /**
@@ -1406,7 +1583,7 @@ void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
     } else if (pdinfo->structure != CROSS_SECTION) {
 	char tmp[OBSLEN];
 
-	ntodate_full(tmp, t, pdinfo);
+	ntodate(tmp, t, pdinfo);
 	if (quarterly_or_monthly(pdinfo)) {
 	    modify_date_for_csv(tmp, pdinfo->pd);
 	}
@@ -1416,23 +1593,21 @@ void csv_obs_to_prn (int t, const DATAINFO *pdinfo, PRN *prn)
 	
 /**
  * print_time:
- * @timep: time to print.
+ * @s: string into which to print: must be at least 48 bytes.
  *
- * Returns: pointer to a static string containing a locale-dependent
- * representation of @timep.  In English, this will be in the format
- * Y/m/d H:M.
+ * Returns: @s, which will contain a locale-dependent representation 
+ * of the current time.  In English, this will be in the format Y/m/d H:M.
  */
 
-const char *print_time (const time_t *timep)
+char *print_time (char *s)
 {
-    static char timestr[48];
+    time_t now = time(NULL);
     struct tm *local;
 
-    local = localtime(timep);
+    local = localtime(&now);
+    strftime(s, 47, "%Y-%m-%d %H:%M", local);
 
-    strftime(timestr, 47, "%Y/%m/%d %H:%M", local);
-
-    return timestr;
+    return s;
 }
 
 /**
@@ -1481,7 +1656,7 @@ char *gretl_xml_encode (const char *str)
 
     targ = malloc(len);
     if (targ == NULL) {
-	sprintf(gretl_errmsg, _("out of memory in XML encoding"));
+	gretl_errmsg_set(_("out of memory in XML encoding"));
 	return NULL;
     }
 
@@ -1619,7 +1794,7 @@ void unescape_url (char *url)
  * @v: the ID number for the new variable.
  * @pdinfo: dataset information.
  *
- * Given a tenative name for a new variable, check that it
+ * Given a tentative name for a new variable, check that it
  * is not a duplicate of an existing varname.  If it is,
  * modify the new name so that it becomes unique. The ID
  * number @v is required so that, if the variable has already
@@ -1666,6 +1841,7 @@ int fix_varname_duplicates (DATAINFO *pdinfo)
     for (i=1; i<pdinfo->v; i++) {
 	for (j=i+1; j<pdinfo->v; j++) {
 	    if (!strcmp(pdinfo->varname[i], pdinfo->varname[j])) {
+		fprintf(stderr, "'%s' duplicated variable name\n", pdinfo->varname[i]);
 		dups = 1;
 		make_varname_unique(pdinfo->varname[j], j, pdinfo);
 	    }
@@ -1702,7 +1878,7 @@ char *append_dir (char *fname, const char *dir)
  * @targ: target string to write to (must be pre-allocated).
  * @dirname: first part of path.
  * @fname: filename.
- * @ext: filename extension to be appended (or %NULL).
+ * @ext: filename extension to be appended (or NULL).
  *
  * Writes to @targ a full path composed of @dirname,
  * @fname and (optionally) @ext.  This function ensures
@@ -1768,7 +1944,7 @@ const char *path_last_element (const char *path)
 
 #ifdef WIN32
     if (p == NULL) {
-	p = strrchr(path, '\\');
+	p = strrchr(path, '/');
     }
 #endif
 
@@ -1781,26 +1957,6 @@ const char *path_last_element (const char *path)
     return p;
 }
 
-/**
- * ensure_slash:
- * @s: string to work on.
- *
- * If @s does not ends with #SLASH, append this character.
- *
- * Returns: the (possibly) modified string.
- */
-
-char *ensure_slash (char *s)
-{
-    int n = strlen(s);
-
-    if (n > 0 && s[n-1] != SLASH) {
-	strcat(s, SLASHSTR);
-    }
-
-    return s;
-}
-
 /**
  * trim_slash:
  * @s: string to work on.
@@ -1844,57 +2000,24 @@ int gretl_string_ends_with (const char *s, const char *test)
     return ret;
 }
 
-int *varname_match_list (const DATAINFO *pdinfo, const char *pattern)
+/**
+ * get_column_widths:
+ * @strs: array of @n strings.
+ * @widths: array of @n default column widths.
+ * @n: number of columns.
+ *
+ * If need be, increases the column widths in @widths to
+ * accomodate the current translations of @strs.
+ */
+
+void get_column_widths (const char **strs, int *widths, int n)
 {
-    GPatternSpec *pspec;
-    int *list = NULL;
-    int i, n = 0;
+    int i, len;
 
-    pspec = g_pattern_spec_new(pattern);
-
-    for (i=1; i<pdinfo->v; i++) { 
-	if (var_is_series(pdinfo, i) &&
-	    g_pattern_match_string(pspec, pdinfo->varname[i])) {
-	    n++;
+    for (i=0; i<n; i++) {
+	len = g_utf8_strlen(_(strs[i]), -1);
+	if (len > widths[i]) {
+	    widths[i] = len;
 	}
     }
-
-    if (n > 0) {
-	list = malloc((n + 1) * sizeof *list);
-	if (list != NULL) {
-	    int j = 1;
-
-	    list[0] = n;
-	    for (i=1; i<pdinfo->v; i++) { 
-		if (var_is_series(pdinfo, i) &&
-		    g_pattern_match_string(pspec, pdinfo->varname[i])) {
-		    list[j++] = i;
-		}
-	    }
-	}
-    }
-
-    g_pattern_spec_free(pspec);
-
-    return list;
-}
-
-int varname_match_any (const DATAINFO *pdinfo, const char *pattern)
-{
-    GPatternSpec *pspec;
-    int i, ret = 0;
-
-    pspec = g_pattern_spec_new(pattern);
-
-    for (i=1; i<pdinfo->v; i++) { 
-	if (var_is_series(pdinfo, i) &&
-	    g_pattern_match_string(pspec, pdinfo->varname[i])) {
-	    ret = 1;
-	    break;
-	}
-    }
-
-    g_pattern_spec_free(pspec);
-
-    return ret;
 }
