@@ -21,6 +21,16 @@
 
 #define TRDEBUG 0
 
+/**
+ * SECTION:transforms
+ * @short_description: standard transformations of series in the dataset
+ * @title: Transformations 
+ * @include: libgretl.h
+ *
+ * Functions to generate standard transformations (logs, lags, first
+ * differences and so on) of series in the dataset.
+ */
+
 enum {
     VARS_IDENTICAL,
     X_HAS_MISSING,
@@ -82,6 +92,9 @@ make_transform_varname (char *vname, const char *orig, int ci,
     } else if (ci == SDIFF) {
 	strcpy(vname, "sd_");
 	strncat(vname, orig, len - 3);
+    } else if (ci == ORTHDEV) {
+	strcpy(vname, "o_");
+	strncat(vname, orig, len - 2);
     } else if (ci == LOGS) {
 	strcpy(vname, "l_");
 	strncat(vname, orig, len - 2);
@@ -152,39 +165,109 @@ make_transform_label (char *label, const char *parent,
     return err;
 }
 
+/**
+ * is_standard_lag:
+ * @v: ID number of variable to test.
+ * @pdinfo: dataset information.
+ * @parent: location to receive ID number of parent variable,
+ * or NULL.
+ *
+ * Returns: 1 if the variable @v is marked as being a
+ * lag of some "parent" variable in the dataset,
+ * otherwise 0.
+ */
+
 int is_standard_lag (int v, const DATAINFO *pdinfo, int *parent)
 {
     int pv = 0, ret = 0;
 
+    if (pdinfo == NULL || v <= 0 || v >= pdinfo->v) {
+	return 0;
+    }
+
     if (pdinfo->varinfo[v]->transform == LAGS) {
-	pv = varindex(pdinfo, pdinfo->varinfo[v]->parent);
+	pv = series_index(pdinfo, pdinfo->varinfo[v]->parent);
 	pv = (pv < pdinfo->v)? pv : 0;
+	ret = pdinfo->varinfo[v]->lag;
     }
 
     if (pv > 0) {
-	if (parent != NULL) {
-	    *parent = pv;
+	int n = strcspn(pdinfo->varname[v], "_");
+
+	if (!strncmp(pdinfo->varname[v], pdinfo->varname[pv], n)) {
+	    if (parent != NULL) {
+		*parent = pv;
+	    }
+	} else {
+	    /* if the lag has a "custom" name, don't consider it
+	       a child to parent variable pv */
+	    ret = 0;
 	}
-	ret = 1;
     }
 
     return ret;
 }
 
+/**
+ * is_standard_diff:
+ * @v: ID number of variable to test.
+ * @pdinfo: dataset information.
+ * @parent: location to receive ID number of parent variable,
+ * or NULL.
+ *
+ * Returns: 1 if the variable @v is marked as being the first
+ * difference of some "parent" variable in the dataset,
+ * otherwise 0.
+ */
+
+int is_standard_diff (int v, const DATAINFO *pdinfo, int *parent)
+{
+    int pv = 0, ret = 0;
+
+    if (v <= 0 || v >= pdinfo->v) {
+	return 0;
+    }
+
+    if (pdinfo->varinfo[v]->transform == DIFF) {
+	pv = series_index(pdinfo, pdinfo->varinfo[v]->parent);
+	pv = (pv < pdinfo->v)? pv : 0;
+	if (pv > 0) {
+	    if (parent != NULL) {
+		*parent = pv;
+	    }
+	    ret = 1;
+	}
+    }
+
+    return ret;
+}
+
+/**
+ * is_dummy_child:
+ * @v: ID number of variable to test.
+ * @pdinfo: dataset information.
+ * @parent: location to receive ID number of parent variable,
+ * or NULL.
+ *
+ * Returns: 1 if the variable @v is marked as being a dummy
+ * variable that codes for a specific value of some "parent" 
+ * variable in the dataset, otherwise 0.
+ */
+
 int is_dummy_child (int v, const DATAINFO *pdinfo, int *parent)
 {
     int pv = pdinfo->v;
     int i = 0, ret = 0;
 
     if (pdinfo->varinfo[v]->transform == DUMMIFY) {
-	pv = varindex(pdinfo, pdinfo->varinfo[v]->parent);
+	pv = series_index(pdinfo, pdinfo->varinfo[v]->parent);
     } else if (!strncmp(pdinfo->varname[v], "dt_", 3)) {
 	if (sscanf(pdinfo->varname[v] + 3, "%d", &i) && i > 1) {
-	    pv = varindex(pdinfo, "dt_1");
+	    pv = series_index(pdinfo, "dt_1");
 	}
     } else if (!strncmp(pdinfo->varname[v], "du_", 3)) {
 	if (sscanf(pdinfo->varname[v] + 3, "%d", &i) && i > 1) {
-	    pv = varindex(pdinfo, "du_1");
+	    pv = series_index(pdinfo, "du_1");
 	}
     }	
 
@@ -198,16 +281,26 @@ int is_dummy_child (int v, const DATAINFO *pdinfo, int *parent)
     return ret;
 }
 
-static void make_xp_varname (char *vname, const char *v1, 
-			     const char *v2, int len)
+static void make_xp_varname (char *vname, int v1, int v2,
+			     const DATAINFO *pdinfo,
+			     int len)
 {
-    int v2len = len / 2;
-    int v1len = (len % 2)? v2len : v2len - 1;
+    int n1 = strlen(pdinfo->varname[v1]);
+    int n2 = strlen(pdinfo->varname[v2]);
+    int cut = n1 + n2 + 1 - len;
+    int cut1 = 0, cut2 = 0;
+
+    if (cut > 0) {
+	cut1 = cut2 = cut/2;
+	if (cut % 2) {
+	    cut2++;
+	}
+    }
 
     *vname = '\0';
-    strncat(vname, v1, v1len);
+    strncat(vname, pdinfo->varname[v1], n1 - cut1);
     strcat(vname, "_");
-    strncat(vname, v2, v2len);
+    strncat(vname, pdinfo->varname[v2], n2 - cut2);    
 }
 
 /* Array into which to write a generated variable, prior to testing
@@ -249,6 +342,13 @@ static double *testvec (int n)
     return x;
 }
 
+/**
+ * gretl_transforms_cleanup:
+ *
+ * Called by libgretl_cleanup(). Frees any memory allocated
+ * as workspace for the creation of transformed variables.
+ */
+
 void gretl_transforms_cleanup (void)
 {
     testvec(0);
@@ -315,9 +415,10 @@ static int get_log (int v, double *logvec, const double **Z,
     int t, err = 0;
 
     for (t=pdinfo->t1; t<=pdinfo->t2 && !err; t++) {
-	xx = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
+	xx = Z[v][t];
 	if (na(xx) || xx <= 0.0) {
 	    logvec[t] = NADBL;
+	    set_gretl_warning(W_GENMISS);
 	} else {
 	    logvec[t] = log(xx); 
 	}
@@ -360,6 +461,14 @@ static int get_diff (int v, double *diffvec, int ci,
     return 0;
 }
 
+/* orthogonal deviations */
+
+static int get_orthdev (int v, double *xvec, const double **Z, 
+			const DATAINFO *pdinfo)
+{
+    return orthdev_series(Z[v], xvec, pdinfo);
+}
+
 /* write square or cross-product into xvec */
 
 static int get_xpx (int vi, int vj, double *xvec, const double **Z, 
@@ -369,8 +478,8 @@ static int get_xpx (int vi, int vj, double *xvec, const double **Z,
     int t;
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	xit = (var_is_series(pdinfo, vi))? Z[vi][t] : Z[vi][0];
-	xjt = (var_is_series(pdinfo, vj))? Z[vj][t] : Z[vj][0];
+	xit = Z[vi][t];
+	xjt = Z[vj][t];
 	if (na(xit) || na(xjt)) {
 	    xvec[t] = NADBL;
 	} else {
@@ -390,7 +499,7 @@ static int get_inverse (int v, double *xvec, const double **Z,
     int t;
 
     for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	xt = (var_is_series(pdinfo, v))? Z[v][t] : Z[v][0];
+	xt = Z[v][t];
 	if (na(xt) || xt == 0.0) {
 	    xvec[t] = NADBL;
 	} else {
@@ -432,8 +541,8 @@ enum transform_results {
 
 #define TR_OVERWRITE 1
 
-static int transform_handle_duplicate (int v, const double *x,
-				       const char *label,
+static int transform_handle_duplicate (int ci, int lag, int v, 
+				       const double *x, const char *label,
 				       DATAINFO *pdinfo, double **Z,
 				       int origv)
 {
@@ -454,7 +563,7 @@ static int transform_handle_duplicate (int v, const double *x,
 
     if (ok) {
 #if TRDEBUG
-	fprintf(stderr, "check_add_transform: updating var %d (%s)\n",
+	fprintf(stderr, "transform_handle_duplicate: updating var %d (%s)\n",
 		v, pdinfo->varname[v]);
 #endif
 	for (t=0; t<pdinfo->n; t++) {
@@ -463,6 +572,8 @@ static int transform_handle_duplicate (int v, const double *x,
 	if (*label != '\0') {
 	    strcpy(VARLABEL(pdinfo, v), label);
 	}
+	pdinfo->varinfo[v]->transform = ci;
+	pdinfo->varinfo[v]->lag = lag;
 	pdinfo->varinfo[v]->flags = 0;
 	ret = VAR_EXISTS_OK;
     }	
@@ -471,7 +582,7 @@ static int transform_handle_duplicate (int v, const double *x,
 }
 
 static int 
-check_add_transform (int vnum, const double *x,
+check_add_transform (int ci, int lag, int vnum, const double *x,
 		     const char *vname, const char *label,
 		     DATAINFO *pdinfo, double ***pZ,
 		     int origv)
@@ -493,7 +604,8 @@ check_add_transform (int vnum, const double *x,
 
 	if (chk == VARS_IDENTICAL) {
 	    ret = VAR_EXISTS_OK;
-	} else if (chk == X_HAS_MISSING) { /* is this right? */
+	} else if (chk == X_HAS_MISSING) { 
+	    /* is this right? */
 	    ret = VAR_EXISTS_OK;
 	} else if (chk == Y_HAS_MISSING) {
 	    for (t=0; t<pdinfo->n; t++) {
@@ -501,7 +613,7 @@ check_add_transform (int vnum, const double *x,
 	    }
 	    ret = VAR_EXISTS_OK;
 	} else {
-	    ret = transform_handle_duplicate(vnum, x, label, pdinfo,
+	    ret = transform_handle_duplicate(ci, lag, vnum, x, label, pdinfo,
 					     *pZ, origv);
 	}
     } else {
@@ -554,7 +666,8 @@ static int get_transform (int ci, int v, int aux, double x,
 {
     char vname[VNAMELEN] = {0};
     char label[MAXLABEL] = {0};
-    int len, vno = -1, err = 0;
+    int vno = -1, err = 0;
+    int len, lag = 0;
     const char *srcname;
     double *vx;
 
@@ -564,12 +677,14 @@ static int get_transform (int ci, int v, int aux, double x,
     }
 
     if (ci == LAGS) {
-	/* "aux" = lag */
-	err = get_lag(v, aux, vx, (const double **) *pZ, pdinfo);
+	lag = aux;
+	err = get_lag(v, lag, vx, (const double **) *pZ, pdinfo);
     } else if (ci == LOGS) {
 	err = get_log(v, vx, (const double **) *pZ, pdinfo);
     } else if (ci == DIFF || ci == LDIFF || ci == SDIFF) {
 	err = get_diff(v, vx, ci, (const double **) *pZ, pdinfo);
+    } else if (ci == ORTHDEV) {
+	err = get_orthdev(v, vx, (const double **) *pZ, pdinfo);
     } else if (ci == SQUARE) {
 	/* "aux" = second variable number */
 	err = get_xpx(v, aux, vx, (const double **) *pZ, pdinfo);
@@ -586,7 +701,7 @@ static int get_transform (int ci, int v, int aux, double x,
 
     if (ci == LAGS && (vno = get_lag_ID(v, aux, pdinfo)) > 0) {
 	/* special case: pre-existing lag */
-	err = check_add_transform(vno, vx, vname, label, pdinfo, pZ, origv);
+	err = check_add_transform(ci, lag, vno, vx, vname, label, pdinfo, pZ, origv);
 	if (err != VAR_EXISTS_OK) {
 	    vno = -1;
 	}
@@ -612,14 +727,13 @@ static int get_transform (int ci, int v, int aux, double x,
 	    /* last resort: hack the name */
 	    make_varname_unique(vname, 0, pdinfo);
 	} else if (ci == SQUARE && v != aux) {
-	    make_xp_varname(vname, pdinfo->varname[v],
-			    pdinfo->varname[aux], len);
+	    make_xp_varname(vname, v, aux, pdinfo, len);
 	} else {
 	    make_transform_varname(vname, srcname, ci, aux, len);
 	}
-	vno = varindex(pdinfo, vname);
+	vno = series_index(pdinfo, vname);
 
-	err = check_add_transform(vno, vx, vname, label, pdinfo, pZ, origv);
+	err = check_add_transform(ci, lag, vno, vx, vname, label, pdinfo, pZ, origv);
 	if (err != VAR_EXISTS_OK && err != VAR_ADDED_OK) {
 	    vno = -1;
 	}
@@ -637,9 +751,12 @@ static int get_transform (int ci, int v, int aux, double x,
 	} else if (ci == LAGS || ci == DIFF) {
 	    strcpy(pdinfo->varinfo[vno]->parent, pdinfo->varname[v]);
 	    pdinfo->varinfo[vno]->transform = ci;
-	    if (ci == LAGS) {
-		pdinfo->varinfo[vno]->lag = aux;
-	    }
+	}
+
+	if (ci == LAGS) {
+	    pdinfo->varinfo[vno]->lag = aux;
+	} else {
+	    pdinfo->varinfo[vno]->lag = 0;
 	}
     }
 
@@ -663,11 +780,8 @@ int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
 {
     int lno;
 
-    if (var_is_scalar(pdinfo, v)) {
-	sprintf(gretl_errmsg, _("variable %s is a scalar"), pdinfo->varname[v]);
-	lno = -1;
-    } else if (lag > pdinfo->n || -lag > pdinfo->n) {
-	sprintf(gretl_errmsg, _("Invalid lag order %d"), lag);
+    if (lag > pdinfo->n || -lag > pdinfo->n) {
+	gretl_errmsg_sprintf(_("Invalid lag order %d"), lag);
 	lno = -1;
     } else if (lag == 0) {
 	lno = v;
@@ -691,7 +805,7 @@ int laggenr (int v, int lag, double ***pZ, DATAINFO *pdinfo)
  * Creates the specified lags of variable @v if they do not
  * already exist.
  *
- * Returns: list of lag variables, or %NULL or on error.
+ * Returns: list of lag variables, or NULL or on error.
  */
 
 int *laggenr_from_to (int v, int minlag, int maxlag, double ***pZ, 
@@ -779,7 +893,7 @@ int invgenr (int v, double ***pZ, DATAINFO *pdinfo)
 /**
  * diffgenr: 
  * @v: ID number in dataset of source variable.
- * @ci: %DIFF (first difference), %LDIFF (log difference) or %SDIFF
+ * @ci: DIFF (first difference), LDIFF (log difference) or SDIFF
  * (seasonal difference).
  * @pZ: pointer to data array.
  * @pdinfo: information on dataset.
@@ -793,10 +907,6 @@ int invgenr (int v, double ***pZ, DATAINFO *pdinfo)
 
 int diffgenr (int v, int ci, double ***pZ, DATAINFO *pdinfo)
 {
-    if (var_is_scalar(pdinfo, v)) {
-	return -1;
-    }
-
     if (ci != DIFF && ci != LDIFF && ci != SDIFF) {
 	return -1;
     }
@@ -988,23 +1098,19 @@ transform_preprocess_list (int *list, double **Z, const DATAINFO *pdinfo,
 	ok = 1;
 
 	if (f == LOGS || f == SQUARE) {
-	    if (v == 0 || var_is_scalar(pdinfo, v)) {
+	    if (v == 0) { /* FIXME?? */
 		ok = 1; 
 	    }
 	    if (gretl_isdummy(pdinfo->t1, pdinfo->t2, Z[v])) {
 		ok = 0;
 	    }
 	} else if (f == LAGS) {
-	    if (v == 0 || var_is_scalar(pdinfo, v)) {
-		ok = 0;
-	    }
-	} else if (f == DIFF || f == LDIFF || f == SDIFF) {
-	    if (var_is_scalar(pdinfo, v)) {
+	    if (v == 0) {
 		ok = 0;
 	    }
 	} else if (f == DUMMIFY) {
 	    ok = 0; /* reverse burden of proof */
-	    if (v > 0 && var_is_series(pdinfo, v)) {
+	    if (v > 0) {
 		if (var_is_discrete(pdinfo, v)) {
 		    /* pre-approved */
 		    ok = 1;
@@ -1032,9 +1138,9 @@ transform_preprocess_list (int *list, double **Z, const DATAINFO *pdinfo,
 	    for (i=0; i<longnames && !herr; i++) {
 		for (j=i+1; j<longnames && !herr; j++) {
 		    if (!strncmp(S[i], S[j], maxc)) {
-			v = varindex(pdinfo, S[i]);
+			v = series_index(pdinfo, S[i]);
 			herr = make_mangled_name(v, S[i], maxc);
-			v = varindex(pdinfo, S[j]);
+			v = series_index(pdinfo, S[j]);
 			herr += make_mangled_name(v, S[j], maxc);
 		    }
 		}
@@ -1099,7 +1205,7 @@ static int *make_lags_list (int *list, int order, DATAINFO *pdinfo)
 
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
-	if (v > 0 && var_is_series(pdinfo, v)) {
+	if (v > 0) {
 	    nl += order;
 	}
     }
@@ -1131,7 +1237,7 @@ int list_laggenr (int **plist, int order, double ***pZ, DATAINFO *pdinfo)
     int err;
 
     if (order < 0) {
-	sprintf(gretl_errmsg, _("Invalid lag order %d"), order);
+	gretl_errmsg_sprintf(_("Invalid lag order %d"), order);
 	return E_DATA;
     }
 
@@ -1188,7 +1294,6 @@ int list_laggenr (int **plist, int order, double ***pZ, DATAINFO *pdinfo)
  *
  * Returns: the default lag order for generating lags, performing
  * autocorrelation test, and so on.
- *
  */
 
 int default_lag_order (const DATAINFO *pdinfo)
@@ -1206,14 +1311,14 @@ int default_lag_order (const DATAINFO *pdinfo)
  * list_diffgenr:
  * @list: on entry, list of variables to process; on exit,
  * ID numbers of the generated variables.
- * @ci: must be %DIFF, %LDIFF or %SDIFF.
+ * @ci: must be DIFF, LDIFF or SDIFF.
  * @pZ: pointer to data matrix.
  * @pdinfo: data information struct.
  *
  * Generate differences of the variables in @list, and add them
- * to the data set.  If @ci is %DIFF these are ordinary first
- * differences; if @ci is %LDIFF they are log differences; and
- * if @ci is %SDIFF they are seasonal differences.
+ * to the data set.  If @ci is DIFF these are ordinary first
+ * differences; if @ci is LDIFF they are log differences; and
+ * if @ci is SDIFF they are seasonal differences.
  *
  * Returns: 0 on successful completion, 1 on error.
  */
@@ -1262,6 +1367,58 @@ int list_diffgenr (int *list, int ci, double ***pZ, DATAINFO *pdinfo)
     return err;
 }
 
+/**
+ * list_orthdev:
+ * @list: list of variables to process.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ *
+ * Generate orthogonal deviations of the variables in @list, and add
+ * them to the data set.
+ *
+ * Returns: 0 on success, error code on error.
+ */
+
+int list_orthdev (int *list, double ***pZ, DATAINFO *pdinfo)
+{
+    int origv = pdinfo->v;
+    int i, v, startlen;
+    int tnum, l0 = 0;
+    int err;
+
+    if (list[0] == 0) {
+	return 0;
+    }
+
+    if (!dataset_is_panel(pdinfo)) {
+	return E_PDWRONG;
+    } 
+
+    err = transform_preprocess_list(list, *pZ, pdinfo, ORTHDEV);
+    if (err) {
+	return err;
+    }
+
+    startlen = get_starting_length(list, pdinfo, 2);
+    
+    for (i=1; i<=list[0] && !err; i++) {
+	v = list[i];
+	tnum = get_transform(ORTHDEV, v, 0, 0.0, pZ, pdinfo, startlen, origv);
+	if (tnum < 0) {
+	    err = 1;
+	} else {
+	    list[i] = tnum;
+	    l0++;
+	}
+    }
+
+    list[0] = l0;
+
+    destroy_mangled_names();
+
+    return err;
+}
+
 /**
  * list_xpxgenr:
  * @plist: pointer to list of variables to process.  On exit
@@ -1272,7 +1429,7 @@ int list_diffgenr (int *list, int ci, double ***pZ, DATAINFO *pdinfo)
  * @opt: If OPT_O, both squares and cross-products are generated,
  * otherwise only squares.
  *
- * Generates and adds to the data set squares and (if @opt is %OPT_O) 
+ * Generates and adds to the data set squares and (if @opt is OPT_O) 
  * cross-products of the variables given in the list pointed to
  * by @plist.
  *
@@ -1344,29 +1501,10 @@ int list_xpxgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 
 #define DUMDEBUG 0
 
-/**
- * list_dumgenr:
- * @plist: pointer to list of variables to process; on exit
- * the list holds the ID numbers of the generated dummies.
- * @pZ: pointer to data matrix.
- * @pdinfo: data information struct.
- * @opt: can be %OPT_F to drop the first value, %OPT_L to drop
- * the last, or %OPT_NONE.
- *
- * For each of the variables given in the list to which @plist
- * points, generates and adds to the data set k dummy variables 
- * coding for the k distinct values of the variable in question.
- * All these variables must have already been marked as discrete.
- * If the %OPT_F or %OPT_L option is given, either the first or
- * the last value of each variable is taken as the "base" and is
- * not given a dummy encoding (that is, only k - 1 dummies are
- * added for each variable).
- *
- * Returns: 0 on success, error code on error.
-*/
+#define skip_j (x, xs) (!na(xs) && (xs == x))
 
-int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
-		  gretlopt opt)
+static int real_list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
+			      double oddval, gretlopt opt)
 {
     int origv = pdinfo->v;
     int *list = *plist;
@@ -1393,7 +1531,7 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 	goto bailout;
     }
 
-     startlen = get_starting_length(list, pdinfo, 3);
+    startlen = get_starting_length(list, pdinfo, 3);
 
     for (i=1; i<=list[0] && !err; i++) {
 	int vi = list[i];
@@ -1402,7 +1540,7 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 	double xt;
 
 	n = 0;
-	for (t=0; t<pdinfo->n; t++) {
+	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    xt = (*pZ)[vi][t];
 	    if (!na(xt)) {
 		x[n++] = xt;
@@ -1429,28 +1567,30 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
 	fprintf(stderr, "variable %d has %d distinct values\n", vi, nuniq);
 	if (opt & OPT_F) fprintf(stderr, "skipping lowest value\n");
 	if (opt & OPT_L) fprintf(stderr, "skipping highest value\n");
-	fprintf(stderr, "jmin = %d, jmax = %d\n", jmin, jmax);
+	fprintf(stderr, "jskip = %d\n", jskip);
 #endif
 
 	for (j=jmin; j<jmax && !err; j++) {
-	    tnum = get_transform(DUMMIFY, vi, j+1, x[j], pZ, pdinfo, 
-				 startlen, origv);
+	    if (x[j] != oddval) {
+		tnum = get_transform(DUMMIFY, vi, j+1, x[j], pZ, pdinfo, 
+				     startlen, origv);
 #if DUMDEBUG   
-	    fprintf(stderr, "VALUE = %g, tnum = %d\n", x[j], tnum);
+		fprintf(stderr, "VALUE = %g, tnum = %d\n", x[j], tnum);
 #endif
-	    if (tnum > 0) {
-		tmplist = gretl_list_append_term(&tmplist, tnum);
-		if (tmplist == NULL) {
-		    err = E_ALLOC;
-		} 
-	    } else {
-		err = E_DATA;
+		if (tnum > 0) {
+		    tmplist = gretl_list_append_term(&tmplist, tnum);
+		    if (tmplist == NULL) {
+			err = E_ALLOC;
+		    } 
+		} else {
+		    err = E_DATA;
+		}
 	    }
 	}
     }
 
     if (!err && tmplist[0] == 0) {
-	strcpy(gretl_errmsg, _("dummify: no suitable variables were found"));
+	gretl_errmsg_set(_("dummify: no suitable variables were found"));
 	err = E_DATA;
     }
 
@@ -1473,14 +1613,66 @@ int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+/**
+ * list_dumgenr:
+ * @plist: pointer to list of variables to process; on exit
+ * the list holds the ID numbers of the generated dummies.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ * @opt: can include OPT_F to drop the first value, OPT_L to drop
+ * the last value.
+ *
+ * For each of the variables given in the list to which @plist
+ * points, generates and adds to the data set k dummy variables 
+ * coding for the k distinct values of the variable in question.
+ * All these variables must have already been marked as discrete.
+ * If the OPT_F or OPT_L option is given, either the first or
+ * the last value of each variable is taken as the "base" and is
+ * not given a dummy encoding (that is, only k - 1 dummies are
+ * added for each variable).
+ *
+ * Returns: 0 on success, error code on error.
+*/
+
+int list_dumgenr (int **plist, double ***pZ, DATAINFO *pdinfo,
+		  gretlopt opt)
+{
+    return real_list_dumgenr(plist, pZ, pdinfo, NADBL, opt);
+}
+
+/**
+ * dumgenr_with_oddval:
+ * @plist: pointer to list of variables to process; on exit
+ * the list holds the ID numbers of the generated dummies.
+ * @pZ: pointer to data matrix.
+ * @pdinfo: data information struct.
+ * @oddval: value which should be skipped when encoding the
+ * input values as dummies.
+ *
+ * For each of the variables given in the list to which @plist
+ * points, generates and adds to the data set k dummy variables 
+ * coding for the k distinct values of the variable in question.
+ * All these variables must have already been marked as discrete.
+ * if @oddval is not %NADBL, it is treated as the omitted
+ * category and only k - 1 dummies are added for each variable).
+ *
+ * Returns: 0 on success, error code on error.
+*/
+
+int dumgenr_with_oddval (int **plist, double ***pZ, DATAINFO *pdinfo,
+			 double oddval)
+{
+    return real_list_dumgenr(plist, pZ, pdinfo, oddval, OPT_NONE);
+}
+
 /**
  * list_makediscrete:
  * @list: list of variables to process.
  * @pdinfo: data information struct.
- * @opt: if %OPT_R, reverse the operation.
+ * @opt: if OPT_R, reverse the operation.
  *
  * Sets the variables given in @list as discrete, unless
- * opt is %OPT_R, in which case the variables are set as
+ * opt is OPT_R, in which case the variables are set as
  * continuous.
  *
  * Returns: 0 on success, error code on error.
@@ -1493,42 +1685,14 @@ int list_makediscrete (const int *list, DATAINFO *pdinfo, gretlopt opt)
 
     for (i=1; i<=list[0]; i++) {
 	v = list[i];
-	if (v == 0 || var_is_scalar(pdinfo, v)) {
-	    continue;
+	if (v > 0) {
+	    set_var_discrete(pdinfo, v, disc);
 	}
-	set_var_discrete(pdinfo, v, disc);
     }
 
     return err;
 }
 
-int gettrend (double ***pZ, DATAINFO *pdinfo, int square)
-{
-    int idx, t, v = pdinfo->v;
-    double x;
 
-    idx = varindex(pdinfo, (square)? "timesq" : "time");
 
-    if (idx < v) {
-	return idx;
-    }
 
-    if (dataset_add_series(1, pZ, pdinfo)) {
-	return 0; /* error: valid value cannot == 0 */
-    }
-
-    for (t=0; t<pdinfo->n; t++) {
-	x = (double) t + 1; 
-	(*pZ)[v][t] = (square)? x * x : x;
-    }
-
-    if (square) {
-	strcpy(pdinfo->varname[v], "timesq");
-	strcpy(VARLABEL(pdinfo, v), _("squared time trend variable"));
-    } else {
-	strcpy(pdinfo->varname[v], "time");
-	strcpy(VARLABEL(pdinfo, v), _("time trend variable"));
-    }
-	    
-    return idx;
-}
