@@ -23,6 +23,12 @@
 #include "gretl_matrix.h"
 #include "gretl_restrict.h"
 
+enum Detflags {
+    DET_CONST = 1 << 0,
+    DET_TREND = 1 << 1,
+    DET_SEAS  = 1 << 2
+};
+
 typedef struct JohansenInfo_ JohansenInfo;
 
 struct GRETL_VAR_ {
@@ -30,29 +36,31 @@ struct GRETL_VAR_ {
     int refcount;        /* for saving/deleting */
     int err;             /* error code */
     int neqns;           /* number of equations in system */
-    int order;           /* lag order */
+    int order;           /* maximum lag order */
     int t1;              /* starting observation */
     int t2;              /* ending observation */
     int T;               /* number of observations */
     int df;              /* T - average coeffs per equation */
     int ifc;             /* equations include a constant (1) or not (0) */
     int ncoeff;          /* total coefficients per equation */
+    int k;               /* maximum columns of Y (VECM special) */
+    int *lags;           /* list of specific lags */
     int *ylist;          /* list of stochastic vars */
     int *xlist;          /* list of exogenous variables */
     int *rlist;          /* restricted exogenous variables (VECM only) */
     int detflags;        /* record of automatic deterministic vars added */
     int robust;          /* computing robust std errors? */
-    int qr;              /* using QR decomposition? */
     gretl_matrix *Y;     /* matrix of dependent variables */
     gretl_matrix *X;     /* matrix of independent variables */
     gretl_matrix *B;     /* basic coefficient matrix */
-    gretl_matrix *XTX;   /* X'X */
+    gretl_matrix *XTX;   /* X'X inverse */
     gretl_matrix *A;     /* augmented coefficient matrix (companion form) */
     gretl_matrix *L;     /* lambda: inverse roots of A(L) polynomial */
     gretl_matrix *E;     /* residuals matrix */
     gretl_matrix *C;     /* augmented Cholesky-decomposed error matrix */
     gretl_matrix *S;     /* cross-equation variance matrix */
     gretl_matrix *F;     /* optional forecast matrix */
+    gretl_matrix *ord;   /* optional Cholesky-ordering vector */
     MODEL **models;      /* pointers to individual equation estimates */
     double *Fvals;       /* hold results of F-tests */
     double *Ivals;       /* hold results of info criteria comparisons */
@@ -68,11 +76,14 @@ struct GRETL_VAR_ {
     char *name;          /* for use in session management */
 };
 
+#define var_n_lags(v) ((v->lags != NULL)? v->lags[0] : v->order)
+#define var_max_lag(v) ((v->lags != NULL)? v->lags[v->lags[0]] : v->order)
+
 int var_max_order (const int *list, const DATAINFO *pdinfo);
 
 GRETL_VAR *gretl_VAR (int order, int *list, 
 		      const double **Z, const DATAINFO *pdinfo,
-		      gretlopt opt, PRN *prn, int *err);
+		      gretlopt opt, PRN *prn, int *errp);
 
 GRETL_VAR *gretl_VECM (int order, int rank, int *list, 
 		       const double **Z, const DATAINFO *pdinfo,
@@ -91,9 +102,10 @@ gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods,
 			    int *errp);
 
 int 
-gretl_VAR_do_error_decomp (const gretl_matrix *S, gretl_matrix *C);
+gretl_VAR_do_error_decomp (const gretl_matrix *S, gretl_matrix *C,
+			   const gretl_matrix *ord);
 
-const gretl_matrix *gretl_VAR_get_roots (GRETL_VAR *var);
+const gretl_matrix *gretl_VAR_get_roots (GRETL_VAR *var, int *err);
 
 int gretl_VAR_autocorrelation_test (GRETL_VAR *var, int order, 
 				    double ***pZ, DATAINFO *pdinfo,
@@ -122,7 +134,7 @@ int default_VAR_horizon (const DATAINFO *pdinfo);
 gretl_matrix *
 gretl_VAR_get_impulse_response (GRETL_VAR *var, 
 				int targ, int shock,
-				int periods,
+				int periods, double alpha,
 				const double **Z,
 				const DATAINFO *pdinfo);
 
@@ -140,14 +152,24 @@ int gretl_VAR_get_t2 (const GRETL_VAR *var);
 
 const MODEL *gretl_VAR_get_model (const GRETL_VAR *var, int i);
 
-int gretl_VAR_add_resids_to_dataset (GRETL_VAR *var, int eqnum,
-				     double ***pZ, DATAINFO *pdinfo);
+double *gretl_VAR_get_resid_series (GRETL_VAR *var, int eqnum,
+				    int *err);
+
+int gretl_VAR_set_ordering (GRETL_VAR *var, gretl_matrix *ord);
 
 int gretl_VAR_do_irf (GRETL_VAR *var, const char *line,
 		      const double **Z, const DATAINFO *pdinfo);
 
 int gretl_VAR_get_highest_variable (const GRETL_VAR *var);
 
+GRETL_VAR *johansen_test (int order, const int *list, 
+			  const double **Z, const DATAINFO *pdinfo,
+			  gretlopt opt, PRN *prn);
+
+int johansen_test_simple (int order, const int *list, 
+			  const double **Z, const DATAINFO *pdinfo,
+			  gretlopt opt, PRN *prn);
+
 int gretl_VECM_n_beta (const GRETL_VAR *vecm);
 
 int gretl_VECM_n_alpha (const GRETL_VAR *vecm);
@@ -199,9 +221,11 @@ int transcribe_VAR_models (GRETL_VAR *var,
 			   const DATAINFO *pdinfo,
 			   const gretl_matrix *XTX);
 
-#ifndef GRETLCLI
+#ifdef FULL_XML_HEADERS
 
-GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, int *err);
+GRETL_VAR *gretl_VAR_from_XML (xmlNodePtr node, xmlDocPtr doc, 
+			       const DATAINFO *pdinfo,
+			       int *err);
 
 #endif
 
