@@ -19,13 +19,16 @@
 
 #include "libgretl.h" 
 #include "var.h"  
+#include "vartest.h" 
 #include "johansen.h"
 #include "varprint.h"
 #include "libset.h"
 #include "texprint.h"
 
+#include <glib.h>
+
 /**
- * gretl_VAR_print_VCV:
+ * gretl_VAR_print_sigma:
  * @var: pointer to gretl VAR structure.
  * @prn: printing object.
  *
@@ -35,7 +38,7 @@
  * Returns: 0 on success, 1 on failure.
  */
 
-int gretl_VAR_print_VCV (const GRETL_VAR *var, PRN *prn)
+int gretl_VAR_print_sigma (const GRETL_VAR *var, PRN *prn)
 {
     int err = 0;
 
@@ -48,21 +51,6 @@ int gretl_VAR_print_VCV (const GRETL_VAR *var, PRN *prn)
     return err;
 }
 
-static void tex_print_double (double x, PRN *prn)
-{
-    char number[16];
-
-    x = screen_zero(x);
-
-    sprintf(number, "%#.*g", GRETL_DIGITS, x);
-
-    if (x < 0.) {
-	pprintf(prn, "$-$%s", number + 1);
-    } else {
-	pputs(prn, number);
-    }
-}
-
 /* printing of impulse responses and variance decompositions */
 
 #define IRF_ROW_MAX 4
@@ -243,7 +231,6 @@ static int varprint_namelen (const GRETL_VAR *var, const DATAINFO *pdinfo,
  * @shock: index number of the "shock" variable.
  * @periods: number of periods over which to print response.
  * @pdinfo: dataset information.
- * @pause: if non-zero, pause between sections of output.
  * @prn: gretl printing object.
  *
  * Prints to @prn the estimated responses of the endogenous
@@ -259,15 +246,15 @@ static int varprint_namelen (const GRETL_VAR *var, const DATAINFO *pdinfo,
 int 
 gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 				  int periods, const DATAINFO *pdinfo, 
-				  int pause, PRN *prn)
+				  PRN *prn)
 {
-    gretl_matrix *rtmp, *ctmp;
+    gretl_matrix *rtmp = NULL, *ctmp = NULL;
+    gretl_matrix *C = var->C;
     int rows = var->neqns * effective_order(var);
     int block, blockmax;
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
-    int vsrc;
-    int i, t, err = 0;
+    int i, t, vsrc, err = 0;
 
     if (prn == NULL) {
 	return 0;
@@ -278,15 +265,19 @@ gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 	return 1;
     }  
 
-    rtmp = gretl_matrix_alloc(rows, var->neqns);
-    if (rtmp == NULL) {
-	return E_ALLOC;
-    }
+    if (var->ord != NULL) {
+	C = reorder_responses(var, &err);
+	if (err) {
+	    return err;
+	}
+    } 
 
+    rtmp = gretl_matrix_alloc(rows, var->neqns);
     ctmp = gretl_matrix_alloc(rows, var->neqns);
-    if (ctmp == NULL) {
-	gretl_matrix_free(rtmp);
-	return E_ALLOC;
+
+    if (rtmp == NULL || ctmp == NULL) {
+	err = E_ALLOC;
+	goto bailout;
     }
 
     vsrc = var->ylist[shock + 1];
@@ -326,7 +317,7 @@ gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 	    VAR_info_print_period(t + 1, prn);
 	    if (t == 0) {
 		/* calculate initial estimated responses */
-		err = gretl_matrix_copy_values(rtmp, var->C);
+		err = gretl_matrix_copy_values(rtmp, C);
 	    } else {
 		/* calculate further estimated responses */
 		err = gretl_matrix_multiply(var->A, rtmp, ctmp);
@@ -360,36 +351,36 @@ gretl_VAR_print_impulse_response (GRETL_VAR *var, int shock,
 	}
 
 	VAR_info_end_table(prn);
-
-	if (pause && block < blockmax - 1) {
-	    scroll_pause();
-	}
     }
 
+ bailout:
+
     if (rtmp != NULL) gretl_matrix_free(rtmp);
     if (ctmp != NULL) gretl_matrix_free(ctmp);
 
+    if (C != var->C) {
+	gretl_matrix_free(C);
+    }
+
     return err;
 }
 
 int gretl_VAR_print_all_impulse_responses (GRETL_VAR *var, const DATAINFO *pdinfo, 
 					   int horizon, PRN *prn)
 {
-    int i, pause = 0, err = 0;
+    int i, err = 0;
 
     if (horizon <= 0) {
 	horizon = default_VAR_horizon(pdinfo);
     }
 
-    if (plain_format(prn)) {
-	pause = gretl_get_text_pause();
-    } else if (rtf_format(prn)) {
+    if (rtf_format(prn)) {
 	pputs(prn, "{\\rtf1\\par\n\\qc ");
     }
 
     for (i=0; i<var->neqns && !err; i++) {
 	err = gretl_VAR_print_impulse_response(var, i, horizon, pdinfo, 
-					       pause, prn);
+					       prn);
     }
 
     if (rtf_format(prn)) {
@@ -405,7 +396,6 @@ int gretl_VAR_print_all_impulse_responses (GRETL_VAR *var, const DATAINFO *pdinf
  * @targ:
  * @periods: number of periods over which to print decomposition.
  * @pdinfo: dataset information.
- * @pause: if non-zero, pause between sections of output.
  * @prn: gretl printing struct.
  *
  *
@@ -415,15 +405,14 @@ int gretl_VAR_print_all_impulse_responses (GRETL_VAR *var, const DATAINFO *pdinf
 int 
 gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 			      int periods, const DATAINFO *pdinfo, 
-			      int pause, PRN *prn)
+			      PRN *prn)
 {
-    int i, t;
     int vtarg;
     gretl_matrix *vd = NULL;
     int block, blockmax;
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
-    int err = 0;
+    int i, t, err = 0;
 
     if (prn == NULL) {
 	return 0;
@@ -519,10 +508,6 @@ gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 	}
 
 	VAR_info_end_table(prn);
-
-	if (pause && block < blockmax - 1) {
-	    scroll_pause();
-	}
     }
 
     if (vd != NULL) {
@@ -535,21 +520,19 @@ gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 int gretl_VAR_print_all_fcast_decomps (GRETL_VAR *var, const DATAINFO *pdinfo, 
 				       int horizon, PRN *prn)
 {
-    int i, pause = 0, err = 0;
+    int i, err = 0;
 
     if (horizon <= 0) {
 	horizon = default_VAR_horizon(pdinfo);
     }
 
-    if (plain_format(prn)) {
-	pause = gretl_get_text_pause();
-    } else if (rtf_format(prn)) {
+    if (rtf_format(prn)) {
 	pputs(prn, "{\\rtf1\\par\n\\qc ");
     }
 
     for (i=0; i<var->neqns && !err; i++) {
 	err = gretl_VAR_print_fcast_decomp(var, i, horizon, pdinfo, 
-					   pause, prn);
+					   prn);
     }
 
     if (rtf_format(prn)) {
@@ -620,6 +603,7 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
 		       PRN *prn)
 {
     JohansenInfo *jv = jvar->jinfo;
+    int utf = plain_format(prn);
     int rtf = rtf_format(prn);
     char namefmt[8];
     char s[16], vname[32];
@@ -628,9 +612,11 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
     int i, j;
     double x;
 
-    pprintf(prn, "beta (%s", _("cointegrating vectors"));
+    pprintf(prn, "beta (%s", (utf)? _("cointegrating vectors") :
+	    I_("cointegrating vectors"));
     if (jv->Bse != NULL) {
-	pprintf(prn, ", %s)", _("standard errors in parentheses"));
+	pprintf(prn, ", %s)", (utf)? _("standard errors in parentheses") :
+		I_("standard errors in parentheses"));
     } else {
 	pputc(prn, ')');
     }
@@ -684,9 +670,11 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
 
     rows = gretl_matrix_rows(jv->Alpha);
 
-    pprintf(prn, "alpha (%s", _("adjustment vectors"));
+    pprintf(prn, "alpha (%s", (utf)? _("adjustment vectors") :
+	    I_("adjustment vectors"));
     if (jv->Ase != NULL) {
-	pprintf(prn, ", %s)", _("standard errors in parentheses"));
+	pprintf(prn, ", %s)", (utf)? _("standard errors in parentheses") :
+		I_("standard errors in parentheses"));
     } else {
 	pputc(prn, ')');
     }
@@ -750,13 +738,15 @@ static int max_vlen (const int *list, const DATAINFO *pdinfo)
 
 static void print_VECM_omega (GRETL_VAR *jvar, const DATAINFO *pdinfo, PRN *prn)
 {
+    int utf = plain_format(prn);
     int rtf = rtf_format(prn);
     int *list = jvar->ylist;
     char s[32];
     int w0, wi = 12;
     int i, j;
 
-    pprintf(prn, "%s\n", _("Cross-equation covariance matrix"));
+    pprintf(prn, "%s:\n", (utf)? _("Cross-equation covariance matrix") :
+	    I_("Cross-equation covariance matrix"));
     gretl_prn_newline(prn);
 
     w0 = max_vlen(list, pdinfo) + 1;
@@ -812,7 +802,9 @@ static void print_VECM_omega (GRETL_VAR *jvar, const DATAINFO *pdinfo, PRN *prn)
 
     gretl_prn_newline(prn);
 
-    pprintf(prn, "%s = %g", _("determinant"), exp(jvar->ldet));
+    pprintf(prn, "%s = %g", (utf)? _("determinant") : I_("determinant"),
+	    exp(jvar->ldet));
+
     gretl_prn_newline(prn);
 }
 
@@ -845,7 +837,11 @@ static void vecm_print_restrictions (GRETL_VAR *vecm, PRN *prn)
 
 static void print_LR_stat (double x, int df, PRN *prn)
 {
-    double pv = chisq_cdf_comp(x, df);
+    double pv = chisq_cdf_comp(df, x);
+
+    if (na(pv)) {
+	return;
+    }
 
     if (tex_format(prn)) {
 	pprintf(prn, "$2 (l_u - l_r) = %g$", x);
@@ -948,23 +944,55 @@ static void VAR_print_LB_stat (const GRETL_VAR *var, PRN *prn)
 {
     int k = var->order + (var->ci == VECM);
     int df = var->neqns * var->neqns * (var->LBs - k);
-    double pv = chisq_cdf_comp(var->LB, df);
+    double pv = chisq_cdf_comp(df, var->LB);
 
     if (tex_format(prn)) {
-	pprintf(prn, "\\noindent\n%s: LB(%d) = %g (%s = %d, %s %f)\\par\n",
+	pprintf(prn, "\\noindent\n%s: LB(%d) = %g, %s = %d [%.4f]\\par\n",
 		I_("Portmanteau test"), var->LBs, var->LB,
-		I_("df"), df, I_("p-value"), pv);	    
+		I_("df"), df, pv);	    
     } else if (rtf_format(prn)) {
-	pprintf(prn, "%s: LB(%d) = %g (%s = %d, %s %f)\\par\n",
+	pprintf(prn, "%s: LB(%d) = %g, %s = %d [%.4f]\\par\n",
 		I_("Portmanteau test"), var->LBs, var->LB,
-		I_("df"), df, I_("p-value"), pv);
+		I_("df"), df, pv);
     } else {
-	pprintf(prn, "%s: LB(%d) = %g (%s = %d, %s %f)\n",
+	pprintf(prn, "%s: LB(%d) = %g, %s = %d [%.4f]\n",
 		_("Portmanteau test"), var->LBs, var->LB,
-		_("df"), df, _("p-value"), pv);
+		_("df"), df, pv);
     }
 }
 
+static int Ivals_ok (const GRETL_VAR *var)
+{
+    return var->Ivals != NULL && !na(var->Ivals[0])
+	&& !na(var->Ivals[1]) && !na(var->Ivals[2]);
+}
+
+static int max_Ftest_label_len (GRETL_VAR *var, const DATAINFO *pdinfo,
+				char *s, int maxlag)
+{
+    int len, len1, len2;
+    int maxnamelen = 0;
+    int i, v;
+
+    for (i=0; i<var->neqns; i++) {
+	v = (var->models[i])->list[1];
+	len = strlen(pdinfo->varname[v]);
+	if (len > maxnamelen) {
+	    maxnamelen = len;
+	}
+    }
+
+    sprintf(s, _("All lags of %s"), "x");
+    len1 = g_utf8_strlen(s, -1) + maxnamelen - 1;
+
+    sprintf(s, _("All vars, lag %d"), maxlag);
+    len2 = g_utf8_strlen(s, -1);
+
+    len = (len1 > len2)? len1 : len2;
+
+    return (len > 25)? (len + 1) : 25;
+}
+
 /**
  * gretl_VAR_print:
  * @var: pointer to VAR struct.
@@ -984,20 +1012,31 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 		     PRN *prn)
 {
     char startdate[OBSLEN], enddate[OBSLEN];
-    char Vstr[72];
+    char label[72];
     int vecm = (var->ci == VECM);
     int dfd = var->models[0]->dfd;
     int tex = tex_format(prn);
     int rtf = rtf_format(prn);
     int quiet = (opt & OPT_Q);
+    int nlags, maxlag, nextlag;
+    int llen, fwidth = 0;
     int lldone = 0;
-    int pause = 0;
+    double pv;
     int i, j, k, v;
 
     if (prn == NULL) {
 	return 0;
     }
 
+    nlags = var_n_lags(var);
+    maxlag = var_max_lag(var);
+    
+    if (var->lags != NULL) {
+	nextlag = var->lags[nlags - 1];
+    } else {
+	nextlag = maxlag - 1;
+    }
+
     ntodate(startdate, var->t1, pdinfo);
     ntodate(enddate, var->t2, pdinfo);
 
@@ -1007,21 +1046,21 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 
     if (vecm) {
 	if (tex || rtf) {
-	    sprintf(Vstr, I_("VECM system, lag order %d"), var->order + 1);
+	    sprintf(label, I_("VECM system, lag order %d"), var->order + 1);
 	} else {
-	    sprintf(Vstr, _("VECM system, lag order %d"), var->order + 1);
+	    sprintf(label, _("VECM system, lag order %d"), var->order + 1);
 	}
     } else {
 	if (tex || rtf) {
-	    sprintf(Vstr, I_("VAR system, lag order %d"), var->order);
+	    sprintf(label, I_("VAR system, lag order %d"), var->order);
 	} else {
-	    sprintf(Vstr, _("VAR system, lag order %d"), var->order);
+	    sprintf(label, _("VAR system, lag order %d"), var->order);
 	}
     }
 
     if (tex) {
 	pputs(prn, "\\begin{center}");
-	pprintf(prn, "\n%s\\\\\n", Vstr);
+	pprintf(prn, "\n%s\\\\\n", label);
 	pprintf(prn, I_("%s estimates, observations %s--%s ($T=%d$)"),
 		(vecm)? I_("Maximum likelihood") : I_("OLS"), startdate, enddate, var->T);
 	if (vecm) {
@@ -1030,7 +1069,7 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	pputs(prn, "\n\\end{center}\n");
     } else if (rtf) {
 	gretl_print_toggle_doc_flag(prn);
-	pprintf(prn, "\n%s\\par\n", Vstr);
+	pprintf(prn, "\n%s\\par\n", label);
 	pprintf(prn, I_("%s estimates, observations %s-%s (T = %d)"),
 		(vecm)? I_("Maximum likelihood") : I_("OLS"), startdate, enddate, var->T);
 	if (vecm) {
@@ -1038,10 +1077,9 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	}	
 	pputs(prn, "\\par\n\n");
     } else {
-	pause = gretl_get_text_pause();
-	pprintf(prn, "\n%s\n", Vstr);
+	pprintf(prn, "\n%s\n", label);
 	pprintf(prn, _("%s estimates, observations %s-%s (T = %d)"),
-		(vecm)? ("Maximum likelihood") : _("OLS"), startdate, enddate, var->T);
+		(vecm)? _("Maximum likelihood") : _("OLS"), startdate, enddate, var->T);
 	if (vecm) {
 	    print_vecm_header_info(var, &lldone, prn);
 	}
@@ -1110,10 +1148,6 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	    }
 	}
 
-	if (pause) {
-	    scroll_pause();
-	}
-
 	if (vecm) {
 	    continue;
 	}
@@ -1125,52 +1159,60 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 	} else if (rtf) {
 	    pprintf(prn, "%s:\\par\n\n", I_("F-tests of zero restrictions"));
 	} else {
-	    pprintf(prn, "  %s:\n\n", _("F-tests of zero restrictions"));
+	    pprintf(prn, "%s:\n\n", _("F-tests of zero restrictions"));
+	}
+
+	if (!tex) {
+	    fwidth = max_Ftest_label_len(var, pdinfo, label, maxlag);
 	}
 
 	for (j=0; j<var->neqns; j++) {
+	    pv = snedecor_cdf_comp(nlags, dfd, var->Fvals[k]);
 	    v = (var->models[j])->list[1];
 	    if (tex) {
-		pprintf(prn, I_("All lags of %-15s "), pdinfo->varname[v]);
-		pputs(prn, "& ");
-		pprintf(prn, "$F(%d, %d) = %g$ & ", var->order, dfd, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\\\\\n", I_("p-value"), 
-			snedecor_cdf_comp(var->Fvals[k], var->order, dfd));
+		pprintf(prn, I_("All lags of %s"), pdinfo->varname[v]);
+		pputs(prn, " & ");
+		pprintf(prn, "$F(%d, %d) = %g$ & ", nlags, dfd, var->Fvals[k]);
+		pprintf(prn, "[%.4f]\\\\\n", pv);
 	    } else if (rtf) {
-		pprintf(prn, I_("All lags of %-15s "), pdinfo->varname[v]);
-		pprintf(prn, "F(%d, %d) = %8.5g, ", var->order, dfd, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\\par\n", I_("p-value"), 
-			snedecor_cdf_comp(var->Fvals[k], var->order, dfd));
+		sprintf(label, I_("All lags of %s"), pdinfo->varname[v]);
+		llen = strlen(label);
+		pputs(prn, label);
+		bufspace(fwidth - llen, prn);
+		pprintf(prn, "F(%d, %d) = %8.5g ", nlags, dfd, var->Fvals[k]);
+		pprintf(prn, "[%.4f]\\par\n", pv);
 	    } else {
-		pputs(prn, "  ");
-		pprintf(prn, _("All lags of %-15s "), pdinfo->varname[v]);
-		sprintf(Fstr, "F(%d, %d)", var->order, dfd);
-		pprintf(prn, "%12s = %#8.5g, ", Fstr, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\n", _("p-value"), 
-			snedecor_cdf_comp(var->Fvals[k], var->order, dfd));
+		sprintf(label, _("All lags of %s"), pdinfo->varname[v]);
+		llen = g_utf8_strlen(label, -1);
+		pputs(prn, label);
+		bufspace(fwidth - llen, prn);
+		sprintf(Fstr, "F(%d, %d)", nlags, dfd);
+		pprintf(prn, "%12s = %#8.5g [%.4f]\n", Fstr, var->Fvals[k], pv);
 	    }
 	    k++;
 	}
 
 	if (var->order > 1) {
+	    pv = snedecor_cdf_comp(var->neqns, dfd, var->Fvals[k]);
 	    if (tex) {
-		pprintf(prn, I_("All vars, lag %-13d "), var->order);
-		pputs(prn, "& ");
+		pprintf(prn, I_("All vars, lag %d"), maxlag);
+		pputs(prn, " & ");
 		pprintf(prn, "$F(%d, %d) = %g$ & ", var->neqns, dfd, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\\\\\n", I_("p-value"), 
-			snedecor_cdf_comp(var->Fvals[k], var->neqns, dfd));
+		pprintf(prn, "[%.4f]\\\\\n", pv);
 	    } else if (rtf) {
-		pprintf(prn, I_("All vars, lag %-13d "), var->order);
-		pprintf(prn, "F(%d, %d) = %8.5g, ", var->neqns, dfd, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\\par\n", I_("p-value"), 
-			snedecor_cdf_comp(var->Fvals[k], var->neqns, dfd));
+		sprintf(label, I_("All vars, lag %d"), maxlag);
+		llen = strlen(label);
+		pputs(prn, label);
+		bufspace(fwidth - llen, prn);
+		pprintf(prn, "F(%d, %d) = %8.5g ", var->neqns, dfd, var->Fvals[k]);
+		pprintf(prn, "[%.4f]\\par\n", pv);
 	    } else {
-		pputs(prn, "  ");
-		pprintf(prn, _("All vars, lag %-13d "), var->order);
+		sprintf(label, _("All vars, lag %d"), maxlag);
+		llen = g_utf8_strlen(label, -1);
+		pputs(prn, label);
+		bufspace(fwidth - llen, prn);
 		sprintf(Fstr, "F(%d, %d)", var->neqns, dfd);
-		pprintf(prn, "%12s = %#8.5g, ", Fstr, var->Fvals[k]);
-		pprintf(prn, "%s %.4f\n", _("p-value"), 
-			snedecor_cdf_comp(var->Fvals[k], var->neqns, dfd));
+		pprintf(prn, "%12s = %#8.5g [%.4f]\n", Fstr, var->Fvals[k], pv);
 	    } 
 	    k++;
 	}
@@ -1181,60 +1223,59 @@ int gretl_VAR_print (GRETL_VAR *var, const DATAINFO *pdinfo, gretlopt opt,
 		  "\\clearpage\n\n");
 	} else if (rtf) {
 	    pputs(prn, "\\par\\n\n");
-	} else if (pause) {
-	    scroll_pause();
-	}
+	} 
     }
 
-    pputc(prn, '\n');
-
     /* global LR test on max lag */
     if (!na(var->LR)) {
 	char h0str[64];
 	char h1str[64];
 	int df = var->neqns * var->neqns;
 
-	if (tex || rtf) {
-	    sprintf(h0str, I_("the longest lag is %d"), var->order - 1);
-	    sprintf(h1str, I_("the longest lag is %d"), var->order);
+	pputc(prn, '\n');
+
+	if (rtf || tex) {
+	    sprintf(h0str, I_("the longest lag is %d"), nextlag);
+	    sprintf(h1str, I_("the longest lag is %d"), maxlag);
 	} else {
-	    sprintf(h0str, _("the longest lag is %d"), var->order - 1);
-	    sprintf(h1str, _("the longest lag is %d"), var->order);
-	}	    
+	    sprintf(h0str, _("the longest lag is %d"), nextlag);
+	    sprintf(h1str, _("the longest lag is %d"), maxlag);
+	}
+
+	pv = chisq_cdf_comp(df, var->LR);
 
 	if (tex) {
 	    pprintf(prn, "\\noindent %s ---\\par\n", I_("For the system as a whole"));
 	    pprintf(prn, "%s: %s\\par\n", I_("Null hypothesis"), h0str);
 	    pprintf(prn, "%s: %s\\par\n", I_("Alternative hypothesis"), h1str);
-	    pprintf(prn, "%s: $\\chi^2_{%d}$ = %.3f (%s %f)\\par\n",
-		    I_("Likelihood ratio test"), 
-		    df, var->LR, I_("p-value"), chisq_cdf_comp(var->LR, df));
+	    pprintf(prn, "%s: $\\chi^2_{%d}$ = %.3f [%.4f]\\par\n",
+		    I_("Likelihood ratio test"), df, var->LR, pv);
 	} else if (rtf) {
 	    pprintf(prn, "\\par %s\n", I_("For the system as a whole"));
 	    pprintf(prn, "\\par %s: %s\n", I_("Null hypothesis"), h0str);
 	    pprintf(prn, "\\par %s: %s\n", I_("Alternative hypothesis"), h1str);
-	    pprintf(prn, "\\par %s: %s(%d) = %g (%s %f)\n",
-		    I_("Likelihood ratio test"), I_("Chi-square"), 
-		    df, var->LR, I_("p-value"), chisq_cdf_comp(var->LR, df));
+	    pprintf(prn, "\\par %s: %s(%d) = %g [%.4f]\n", I_("Likelihood ratio test"), 
+		    I_("Chi-square"), df, var->LR, pv);
 	} else {
 	    int ordlen = (var->order > 10)? 2 : 1;
 
 	    pprintf(prn, "%s:\n\n", _("For the system as a whole"));
 	    pprintf(prn, "  %s: %s\n", _("Null hypothesis"), h0str);
 	    pprintf(prn, "  %s: %s\n", _("Alternative hypothesis"), h1str);
-	    pprintf(prn, "  %s: %s(%d) = %g (%s %f)\n",
-		    _("Likelihood ratio test"), _("Chi-square"), 
-		    df, var->LR, _("p-value"), chisq_cdf_comp(var->LR, df));
-	    /* Info criteria comparison */
-	    pprintf(prn, "\n  %s:\n", _("Comparison of information criteria"));
-	    pputs(prn, "  ");
-	    pprintf(prn, _("Lag order %*d"), ordlen, var->order);
-	    pprintf(prn, ": AIC = %#.6g, BIC = %#.6g, HQC = %#.6g\n", 
-		    var->AIC, var->BIC, var->HQC);
-	    pputs(prn, "  ");
-	    pprintf(prn, _("Lag order %*d"), ordlen, var->order - 1);
-	    pprintf(prn, ": AIC = %#.6g, BIC = %#.6g, HQC = %#.6g\n", 
-		    var->Ivals[0], var->Ivals[1], var->Ivals[2]);
+	    pprintf(prn, "  %s: %s(%d) = %g [%.4f]\n", _("Likelihood ratio test"), 
+		    _("Chi-square"), df, var->LR, pv);
+	    if (Ivals_ok(var)) {
+		/* Info criteria comparison */
+		pprintf(prn, "\n  %s:\n", _("Comparison of information criteria"));
+		pputs(prn, "  ");
+		pprintf(prn, _("Lag order %*d"), ordlen, var->order);
+		pprintf(prn, ": AIC = %#.6g, BIC = %#.6g, HQC = %#.6g\n", 
+			var->AIC, var->BIC, var->HQC);
+		pputs(prn, "  ");
+		pprintf(prn, _("Lag order %*d"), ordlen, var->order - 1);
+		pprintf(prn, ": AIC = %#.6g, BIC = %#.6g, HQC = %#.6g\n", 
+			var->Ivals[0], var->Ivals[1], var->Ivals[2]);
+	    }
 	}
     }
 
