@@ -45,6 +45,10 @@ int gretl_VAR_autocorrelation_test (GRETL_VAR *var, int order,
     double lb;
     int i, err = 0;
 
+    if (order == 0) {
+	order = pdinfo->pd;
+    }
+
     for (i=0; i<var->neqns && !err; i++) {
 	pmod = var->models[i];
 	pprintf(prn, "%s %d:\n", _("Equation"), i + 1);
@@ -54,7 +58,7 @@ int gretl_VAR_autocorrelation_test (GRETL_VAR *var, int order,
 	    if (!err) {
 		pprintf(prn, "Ljung-Box Q' = %g %s = P(%s(%d) > %g) = %.3g\n", 
 			lb, _("with p-value"), _("Chi-square"), order,
-			lb, chisq_cdf_comp(lb, order));
+			lb, chisq_cdf_comp(order, lb));
 		pputc(prn, '\n');
 	    }
 	} else {
@@ -178,62 +182,12 @@ int VAR_portmanteau_test (GRETL_VAR *var)
     return err;
 }
 
-/* make and record residuals for LR test on last lag */
-
-int last_lag_LR_prep (GRETL_VAR *var, int ifc)
+int VAR_LR_lag_test (GRETL_VAR *var, const gretl_matrix *E)
 {
-    gretl_matrix *X = NULL;
-    gretl_matrix *B = NULL;
-    double x;
-    int g = var->ncoeff - var->neqns;
-    int lag, lmax;
-    int i, t, k, err = 0;
-
-    if (var->F == NULL) {
-	var->F = gretl_matrix_alloc(var->T, var->neqns);
-	if (var->F == NULL) {
-	    return E_ALLOC;
-	}
-    }   
-
-    X = gretl_matrix_alloc(var->T, g);
-    B = gretl_matrix_alloc(g, var->neqns);
-    if (X == NULL || B == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }
-
-    lag = (ifc)? 0 : 1;
-    lmax = ifc + var->neqns * var->order;
-    k = 0;
-
-    for (i=0; i<var->ncoeff; i++) {
-	if (lag != var->order) {
-	    for (t=0; t<var->T; t++) {
-		x = gretl_matrix_get(var->X, t, i);
-		gretl_matrix_set(X, t, k, x);
-	    }
-	    k++;
-	}
-	lag = (i >= lmax)? 0 : (lag < var->order)? lag + 1 : 1;
-    }
-
-    err = gretl_matrix_multi_ols(var->Y, X, B, var->F, NULL);
-
- bailout:
-
-    gretl_matrix_free(X);
-    gretl_matrix_free(B);
-
-    return err;
-}
-
-int VAR_LR_lag_test (GRETL_VAR *var)
-{
-    double ldet;
+    double test_ldet;
     int err = 0;
 
-    ldet = gretl_VAR_ldet(var, &err);
+    test_ldet = gretl_VAR_ldet(var, E, &err);
 
     if (!err) {
 	double ll, AIC, BIC, HQC;
@@ -242,9 +196,9 @@ int VAR_LR_lag_test (GRETL_VAR *var)
 	int m = var->ncoeff - g;
 	int k = g * m;
 
-	var->LR = T * (ldet - var->ldet);
+	var->LR = T * (test_ldet - var->ldet);
 
-	ll = -(g * T / 2.0) * (LN_2_PI + 1) - (T / 2.0) * ldet;
+	ll = -(g * T / 2.0) * (LN_2_PI + 1) - (T / 2.0) * test_ldet;
 	AIC = (-2.0 * ll + 2.0 * k) / T;
 	BIC = (-2.0 * ll + k * log(T)) / T;
 	HQC = (-2.0 * ll + 2.0 * k * log(log(T))) / T;
@@ -253,10 +207,6 @@ int VAR_LR_lag_test (GRETL_VAR *var)
 	var->Ivals[2] = HQC;
     }
 
-    /* we're done with this set of residuals */
-    gretl_matrix_free(var->F);
-    var->F = NULL;
-
     return err;
 }
 
@@ -310,7 +260,7 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
 {
     gretl_matrix *crittab = NULL;
     gretl_matrix *lltab = NULL;
-
+    gretl_matrix *E = NULL;
     int p = var->order;
     int r = p - 1;
     int T = var->T;
@@ -330,12 +280,8 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
 	return 0;
     }
 
-    if (var->F != NULL) {
-	gretl_matrix_free(var->F);
-    }
-
-    var->F = gretl_matrix_alloc(T, n);
-    if (var->F == NULL) {
+    E = gretl_matrix_alloc(T, n);
+    if (E == NULL) {
 	return E_ALLOC;
     }
 
@@ -346,7 +292,7 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
 	goto bailout;
     }
 
-    /* # of cols in X that are not Y lags */
+    /* number of cols in X that are not Y lags */
     cols0 = var->ncoeff - p * n; 
 
     for (j=1; j<p && !err; j++) {
@@ -358,10 +304,10 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
 	gretl_matrix_reuse(var->B, jxcols, n);
 
 	err = gretl_matrix_multi_ols(var->Y, var->X, var->B, 
-				     var->F, NULL);
+				     E, NULL);
 
 	if (!err) {
-	    ldet = gretl_VAR_ldet(var, &err);
+	    ldet = gretl_VAR_ldet(var, E, &err);
 	}
 
 	if (!err) {
@@ -379,7 +325,7 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
 		gretl_matrix_set(lltab, m, 1, 0);
 	    } else {
 		LRtest = 2.0 * (ll - gretl_matrix_get(lltab, m-1, 0));
-		gretl_matrix_set(lltab, m, 1, chisq_cdf_comp(LRtest, n * n));
+		gretl_matrix_set(lltab, m, 1, chisq_cdf_comp(n * n, LRtest));
 	    }	
 	    
 	    for (c=0; c<N_IVALS; c++) {
@@ -397,7 +343,7 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
     if (!err) {
 	gretl_matrix_set(lltab, m, 0, var->ll);
 	LRtest = 2.0 * (var->ll - gretl_matrix_get(lltab, m - 1, 0));
-	gretl_matrix_set(lltab, m, 1, chisq_cdf_comp(LRtest, n * n));
+	gretl_matrix_set(lltab, m, 1, chisq_cdf_comp(n * n, LRtest));
 	gretl_matrix_set(crittab, m, 0, var->AIC);
 	gretl_matrix_set(crittab, m, 1, var->BIC);
 	gretl_matrix_set(crittab, m, 2, var->HQC);
@@ -408,44 +354,89 @@ int VAR_do_lagsel (GRETL_VAR *var, const double **Z,
 
     gretl_matrix_free(crittab);
     gretl_matrix_free(lltab);
-
-    gretl_matrix_free(var->F);
-    var->F = NULL;
+    gretl_matrix_free(E);
 
     return err;
 }
 
-/* (X'X)^{-1} * X'\Omega X * (X'X)^{-1} : right now
-   we're supporting only HC0 and HC1 */
+static gretl_matrix *VAR_get_hvec (const gretl_matrix *X,
+				   const gretl_matrix *XTX,
+				   int *err)
+{
+    gretl_matrix *hvec;
+    gretl_matrix *Xt;
+    int t, T = X->rows;
+    int k = X->cols;
+    double ht;
+
+    Xt = gretl_matrix_alloc(1, k);
+    hvec = gretl_column_vector_alloc(T);
+
+    if (Xt == NULL || hvec == NULL) {
+	gretl_matrix_free(Xt);
+	gretl_matrix_free(hvec);
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (t=0; t<T && !*err; t++) {
+	gretl_matrix_get_row(X, t, Xt);
+	ht = gretl_scalar_qform(Xt, XTX, err);
+	gretl_vector_set(hvec, t, ht);
+    }
+
+    gretl_matrix_free(Xt);
+
+    return hvec;
+}
+
+/* (X'X)^{-1} * X'\Omega X * (X'X)^{-1} */
 
 static int VAR_robust_vcv (GRETL_VAR *var, gretl_matrix *V,
 			   MODEL *pmod, int hcv, int k)
 {
     gretl_matrix *XOX = NULL;
-    double xij, xti, xtj, utk;
+    gretl_matrix *hvec = NULL;
+    double xij, xti, xtj, utk, u2, ht;
     int T = var->T;
     int g = var->ncoeff;
     int i, j, t;
+    int err = 0;
 
     XOX = gretl_matrix_alloc(g, g);
     if (XOX == NULL) {
 	return E_ALLOC;
     }
 
+    if (hcv > 1) {
+	hvec = VAR_get_hvec(var->X, var->XTX, &err);
+	if (err) {
+	    gretl_matrix_free(XOX);
+	    return err;
+	}
+    }
+    
     /* form X' \Omega X */
     for (i=0; i<g; i++) {
 	for (j=i; j<g; j++) {
 	    xij = 0.0;
 	    for (t=0; t<T; t++) {
+		utk = gretl_matrix_get(var->E, t, k);
+		u2 = utk * utk;
+		if (hcv > 1) {
+		    ht = gretl_vector_get(hvec, t);
+		    u2 /= 1.0 - ht;
+		    if (hcv > 2) {
+			u2 /= 1.0 - ht;
+		    }
+		}
 		xti = gretl_matrix_get(var->X, t, i);
 		xtj = gretl_matrix_get(var->X, t, j);
-		utk = gretl_matrix_get(var->E, t, k);
-		xij += utk * utk * xti * xtj;
+		xij += u2 * xti * xtj;
 	    }
-	    if (hcv > 0) {
-		/* cheating here, for now */
+	    if (hcv == 1) {
 		xij *= (double) T / (T - g);
-	    }
+	    } 
 	    gretl_matrix_set(XOX, i, j, xij);
 	    if (i != j) {
 		gretl_matrix_set(XOX, j, i, xij);
@@ -456,12 +447,10 @@ static int VAR_robust_vcv (GRETL_VAR *var, gretl_matrix *V,
     gretl_matrix_qform(var->XTX, GRETL_MOD_TRANSPOSE, XOX,
 		       V, GRETL_MOD_NONE);
 
-    gretl_model_set_int(pmod, "hc", 1);
-    if (hcv > 0) {
-	gretl_model_set_int(pmod, "hc_version", 1);
-    }
+    gretl_model_set_vcv_info(pmod, VCV_HC, hcv);
 
     gretl_matrix_free(XOX);
+    gretl_matrix_free(hvec);
 
     return 0;
 }
@@ -469,23 +458,23 @@ static int VAR_robust_vcv (GRETL_VAR *var, gretl_matrix *V,
 /* Run the various per-equation omit tests (all lags of each var in
    turn, last lag of all vars) using the Wald method.  We also
    add the standard errors to the models here, since we have the
-   covariance matrix to hand.
+   per-equation covariance matrices to hand.
 */
 
-int VAR_wald_omit_tests (GRETL_VAR *var, int ifc)
+int VAR_wald_omit_tests (GRETL_VAR *var)
 {
     gretl_matrix *V = NULL;
     gretl_matrix *C = NULL;
     gretl_vector *b = NULL;
     int hcv = libset_get_int(HC_VERSION);
-    int p = var->order;
+    int p = (var->lags != NULL)? var->lags[0] : var->order;
     int n = var->neqns;
     int g = var->ncoeff;
     int dim = (p > n)? p : n;
     int i, j, k, m = 0;
     int err = 0;
 
-    if (ifc && var->robust && g - 1 > dim) {
+    if (var->ifc && var->robust && g - 1 > dim) {
 	/* need bigger arrays for robust overall F-test */
 	dim = g - 1;
     }
@@ -498,9 +487,9 @@ int VAR_wald_omit_tests (GRETL_VAR *var, int ifc)
 	return E_ALLOC;
     }     
 
-    for (i=0; i<n && !err; i++) {
+    for (i=0; i<var->neqns && !err; i++) {
 	MODEL *pmod = var->models[i];
-	int ii, jj, jpos, ipos = ifc;
+	int ii, jj, jpos, ipos = var->ifc;
 	double w, vij;
 
 	gretl_matrix_reuse(V, g, g);
@@ -525,7 +514,7 @@ int VAR_wald_omit_tests (GRETL_VAR *var, int ifc)
 	gretl_matrix_reuse(C, p, p);
 	gretl_matrix_reuse(b, p, 1);
 
-	for (j=0; j<n && !err; j++) {
+	for (j=0; j<var->neqns && !err; j++) {
 	    double w = NADBL;
 
 	    gretl_matrix_extract_matrix(C, V, ipos, ipos, GRETL_MOD_NONE);
@@ -549,9 +538,9 @@ int VAR_wald_omit_tests (GRETL_VAR *var, int ifc)
 	    gretl_matrix_reuse(C, n, n);
 	    gretl_matrix_reuse(b, n, 1);
 
-	    ipos = ifc + p - 1;
+	    ipos = var->ifc + p - 1;
 	    for (ii=0; ii<n; ii++) {
-		jpos = ifc + p - 1;
+		jpos = var->ifc + p - 1;
 		for (jj=0; jj<n; jj++) {
 		    vij = gretl_matrix_get(V, ipos, jpos);
 		    gretl_matrix_set(C, ii, jj, vij);
@@ -572,7 +561,7 @@ int VAR_wald_omit_tests (GRETL_VAR *var, int ifc)
 
 	/* exclusion of all but const? */
 
-	if (ifc && var->robust) {
+	if (var->ifc && var->robust) {
 	    gretl_matrix_reuse(C, g-1, g-1);
 	    gretl_matrix_reuse(b, g-1, 1);
 
@@ -610,7 +599,7 @@ const int *gretl_VAR_get_exo_list (const GRETL_VAR *var)
 
 static int *build_VAR_list (const GRETL_VAR *var, int *exolist, int *err)
 {
-    return list_composite(var->ylist, exolist, var->rlist);
+    return VAR_list_composite(var->ylist, exolist, var->rlist);
 }
 
 static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
@@ -640,7 +629,7 @@ static int gretl_VAR_real_omit_test (const GRETL_VAR *orig,
 
     LR = orig->T * (new->ldet - orig->ldet);
     df = orig->neqns * omitlist[0];
-    pval = chisq_cdf_comp(LR, df);
+    pval = chisq_cdf_comp(df, LR);
     
     pputs(prn, _("\n  Null hypothesis: the regression parameters are "
 		 "zero for the variables\n\n"));
