@@ -17,13 +17,13 @@
  * 
  */
 
-#ifndef GRETL_VERSION_H
-#define GRETL_VERSION_H
+#ifndef GRETL__VERSION_H
+#define GRETL__VERSION_H
 
 #define LIBGRETL_CURRENT   0
-#define LIBGRETL_REVISION  44
+#define LIBGRETL_REVISION  59
 #define LIBGRETL_AGE       0
 
-#define GRETL_VERSION "1.7.4"
+#define GRETL_VERSION "1.9.1"
 
-#endif
+#endif /* GRETL__VERSION_H */
