@@ -22,6 +22,7 @@
 #include "libgretl.h"
 #include "bhhh_max.h"
 #include "libset.h"
+#include "arma_priv.h"
 
 #include <glib.h>
 
@@ -121,7 +122,7 @@ static int add_unique_output_file (MODEL *pmod, const char *path)
     sprintf(unique, "%s.XXXXXX", fname);
     if (mktemp(unique) == NULL) return 1;
 
-    err = rename(fname, unique);
+    err = gretl_rename(fname, unique);
     if (!err) {
 	gretl_model_set_data(pmod, "x12a_output", g_strdup(unique),
 			     GRETL_TYPE_STRING, strlen(fname) + 1);
@@ -155,17 +156,23 @@ static int print_iterations (const char *path, PRN *prn)
     return err;
 }
 
-#define daily_frequency(f) (f == 5 || f == 6 || f == 7)
+#define non_yearly_frequency(f) (f != 1 && f != 4 && f != 12)
 
 static int x12_date_to_n (const char *s, const DATAINFO *pdinfo)
 {
     char date[12] = {0};
 
-    if (daily_frequency(pdinfo->pd)) {
-	int t, maj, min;
+    if (non_yearly_frequency(pdinfo->pd)) {
+	int t, maj = 0, min = 0, n = strlen(s);
+	char fmt[16];
+
+	sprintf(fmt, "%%%dd%%2d", n - 2);
+	if (sscanf(s, fmt, &maj, &min) == 2) {
+	    t = (maj - 1) * pdinfo->pd + min - 1;
+	} else {
+	    t = -1;
+	}
 
-	sscanf(s, "%1d%2d", &maj, &min);
-	t = (maj - 1) * pdinfo->pd + min - 1;
 	return t;
     }
 
@@ -223,11 +230,9 @@ static int get_ll_stats (const char *fname, MODEL *pmod)
 
     fclose(fp);
 
-    if (nobs != nefobs && nefobs > 0) {
+    if (nefobs > 0) {
 	pmod->nobs = nefobs;
-	pmod->t1 += nobs - nefobs;
-    } else {
-	pmod->nobs = nobs;
+	pmod->t1 = pmod->t2 - nefobs + 1;
     }
 
     return 0;
@@ -236,7 +241,7 @@ static int get_ll_stats (const char *fname, MODEL *pmod)
 /* Parse the roots information from the X12ARIMA output file foo.rts */
 
 static int get_roots (const char *fname, MODEL *pmod, 
-		      struct arma_info *ainfo)
+		      arma_info *ainfo)
 {
     FILE *fp;
     char line[132];
@@ -354,7 +359,7 @@ static int get_x12a_vcv (const char *fname, MODEL *pmod, int nc)
 */
 
 static int 
-get_estimates (const char *fname, MODEL *pmod, struct arma_info *ainfo)
+get_estimates (const char *fname, MODEL *pmod, arma_info *ainfo)
 {
     double *ar_coeff = pmod->coeff + ainfo->ifc;
     double *ma_coeff = ar_coeff + ainfo->np + ainfo->P;
@@ -475,9 +480,9 @@ get_uhat (const char *fname, MODEL *pmod, const DATAINFO *pdinfo)
 }
 
 static void 
-populate_arma_model (MODEL *pmod, const int *list, const char *path, 
+populate_arma_model (MODEL *pmod, const char *path, 
 		     const double **Z, const DATAINFO *pdinfo, 
-		     struct arma_info *ainfo)
+		     arma_info *ainfo)
 {
     char fname[MAXLEN];
     int t, err = 0;
@@ -529,7 +534,7 @@ populate_arma_model (MODEL *pmod, const int *list, const char *path,
 	fprintf(stderr, "problem reading X-12-ARIMA model info\n");
 	pmod->errcode = err;
     } else {
-	write_arma_model_stats(pmod, list, ainfo, Z, pdinfo);
+	write_arma_model_stats(pmod, ainfo, Z, pdinfo);
     }
 }
 
@@ -546,7 +551,7 @@ output_series_to_spc (const int *list, const double **Z,
 	    if (na(Z[list[i]][t])) {
 		fputs("-9999.0 ", fp);
 	    } else {
-		fprintf(fp, "%g ", Z[list[i]][t]);
+		fprintf(fp, "%.15g ", Z[list[i]][t]);
 	    }
 	}
 	fputc('\n', fp);
@@ -555,8 +560,7 @@ output_series_to_spc (const int *list, const double **Z,
     fputs(" )\n", fp);
 }
 
-static int *
-arma_info_get_x_list (struct arma_info *ainfo, const int *alist)
+static int *arma_info_get_x_list (arma_info *ainfo)
 {
     int *xlist = NULL;
     int start = arma_list_y_position(ainfo);
@@ -566,7 +570,7 @@ arma_info_get_x_list (struct arma_info *ainfo, const int *alist)
 
     if (xlist != NULL) {
 	for (i=1; i<=xlist[0]; i++) {
-	    xlist[i] = alist[i + start];
+	    xlist[i] = ainfo->alist[i + start];
 	}
     }
 
@@ -580,10 +584,9 @@ make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
     int yr, subper = 0;
     char *s;
 
-    /* daily data: FIXME when dated? */
-    if (daily_frequency(pdinfo->pd)) {
+    if (non_yearly_frequency(pdinfo->pd)) {
 	int maj = t / pdinfo->pd + 1;
-	int min = (t + 1) % pdinfo->pd;
+	int min = t % pdinfo->pd + 1;
 
 	sprintf(str, "%d.%d", maj, min);
 	return;
@@ -608,7 +611,7 @@ make_x12a_date_string (int t, const DATAINFO *pdinfo, char *str)
     }    
 }
 
-static void x12_pdq_string (struct arma_info *ainfo, FILE *fp)
+static void x12_pdq_string (arma_info *ainfo, FILE *fp)
 {
     fputc('(', fp);
     
@@ -651,24 +654,24 @@ static void x12_pdq_string (struct arma_info *ainfo, FILE *fp)
     fputc(')', fp);
 }
 
-#define MAXOBS 720
-#define MAXFCAST 60
-
 static int write_spc_file (const char *fname, 
 			   const double **Z, const DATAINFO *pdinfo,
-			   struct arma_info *ainfo, const int *alist,
+			   arma_info *ainfo, int pdmax, 
 			   gretlopt opt)
 {
+    int maxobs = pdmax * 50;
+    int maxfcast = pdmax * 5;
     int ylist[2];
     int *xlist = NULL;
     FILE *fp;
     char datestr[12];
     int nfcast = 0;
+    int t1 = ainfo->t1;
     int tmax;
     int i;
 
     if (ainfo->nexo > 0) {
-	xlist = arma_info_get_x_list(ainfo, alist);
+	xlist = arma_info_get_x_list(ainfo);
 	if (xlist == NULL) {
 	    return E_ALLOC;
 	}
@@ -682,10 +685,12 @@ static int write_spc_file (const char *fname,
 
     gretl_push_c_numeric_locale();
 
-    fprintf(fp, "series {\n period = %d\n title = \"%s\"\n", pdinfo->pd, 
-	    pdinfo->varname[ainfo->yno]);
+    fprintf(fp, "series {\n title = \"%s\"\n period = %d\n", 
+	    pdinfo->varname[ainfo->yno], pdinfo->pd);
 
-    make_x12a_date_string(ainfo->t1, pdinfo, datestr);
+    t1 -= ainfo->maxlag;
+    
+    make_x12a_date_string(t1, pdinfo, datestr);
     fprintf(fp, " start = %s\n", datestr);
 
     ylist[0] = 1;
@@ -699,21 +704,21 @@ static int write_spc_file (const char *fname,
 	/* FIXME ensure we don't print any NAs */
 
 	tmax = pdinfo->n - 1;
-	nobs = tmax - ainfo->t1 + 1;
-	if (nobs > MAXOBS) {
-	    tmax -= nobs - MAXOBS;
+	nobs = tmax - ainfo->t1 + 1; /* t1? */
+	if (nobs > maxobs) {
+	    tmax -= nobs - maxobs;
 	}
 	nfcast = tmax - ainfo->t2;
-	if (nfcast > MAXFCAST) {
-	    tmax -= nfcast - MAXFCAST;
-	    nfcast -= nfcast - MAXFCAST;
+	if (nfcast > maxfcast) {
+	    tmax -= nfcast - maxfcast;
+	    nfcast -= nfcast - maxfcast;
 	}
 #if 0
 	fprintf(stderr, "x12a: doing forecast: nfcast = %d\n", nfcast);
 #endif
     } 
 
-    output_series_to_spc(ylist, Z, ainfo->t1, tmax, fp);
+    output_series_to_spc(ylist, Z, t1, tmax, fp);
 
     if (tmax > ainfo->t2) {
 	make_x12a_date_string(ainfo->t2, pdinfo, datestr);
@@ -733,7 +738,7 @@ static int write_spc_file (const char *fname,
 	    fprintf(fp, "%s ", pdinfo->varname[xlist[i]]);
 	}
 	fputs(")\n", fp);
-	output_series_to_spc(xlist, Z, ainfo->t1, tmax, fp);
+	output_series_to_spc(xlist, Z, t1, tmax, fp);
 	free(xlist);
     }
     fputs("}\n", fp);
@@ -749,6 +754,10 @@ static int write_spc_file (const char *fname,
 
     fputs("estimate {\n", fp);
 
+    /* could enable here: "tol = XX, maxiter = NN"  
+       the default is tol = 1.0e-5, maxiter = 200 
+    */
+
     if (opt & OPT_V) {
 	fputs(" print = (acm itr lkf lks mdl est rts rcm)\n", fp);
     } else {
@@ -775,6 +784,20 @@ static int write_spc_file (const char *fname,
     return 0;
 }
 
+static void print_x12a_error_file (const char *fname, PRN *prn)
+{
+    FILE *fp = gretl_fopen(fname, "r");
+
+    if (fp != NULL) {
+	char line[512];
+
+	while (fgets(line, sizeof line, fp)) {
+	    pputs(prn, line);
+	}
+	fclose(fp);
+    }
+}
+
 static void delete_old_files (const char *path)
 {
     const char *exts[] = {
@@ -788,45 +811,42 @@ static void delete_old_files (const char *path)
 	*old = '\0';
 	strncat(old, path, n - 3);
 	strcat(old, exts[i]);
-	remove(old);
+	gretl_remove(old);
     }
 }
 
 MODEL arma_x12_model (const int *list, const char *pqspec,
 		      const double **Z, const DATAINFO *pdinfo,
-		      gretlopt opt, PRN *prn)
+		      int pdmax, gretlopt opt, PRN *prn)
 {
     int verbose = (opt & OPT_V);
     const char *prog = gretl_x12_arima();
     const char *workdir = gretl_x12_arima_dir();
     char yname[VNAMELEN], path[MAXLEN];
-    int *alist = NULL;
-    PRN *vprn = NULL;
     MODEL armod;
-    struct arma_info ainfo;
+    arma_info ainfo;
 #ifdef WIN32
     char *cmd;
 #endif
     int err = 0;
 
-    vprn = set_up_verbose_printer(opt, prn);
-
     if (pdinfo->t2 < pdinfo->n - 1) {
 	/* FIXME this is temporary (OPT_F -> generate forecast) */
 	opt |= OPT_F;
     }
 
-    arma_info_init(&ainfo, ARMA_X12A, pqspec, pdinfo);
+    arma_info_init(&ainfo, opt | OPT_X, pqspec, pdinfo);
+    ainfo.prn = set_up_verbose_printer(opt, prn);
     gretl_model_init(&armod); 
     gretl_model_smpl_init(&armod, pdinfo);
 
-    alist = gretl_list_copy(list);
-    if (alist == NULL) {
+    ainfo.alist = gretl_list_copy(list);
+    if (ainfo.alist == NULL) {
 	err = E_ALLOC;
     }
 
     if (!err) {
-	err = arma_check_list(alist, opt, Z, pdinfo, &ainfo);
+	err = arma_check_list(&ainfo, Z, pdinfo, opt);
     }
 
     if (err) {
@@ -838,21 +858,21 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
     calc_max_lag(&ainfo);
 
     /* adjust sample range if need be */
-    if (arma_adjust_sample(pdinfo, Z, alist, &ainfo)) {
+    if (arma_adjust_sample(&ainfo, Z, pdinfo)) {
         armod.errcode = E_DATA;
 	goto bailout;
     }
 
     /* create differenced series if needed */
     if (ainfo.d > 0 || ainfo.D > 0) {
-	err = arima_difference(Z[ainfo.yno], &ainfo);
+	err = arima_difference(&ainfo, Z, pdinfo);
     }  
 
-    sprintf(yname, pdinfo->varname[ainfo.yno]);
+    strcpy(yname, pdinfo->varname[ainfo.yno]);
 
     /* write out an .spc file */
     sprintf(path, "%s%c%s.spc", workdir, SLASH, yname);
-    write_spc_file(path, Z, pdinfo, &ainfo, alist, opt);
+    write_spc_file(path, Z, pdinfo, &ainfo, pdmax, opt);
 
     /* remove any files from on old run, in case of error */
     delete_old_files(path);
@@ -871,37 +891,40 @@ MODEL arma_x12_model (const int *list, const char *pqspec,
 	sprintf(path, "%s%c%s", workdir, SLASH, yname); 
 	armod.t1 = ainfo.t1;
 	armod.t2 = ainfo.t2;
-	armod.nobs = armod.t2 - armod.t1 + 1;
-	populate_arma_model(&armod, alist, path, Z, pdinfo, &ainfo);
+	populate_arma_model(&armod, path, Z, pdinfo, &ainfo);
 	if (verbose && !armod.errcode) {
-	    print_iterations(path, vprn);
+	    print_iterations(path, ainfo.prn);
 	}
 	if (!armod.errcode) {
-	    int acode = ARMA_X12A;
-
 	    if (gretl_in_gui_mode()) {
 		add_unique_output_file(&armod, path);
 	    }
-	    if (!(opt & OPT_C)) {
-		acode |= ARMA_EXACT;
-	    }
-	    gretl_model_set_int(&armod, "arma_flags", acode);
+	    gretl_model_set_int(&armod, "arma_flags", (int) ainfo.flags);
 	}	
     } else {
 	armod.errcode = E_UNSPEC;
 	gretl_errmsg_set(_("Failed to execute x12arima"));
     }
 
-    if (vprn != NULL && vprn != prn) {
-	iter_print_callback(0, vprn);
-	close_down_verbose_printer(vprn);
+    if (armod.errcode && ainfo.prn != NULL) {
+	sprintf(path, "%s%c%s.err", workdir, SLASH, yname);
+	print_x12a_error_file(path, ainfo.prn);
+    }
+
+    if (ainfo.prn != NULL && ainfo.prn != prn) {
+	iter_print_callback(0, ainfo.prn);
+	close_down_verbose_printer(ainfo.prn);
     }
 
  bailout:
 
-    free(alist);
     arma_info_cleanup(&ainfo);
 
+    if (armod.errcode && (opt & OPT_U)) {
+	/* continue on error */
+	armod.opt |= OPT_U;
+    }
+
     return armod;
 }
 
