@@ -17,6 +17,10 @@
  * 
  */
 
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)
+# include "gtk_compat.h"
+#endif
+
 static void invalid_varname (PRN *prn)
 {
     pputs(prn, gretl_errmsg_get());
@@ -25,309 +29,61 @@ static void invalid_varname (PRN *prn)
 
 #ifndef ODS_IMPORTER
 
-static int label_is_date (char *str, int *submax)
-{
-    int len = strlen(str);
-    int i, d, sub = 0, pd = 0;
-    char *p;
-    double dd;
-
-#if 0
-    fprintf(stderr, "label_is_date: looking at '%s'\n", str);
-#endif
-
-    for (i=0; i<len; i++) {
-	if (str[i] == ':' || str[i] == 'Q') {
-	    str[i] = '.';
-	    break;
-	}
-    }
-
-    p = strchr(str, '.');
-
-    if (len == 4 && sscanf(str, "%d", &d) && d > 0 && d < 3000) {
-	pd = 1;
-    } else if (p != NULL) {
-	if (len == 6) {
-	    if (sscanf(str, "%lf", &dd) && dd > 0 && dd < 3000) { 
-		sub = atoi(p + 1);
-		if (sub > 0 && sub < 5) {
-		    pd = 4;
-		}
-	    }
-	} else if (len == 7) {
-	    if (sscanf(str, "%lf", &dd) && dd > 0 && dd < 3000) {
-		sub = atoi(p + 1);
-		if (sub > 0 && sub < 13) {
-		    pd = 12;
-		}
-	    }
-	}
-	if (sub > *submax) {
-	    *submax = sub;
-	}
-    }
-
-    return pd;
-}
-
-static int pd_from_dmult (double dm)
-{
-    int pd = 0;
-
-    if (dm > 250.0) {
-	pd = 1;
-    } else if (dm > 90.0) {
-	pd = 4;
-    } else if (dm > 29.0) {
-	pd = 12;
-    } else if (dm > 6.0) {
-	pd = 52;
-    } else if (dm > 1.4) {
-	pd = 5;
-    } else if (dm > 1.16) {
-	pd = 6;
-    } else if (dm > 0.9) {
-	pd = 7;
-    }
-
-    return pd;
-}
-
-#define dmax(f) ((f == 1)? 366 : (f == 4)? 92 : 31)
-
-static int 
-calendar_missing_obs (int diff, int pd, double d1, BookFlag flags)
-{
-    int mc = 0;
-
-    if (pd == 52) {
-	if (diff > 7) {
-	    mc = (diff / 7) - 1;
-	}
-    } else if (pd == 7) {
-	if (diff > 1) {
-	    mc = diff - 1;
-	}
-    } else if (pd == 1 || pd == 4 || pd == 12) {
-	if (diff > dmax(pd)) {
-	    double xmc = diff / (365.0 / pd);
-
-	    mc = floor(xmc - .5);
-	}
-    } else if ((pd == 5 || pd == 6) && diff > 1) {
-	char dstr[12];
-	int wday;
-
-	MS_excel_date_string(dstr, d1, 0, flags & BOOK_DATE_BASE_1904);
-	wday = get_day_of_week(dstr);
-
-	if (wday == 1) {
-	    if (pd == 5) {
-		mc = diff - 3;
-	    } else {
-		mc = diff - 2;
-	    }
-	} else {
-	    mc = diff - 1;
-	}
-    }
-
-    return mc;
-}
-
 #ifdef EXCEL_IMPORTER
-# define cell_val(t,o) (rows[t].cells[o])
+# define cell_string(i,j) ((rows[i].cells != NULL)? rows[i].cells[j] : NULL)
 #else
-# define cell_val(t,o) (labels[t])
+# define cell_string(i,j) (labels[i])
 #endif
 
-static int pd_from_numeric_dates (int nrows, int row_offset, int col_offset,
-				  char **labels, wbook *book)
-{
-    char dstr[12];
-    int tstart = 1 + row_offset;
-    int ndays, nobs = nrows - tstart;
-    int mc, t, pd = 0;
-    double x1, x2, dmult, diff;
-    char *test;
-
-    fprintf(stderr, "check for consistent numeric dates in col %d (nobs = %d)\n", 
-	    col_offset, nobs);
-
-    /* look at first date */
-    test = cell_val(tstart, col_offset);
-    if (sscanf(test, "%lf", &x1)) {
-	MS_excel_date_string(dstr, (int) x1, 0, book_base_1904(book));
-	fprintf(stderr, "numeric date on row %d = %g (%s)\n", tstart, 
-		x1, dstr);
-    } else {
-	fprintf(stderr, "failed to read starting\n");
-	return 0;
-    }
-
-    /* look at last date */
-    test = cell_val(nrows - 1, col_offset);
-    if (sscanf(test, "%lf", &x2)) {
-	MS_excel_date_string(dstr, (int) x2, 0, book_base_1904(book));
-	fprintf(stderr, "numeric date on row %d = %g (%s)\n", nrows - 1, 
-		x2, dstr);
-    } else {
-	fprintf(stderr, "failed to read ending date\n");
-	return 0;
-    }
-
-    /* compare number of obs and calendar span of data */
-    ndays = (int) x2 - (int) x1 + 1;
-    dmult = (double) ndays / nobs;
-    fprintf(stderr, "Calendar interval = %d days\n", ndays);
-    fprintf(stderr, "Calendar days per observation = %g\n", dmult);
-    pd = pd_from_dmult(dmult);
-
-    if (pd > 0) {
-	fprintf(stderr, "provisional data frequency = %d\n", pd);
-    } else {
-	fputs("Can't make sense of this\n", stderr);
-	return 0;
-    }
-
-    book->totmiss = 0;
-
-    for (t=tstart; t<nrows; t++) {
-	test = cell_val(t, col_offset);
-
-	if (sscanf(test, "%lf", &x1) != 1) {
-	    fprintf(stderr, "Problem: blank cell at row %d\n", t + 1);
-	    return 0;
-	}
-
-	if (t > tstart) {
-	    diff = x1 - x2;
-	    mc = calendar_missing_obs((int) diff, pd, x1, book->flags);
-	    if (mc > 0) {
-		fprintf(stderr, "row %d: calendar gap = %g, %d values missing?\n", 
-			t, diff, mc);
-		book->totmiss += mc;
-	    }
-	}
-
-	x2 = x1;
-    }
-
-    if (book->totmiss > 0) {
-	int i, s = 0;
-
-	fprintf(stderr, "Total missing values = %d\n", book->totmiss);
-	book->missmask = calloc(nobs + book->totmiss, 1);
-
-	if (book->missmask == NULL) {
-	    fprintf(stderr, "Out of memory allocating missing obs mask\n");
-	    return 0;
-	}
-
-	for (t=tstart; t<nrows; t++) {
-	    test = cell_val(t, col_offset);
-	    sscanf(test, "%lf", &x1);
-	    if (t > tstart) {
-		mc = calendar_missing_obs((int) (x1 - x2), pd, x1, book->flags);
-		for (i=0; i<mc; i++) {
-		    book->missmask[s++] = 1;
-		}
-	    }
-	    x2 = x1;
-	    s++;
-	}
-    }
-
-    fprintf(stderr, "Setting data frequency = %d\n", pd);
-
-    return pd;
-}
-
 static int 
-consistent_date_labels (int nrows, int row_offset, int col_offset, char **labels)
-{
-    int t, tstart = 1 + row_offset;
-    int pd = 0, pdbak = 0;
-    int submax = 0;
-    double x, xbak = 0.0;
-    char *test;
-
-    fprintf(stderr, "testing for consistent date labels in col %d\n", 
-	    col_offset);
-
-    for (t=tstart; t<nrows; t++) {
-	test = cell_val(t, col_offset);
-
-	if (*test == '\0') {
-	    fprintf(stderr, " no: blank cell at row %d\n", t + 1);
-	    return 0;
-	} else if (*test == '"' || *test == '\'') {
-	    test++;
-	}
-
-	pd = label_is_date(test, &submax);
-	if (pd == 0) {
-	    fprintf(stderr, " no: label '%s' on row %d is not a valid date\n", 
-		    test, t + 1);
-	    return 0;
-	}
-
-	if (pd == 12 && t - tstart > 3 && submax == 4) {
-	    /* appeared to be monthly but really quarterly? */
-	    pd = pdbak = 4;
-	}
-
-	x = atof(test);
-
-	if (t > tstart) {
-	    if (pd != pdbak) {
-		fprintf(stderr, " no: got inconsistent data frequencies %d and %d\n",
-			pdbak, pd);
-		return 0;
-	    }
-	    if (x <= xbak) {
-		fprintf(stderr, " no: got %g <= %g\n", x, xbak);
-		return 0;
-	    }
-	}
-
-	pdbak = pd;
-	xbak = x;
-    }
-
-    fprintf(stderr, " yes: data frequency = %d\n", pd);
-
-    return pd;
-}
-
-static int obs_column_heading (const char *label)
+importer_dates_check (int row_offset, int col_offset, 
+		      BookFlag *pflags, char **labels, 
+		      DATAINFO *newinfo, PRN *prn, int *err)
 {
+    int d, t;
+    char dstr[12];
+    char *s;
     int ret = 0;
 
-    if (label == NULL) {
-	ret = 1;
-    } else {
-#if 1
-	fprintf(stderr, "obs_column_heading: looking at '%s'\n", label);
-#endif
-	if (*label == '"') {
-	    label++;
+    for (t=0; t<newinfo->n; t++) {
+	s = cell_string(t + row_offset, col_offset);
+	if (s == NULL || *s == '\0') {
+	    fprintf(stderr, "importer_dates_check: got blank label\n");
+	    return 0;
 	}
-	if (*label == '\0') {
-	    ret = 1;    
-	} else {
-	    gchar *test = g_strdup(label);
+    }
 
-	    lower(test);
-	    if (strncmp(test, "obs", 3) == 0 ||
-		strcmp(test, "date") == 0 ||
-		strcmp(test, "year") == 0 ||
-		strcmp(test, "yr") == 0) {
-		ret = 1;
+    *err = dataset_allocate_obs_markers(newinfo);
+    if (*err) {
+	return 0;
+    }
+
+    for (t=0; t<newinfo->n && !*err; t++) {
+	s = cell_string(t + row_offset, col_offset);
+	if (*s == '"' || *s == '\'') s++;
+	if (*pflags & BOOK_NUMERIC_DATES) {
+	    if (sscanf(s, "%d", &d)) {
+		MS_excel_date_string(dstr, d, 0, *pflags & BOOK_DATE_BASE_1904);
+		s = dstr;
+	    } else {
+		pprintf(prn, "Bad date on row %d: '%s'\n", t+1, s);
+		*err = E_DATA;
 	    }
-	    g_free(test);
 	}
+	strncat(newinfo->S[t], s, OBSLEN - 1);
+    }
+
+    if (!*err) {
+	int reversed = 0;
+
+	ret = test_markers_for_dates(NULL, newinfo, &reversed, NULL, prn);
+	if (reversed) {
+	    *pflags |= BOOK_DATA_REVERSED;
+	}
+    }
+
+    if (newinfo->markers != DAILY_DATE_STRINGS) {
+	dataset_destroy_obs_markers(newinfo);
     }
 
     return ret;
@@ -358,27 +114,74 @@ static void wbook_free (wbook *book)
 	free(book->sheetnames[i]);
     }
     free(book->sheetnames);
+    free(book->targname);
     free(book->byte_offsets);
     free(book->xf_list);
-    free(book->missmask);
+}
+
+static int wbook_check_params (wbook *book)
+{
+    if (book->targname != NULL) {
+	int i;
+
+	book->selected = -1;
+	for (i=0; i<book->nsheets; i++) {
+	    if (!strcmp(book->targname, book->sheetnames[i])) {
+		book->selected = i;
+	    }
+	}
+	if (book->selected < 0) {
+	    gretl_errmsg_sprintf("\"%s\": no such sheet", book->targname);
+	    return E_DATA;
+	}
+    }
+
+    if (book->selected < 0 || book->selected >= book->nsheets) {
+	return E_DATA;
+    } else if (book->col_offset < 0 || book->row_offset < 0) {
+	return E_DATA;
+    } else {
+	return 0;
+    }
+}
+
+static void wbook_record_params (wbook *book, int *list)
+{
+    if (list != NULL && list[0] == 3) {
+	list[1] = book->selected + 1;
+	list[2] = book->col_offset;
+	list[3] = book->row_offset;
+    }
 }
 
 #endif /* !ODS_IMPORTER */
 
-static void wbook_init (wbook *book)
+static void wbook_init (wbook *book, const int *list, char *sheetname)
 {
     book->version = 0;
     book->nsheets = 0;
     book->col_offset = book->row_offset = 0;
+    book->targname = NULL;
     book->sheetnames = NULL;
     book->byte_offsets = NULL;
     book->selected = 0;
     book->flags = 0;
     book->xf_list = NULL;
-    book->totmiss = 0;
-    book->missmask = NULL;
     book->get_min_offset = NULL;
     book->data = NULL;
+
+    if (sheetname != NULL && *sheetname != '\0') {
+	book->targname = gretl_strdup(sheetname);
+	tailstrip(book->targname);
+    }
+
+    if (list != NULL && list[0] == 3) {
+	if (book->targname == NULL && list[1] > 0) {
+	    book->selected = list[1] - 1;
+	}
+	book->col_offset = list[2];
+	book->row_offset = list[3];
+    }
 }
 
 static const char *column_label (int col)
@@ -429,6 +232,7 @@ void infobox (const char *template, ...)
 				     GTK_DIALOG_DESTROY_WITH_PARENT,
 				     GTK_MESSAGE_INFO,
 				     GTK_BUTTONS_CLOSE,
+				     "%s",
 				     msg);
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
@@ -559,14 +363,14 @@ void add_sheets_list (GtkWidget *vbox, wbook *book)
 
     store = gtk_list_store_new(1, G_TYPE_STRING);
     view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-    g_object_unref (G_OBJECT(store));
+    g_object_unref(G_OBJECT(store));
 
-    renderer = gtk_cell_renderer_text_new ();
-    g_object_set (renderer, "ypad", 0, NULL);
-    column = gtk_tree_view_column_new_with_attributes (NULL,
-                                                       renderer,
-                                                       "text", 
-                                                       0, NULL);
+    renderer = gtk_cell_renderer_text_new();
+    g_object_set(renderer, "ypad", 0, NULL);
+    column = gtk_tree_view_column_new_with_attributes(NULL,
+						      renderer,
+						      "text", 
+						      0, NULL);
     gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);   
     gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
 
@@ -581,20 +385,25 @@ void add_sheets_list (GtkWidget *vbox, wbook *book)
     /* now set up the widgets */
 
     hsep = gtk_hseparator_new();
-    gtk_container_add(GTK_CONTAINER(vbox), hsep);
+    gtk_box_pack_start(GTK_BOX(vbox), hsep, FALSE, FALSE, 5);
 
     label = gtk_label_new(_("Sheet to import:"));
-    gtk_container_add(GTK_CONTAINER(vbox), label);
+    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
 
-    sw = gtk_scrolled_window_new (NULL, NULL);
+    sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 5);
 
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-                                    GTK_POLICY_AUTOMATIC,
-                                    GTK_POLICY_AUTOMATIC);
-    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
-                                         GTK_SHADOW_IN);
-    gtk_container_add (GTK_CONTAINER(sw), view); 
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
+				   GTK_POLICY_AUTOMATIC,
+				   GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
+					GTK_SHADOW_IN);
+    gtk_container_add(GTK_CONTAINER(sw), view); 
+}
+
+static void make_wmenu_modal (GtkWidget *w, gpointer p)
+{
+    gtk_window_set_modal(GTK_WINDOW(w), TRUE);
 }
 
 static void wsheet_menu (wbook *book, int multisheet)
@@ -611,8 +420,10 @@ static void wsheet_menu (wbook *book, int multisheet)
 			   G_CALLBACK(wsheet_menu_cancel), book);
     g_signal_connect(G_OBJECT(w), "destroy",  
 		     G_CALLBACK(gtk_main_quit), NULL);
+    g_signal_connect(G_OBJECT(w), "realize",
+		     G_CALLBACK(make_wmenu_modal), NULL);
 
-    vbox = GTK_DIALOG(w)->vbox;
+    vbox = gtk_dialog_get_content_area(GTK_DIALOG(w));
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
     /* selection of starting column and row */
@@ -620,12 +431,12 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
 
     hbox = gtk_hbox_new(FALSE, 5);
-    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 
     /* starting column spinner */
     tmp = gtk_label_new(_("column:"));
     offmin = book->col_offset + 1;
-    c_adj = gtk_adjustment_new(offmin, offmin, 256, 1, 1, 1);
+    c_adj = gtk_adjustment_new(offmin, offmin, 256, 1, 1, 0);
     book->colspin = gtk_spin_button_new(GTK_ADJUSTMENT(c_adj), 1, 0);
     g_signal_connect(c_adj, "value_changed",
 		     G_CALLBACK(wbook_set_col_offset), book);
@@ -637,7 +448,7 @@ static void wsheet_menu (wbook *book, int multisheet)
     /* starting row spinner */
     tmp = gtk_label_new(_("row:"));
     offmin = book->row_offset + 1;
-    r_adj = gtk_adjustment_new(offmin, offmin, 256, 1, 1, 1);
+    r_adj = gtk_adjustment_new(offmin, offmin, 256, 1, 1, 0);
     book->rowspin = gtk_spin_button_new(GTK_ADJUSTMENT(r_adj), 1, 0);
     g_signal_connect(r_adj, "value_changed",
 		     G_CALLBACK(wbook_set_row_offset), book);
@@ -647,10 +458,10 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_box_pack_start(GTK_BOX(hbox), book->rowspin, FALSE, FALSE, 5);
 
     /* column label feedback */
-    hbox = gtk_hbox_new (FALSE, 5);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+    hbox = gtk_hbox_new(FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     label = gtk_label_new("(A)");
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
     g_signal_connect(GTK_EDITABLE(book->colspin), "changed",
 		     G_CALLBACK(colspin_changed), label);
 
@@ -664,12 +475,12 @@ static void wsheet_menu (wbook *book, int multisheet)
     g_signal_connect(G_OBJECT(tmp), "toggled", G_CALLBACK(debug_callback), 
 		     book);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), FALSE);
-    gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, TRUE, 5);
+    gtk_box_pack_start(GTK_BOX(vbox), tmp, FALSE, FALSE, 5);
 #endif
 
-    hbox = GTK_DIALOG(w)->action_area;
+    hbox = gtk_dialog_get_action_area(GTK_DIALOG(w));
     gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_END);
-    gtk_button_box_set_spacing(GTK_BUTTON_BOX(hbox), 10);
+    gtk_box_set_spacing(GTK_BOX(hbox), 10);
 
     /* Cancel button */
     tmp = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
@@ -693,10 +504,6 @@ static void wsheet_menu (wbook *book, int multisheet)
     gtk_entry_set_activates_default(GTK_ENTRY(book->rowspin), TRUE);
 
     gtk_widget_show_all(w);
-
-    gtk_window_set_modal(GTK_WINDOW(w), TRUE);
-
     gtk_main();
 }
 
-
