@@ -19,6 +19,8 @@
 
 #include <gtk/gtk.h>
 
+#define FULL_XML_HEADERS
+
 #include "libgretl.h"
 #include "gretl_xml.h"
 #include "csvdata.h"
@@ -35,16 +37,16 @@
 
 static void remove_temp_dir (char *dname)
 {
-    const char *udir = gretl_dot_dir();
+    const char *udir = gretl_dotdir();
 
 #ifdef G_OS_WIN32
     char *fullpath = g_strdup_printf("%s%s", udir, dname);
 
-    chdir(udir);
+    gretl_chdir(udir);
     win32_delete_dir(fullpath);
     g_free(fullpath);
 #else
-    chdir(udir);
+    gretl_chdir(udir);
     gretl_deltree(dname);
 #endif
 }
@@ -444,6 +446,7 @@ static int real_read_cell (xmlNodePtr cur,
     t = iread - vnames;
 
     if (v >= sheet->dinfo->v || t >= sheet->dinfo->n) {
+	fprintf(stderr, "v = %d, t = %d: out of bounds?\n", v, t);
 	return E_DATA;
     }
 
@@ -548,6 +551,7 @@ static int real_read_cell (xmlNodePtr cur,
 	}
 	free(val);
     } else {
+	fprintf(stderr, " vtype = %d??\n", vtype); 
 	err = E_DATA;
     }
 
@@ -568,12 +572,13 @@ static int read_data_row (xmlNodePtr cur,
 			  int readrow,
 			  PRN *prn)
 {
+    int readmax = tab->cols - sheet->xoffset;
     int tabcol = 0, readcol = 0;
     int err = 0;
 
     cur = cur->xmlChildrenNode;
 
-    while (cur != NULL && !err && readcol < tab->cols) {
+    while (cur != NULL && !err && readcol < readmax) {
 	if (!xmlStrcmp(cur->name, (XUC) "table-cell")) {
 	    if (tabcol >= sheet->xoffset) {
 		err = real_read_cell(cur, sheet, 
@@ -739,7 +744,7 @@ static int repeat_data_row (ods_sheet *sheet, int iread,
     int i, t = iread - vnames;
 
     if (t < 1 || t >= sheet->dinfo->n) {
-	pprintf(prn, "Found a repeated row in the wrong place");
+	pprintf(prn, "Found a repeated row in the wrong place\n");
 	return E_DATA;
     }
 
@@ -758,7 +763,8 @@ static int read_table_content (ods_sheet *sheet, PRN *prn)
 {
     ods_table *tab;
     xmlNodePtr cur;
-    int i, nr, tabrow = 0, readrow = 0;
+    int i, nr, maxrow;
+    int tabrow = 0, readrow = 0;
     int err = 0;
     
     if (sheet->seltab < 0 || sheet->seltab >= sheet->n_tables) {
@@ -776,12 +782,13 @@ static int read_table_content (ods_sheet *sheet, PRN *prn)
     if (err) {
 	return err;
     }
-    
+
+    maxrow = tab->rows - sheet->yoffset;
     cur = tab->node->xmlChildrenNode;
 
     gretl_push_c_numeric_locale();
 
-    while (cur != NULL && !err && readrow < tab->rows) {
+    while (cur != NULL && !err && readrow < maxrow) {
 	if (!xmlStrcmp(cur->name, (XUC) "table-row")) {
 	    nr = ods_row_height(cur);
 	    if (tabrow >= sheet->yoffset) {
@@ -889,7 +896,7 @@ get_table_dimensions (xmlNodePtr cur, ods_sheet *sheet)
     return err;
 }
 
-static ods_sheet *read_content (PRN *prn, int *err) 
+static ods_sheet *ods_read_content (PRN *prn, int *err) 
 {
     ods_sheet *sheet = NULL;
     xmlDocPtr doc = NULL;
@@ -989,7 +996,7 @@ static int gretl_make_tempdir (char *dname)
     s = mkdtemp(dname);
 
     if (s == NULL) {
-	gretl_errmsg_set_from_errno();
+	gretl_errmsg_set_from_errno("gretl_make_tempdir");
 	err = E_FOPEN;
     } 
 
@@ -1000,19 +1007,24 @@ static int gretl_make_tempdir (char *dname)
 
 static int ts_check (ods_sheet *sheet, PRN *prn)
 {
+    int reversed = 0;
     int mpd = -1;
 
     if (sheet->dinfo->S == NULL) {
 	return 0;
     }
 
-    mpd = test_markers_for_dates(&sheet->Z, sheet->dinfo, NULL, prn);
+    mpd = test_markers_for_dates(&sheet->Z, sheet->dinfo, &reversed,
+				 NULL, prn);
 
     if (mpd > 0) {
 	pputs(prn, _("taking date information from row labels\n\n"));
 	if (sheet->dinfo->markers != DAILY_DATE_STRINGS) {
 	    dataset_destroy_obs_markers(sheet->dinfo);
 	}
+	if (reversed) {
+	    sheet->flags |= BOOK_DATA_REVERSED;
+	}
     } 
 
 #if ODEBUG
@@ -1048,11 +1060,11 @@ static int ods_min_offset (wbook *book, int k)
     return ret;
 }
 
-static int ods_book_init (wbook *book, ods_sheet *sheet)
+static int ods_book_init (wbook *book, ods_sheet *sheet, char *sheetname)
 {
     int i, err = 0;
 
-    wbook_init(book);
+    wbook_init(book, NULL, sheetname);
 
     if (sheet->n_tables > 0) {
 	book->sheetnames = strings_array_new(sheet->n_tables);
@@ -1074,11 +1086,71 @@ static int ods_book_init (wbook *book, ods_sheet *sheet)
     return err;
 }
 
+static void record_ods_params (ods_sheet *sheet, int *list)
+{
+    if (list != NULL && list[0] == 3) {
+	list[1] = sheet->seltab + 1;
+	list[2] = sheet->xoffset;
+	list[3] = sheet->yoffset;
+    }
+}
+
+static int set_ods_params_from_cli (ods_sheet *sheet, 
+				    const int *list,
+				    char *sheetname)
+{
+    int gotname = (sheetname != NULL && *sheetname != '\0');
+    int gotlist = (list != NULL && list[0] == 3);
+    int i;
+
+    if (!gotname && !gotlist) {
+	sheet->seltab = 0;
+	sheet->xoffset = sheet->tables[0]->xoffset;
+	sheet->yoffset = sheet->tables[0]->yoffset;
+	return 0;
+    }
+
+    /* invalidate this */
+    sheet->seltab = -1;
+
+    if (gotname) {
+	for (i=0; i<sheet->n_tables; i++) {
+	    if (!strcmp(sheetname, sheet->tables[i]->name)) {
+		sheet->seltab = i;
+		break;
+	    }
+	}
+	if (sheet->seltab < 0 && integer_string(sheetname)) {
+	    i = atoi(sheetname);
+	    if (i >= 1 && i <= sheet->n_tables) {
+		sheet->seltab = i - 1;
+	    }
+	}
+    }
+
+    if (gotlist) {
+	if (!gotname) {
+	    /* convert to zero-based */
+	    sheet->seltab = list[1] - 1;
+	}
+	sheet->xoffset = list[2];
+	sheet->yoffset = list[3];
+    }
+
+    if (sheet->seltab < 0 || sheet->seltab >= sheet->n_tables ||
+	sheet->xoffset < 0 || sheet->yoffset < 0) {
+	gretl_errmsg_set(_("Invalid argument for worksheet import"));
+	return E_DATA;
+    }
+
+    return 0;
+}
+
 static int ods_sheet_dialog (ods_sheet *sheet, int *err)
 {
     wbook book;   
 
-    *err = ods_book_init(&book, sheet);
+    *err = ods_book_init(&book, sheet, NULL);
     if (*err) {
 	return -1;
     }
@@ -1144,6 +1216,9 @@ static int finalize_ods_import (double ***pZ, DATAINFO *pdinfo,
     if (!err) {
 	tprn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
 	ts_check(sheet, tprn);
+	if (sheet->flags & BOOK_DATA_REVERSED) {
+	    reverse_data(sheet->Z, sheet->dinfo, tprn);
+	}
 	gretl_print_destroy(tprn);
     }
 
@@ -1168,14 +1243,14 @@ static char *get_absolute_path (const char *fname)
     return ret;
 }
 
-int ods_get_data (const char *fname, 
+int ods_get_data (const char *fname, int *list, char *sheetname,
 		  double ***pZ, DATAINFO *pdinfo,
 		  gretlopt opt, PRN *prn)
 {
     int gui = (opt & OPT_G);
     ods_sheet *sheet = NULL;
     int (*gretl_unzip_file)(const char *, GError **);
-    const char *udir = gretl_dot_dir();
+    const char *udir = gretl_dotdir();
     char *abspath = NULL;
     void *handle;
     char dname[32];
@@ -1199,16 +1274,16 @@ int ods_get_data (const char *fname,
     }
 
     /* cd to user dir */
-    if (chdir(udir)) {
-	gretl_errmsg_set_from_errno();
+    if (gretl_chdir(udir)) {
+	gretl_errmsg_set_from_errno(udir);
 	return E_FOPEN;
     }
 
     err = gretl_make_tempdir(dname);
     if (!err) {
-	err = chdir(dname);
+	err = gretl_chdir(dname);
 	if (err) {
-	    remove(dname);
+	    gretl_remove(dname);
 	}
     }
     
@@ -1219,7 +1294,7 @@ int ods_get_data (const char *fname,
     gretl_unzip_file = get_plugin_function("gretl_unzip_file", 
 					   &handle);
     if (gretl_unzip_file == NULL) {
-	remove(dname);
+	gretl_remove(dname);
         return E_FOPEN;
     }
 
@@ -1244,7 +1319,7 @@ int ods_get_data (const char *fname,
     }
 
     if (!err) {
-	sheet = read_content(prn, &err);
+	sheet = ods_read_content(prn, &err);
     }
 
     remove_temp_dir(dname);
@@ -1255,6 +1330,9 @@ int ods_get_data (const char *fname,
 	err = ods_sheet_prune(sheet, prn);
     }
 
+    printlist(list, "ods list");
+    fprintf(stderr, "sheetname='%s'\n", sheetname);
+
     if (!err) {
 	if (gui) {
 	    int resp = ods_sheet_dialog(sheet, &err);
@@ -1264,6 +1342,8 @@ int ods_get_data (const char *fname,
 		ods_sheet_free(sheet);
 		return -1;
 	    } 
+	} else if (list != NULL && list[0] == 3) {
+	    err = set_ods_params_from_cli(sheet, list, sheetname);
 	} else {
 	    sheet->seltab = 0;
 	    sheet->xoffset = sheet->tables[0]->xoffset;
@@ -1277,12 +1357,12 @@ int ods_get_data (const char *fname,
 
     if (!err) {
 	err = finalize_ods_import(pZ, pdinfo, sheet, opt, prn); 
+	if (!err && gui) {
+	    record_ods_params(sheet, list);
+	}
     }
 
     ods_sheet_free(sheet);
 
     return err;
 }
-
-
-
