@@ -47,11 +47,12 @@ static int
 do_range_mean_plot (int n, const double **Z, double a, double b,
 		    const char *vname)
 {
-    FILE *fp = NULL;
+    FILE *fp;
     int fitline = 0;
-    int t, err;
+    int t, err = 0;
 
-    if ((err = gnuplot_init(PLOT_RANGE_MEAN, &fp))) {
+    fp = get_plot_input_stream(PLOT_RANGE_MEAN, &err);
+    if (err) {
 	return err;
     }
 
@@ -62,10 +63,10 @@ do_range_mean_plot (int n, const double **Z, double a, double b,
     fprintf(fp, "# for %s\n", vname);
     fputs("set nokey\n", fp);
     fprintf(fp, "set title '%s %s %s'\n", 
-	    G_("range-mean plot for"), vname, 
-	    (fitline)? G_("with least squares fit") : "");
+	    _("range-mean plot for"), vname, 
+	    (fitline)? _("with least squares fit") : "");
     fprintf(fp, "set xlabel '%s'\nset ylabel '%s'\n",
-	    G_("mean"), G_("range"));
+	    _("mean"), _("range"));
     fputs("plot \\\n", fp);
 
     gretl_push_c_numeric_locale();
@@ -84,7 +85,7 @@ do_range_mean_plot (int n, const double **Z, double a, double b,
 
     fclose(fp);
 
-    return 0;
+    return gnuplot_make_graph();
 }
 
 /* drop first/last observations from sample if missing obs 
@@ -110,7 +111,8 @@ rm_adjust_sample (int v, const double **Z, int *t1, int *t2)
     return 0;
 }
 
-int range_mean_graph (int vnum, const double **Z, DATAINFO *pdinfo, PRN *prn)
+int range_mean_graph (int vnum, const double **Z, DATAINFO *pdinfo, 
+		      gretlopt opt, PRN *prn)
 {
     double **rmZ;
     DATAINFO *rminfo;
@@ -118,6 +120,7 @@ int range_mean_graph (int vnum, const double **Z, DATAINFO *pdinfo, PRN *prn)
     int rmlist[4] = { 3, 1, 0, 2 };
     int k, t, m, nsamp, err = 0;
     int start, end, extra;
+    int verbose = !(opt & OPT_Q);
     double mean, range, tpval;
     char startdate[OBSLEN], enddate[OBSLEN];
     double a, b;
@@ -132,7 +135,7 @@ int range_mean_graph (int vnum, const double **Z, DATAINFO *pdinfo, PRN *prn)
     if (nsamp < 16) {
 	pputs(prn, _("Sample is too small for range-mean graph\n"));
 	errmsg(err, prn);
-	return 1;
+	return E_DATA;
     } 	
 
     if (pdinfo->pd > 1 && nsamp >= 3 * pdinfo->pd) {
@@ -149,14 +152,16 @@ int range_mean_graph (int vnum, const double **Z, DATAINFO *pdinfo, PRN *prn)
 	return E_ALLOC;
     }
 
-    pprintf(prn, _("Range-mean statistics for %s\n"), 
-	    pdinfo->varname[vnum]);
-    pprintf(prn, _("using %d sub-samples of size %d\n\n"),
-	    m, k);
-
-    pprintf(prn, "%30s%16s\n", _("range"), _("mean"));
+    if (verbose) {
+	pprintf(prn, _("Range-mean statistics for %s\n"), 
+		pdinfo->varname[vnum]);
+	pprintf(prn, _("using %d sub-samples of size %d\n\n"),
+		m, k);
+	pprintf(prn, "%30s%16s\n", _("range"), _("mean"));
+    }
 
     /* find group means and ranges */
+
     for (t=0; t<m; t++) {
 	char obsstr[32];
 
@@ -172,14 +177,16 @@ int range_mean_graph (int vnum, const double **Z, DATAINFO *pdinfo, PRN *prn)
 	rmZ[1][t] = range;
 	rmZ[2][t] = mean;
 
-	ntodate(startdate, start, pdinfo);
-	ntodate(enddate, end, pdinfo);
-	sprintf(obsstr, "%s - %s", startdate, enddate);
-	pputs(prn, obsstr);
-	bufspace(20 - strlen(obsstr), prn);
-	gretl_print_fullwidth_double(rmZ[1][t], GRETL_DIGITS, prn);
-	gretl_print_fullwidth_double(rmZ[2][t], GRETL_DIGITS, prn);
-	pputc(prn, '\n');
+	if (verbose) {
+	    ntodate(startdate, start, pdinfo);
+	    ntodate(enddate, end, pdinfo);
+	    sprintf(obsstr, "%s - %s", startdate, enddate);
+	    pputs(prn, obsstr);
+	    bufspace(20 - strlen(obsstr), prn);
+	    gretl_print_fullwidth_double(rmZ[1][t], GRETL_DIGITS, prn);
+	    gretl_print_fullwidth_double(rmZ[2][t], GRETL_DIGITS, prn);
+	    pputc(prn, '\n');
+	}
     }
 
     strcpy(rminfo->varname[1], "range");
@@ -193,12 +200,19 @@ int range_mean_graph (int vnum, const double **Z, DATAINFO *pdinfo, PRN *prn)
 	pputs(prn, _("Error estimating range-mean model\n"));
 	errmsg(err, prn);
     } else {
-	pputc(prn, '\n');
-	pprintf(prn, _("slope of range against mean = %g\n"),
-		rmmod.coeff[1]);
+	if (verbose) {
+	    pputc(prn, '\n');
+	    pprintf(prn, _("slope of range against mean = %g\n"),
+		    rmmod.coeff[1]);
+	}
 	if (rmmod.sderr[1] > 0) {
-	    tpval = student_pvalue_2(rmmod.coeff[1] / rmmod.sderr[1], rmmod.dfd);
-	    pprintf(prn, _("p-value for H0: slope = 0 is %g\n"), tpval);
+	    double tval = rmmod.coeff[1] / rmmod.sderr[1];
+
+	    tpval = student_pvalue_2(rmmod.dfd, tval);
+	    record_test_result(tval, tpval, _("range-mean test"));
+	    if (verbose) {
+		pprintf(prn, _("p-value for H0: slope = 0 is %g\n"), tpval);
+	    }
 	} else {
 	    tpval = 1.0;
 	}
@@ -209,7 +223,7 @@ int range_mean_graph (int vnum, const double **Z, DATAINFO *pdinfo, PRN *prn)
 	} 
     }
 
-    if (!gretl_in_batch_mode() && !gretl_looping()) {
+    if (verbose && !gretl_in_batch_mode() && !gretl_looping()) {
 	err = do_range_mean_plot(m, (const double **) rmZ, a, b, 
 				 pdinfo->varname[vnum]);
     }
