@@ -20,13 +20,18 @@
 #include "libgretl.h"
 #include "gretl_matrix.h"
 #include "system.h"
+#include "tsls.h"
 #include "sysml.h"
 
 #define SDEBUG 0
 #define SYS_USE_SVD 0
 
-/* insert the elements of sub-matrix M, multuplied by scale, in the
-   appropriate position within the big matrix X */
+#define sys_ols_ok(s) (s->method == SYS_METHOD_SUR || \
+                       s->method == SYS_METHOD_OLS || \
+                       s->method == SYS_METHOD_WLS)
+
+/* insert the elements of sub-matrix @M, multiplied by @scale, in the
+   appropriate position within the big matrix @X */
 
 static void 
 kronecker_place (gretl_matrix *X, const gretl_matrix *M,
@@ -46,8 +51,44 @@ kronecker_place (gretl_matrix *X, const gretl_matrix *M,
     }
 }
 
-/* retrieve the special k-class transformed data wanted for LIML
-   estimation */
+/* Retrieve the data placed on @pmod in the course of 2sls estimation:
+   for endogenous regressors these values are the first-stage fitted
+   values, otherwise they're just the original data.  Note that @endog
+   is a mask with nonzero values identifying the endogenous
+   regressors, and the special array @X contains a column for each
+   endogenous variable.
+*/
+
+double *model_get_Xi (const MODEL *pmod, double **Z, int i)
+{
+    const char *endog = gretl_model_get_data(pmod, "endog");
+    double **X;
+    double *xi = NULL;
+
+    if (endog == NULL || !endog[i]) {
+	/* return the original data */
+	return Z[pmod->list[i+2]];
+    }
+
+    X = gretl_model_get_data(pmod, "tslsX");
+
+    if (X != NULL) {
+	/* find and return the correct column */
+	int j, k = 0;
+
+	for (j=0; j<i; j++) {
+	    if (endog[j]) k++;
+	}
+	xi = X[k];
+    }
+
+    return xi;
+}
+
+/* retrieve the special k-class data wanted for LIML estimation: these
+   represent a further transformation of the data placed on the model
+   via 2sls estimation.
+*/
 
 static int make_liml_X_block (gretl_matrix *X, const MODEL *pmod,
 			      double **Z, int t1)
@@ -58,7 +99,7 @@ static int make_liml_X_block (gretl_matrix *X, const MODEL *pmod,
     X->cols = pmod->ncoeff;
 
     for (i=0; i<X->cols; i++) {
-	Xi = tsls_get_Xi(pmod, Z, i);
+	Xi = model_get_Xi(pmod, Z, i);
 	if (Xi == NULL) {
 	    return 1;
 	}
@@ -87,7 +128,7 @@ make_sys_X_block (gretl_matrix *X, const MODEL *pmod,
 	if (method == SYS_METHOD_3SLS || 
 	    method == SYS_METHOD_FIML || 
 	    method == SYS_METHOD_TSLS) {
-	    Xi = tsls_get_Xi(pmod, Z, i);
+	    Xi = model_get_Xi(pmod, Z, i);
 	} else {
 	    Xi = Z[pmod->list[i+2]];
 	}
@@ -150,12 +191,20 @@ gls_sigma_from_uhat (equation_system *sys, gretl_matrix *sigma)
     return 0;
 }
 
+/* Note: this was changed in Revision 1.86 of Fri Apr 30 2010. Up till
+   then we were reporting the regular R^2 for models estimated as part
+   of a system, which seems wrong.
+*/
+
+#define SYS_CORR_RSQ 1
+
 /* compute residuals, for all cases other than FIML */
 
 static void 
 sys_resids (equation_system *sys, int eq, const double **Z)
 {
     MODEL *pmod = sys->models[eq];
+    int yno = pmod->list[1];
     double yh;
     int i, t;
 
@@ -167,7 +216,7 @@ sys_resids (equation_system *sys, int eq, const double **Z)
 	    yh += pmod->coeff[i] * Z[pmod->list[i+2]][t];
 	}
 	pmod->yhat[t] = yh;
-	pmod->uhat[t] = Z[pmod->list[1]][t] - yh;
+	pmod->uhat[t] = Z[yno][t] - yh;
 	/* for cross-equation vcv */
 	gretl_matrix_set(sys->E, t - pmod->t1, pmod->ID, pmod->uhat[t]);
 	pmod->ess += pmod->uhat[t] * pmod->uhat[t];
@@ -179,6 +228,21 @@ sys_resids (equation_system *sys, int eq, const double **Z)
     } else {
 	pmod->sigma = sqrt(pmod->ess / pmod->nobs);
     }
+
+    if (pmod->ifc && pmod->tss > 0) {
+	/* R-squared */
+	double r;
+
+#if SYS_CORR_RSQ
+	pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, Z[yno], pmod->yhat);
+#else
+	pmod->rsq = 1 - (pmod->ess / pmod->tss);
+#endif
+	r = 1 - pmod->rsq;
+	pmod->adjrsq = 1.0 - (r * (pmod->nobs - 1) / pmod->dfd);
+    } else {
+	pmod->rsq = pmod->adjrsq = NADBL;
+    }
 }
 
 static void
@@ -283,7 +347,7 @@ calculate_sys_coeffs (equation_system *sys,
 
     vcv = gretl_matrix_copy(X);
     if (vcv == NULL) {
-	return 1;
+	return E_ALLOC;
     }
 
     err = gretl_LU_solve(X, y);
@@ -368,17 +432,6 @@ calculate_sys_coeffs (equation_system *sys,
     return err;
 }
 
-static int in_list (const int *list, int k)
-{
-    int i;
-
-    for (i=1; i<=list[0]; i++) {
-	if (k == list[i]) return 1;
-    }
-
-    return 0;
-}
-
 static int *
 system_model_list (equation_system *sys, int i, int *freeit)
 {
@@ -386,27 +439,30 @@ system_model_list (equation_system *sys, int i, int *freeit)
 
     *freeit = 0;
 
-    if (sys->method == SYS_METHOD_SUR || 
-	sys->method == SYS_METHOD_3SLS ||
-	sys->method == SYS_METHOD_OLS || 
-	sys->method == SYS_METHOD_TSLS ||
-	sys->method == SYS_METHOD_WLS) {
+    if (sys_ols_ok(sys)) {
+	return system_get_list(sys, i);
+    }
+
+    if (sys->method != SYS_METHOD_FIML) {
 	list = system_get_list(sys, i);
     }
 
     if (sys->method == SYS_METHOD_3SLS || 
-	sys->method == SYS_METHOD_TSLS) {
-	/* is list already in tsls form? */
-	if (list != NULL && !in_list(list, LISTSEP)) {
+	sys->method == SYS_METHOD_TSLS ||
+	sys->method == SYS_METHOD_LIML) {
+	/* Is the list already in ivreg form? 
+	   If not, ignore it. */
+	if (list != NULL && !gretl_list_has_separator(list)) {
 	    list = NULL;
 	}
     }
 
-    if (sys->method == SYS_METHOD_FIML || 
-	sys->method == SYS_METHOD_LIML ||
-	((sys->method == SYS_METHOD_3SLS || 
-	  sys->method == SYS_METHOD_TSLS) && list == NULL)) {
-	list = compose_tsls_list(sys, i);
+    if ((sys->method == SYS_METHOD_FIML || 
+	 sys->method == SYS_METHOD_LIML ||
+	 sys->method == SYS_METHOD_3SLS || 
+	 sys->method == SYS_METHOD_TSLS) 
+	&& list == NULL) {
+	list = compose_ivreg_list(sys, i);
 	*freeit = 1;
     }
 
@@ -425,17 +481,14 @@ static int hansen_sargan_test (equation_system *sys,
 			       const double **Z)
 {
     const int *exlist = system_get_instr_vars(sys);
+    const double *Wi, *Wj;
     int nx = exlist[0];
     int m = sys->neqns;
     int T = sys->T;
     int df = system_get_overid_df(sys);
-
-    const double *Wi, *Wj;
-
     gretl_matrix *WTW = NULL;
     gretl_matrix *eW = NULL;
     gretl_matrix *tmp = NULL;
-
     double x, X2;
     int i, j, t;
     int err = 0;
@@ -517,7 +570,7 @@ static int hansen_sargan_test (equation_system *sys,
 
 #if SDEBUG
     fprintf(stderr, "Hansen-Sargan: Chi-square(%d) = %g (p-value %g)\n", 
-	    df, X2, chisq_cdf_comp(X2, df));
+	    df, X2, chisq_cdf_comp(df, X2));
 #endif
     sys->X2 = X2;
 
@@ -541,6 +594,9 @@ static int basic_system_allocate (equation_system *sys,
 
     /* allocate a model for each stochastic equation */
     sys->models = gretl_model_array_new(m);
+    if (sys->models == NULL) {
+	return E_ALLOC;
+    }    
 
     sys->E = gretl_matrix_alloc(T, m);
     if (sys->E == NULL) {
@@ -571,14 +627,16 @@ double sur_ll (equation_system *sys)
 {
     int m = sys->neqns;
     int T = sys->T;
-    gretl_matrix *sigtmp;
+    gretl_matrix *tmp;
     double ldet;
 
-    sigtmp = gretl_matrix_alloc(m, m);
-    if (sigtmp == NULL) return NADBL;
+    tmp = gretl_matrix_alloc(m, m);
+    if (tmp == NULL) {
+	return NADBL;
+    }
 
-    gls_sigma_from_uhat(sys, sigtmp);
-    ldet = gretl_vcv_log_determinant(sigtmp);
+    gls_sigma_from_uhat(sys, tmp);
+    ldet = gretl_vcv_log_determinant(tmp);
 
     if (na(ldet)) {
 	sys->ll = NADBL;
@@ -587,7 +645,7 @@ double sur_ll (equation_system *sys)
 	sys->ll -= (T / 2.0) * ldet;
     }
 
-    gretl_matrix_free(sigtmp);
+    gretl_matrix_free(tmp);
 
     return sys->ll;
 }
@@ -638,7 +696,9 @@ augment_y_with_restrictions (gretl_matrix *y, int mk, int nr,
 {
     int i;
 
-    if (sys->q == NULL) return 1;
+    if (sys->q == NULL) {
+	return 1;
+    }
 
     for (i=0; i<nr; i++) {
 	gretl_vector_set(y, mk + i, gretl_vector_get(sys->q, i));
@@ -705,24 +765,35 @@ static int sys_converged (equation_system *sys,
 
 static void clean_up_models (equation_system *sys)
 {
-    double ess = 0.0;
     int i;
 
+    if (sys->models == NULL) {
+	/* "can't happen" */
+	return;
+    }
+
+    sys->ess = 0.0;
+
     for (i=0; i<sys->neqns; i++) {
-	ess += sys->models[i]->ess;
+	sys->ess += sys->models[i]->ess;
 	if (sys->method == SYS_METHOD_3SLS || 
 	    sys->method == SYS_METHOD_FIML || 
 	    sys->method == SYS_METHOD_TSLS || 
 	    sys->method == SYS_METHOD_LIML) {
 	    tsls_free_data(sys->models[i]);
 	}
-	gretl_model_free(sys->models[i]);
+	if (sys->neqns > 1) {
+	    gretl_model_free(sys->models[i]);
+	}
     }
 
-    free(sys->models);
-    sys->models = NULL;
-
-    sys->ess = ess;
+    if (sys->neqns == 1) {
+	/* ivreg (LIML) */
+	sys->models[0]->rho = sys->models[0]->dw = NADBL;
+    } else {
+	free(sys->models);
+	sys->models = NULL;
+    }
 }
 
 static int drop_redundant_instruments (equation_system *sys,
@@ -754,24 +825,39 @@ static int drop_redundant_instruments (equation_system *sys,
     return err;
 }
 
+/* options to be passed in running initial 2sls */
+
+static gretlopt sys_tsls_opt (const equation_system *sys)
+{
+    gretlopt opt = (OPT_E | OPT_A);
+
+    if (sys->method == SYS_METHOD_LIML) {
+	opt |= OPT_N; /* ML: no df correction */
+    }
+
+    if (sys->flags & SYSTEM_SINGLE) {
+	opt |= OPT_H; /* add "hatlist" of instrumented vars */
+    }
+
+    return opt;
+}
+
 /* general function that forms the basis for all specific system
    estimators */
 
 int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo, 
 		     gretlopt opt, PRN *prn)
 {
-    int i, j, k, T, t, m = 0;
+    int i, j, k, T, t;
     int v, l, mk, krow, nr;
     int orig_t1 = pdinfo->t1;
     int orig_t2 = pdinfo->t2;
-
     gretl_matrix *X = NULL;
     gretl_matrix *y = NULL;
     gretl_matrix *Xi = NULL;
     gretl_matrix *Xj = NULL;
     gretl_matrix *M = NULL;
     MODEL **models = NULL;
-
     int method = sys->method;
     double llbak = -1.0e9;
     int single_equation = 0;
@@ -798,15 +884,16 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	rtsls = 1;
     }
 
-    /* get uniform sample starting and ending points */
-    if (system_adjust_t1t2(sys, &pdinfo->t1, &pdinfo->t2, 
-			   (const double **) *pZ)) {
-	err = E_DATA;
-	goto cleanup;
+    /* get uniform sample starting and ending points and check for
+       missing data */
+    err = system_adjust_t1t2(sys, (const double **) *pZ, pdinfo);
+    if (err) {
+	return err;
     } 
 
-    /* number of equations */
-    m = sys->neqns;
+    /* set sample for auxiliary regressions */
+    pdinfo->t1 = sys->t1;
+    pdinfo->t2 = sys->t2;
 
     /* max indep vars per equation */
     k = system_max_indep_vars(sys);
@@ -814,12 +901,18 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     /* total indep vars, all equations */
     mk = system_n_indep_vars(sys);
 
+    /* set sample for auxiliary regressions */
+    pdinfo->t1 = sys->t1;
+    pdinfo->t2 = sys->t2;
+
     /* number of observations per series */
     T = sys->T;
 
     /* allocate models etc */
     err = basic_system_allocate(sys, mk, nr, &X, &y);
-    if (err) goto cleanup;
+    if (err) {
+	goto cleanup;
+    }
 
     /* convenience pointers */
     models = sys->models;
@@ -836,7 +929,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
        straight equation-by-equation estimation.
     */
 
-    for (i=0; i<m; i++) {
+    for (i=0; i<sys->neqns; i++) {
 	int freeit = 0;
 	int *list = system_model_list(sys, i, &freeit);
 	const int *droplist = NULL;
@@ -847,16 +940,10 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	    break;
 	}
 
-	if (method == SYS_METHOD_SUR || 
-	    method == SYS_METHOD_OLS || 
-	    method == SYS_METHOD_WLS) {
+	if (sys_ols_ok(sys)) {
 	    *pmod = lsq(list, pZ, pdinfo, OLS, OPT_A);
-	} else if (method == SYS_METHOD_3SLS || 
-		   method == SYS_METHOD_FIML || 
-		   method == SYS_METHOD_TSLS) {
-	    *pmod = tsls_func(list, SYSTEM, pZ, pdinfo, OPT_NONE);
-	} else if (method == SYS_METHOD_LIML) {
-	    *pmod = tsls_func(list, SYSTEM, pZ, pdinfo, OPT_N);
+	} else {
+	    *pmod = tsls(list, pZ, pdinfo, sys_tsls_opt(sys));
 	}
 
 	if (freeit) {
@@ -898,7 +985,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 
     if (method == SYS_METHOD_LIML) {
 	/* compute the minimum eigenvalues and generate the
-	   suitably transformed data matrices */
+	   k-class data matrices */
 	err = liml_driver(sys, pZ, pdinfo, prn);
 	if (err) goto cleanup;
     }
@@ -953,12 +1040,12 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     */
 
     krow = 0;
-    for (i=0; i<m && !err; i++) {
+    for (i=0; i<sys->neqns && !err; i++) {
 	int kcol = 0;
 
 	err = make_sys_X_block(Xi, models[i], *pZ, sys->t1, method);
 
-	for (j=0; j<m && !err; j++) { 
+	for (j=0; j<sys->neqns && !err; j++) { 
 	    const gretl_matrix *Xk;
 	    double sij;
 
@@ -1017,7 +1104,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     /* form stacked Y column vector (m x k) */
 
     v = 0;
-    for (i=0; i<m; i++) { 
+    for (i=0; i<sys->neqns; i++) { 
 
 	/* loop over the m vertically-arranged blocks */
 
@@ -1026,7 +1113,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	for (j=0; j<models[i]->ncoeff; j++) { 
 	    /* loop over the rows within each of the m blocks */
 	    double yv = 0.0;
-	    int lmin = 0, lmax = m;
+	    int lmin = 0, lmax = sys->neqns;
 
 	    if (single_equation || rtsls) {
 		/* no cross terms wanted */
@@ -1078,8 +1165,11 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
        unless, that is, we're just doing restricted OLS, WLS or TSLS
        estimates.
     */
-    calculate_sys_coeffs(sys, (const double **) *pZ, X, 
-			 y, mk, nr, do_iteration);
+    err = calculate_sys_coeffs(sys, (const double **) *pZ, X, 
+			       y, mk, nr, do_iteration);
+    if (err) {
+	goto cleanup;
+    }
 
     if (rtsls) {
 	rtsls = 0;
@@ -1110,7 +1200,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
 	err = fiml_driver(sys, pZ, pdinfo, opt, prn);
     }
 
-    if (!err) {
+    if (!err && !(sys->flags & SYSTEM_SINGLE)) {
 	err = system_save_and_print_results(sys, pZ, pdinfo, 
 					    opt, prn);
     }
@@ -1123,9 +1213,7 @@ int system_estimate (equation_system *sys, double ***pZ, DATAINFO *pdinfo,
     gretl_matrix_free(X);
     gretl_matrix_free(y);
 
-    if (models != NULL) {
-	clean_up_models(sys);
-    }
+    clean_up_models(sys);
 
     pdinfo->t1 = orig_t1;
     pdinfo->t2 = orig_t2;
